@@ -1,5 +1,5 @@
 #!/bin/sh -f
-# $RCSfile: pv.sh,v $ $Revision: 1.1.2.1 $
+# $Id: pv.sh 8104 2007-07-05 10:41:52Z till $
 #
 # pv - preview a specified page of a dvi file in a Ghostscript window
 # usage: pv page file
@@ -25,13 +25,26 @@ fi
 # are held on the system."  So we have deleted this line, and also removed
 # the -D switch from the call of dvips below.
 #
+
+# This definition is changed on install to match the
+# executable name set in the makefile
+GS_EXECUTABLE=gs
+
 TEMPDIR=.
 PAGE=$1
 shift
-FILE=$1
+FILE="$1"
 shift
-trap "rm -rf $TEMPDIR/$FILE.$$.pv" 0 1 2 15
-#dvips -D$RESOLUTION -p $PAGE -n 1 $FILE $* -o $FILE.$$.pv
-dvips -p $PAGE -n 1 $FILE $* -o $FILE.$$.pv
-gs $FILE.$$.pv
+if test -z "$TEMPDIR"; then
+	TEMPDIR=/tmp
+fi
+if which mktemp >/dev/null 2>/dev/null; then
+	tmpfile="`mktemp $TEMPDIR/\"$FILE\".pv.XXXXXX`"
+else
+	tmpfile="$TEMPDIR/$FILE.$$.pv"
+fi
+trap "rm -rf $tmpfile" 0 1 2 15
+#dvips -D$RESOLUTION -p $PAGE -n 1 "$FILE" "$@" -o $tmpfile
+dvips -p $PAGE -n 1 "$FILE" "$@" -o $tmpfile
+$GS_EXECUTABLE $tmpfile
 exit 0
