@@ -1,23 +1,21 @@
-/* Copyright (C) 1992, 1995, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$Id: echogs.c,v 1.3.2.1 2002/02/22 19:45:55 ray Exp $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: echogs.c 8022 2007-06-05 22:23:38Z giles $ */
 /* 'echo'-like utility */
+
 #include "stdpre.h"
 #include <stdio.h>
+#include <stdlib.h>
 
 #if defined(__sun__) && !defined(const)
 /* Apparently, there are archaic Sun platforms which don't include
@@ -25,7 +23,7 @@
  * cause type mismatches if const has been defined (usually with
  * -Dconst=), so it's only included if const is undefined.
  */
-extern int fputc(P2(int, FILE *)), fputs(P2(const char *, FILE *));
+extern int fputc(int, FILE *), fputs(const char *, FILE *);
 #endif
 
 /* Some systems have time_t in sys/types.h rather than time.h. */
@@ -33,9 +31,6 @@ extern int fputc(P2(int, FILE *)), fputs(P2(const char *, FILE *));
 #include <ctype.h>
 #include <string.h>
 #include <time.h>		/* for ctime */
-#ifdef VMS
-#include <stdlib.h>
-#endif
 
 /*
  * This program exists solely to get around omissions, problems, and
@@ -82,7 +77,7 @@ extern int fputc(P2(int, FILE *)), fputs(P2(const char *, FILE *));
  * which writes 'a b'.
  */
 
-static int hputc(P2(int, FILE *)), hputs(P2(const char *, FILE *));
+static int hputc(int, FILE *), hputs(const char *, FILE *);
 
 int
 main(int argc, char *argv[])
@@ -101,8 +96,8 @@ main(int argc, char *argv[])
     char fname[FNSIZE];
     int newline = 1;
     int interact = 0;
-    int (*eputc)(P2(int, FILE *)) = fputc;
-    int (*eputs)(P2(const char *, FILE *)) = fputs;
+    int (*eputc)(int, FILE *) = fputc;
+    int (*eputs)(const char *, FILE *) = fputs;
 #define LINESIZE 1000
     char line[LINESIZE];
     char sw = 0, sp = 0, hexx = 0;
