@@ -1,22 +1,16 @@
-/* Copyright (C) 1996, 2000 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-
-/*$Id: gdevbbox.c,v 1.4.2.1.2.1 2003/01/17 00:49:00 giles Exp $ */
+/*$Id: gdevbbox.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Device for tracking bounding box */
 #include "math_.h"
 #include "memory_.h"
@@ -37,34 +31,34 @@
 public_st_device_bbox();
 
 /* Device procedures */
-private dev_proc_open_device(bbox_open_device);
-private dev_proc_close_device(bbox_close_device);
-private dev_proc_output_page(bbox_output_page);
-private dev_proc_fill_rectangle(bbox_fill_rectangle);
-private dev_proc_copy_mono(bbox_copy_mono);
-private dev_proc_copy_color(bbox_copy_color);
-private dev_proc_get_params(bbox_get_params);
-private dev_proc_put_params(bbox_put_params);
-private dev_proc_copy_alpha(bbox_copy_alpha);
-private dev_proc_fill_path(bbox_fill_path);
-private dev_proc_stroke_path(bbox_stroke_path);
-private dev_proc_fill_mask(bbox_fill_mask);
-private dev_proc_fill_trapezoid(bbox_fill_trapezoid);
-private dev_proc_fill_parallelogram(bbox_fill_parallelogram);
-private dev_proc_fill_triangle(bbox_fill_triangle);
-private dev_proc_draw_thin_line(bbox_draw_thin_line);
-private dev_proc_strip_tile_rectangle(bbox_strip_tile_rectangle);
-private dev_proc_strip_copy_rop(bbox_strip_copy_rop);
-private dev_proc_begin_typed_image(bbox_begin_typed_image);
-private dev_proc_create_compositor(bbox_create_compositor);
-private dev_proc_text_begin(bbox_text_begin);
+static dev_proc_open_device(bbox_open_device);
+static dev_proc_close_device(bbox_close_device);
+static dev_proc_output_page(bbox_output_page);
+static dev_proc_fill_rectangle(bbox_fill_rectangle);
+static dev_proc_copy_mono(bbox_copy_mono);
+static dev_proc_copy_color(bbox_copy_color);
+static dev_proc_get_params(bbox_get_params);
+static dev_proc_put_params(bbox_put_params);
+static dev_proc_copy_alpha(bbox_copy_alpha);
+static dev_proc_fill_path(bbox_fill_path);
+static dev_proc_stroke_path(bbox_stroke_path);
+static dev_proc_fill_mask(bbox_fill_mask);
+static dev_proc_fill_trapezoid(bbox_fill_trapezoid);
+static dev_proc_fill_parallelogram(bbox_fill_parallelogram);
+static dev_proc_fill_triangle(bbox_fill_triangle);
+static dev_proc_draw_thin_line(bbox_draw_thin_line);
+static dev_proc_strip_tile_rectangle(bbox_strip_tile_rectangle);
+static dev_proc_strip_copy_rop(bbox_strip_copy_rop);
+static dev_proc_begin_typed_image(bbox_begin_typed_image);
+static dev_proc_create_compositor(bbox_create_compositor);
+static dev_proc_text_begin(bbox_text_begin);
 
 /* The device prototype */
 /*
- * Normally this would be private, but if the device is going to be used
+ * Normally this would be static, but if the device is going to be used
  * stand-alone, it has to be public.
  */
-/*private */ const
+/*static*/ const
 /*
  * The bbox device sets the resolution to some value R (currently 4000), and
  * the page size in device pixels to slightly smaller than the largest
@@ -131,7 +125,16 @@ gx_device_bbox gs_bbox_device =
      bbox_create_compositor,
      NULL,			/* get_hardware_params */
      bbox_text_begin,
-     NULL			/* finish_copydevice */
+     NULL,			/* finish_copydevice */
+     NULL,			/* begin_transparency_group */
+     NULL,			/* end_transparency_group */
+     NULL,			/* begin_transparency_mask */
+     NULL,			/* end_transparency_mask */
+     NULL,			/* discard_transparency_layer */
+     NULL,			/* get_color_mapping_procs */
+     NULL,			/* get_color_comp_index */
+     NULL,			/* encode_color */
+     NULL			/* decode_color */
     },
     0,				/* target */
     1,				/*true *//* free_standing */
@@ -197,7 +200,7 @@ bbox_default_in_rect(const void *pdata, const gs_fixed_rect *pbox)
 #define BBOX_IN_RECT(bdev, pbox)\
     bdev->box_procs.in_rect(bdev->box_proc_data, pbox)
 
-private const gx_device_bbox_procs_t box_procs_default = {
+static const gx_device_bbox_procs_t box_procs_default = {
     bbox_default_init_box, bbox_default_get_box, bbox_default_add_rect,
     bbox_default_in_rect
 };
@@ -208,7 +211,7 @@ private const gx_device_bbox_procs_t box_procs_default = {
      /* ---------------- Open/close/page ---------------- */
 
 /* Copy device parameters back from the target. */
-private void
+static void
 bbox_copy_params(gx_device_bbox * bdev, bool remap_colors)
 {
     gx_device *tdev = bdev->target;
@@ -226,7 +229,7 @@ bbox_copy_params(gx_device_bbox * bdev, bool remap_colors)
 #define GX_DC_IS_TRANSPARENT(pdevc, bdev)\
   (gx_dc_pure_color(pdevc) == (bdev)->transparent && gx_dc_is_pure(pdevc))
 
-private int
+static int
 bbox_close_device(gx_device * dev)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
@@ -248,19 +251,32 @@ bbox_close_device(gx_device * dev)
 
 /* Initialize a bounding box device. */
 void
-gx_device_bbox_init(gx_device_bbox * dev, gx_device * target)
+gx_device_bbox_init(gx_device_bbox * dev, gx_device * target, gs_memory_t *mem)
 {
     gx_device_init((gx_device *) dev, (const gx_device *)&gs_bbox_device,
-		   (target ? target->memory : NULL), true);
-    gx_device_forward_fill_in_procs((gx_device_forward *) dev);
+		   (target ? target->memory : mem), true);
     if (target) {
+        gx_device_forward_fill_in_procs((gx_device_forward *) dev);
 	set_dev_proc(dev, get_initial_matrix, gx_forward_get_initial_matrix);
 	set_dev_proc(dev, map_rgb_color, gx_forward_map_rgb_color);
 	set_dev_proc(dev, map_color_rgb, gx_forward_map_color_rgb);
 	set_dev_proc(dev, map_cmyk_color, gx_forward_map_cmyk_color);
 	set_dev_proc(dev, map_rgb_alpha_color, gx_forward_map_rgb_alpha_color);
-	set_dev_proc(dev, map_color_rgb_alpha, gx_forward_map_color_rgb_alpha);
+	set_dev_proc(dev, get_color_mapping_procs, gx_forward_get_color_mapping_procs);
+	set_dev_proc(dev, get_color_comp_index, gx_forward_get_color_comp_index);
+	set_dev_proc(dev, encode_color, gx_forward_encode_color);
+	set_dev_proc(dev, decode_color, gx_forward_decode_color);
+	set_dev_proc(dev, pattern_manage, gx_forward_pattern_manage);
+	set_dev_proc(dev, fill_rectangle_hl_color, gx_forward_fill_rectangle_hl_color);
+	set_dev_proc(dev, include_color_space, gx_forward_include_color_space);
+	set_dev_proc(dev, update_spot_equivalent_colors,
+				gx_forward_update_spot_equivalent_colors);
+	set_dev_proc(dev, get_page_device, gx_forward_get_page_device);
+	set_dev_proc(dev, ret_devn_params, gx_forward_ret_devn_params);
 	gx_device_set_target((gx_device_forward *)dev, target);
+    } else {
+	gx_device_fill_in_procs((gx_device *)dev);
+        gx_device_forward_fill_in_procs((gx_device_forward *) dev);
     }
     dev->box_procs = box_procs_default;
     dev->box_proc_data = dev;
@@ -315,7 +331,7 @@ gx_device_bbox_bbox(gx_device_bbox * dev, gs_rect * pbbox)
     }
 }
 
-private int
+static int
 bbox_open_device(gx_device * dev)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
@@ -338,7 +354,7 @@ bbox_open_device(gx_device * dev)
     }
 }
 
-private int
+static int
 bbox_output_page(gx_device * dev, int num_copies, int flush)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
@@ -361,7 +377,7 @@ bbox_output_page(gx_device * dev, int num_copies, int flush)
 
 /* ---------------- Low-level drawing ---------------- */
 
-private int
+static int
 bbox_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 		    gx_color_index color)
 {
@@ -382,7 +398,7 @@ bbox_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     return code;
 }
 
-private int
+static int
 bbox_copy_mono(gx_device * dev, const byte * data,
 	    int dx, int raster, gx_bitmap_id id, int x, int y, int w, int h,
 	       gx_color_index zero, gx_color_index one)
@@ -402,7 +418,7 @@ bbox_copy_mono(gx_device * dev, const byte * data,
     return code;
 }
 
-private int
+static int
 bbox_copy_color(gx_device * dev, const byte * data,
 	    int dx, int raster, gx_bitmap_id id, int x, int y, int w, int h)
 {
@@ -418,7 +434,7 @@ bbox_copy_color(gx_device * dev, const byte * data,
     return code;
 }
 
-private int
+static int
 bbox_copy_alpha(gx_device * dev, const byte * data, int data_x,
 		int raster, gx_bitmap_id id, int x, int y, int w, int h,
 		gx_color_index color, int depth)
@@ -435,7 +451,7 @@ bbox_copy_alpha(gx_device * dev, const byte * data, int data_x,
     return code;
 }
 
-private int
+static int
 bbox_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
    int x, int y, int w, int h, gx_color_index color0, gx_color_index color1,
 			  int px, int py)
@@ -456,7 +472,7 @@ bbox_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
     return code;
 }
 
-private int
+static int
 bbox_strip_copy_rop(gx_device * dev,
 		    const byte * sdata, int sourcex, uint sraster,
 		    gx_bitmap_id id,
@@ -482,7 +498,7 @@ bbox_strip_copy_rop(gx_device * dev,
 /* ---------------- Parameters ---------------- */
 
 /* We implement get_params to provide a way to read out the bounding box. */
-private int
+static int
 bbox_get_params(gx_device * dev, gs_param_list * plist)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
@@ -506,18 +522,23 @@ bbox_get_params(gx_device * dev, gs_param_list * plist)
     bbox[2] = fixed2float(fbox.q.x);
     bbox[3] = fixed2float(fbox.q.y);
     bba.data = bbox, bba.size = 4, bba.persistent = false;
-    return param_write_float_array(plist, "PageBoundingBox", &bba);
+    code = param_write_float_array(plist, "PageBoundingBox", &bba);
+    if (code < 0)
+        return code;
+    code = param_write_bool(plist, "WhiteIsOpaque", &bdev->white_is_opaque);
+    return code;
 }
 
 /* We implement put_params to ensure that we keep the important */
 /* device parameters up to date, and to prevent an /undefined error */
 /* from PageBoundingBox. */
-private int
+static int
 bbox_put_params(gx_device * dev, gs_param_list * plist)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
     int code;
     int ecode = 0;
+    bool white_is_opaque = bdev->white_is_opaque;
     gs_param_name param_name;
     gs_param_float_array bba;
 
@@ -532,33 +553,45 @@ bbox_put_params(gx_device * dev, gs_param_list * plist)
 	    break;
 	default:
 	    ecode = code;
-	  e:param_signal_error(plist, param_name, ecode);
+	    e:param_signal_error(plist, param_name, ecode);
 	case 1:
 	    bba.data = 0;
     }
 
+    switch (code = param_read_bool(plist, (param_name = "WhiteIsOpaque"), &white_is_opaque)) {
+	default:
+	    ecode = code;
+	    param_signal_error(plist, param_name, ecode);
+	case 0:
+        case 1:
+	    break;
+    }
+
     code = gx_forward_put_params(dev, plist);
     if (ecode < 0)
 	code = ecode;
-    if (code >= 0 && bba.data != 0) {
-	BBOX_INIT_BOX(bdev);
-	BBOX_ADD_RECT(bdev, float2fixed(bba.data[0]), float2fixed(bba.data[1]),
-		      float2fixed(bba.data[2]), float2fixed(bba.data[3]));
+    if (code >= 0) {
+        if( bba.data != 0) {
+	    BBOX_INIT_BOX(bdev);
+	    BBOX_ADD_RECT(bdev, float2fixed(bba.data[0]), float2fixed(bba.data[1]),
+	    	          float2fixed(bba.data[2]), float2fixed(bba.data[3]));
+        }
+        bdev->white_is_opaque = white_is_opaque;
     }
-    bbox_copy_params(bdev, true);
+    bbox_copy_params(bdev, bdev->is_open);
     return code;
 }
 
 /* ---------------- Polygon drawing ---------------- */
 
-private fixed
+static fixed
 edge_x_at_y(const gs_fixed_edge * edge, fixed y)
 {
     return fixed_mult_quo(edge->end.x - edge->start.x,
 			  y - edge->start.y,
 			  edge->end.y - edge->start.y) + edge->start.x;
 }
-private int
+static int
 bbox_fill_trapezoid(gx_device * dev,
 		    const gs_fixed_edge * left, const gs_fixed_edge * right,
 		    fixed ybot, fixed ytop, bool swap_axes,
@@ -597,7 +630,7 @@ bbox_fill_trapezoid(gx_device * dev,
     return code;
 }
 
-private int
+static int
 bbox_fill_parallelogram(gx_device * dev,
 			fixed px, fixed py, fixed ax, fixed ay,
 			fixed bx, fixed by, const gx_device_color * pdevc,
@@ -635,7 +668,7 @@ bbox_fill_parallelogram(gx_device * dev,
     return code;
 }
 
-private int
+static int
 bbox_fill_triangle(gx_device * dev,
 		   fixed px, fixed py, fixed ax, fixed ay, fixed bx, fixed by,
 		   const gx_device_color * pdevc, gs_logical_operation_t lop)
@@ -672,7 +705,7 @@ bbox_fill_triangle(gx_device * dev,
     return code;
 }
 
-private int
+static int
 bbox_draw_thin_line(gx_device * dev,
 		    fixed fx0, fixed fy0, fixed fx1, fixed fy1,
 		    const gx_device_color * pdevc, gs_logical_operation_t lop)
@@ -710,7 +743,7 @@ bbox_draw_thin_line(gx_device * dev,
 ((pbox)->p.x -= (adj).x, (pbox)->p.y -= (adj).y,\
  (pbox)->q.x += (adj).x, (pbox)->q.y += (adj).y)
 
-private int
+static int
 bbox_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 	       const gx_fill_params * params, const gx_device_color * pdevc,
 	       const gx_clip_path * pcpath)
@@ -758,7 +791,7 @@ bbox_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 	     */
 	    gx_drawing_color devc;
 
-	    color_set_pure(&devc, bdev->black);  /* any non-white color will do */
+	    set_nonclient_dev_color(&devc, bdev->black);  /* any non-white color will do */
 	    bdev->target = NULL;
 	    code = gx_default_fill_path(dev, pis, ppath, params, &devc, pcpath);
 	    bdev->target = tdev;
@@ -770,7 +803,7 @@ bbox_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 	return fill_path(tdev, pis, ppath, params, pdevc, pcpath);
 }
 
-private int
+static int
 bbox_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 		 const gx_stroke_params * params,
 		 const gx_drawing_color * pdevc, const gx_clip_path * pcpath)
@@ -820,7 +853,7 @@ bbox_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 	    /* fill path into pieces for computing the bounding box. */
 	    gx_drawing_color devc;
 
-	    color_set_pure(&devc, bdev->black);  /* any non-white color will do */
+	    set_nonclient_dev_color(&devc, bdev->black);  /* any non-white color will do */
 	    bdev->target = NULL;
 	    gx_default_stroke_path(dev, pis, ppath, params, &devc, pcpath);
 	    bdev->target = tdev;
@@ -832,7 +865,7 @@ bbox_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
     return code;
 }
 
-private int
+static int
 bbox_fill_mask(gx_device * dev,
 	       const byte * data, int dx, int raster, gx_bitmap_id id,
 	       int x, int y, int w, int h,
@@ -883,16 +916,16 @@ gs_private_st_suffix_add2(st_bbox_image_enum, bbox_image_enum,
   "bbox_image_enum", bbox_image_enum_enum_ptrs, bbox_image_enum_reloc_ptrs,
   st_gx_image_enum_common, pcpath, target_info);
 
-private image_enum_proc_plane_data(bbox_image_plane_data);
-private image_enum_proc_end_image(bbox_image_end_image);
-private image_enum_proc_flush(bbox_image_flush);
-private image_enum_proc_planes_wanted(bbox_image_planes_wanted);
-private const gx_image_enum_procs_t bbox_image_enum_procs = {
+static image_enum_proc_plane_data(bbox_image_plane_data);
+static image_enum_proc_end_image(bbox_image_end_image);
+static image_enum_proc_flush(bbox_image_flush);
+static image_enum_proc_planes_wanted(bbox_image_planes_wanted);
+static const gx_image_enum_procs_t bbox_image_enum_procs = {
     bbox_image_plane_data, bbox_image_end_image,
     bbox_image_flush, bbox_image_planes_wanted
 };
 
-private int
+static int
 bbox_image_begin(const gs_imager_state * pis, const gs_matrix * pmat,
 		 const gs_image_common_t * pic, const gs_int_rect * prect,
 		 const gx_clip_path * pcpath, gs_memory_t * memory,
@@ -935,7 +968,7 @@ bbox_image_begin(const gs_imager_state * pis, const gs_matrix * pmat,
     return 0;
 }
 
-private void
+static void
 bbox_image_copy_target_info(bbox_image_enum * pbe)
 {
     const gx_image_enum_common_t *target_info = pbe->target_info;
@@ -947,7 +980,7 @@ bbox_image_copy_target_info(bbox_image_enum * pbe)
 	   pbe->num_planes * sizeof(pbe->plane_widths[0]));
 }
 
-private int
+static int
 bbox_begin_typed_image(gx_device * dev,
 		       const gs_imager_state * pis, const gs_matrix * pmat,
 		   const gs_image_common_t * pic, const gs_int_rect * prect,
@@ -1003,7 +1036,7 @@ bbox_begin_typed_image(gx_device * dev,
     return 0;
 }
 
-private int
+static int
 bbox_image_plane_data(gx_image_enum_common_t * info,
 		      const gx_image_plane_t * planes, int height,
 		      int *rows_used)
@@ -1043,10 +1076,8 @@ bbox_image_plane_data(gx_image_enum_common_t * info,
 	fixed x0 = float2fixed(corners[0].x), y0 = float2fixed(corners[0].y);
 	fixed bx2 = float2fixed(corners[2].x) - x0, by2 = float2fixed(corners[2].y) - y0;
 
-	gx_make_clip_path_device(&cdev, pcpath);
-	cdev.target = dev;
-	(*dev_proc(&cdev, open_device)) ((gx_device *) & cdev);
-	color_set_pure(&devc, bdev->black);  /* any non-white color will do */
+	gx_make_clip_device_on_stack(&cdev, pcpath, dev);
+	set_nonclient_dev_color(&devc, bdev->black);  /* any non-white color will do */
 	bdev->target = NULL;
 	gx_default_fill_triangle((gx_device *) & cdev, x0, y0,
 				 float2fixed(corners[1].x) - x0,
@@ -1064,7 +1095,7 @@ bbox_image_plane_data(gx_image_enum_common_t * info,
     return code;
 }
 
-private int
+static int
 bbox_image_end_image(gx_image_enum_common_t * info, bool draw_last)
 {
     bbox_image_enum *pbe = (bbox_image_enum *) info;
@@ -1074,7 +1105,7 @@ bbox_image_end_image(gx_image_enum_common_t * info, bool draw_last)
     return code;
 }
 
-private int
+static int
 bbox_image_flush(gx_image_enum_common_t * info)
 {
     bbox_image_enum *pbe = (bbox_image_enum *) info;
@@ -1083,7 +1114,7 @@ bbox_image_flush(gx_image_enum_common_t * info)
     return (target_info ? gx_image_flush(target_info) : 0);
 }
 
-private bool
+static bool
 bbox_image_planes_wanted(const gx_image_enum_common_t * info, byte *wanted)
 {
     /* This is only used if target_info != 0. */
@@ -1094,43 +1125,43 @@ bbox_image_planes_wanted(const gx_image_enum_common_t * info, byte *wanted)
 
 /* Compositing */
 
-private bool
+static bool
 bbox_forward_init_box(void *pdata)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *)pdata;
 
     return BBOX_INIT_BOX(bdev);
 }
-private void
+static void
 bbox_forward_get_box(const void *pdata, gs_fixed_rect *pbox)
 {
     const gx_device_bbox *const bdev = (const gx_device_bbox *)pdata;
 
     BBOX_GET_BOX(bdev, pbox);
 }
-private void
+static void
 bbox_forward_add_rect(void *pdata, fixed x0, fixed y0, fixed x1, fixed y1)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *)pdata;
 
     BBOX_ADD_RECT(bdev, x0, y0, x1, y1);
 }
-private bool
+static bool
 bbox_forward_in_rect(const void *pdata, const gs_fixed_rect *pbox)
 {
     const gx_device_bbox *const bdev = (const gx_device_bbox *)pdata;
 
     return BBOX_IN_RECT(bdev, pbox);
 }
-private const gx_device_bbox_procs_t box_procs_forward = {
+static const gx_device_bbox_procs_t box_procs_forward = {
     bbox_forward_init_box, bbox_forward_get_box, bbox_forward_add_rect,
     bbox_forward_in_rect
 };
 
-private int
+static int
 bbox_create_compositor(gx_device * dev,
 		       gx_device ** pcdev, const gs_composite_t * pcte,
-		       const gs_imager_state * pis, gs_memory_t * memory)
+		       gs_imager_state * pis, gs_memory_t * memory)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
     gx_device *target = bdev->target;
@@ -1154,8 +1185,11 @@ bbox_create_compositor(gx_device * dev,
 	int code = (*dev_proc(target, create_compositor))
 	    (target, &cdev, pcte, pis, memory);
 
-	if (code < 0)
+	/* If the target did not create a new compositor then we are done. */
+	if (code < 0 || target == cdev) {
+	    *pcdev = dev;
 	    return code;
+	}
 	bbcdev = gs_alloc_struct_immovable(memory, gx_device_bbox,
 					   &st_device_bbox,
 					   "bbox_create_compositor");
@@ -1163,7 +1197,7 @@ bbox_create_compositor(gx_device * dev,
 	    (*dev_proc(cdev, close_device)) (cdev);
 	    return_error(gs_error_VMerror);
 	}
-	gx_device_bbox_init(bbcdev, target);
+	gx_device_bbox_init(bbcdev, target, memory);
 	gx_device_set_target((gx_device_forward *)bbcdev, cdev);
 	bbcdev->box_procs = box_procs_forward;
 	bbcdev->box_proc_data = bdev;
@@ -1174,45 +1208,7 @@ bbox_create_compositor(gx_device * dev,
 
 /* ------ Text imaging ------ */
 
-extern_st(st_gs_text_enum);
-
-typedef struct bbox_text_enum_s {
-    gs_text_enum_common;
-    gs_text_enum_t *target_info;
-} bbox_text_enum;
-
-gs_private_st_suffix_add1(st_bbox_text_enum, bbox_text_enum, "bbox_text_enum",
-			bbox_text_enum_enum_ptrs, bbox_text_enum_reloc_ptrs,
-			  st_gs_text_enum, target_info);
-
-private text_enum_proc_resync(bbox_text_resync);
-private text_enum_proc_process(bbox_text_process);
-private text_enum_proc_is_width_only(bbox_text_is_width_only);
-private text_enum_proc_current_width(bbox_text_current_width);
-private text_enum_proc_set_cache(bbox_text_set_cache);
-private text_enum_proc_retry(bbox_text_retry);
-private text_enum_proc_release(bbox_text_release);
-private rc_free_proc(bbox_text_free);
-
-private const gs_text_enum_procs_t bbox_text_procs =
-{
-    bbox_text_resync, bbox_text_process, bbox_text_is_width_only,
-    bbox_text_current_width, bbox_text_set_cache, bbox_text_retry,
-    bbox_text_release
-};
-
-private void
-bbox_text_enum_copy(bbox_text_enum *pbte)
-{
-    rc_header rc_save;
-
-    rc_save = pbte->rc;
-    *(gs_text_enum_t *)pbte = *pbte->target_info;	/* copy common info */
-    pbte->rc = rc_save;
-    pbte->procs = &bbox_text_procs;
-}
-
-private int
+static int
 bbox_text_begin(gx_device * dev, gs_imager_state * pis,
 		const gs_text_params_t * text, gs_font * font,
 		gx_path * path, const gx_device_color * pdcolor,
@@ -1220,105 +1216,14 @@ bbox_text_begin(gx_device * dev, gs_imager_state * pis,
 		gs_memory_t * memory, gs_text_enum_t ** ppenum)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
-    gx_device *tdev = bdev->target;
-    bbox_text_enum *pbte;
-    int code;
-
-    if (tdev == 0)
-	return gx_default_text_begin(dev, pis, text, font, path, pdcolor,
+    int code = gx_default_text_begin(dev, pis, text, font, path, pdcolor,
 				     pcpath, memory, ppenum);
-    rc_alloc_struct_1(pbte, bbox_text_enum, &st_bbox_text_enum, memory,
-		      return_error(gs_error_VMerror),
-		      "bbox_text_begin");
-    pbte->rc.free = bbox_text_free;
-    code =
-	(*dev_proc(tdev, text_begin))
-	(tdev, pis, text, font, path, pdcolor, pcpath, memory,
-	 &pbte->target_info);
-    if (code < 0) {
-	gs_free_object(memory, pbte, "bbox_text_begin");
-	return code;
+
+    if (bdev->target != NULL) {
+        /* See note on imaging_dev in gxtext.h */
+        rc_assign((*ppenum)->imaging_dev, dev, "bbox_text_begin");
     }
-    bbox_text_enum_copy(pbte);
 
-    /* See note on imaging_dev in gxtext.h */
-    rc_assign(pbte->target_info->imaging_dev, dev, "bbox_text_begin");
-
-    *ppenum = (gs_text_enum_t *) pbte;
     return code;
 }
 
-private int
-bbox_text_resync(gs_text_enum_t *pte, const gs_text_enum_t *pfrom)
-{
-    bbox_text_enum *const pbte = (bbox_text_enum *) pte;
-    int code = gs_text_resync(pbte->target_info, pfrom);
-
-    if (code >= 0)
-	bbox_text_enum_copy(pbte);
-    return code;
-}
-
-private int
-bbox_text_process(gs_text_enum_t * pte)
-{
-    bbox_text_enum *const pbte = (bbox_text_enum *) pte;
-    int code = gs_text_process(pbte->target_info);
-
-    gs_text_enum_copy_dynamic(pte, pbte->target_info, true);
-    return code;
-}
-
-private bool
-bbox_text_is_width_only(const gs_text_enum_t *pte)
-{
-    const bbox_text_enum *const pbte = (const bbox_text_enum *)pte;
-
-    return gs_text_is_width_only(pbte->target_info);
-}
-
-private int
-bbox_text_current_width(const gs_text_enum_t *pte, gs_point *pwidth)
-{
-    const bbox_text_enum *const pbte = (const bbox_text_enum *)pte;
-
-    return gs_text_current_width(pbte->target_info, pwidth);
-}
-
-private int
-bbox_text_set_cache(gs_text_enum_t * pte, const double *values,
-		    gs_text_cache_control_t control)
-{
-    bbox_text_enum *const pbte = (bbox_text_enum *) pte;
-    gs_text_enum_t *tpte = pbte->target_info;
-    int code = gs_text_set_cache(tpte, values, control);
-
-    if (code < 0)
-	return code;
-    /* Copy back the dynamic information for the client. */
-    pte->index = tpte->index;
-    return code;
-}
-
-private int
-bbox_text_retry(gs_text_enum_t *pte)
-{
-    bbox_text_enum *const pbte = (bbox_text_enum *) pte;
-
-    return gs_text_retry(pbte->target_info);
-}
-
-private void
-bbox_text_release(gs_text_enum_t *pte, client_name_t cname)
-{
-    bbox_text_enum *const pbte = (bbox_text_enum *)pte;
-
-    gs_text_release(pbte->target_info, cname);
-}
-
-private void
-bbox_text_free(gs_memory_t * memory, void *vpte, client_name_t cname)
-{
-    bbox_text_release((gs_text_enum_t *)vpte, cname);
-    rc_free_struct_only(memory, vpte, cname);
-}
