@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevdrop.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gdevdrop.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Default and device-independent RasterOp algorithms */
 #include "memory_.h"
 #include "gx.h"
@@ -31,7 +31,7 @@
  * multiple-row RasterOp buffer.  (We are always willing to allocate
  * one row, no matter how wide.)
  */
-private const uint max_rop_bitmap = 1000;
+static const uint max_rop_bitmap = 1000;
 
 /* ---------------- Debugging aids ---------------- */
 
@@ -193,7 +193,7 @@ out:
 /* ---------------- Default memory device copy_rop ---------------- */
 
 /* Convert color constants to standard RGB representation. */
-private void
+static void
 unpack_colors_to_standard(gx_device * dev, gx_color_index real_colors[2],
 			  const gx_color_index * colors, int depth)
 {
@@ -218,7 +218,7 @@ unpack_colors_to_standard(gx_device * dev, gx_color_index real_colors[2],
  * Convert RGB to the device's native format.  We special-case this for
  * 1-bit CMYK devices.
  */
-private void
+static void
 pack_cmyk_1bit_from_standard(gx_device * dev, byte * dest, int destx,
 			     const byte * src, int width, int depth,
 			     int src_depth)
@@ -250,7 +250,7 @@ pack_cmyk_1bit_from_standard(gx_device * dev, byte * dest, int destx,
 
 }
 
-private gx_color_index
+static gx_color_index
 map_rgb_to_color_via_cmyk(gx_device * dev, const gx_color_value rgbcv[])
 {
     gx_color_value cmykcv[4];
@@ -266,7 +266,7 @@ map_rgb_to_color_via_cmyk(gx_device * dev, const gx_color_value rgbcv[])
 
     return (*dev_proc(dev, map_cmyk_color)) (dev, cmykcv);
 }
-private void
+static void
 pack_from_standard(gx_device * dev, byte * dest, int destx, const byte * src,
 		   int width, int depth, int src_depth)
 {
@@ -369,6 +369,7 @@ mem_default_strip_copy_rop(gx_device * dev,
     gx_device_memory mdev;
     union { long l; void *p; } mdev_storage[20];
     uint row_raster = bitmap_raster(width * depth);
+    ulong size_from_mem_device;
     gs_rop3_t trans_rop = gs_transparent_rop(lop);
     bool uses_d = rop3_uses_D(trans_rop);
     bool uses_s = rop3_uses_S(trans_rop);
@@ -451,11 +452,13 @@ mem_default_strip_copy_rop(gx_device * dev,
     mdev.width = width;
     mdev.height = block_height;
     mdev.color_info.num_components = rop_depth >> 3;
-    if (gdev_mem_data_size(&mdev, width, block_height) <= sizeof(mdev_storage)) {
+    if (gdev_mem_data_size(&mdev, width, block_height, &size_from_mem_device) >= 0 &&
+	size_from_mem_device <= sizeof(mdev_storage)) {
 	/* Use the locally allocated storage. */
 	mdev.base = (byte *)mdev_storage;
-	mdev.line_ptrs = (byte **)
-	    (mdev.base + gdev_mem_bits_size(&mdev, mdev.width, mdev.height));
+	if ((code = gdev_mem_bits_size(&mdev, mdev.width, mdev.height, &size_from_mem_device)) < 0)
+	    return code;
+	mdev.line_ptrs = (byte **) (mdev.base + size_from_mem_device);
     } else {
 	mdev.bitmap_memory = mem;
     }
