@@ -1,23 +1,16 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/* $RCSfile: gdevhl7x.c,v $ $Revision: 1.3.2.1 $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevhl7x.c 8250 2007-09-25 13:31:24Z giles $ */
 /*
  * Brother HL 720 and 730 driver for Ghostscript
  *
@@ -32,7 +25,7 @@
  *
  * Removal of compression code on 1/17/00 by Ross Martin
  * (ross@ross.interwrx.com, martin@walnut.eas.asu.edu)
- * enables this driver to correctly print tiger.ps on a
+ * enables this driver to correctly print tiger.eps on a
  * Brother MFC6550MC Fax Machine.  Change to the Horizontal
  * Offset fixes incorrect page alignment at 300dpi in
  * Landscape mode with a2ps.
@@ -130,10 +123,10 @@ typedef struct {
 /* We need a boolean : true , we got it from gdevprn.h */
 
 /* Other constants */
-private const int DumpFinished = 0;
-private const int DumpContinue = 1;
-private const int HL7X0_LENGTH = 5; /* Length of a command to tell the size of the data to be sent to the printer*/
-private void  makeCommandsForSequence(Byte     * pSource,
+static const int DumpFinished = 0;
+static const int DumpContinue = 1;
+static const int HL7X0_LENGTH = 5; /* Length of a command to tell the size of the data to be sent to the printer*/
+static void  makeCommandsForSequence(Byte     * pSource,
 				      short      length,
 				      ByteList * pCommandList,
 				      short      offset,
@@ -144,16 +137,16 @@ private void  makeCommandsForSequence(Byte     * pSource,
 
 
 
-private int dumpPage(gx_device_printer * pSource,
+static int dumpPage(gx_device_printer * pSource,
 		      Byte              * pLineTmp,
 		      ByteList          * pCommandList,
 		      Summary           * pSummary
 		      );
-private void initSummary(Summary * s,short pw, short ph, short resolution);
+static void initSummary(Summary * s,short pw, short ph, short resolution);
 
-private void resetPreviousData(Summary * s);
+static void resetPreviousData(Summary * s);
 
-private void makeFullLine( Byte      * pCurrentLine,
+static void makeFullLine( Byte      * pCurrentLine,
 			   Byte      * pPreviousLine,
 			   short       lineWidth,
 			   ByteList  * commandsList,
@@ -165,19 +158,19 @@ private void makeFullLine( Byte      * pCurrentLine,
 /*
  * Initialize a list of Bytes structure
  */
-private void initByteList(ByteList *list, Byte *array, short maxSize,short initCurrent);
-private void addByte(ByteList *list,Byte value );
-private void addArray(ByteList *list, Byte *source, short nb);
-private void addNBytes(ByteList * list, Byte value, short nb);
-private Byte * currentPosition(ByteList * list);
-private void addCodedNumber(ByteList * list, short number);
-private int isThereEnoughRoom(ByteList * list, short biggest);
-private short roomLeft(ByteList * list);
-private void dumpToPrinter(ByteList * list,FILE * printStream);
+static void initByteList(ByteList *list, Byte *array, short maxSize,short initCurrent);
+static void addByte(ByteList *list,Byte value );
+static void addArray(ByteList *list, Byte *source, short nb);
+static void addNBytes(ByteList * list, Byte value, short nb);
+static Byte * currentPosition(ByteList * list);
+static void addCodedNumber(ByteList * list, short number);
+static int isThereEnoughRoom(ByteList * list, short biggest);
+static short roomLeft(ByteList * list);
+static void dumpToPrinter(ByteList * list,FILE * printStream);
 
 /* Real Print function */
 
-private int hl7x0_print_page(P5(gx_device_printer *, FILE *, int, int, ByteList *));
+static int hl7x0_print_page(gx_device_printer *, FILE *, int, int, ByteList *);
 
 
 
@@ -202,7 +195,7 @@ private int hl7x0_print_page(P5(gx_device_printer *, FILE *, int, int, ByteList
 #define LETTER_WIDTH 5100
 #define LEFT_MARGIN  30
 /* The following table is not actually used.... */
-private const PaperFormat tableOfFormats[] = {
+static const PaperFormat tableOfFormats[] = {
     /*  0 P LETTER */ { 2550, 3300 },
     /*  1 P LEGAL  */ { 2550, 4200 },
     /*  2 P EXEC   */ { 2175, 3150 },
@@ -230,7 +223,7 @@ private const PaperFormat tableOfFormats[] = {
 
 
 /* Compute the maximum length of a compressed line */
-private short MaxLineLength(short resolution){
+static short MaxLineLength(short resolution){
 return (((156 * resolution / 150 ) * 5 )/4) + 8;
 } 
 
@@ -265,19 +258,19 @@ return (((156 * resolution / 150 ) * 5 )/4) + 8;
 
 
 /* The device descriptors */
-private dev_proc_open_device(hl7x0_open);
-private dev_proc_close_device(hl7x0_close);
-private dev_proc_print_page(hl720_print_page);
-private dev_proc_print_page(hl730_print_page);
+static dev_proc_open_device(hl7x0_open);
+static dev_proc_close_device(hl7x0_close);
+static dev_proc_print_page(hl720_print_page);
+static dev_proc_print_page(hl730_print_page);
 
 
 
-private const gx_device_procs prn_hl_procs =
+static const gx_device_procs prn_hl_procs =
   prn_params_procs(hl7x0_open, gdev_prn_output_page, hl7x0_close,
 		   gdev_prn_get_params, gdev_prn_put_params);
 
 
-gx_device_printer far_data gs_hl7x0_device =
+const gx_device_printer far_data gs_hl7x0_device =
   prn_device(prn_hl_procs, "hl7x0",
 	DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
 	X_DPI, Y_DPI,
@@ -288,7 +281,7 @@ gx_device_printer far_data gs_hl7x0_device =
 
 /* Open the printer, adjusting the margins if necessary. */
 
-private int 
+static int 
 hl7x0_open(gx_device *pdev)
 {	/* Change the margins if necessary. */
 	static const float m_a4[4] = { HL7X0_MARGINS_A4 };
@@ -302,7 +295,7 @@ hl7x0_open(gx_device *pdev)
 
 
 /* The orders sent are those provided in the Brother DOS example */
-private int 
+static int 
 hl7x0_close(gx_device *pdev)
 {
     gx_device_printer *const ppdev = (gx_device_printer *)pdev;
@@ -317,7 +310,7 @@ hl7x0_close(gx_device *pdev)
 /* ------ Internal routines ------ */
 
 /* The HL 720 can compress*/
-private int
+static int
 hl720_print_page(gx_device_printer *pdev, FILE *prn_stream)
 {
 	Byte prefix[] ={
@@ -343,14 +336,14 @@ hl720_print_page(gx_device_printer *pdev, FILE *prn_stream)
 	       &initCommand);
 }
 /* The HL 730 can compress  */
-private int
+static int
 hl730_print_page(gx_device_printer *pdev, FILE *prn_stream)
 {	return hl720_print_page(pdev, prn_stream);
 }
 
 /* Send the page to the printer.  For speed, compress each scan line, */
 /* since computer-to-printer communication time is often a bottleneck. */
-private int 
+static int 
 hl7x0_print_page(gx_device_printer *pdev, FILE *printStream, int ptype,
   int dots_per_inch, ByteList *initCommand)
 {
@@ -365,7 +358,8 @@ hl7x0_print_page(gx_device_printer *pdev, FILE *printStream, int ptype,
   int num_rows = dev_print_scan_lines(pdev);
   int result;
   int sizeOfBuffer   = MaxLineLength(x_dpi) + 30; 
-  Byte * storage      = (Byte *) gs_malloc(sizeOfBuffer + line_size,
+  Byte * storage      = (Byte *) gs_malloc(pdev->memory, 
+					   sizeOfBuffer + line_size,
 					   1,
 					   "hl7x0_print_page");
 	/*	bool dup = pdev->Duplex; */
@@ -405,7 +399,7 @@ hl7x0_print_page(gx_device_printer *pdev, FILE *printStream, int ptype,
 	dumpToPrinter(&formFeedCommand, printStream);
 		
 	/* free temporary storage */
-	gs_free((char *)storage, storage_size_words, 1, "hl7X0_print_page");
+	gs_free(pdev->memory, (char *)storage, storage_size_words, 1, "hl7X0_print_page");
 
 	return 0; /* If we reach this line, it means there was no error */
 }
@@ -416,7 +410,7 @@ hl7x0_print_page(gx_device_printer *pdev, FILE *printStream, int ptype,
  */
 
 
-private short stripTrailingBlanks(Byte * line, short length){
+static short stripTrailingBlanks(Byte * line, short length){
   short positionOfFirstZero = length - 1;
   while (positionOfFirstZero > 0) {
     if (line[positionOfFirstZero] != 0) {
@@ -439,7 +433,7 @@ private short stripTrailingBlanks(Byte * line, short length){
  * correct, but it now looks to be pretty close visually,
  * and works correctly at 600dpi and 300dpi.
  */
-private short horizontalOffset(short pixWidth,
+static short horizontalOffset(short pixWidth,
 			      short pixOffset,
 			      short resolution){
 return (((LETTER_WIDTH * resolution/600 - pixWidth) + pixOffset * 2) + 7) / 8; 
@@ -451,7 +445,7 @@ return (((LETTER_WIDTH * resolution/600 - pixWidth) + pixOffset * 2) + 7) / 8;
 /*
  * First values in a Summary
  */ 
-private void initSummary(Summary * s,short pw, short ph, short resolution){
+static void initSummary(Summary * s,short pw, short ph, short resolution){
   s->previousSize = -1 ;
   s->nbBlankLines = 1;
   s->nbLinesSent = 0;
@@ -464,7 +458,7 @@ private void initSummary(Summary * s,short pw, short ph, short resolution){
 /*
  * The previous line was blank, so we need to clean the corresponding array
  */
-private void resetPreviousData(Summary * s){
+static void resetPreviousData(Summary * s){
  memset(s->previousData,0,s->pageWidth);
 }
 
@@ -473,7 +467,7 @@ private void resetPreviousData(Summary * s){
  * dumpPage :
  *
  */
-private int dumpPage(gx_device_printer * pSource,
+static int dumpPage(gx_device_printer * pSource,
 		      Byte              * pLineTmp,
 		      ByteList          * pCommandList,
 		      Summary           * pSummary
@@ -601,7 +595,7 @@ private int dumpPage(gx_device_printer * pSource,
 
 
 
-private void makeFullLine( Byte      * pCurrentLine,
+static void makeFullLine( Byte      * pCurrentLine,
 			   Byte      * pPreviousLine,
 			   short       lineWidth,
 			   ByteList  * commandsList,
@@ -652,7 +646,7 @@ private void makeFullLine( Byte      * pCurrentLine,
      * Disabled line-to-line compression, 1/17/00 Ross Martin
      * ross@ross.interwrx.com and/or martin@walnut.eas.asu.edu
      *
-     * The compression here causes problems printing tiger.ps.
+     * The compression here causes problems printing tiger.eps.
      * The problem is vertical streaks.  The printer I'm printing
      * to is a Brother MFC6550MC Fax Machine, which may be
      * slightly different from the hl720 and hl730.  Note that
@@ -661,11 +655,11 @@ private void makeFullLine( Byte      * pCurrentLine,
      * setup from a DOS program included with the printer.  Thus,
      * the hl7x0 driver seems a better choice.  In any case,
      * on the MFC6550MC, some files print fine with compression
-     * turned on, but others such as tiger.ps print with streaks.
+     * turned on, but others such as tiger.eps print with streaks.
      * disabling the compression fixes the problem, so I haven't
      * looked any further at the cause.  It may be that the
      * compression is correct for the hl720 and hl730, and only
-     * different for the MFC6550MC, or it may be that tiger.ps
+     * different for the MFC6550MC, or it may be that tiger.eps
      * won't print correctly with compression enabled on any
      * of these.  It may be that the problem is only with color
      * and/or grayscale prints.  YMMV.  I don't think it likely
@@ -731,13 +725,13 @@ private void makeFullLine( Byte      * pCurrentLine,
 /* 
  *  Declarations of functions that are defined further in the file 
  */
-private void makeSequenceWithoutRepeat(
+static void makeSequenceWithoutRepeat(
 				  Byte     * pSequence,
 				  short      lengthOfSequence,
 				  ByteList * pCommandList, 
 				  short      offset             );
 
-private void makeSequenceWithRepeat(
+static void makeSequenceWithRepeat(
 				  Byte     * pSequence,
 				  short      lengthOfSequence,
 				  ByteList * pCommandList, 
@@ -749,7 +743,7 @@ private void makeSequenceWithRepeat(
  * Process a sequence of new bytes (i.e. different from the ones on the former line)
  */
 
-private void makeCommandsForSequence(Byte     * pSource,
+static void makeCommandsForSequence(Byte     * pSource,
 				     short      length,
 				     ByteList * pCommandList,
 				     short      offset,
@@ -862,7 +856,7 @@ private void makeCommandsForSequence(Byte     * pSource,
 /*
  * makeSequenceWithoutRepeat
  */
-private void makeSequenceWithoutRepeat(
+static void makeSequenceWithoutRepeat(
 				  Byte     * pSequence,
 				  short      lengthOfSequence,
 				  ByteList * pCommandList, 
@@ -915,7 +909,7 @@ private void makeSequenceWithoutRepeat(
 /*
  * makeSequenceWithRepeat
  */
-private void makeSequenceWithRepeat(
+static void makeSequenceWithRepeat(
 				  Byte     * pSequence,
 				  short      lengthOfSequence,
 				  ByteList * pCommandList, 
@@ -969,7 +963,7 @@ private void makeSequenceWithRepeat(
 /*
  * Initialize a list of Bytes structure
  */
-private void initByteList(ByteList *list, Byte *array, short maxSize, short initCurrent) {
+static void initByteList(ByteList *list, Byte *array, short maxSize, short initCurrent) {
   list->current = initCurrent;
   list->maxSize = maxSize;
   list->data = array;
@@ -978,11 +972,11 @@ private void initByteList(ByteList *list, Byte *array, short maxSize, short init
 /*
  * Add a Byte to a list of Bytes
  */
-private void addByte(ByteList *list,Byte value ) {
+static void addByte(ByteList *list,Byte value ) {
  if (list->current < list->maxSize)
   list->data[list->current++] = value;
  else
-   fprintf(stderr,"Could not add byte to command\n");
+   errprintf("Could not add byte to command\n");
 }
 
 
@@ -990,14 +984,14 @@ private void addByte(ByteList *list,Byte value ) {
  * Add a copy of an array to a list of Bytes
  */
 
-private void addArray(ByteList *list, Byte *source, short nb){
+static void addArray(ByteList *list, Byte *source, short nb){
   if (list->current <= list->maxSize - nb)
   {
     memcpy(list->data + list->current, source , (size_t) nb);
     list->current += nb;
   }
   else 
-    fprintf(stderr,"Could not add byte array to command\n");
+    errprintf("Could not add byte array to command\n");
 }
 
 
@@ -1005,7 +999,7 @@ private void addArray(ByteList *list, Byte *source, short nb){
  * Add N bytes to a list of Bytes
  */
 
-private void addNBytes(ByteList * list, Byte value, short nb){
+static void addNBytes(ByteList * list, Byte value, short nb){
   int i;
   if (list->current <= list->maxSize - nb)
   {
@@ -1016,13 +1010,13 @@ private void addNBytes(ByteList * list, Byte value, short nb){
     list->current += nb;
   }
   else 
-    fprintf(stderr,"Could not add %d bytes to command\n",nb);
+    errprintf("Could not add %d bytes to command\n",nb);
 }
 
 /*
  * Get pointer to the current byte
  */
-private Byte * currentPosition(ByteList * list) {
+static Byte * currentPosition(ByteList * list) {
   return &(list->data[list->current]);
 }
 
@@ -1033,7 +1027,7 @@ private Byte * currentPosition(ByteList * list) {
  * where q is the quotient of the number divided by 0xff and r is the
  * remainder.
  */
-private void addCodedNumber(ByteList * list, short number){
+static void addCodedNumber(ByteList * list, short number){
  short q = number / 0xff;
  short r = number % 0xff;
  
@@ -1047,20 +1041,20 @@ private void addCodedNumber(ByteList * list, short number){
  *
  */
 
-private int isThereEnoughRoom(ByteList * list, short biggest){
+static int isThereEnoughRoom(ByteList * list, short biggest){
   return ((list->maxSize-list->current) >= biggest);
 }
 /*
  * Tell how much room is left 
  */
-private short roomLeft(ByteList * list){
+static short roomLeft(ByteList * list){
   return list->maxSize - list->current;
 }
 /*
  * Dump all commands to the printer and reset the structure
  *
  */
-private void dumpToPrinter(ByteList * list,FILE * printStream){
+static void dumpToPrinter(ByteList * list,FILE * printStream){
   short loopCounter;
   /* Actual dump */
   /* Please note that current is the first empty byte */
