@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevmem.h 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gdevmem.h 8250 2007-09-25 13:31:24Z giles $ */
 /* Private definitions for memory devices. */
 
 #ifndef gdevmem_INCLUDED
@@ -72,12 +72,12 @@
 dev_proc_get_initial_matrix(mem_get_initial_matrix);
 dev_proc_close_device(mem_close);
 #define declare_mem_map_procs(map_rgb_color, map_color_rgb)\
-  private dev_proc_map_rgb_color(map_rgb_color);\
-  private dev_proc_map_color_rgb(map_color_rgb)
+  static dev_proc_map_rgb_color(map_rgb_color);\
+  static dev_proc_map_color_rgb(map_color_rgb)
 #define declare_mem_procs(copy_mono, copy_color, fill_rectangle)\
-  private dev_proc_copy_mono(copy_mono);\
-  private dev_proc_copy_color(copy_color);\
-  private dev_proc_fill_rectangle(fill_rectangle)
+  static dev_proc_copy_mono(copy_mono);\
+  static dev_proc_copy_color(copy_color);\
+  static dev_proc_fill_rectangle(fill_rectangle)
 /*
  * We define one relatively low-usage drawing procedure that is common to
  * all memory devices so that we have a reliable way to implement
