@@ -1,23 +1,19 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpcfb.c,v 1.6 2004/09/20 22:14:59 dan Exp $ */
+/* $Id: gdevpcfb.c 8468 2007-12-31 18:07:59Z giles $ */
 /* IBM PC frame buffer (EGA/VGA) drivers */
 #include "memory_.h"
-#include "gconfigv.h"		/* for USE_ASM */
 #include "gx.h"
 #include "gserrors.h"
 #include "gsparam.h"
@@ -34,10 +30,10 @@
 #define fb_dev ((gx_device_ega *)dev)
 
 /* Procedure record */
-private dev_proc_map_rgb_color(ega0_map_rgb_color);
-private dev_proc_map_rgb_color(ega1_map_rgb_color);
+static dev_proc_map_rgb_color(ega0_map_rgb_color);
+static dev_proc_map_rgb_color(ega1_map_rgb_color);
 #define ega2_map_rgb_color pc_4bit_map_rgb_color
-private dev_proc_map_color_rgb(ega01_map_color_rgb);
+static dev_proc_map_color_rgb(ega01_map_color_rgb);
 #define ega2_map_color_rgb pc_4bit_map_color_rgb
 #if ega_bits_of_color == 0
 #   define ega_map_rgb_color ega0_map_rgb_color
@@ -73,14 +69,14 @@ private dev_proc_map_color_rgb(ega01_map_color_rgb);
 	NULL,			/* map_rgb_alpha_color */\
 	gx_page_device_get_page_device
 
-private const gx_device_procs ega_procs =
+static const gx_device_procs ega_procs =
 {
     ega_std_procs(NULL, NULL)
 };
 
-private dev_proc_get_params(svga16_get_params);
-private dev_proc_put_params(svga16_put_params);
-private const gx_device_procs svga16_procs =
+static dev_proc_get_params(svga16_get_params);
+static dev_proc_put_params(svga16_put_params);
+static const gx_device_procs svga16_procs =
 {
     ega_std_procs(svga16_get_params, svga16_put_params)
 };
@@ -99,7 +95,7 @@ gx_device_ega far_data gs_svga16_device =
 ega_device("svga16", svga16_procs, 100, 600, 1.0, 0x29 /*Tseng */ );
 
 /* Save the BIOS state */
-private pcfb_bios_state pcfb_save_state =
+static pcfb_bios_state pcfb_save_state =
 {-1};
 
 /* Initialize the EGA for graphics mode */
@@ -137,7 +133,7 @@ ega_close(gx_device * dev)
 }
 
 /* Get/put the display mode parameter. */
-private int
+static int
 svga16_get_params(gx_device * dev, gs_param_list * plist)
 {
     int code = gx_default_get_params(dev, plist);
@@ -146,7 +142,7 @@ svga16_get_params(gx_device * dev, gs_param_list * plist)
 	return code;
     return param_write_int(plist, "DisplayMode", &fb_dev->video_mode);
 }
-private int
+static int
 svga16_put_params(gx_device * dev, gs_param_list * plist)
 {
     int ecode = 0;
@@ -178,23 +174,25 @@ svga16_put_params(gx_device * dev, gs_param_list * plist)
 }
 
 /* Map a r-g-b color to an EGA color code. */
-#define Nb gx_color_value_bits
-private gx_color_index
+static gx_color_index
 ega0_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
     return pc_4bit_map_rgb_color(dev, cv);
 }
-private gx_color_index
+static gx_color_index
 ega1_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
-#define cvtop (gx_color_value)(1 << (Nb - 1))
-    return pc_4bit_map_rgb_color(dev, cv[0] & cvtop, cv[1] & cvtop, cv[2] & cvtop);
+    const gx_color_value cvtop = (1 << (gx_color_value_bits - 1));
+    gx_color_value cvt[3];
+    cvt[0] = cv[0] & cvtop;
+    cvt[1] = cv[1] & cvtop;
+    cvt[2] = cv[2] & cvtop;
+    return pc_4bit_map_rgb_color(dev, cvt);
 }
-#undef Nb
 
 /* Map a color code to r-g-b. */
 #define icolor (int)color
-private int
+static int
 ega01_map_color_rgb(gx_device * dev, gx_color_index color,
 		    gx_color_value prgb[3])
 {
@@ -227,12 +225,8 @@ typedef rop_params _ss *rop_ptr;
 
 /* Assembly language routines */
 
-#if USE_ASM
-void memsetcol(rop_ptr);	/* dest, draster, height, data */
-#else
-#define memsetcol cmemsetcol
-private void
-cmemsetcol(rop_ptr rop)
+static void
+memsetcol(rop_ptr rop)		/* dest, draster, height, data */
 {
     byte *addr = rop->dest;
     int yc = rop->height;
@@ -245,14 +239,9 @@ cmemsetcol(rop_ptr rop)
 	addr += draster;
     }
 }
-#endif
 
-#if USE_ASM
-void memsetrect(rop_ptr);	/* dest, draster, width, height, data */
-#else
-#define memsetrect cmemsetrect
-private void
-cmemsetrect(rop_ptr rop)
+static void
+memsetrect(rop_ptr rop)       /* dest, draster, width, height, data */
 {
     int yc = rop->height;
     int width = rop->width;
@@ -286,16 +275,10 @@ cmemsetrect(rop_ptr rop)
 	}
     }
 }
-#endif
 
-#if USE_ASM
-void memrwcol(rop_ptr);	/* dest, draster, src, sraster, height, shift, invert */
-#  define memrwcol0(rop) memrwcol(rop)	/* same except shift = 0 */
-#else
-#  define memrwcol cmemrwcol
-#  define memrwcol0 cmemrwcol0
-private void
-cmemrwcol(rop_ptr rop)
+static void
+memrwcol(rop_ptr rop)	/* dest, draster, src, sraster, height, shift, invert */
+
 {
     byte *dp = rop->dest;
     const byte *sp = rop->src;
@@ -310,8 +293,8 @@ cmemrwcol(rop_ptr rop)
 	dp += draster, sp += sraster;
     }
 }
-private void
-cmemrwcol0(rop_ptr rop)
+static void
+memrwcol0(rop_ptr rop)  /* same except shift = 0 */
 {
     byte *dp = rop->dest;
     const byte *sp = rop->src;
@@ -327,14 +310,9 @@ cmemrwcol0(rop_ptr rop)
 	}
 	while (--yc);
 }
-#endif
 
-#if USE_ASM
-void memrwcol2(rop_ptr);	/* dest, draster, src, sraster, height, shift, invert */
-#else
-#define memrwcol2 cmemrwcol2
-private void
-cmemrwcol2(rop_ptr rop)
+static void
+memrwcol2(rop_ptr rop)	/* dest, draster, src, sraster, height, shift, invert */
 {
     byte *dp = rop->dest;
     const byte *sp = rop->src;
@@ -349,12 +327,11 @@ cmemrwcol2(rop_ptr rop)
 	dp += draster, sp += sraster;
     }
 }
-#endif
 
 /* Forward definitions */
 int ega_write_dot(gx_device *, int, int, gx_color_index);
-private void fill_rectangle(rop_ptr, int, int, int);
-private void fill_row_only(byte *, int, int, int);
+static void fill_rectangle(rop_ptr, int, int, int);
+static void fill_row_only(byte *, int, int, int);
 
 /* Clean up after writing */
 #define dot_end()\
@@ -838,7 +815,7 @@ static const byte rmask_tab[9] =
 /* Fill a rectangle specified by pointer into frame buffer, */
 /* starting bit within byte, width, and height. */
 /* Smashes rop->dest. */
-private void
+static void
 fill_rectangle(register rop_ptr rop, int bit, int w, int color)
   /* rop: dest, draster, height */
 {
@@ -875,7 +852,7 @@ fill_rectangle(register rop_ptr rop, int bit, int w, int color)
 /* Fill a single row specified by pointer into frame buffer, */
 /* starting bit within byte, and width; clean up afterwards. */
 #define r_m_w(ptr) (*(ptr))++	/* read & write, data irrelevant */
-private void
+static void
 fill_row_only(byte * dest, int bit, int w, int color)
   /* rop: dest */
 {
