@@ -1,20 +1,17 @@
-/* Copyright (C) 1992, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevpcl.c,v 1.5.2.1 2002/02/22 19:45:56 ray Exp $ */
+/* $Id: gdevpcl.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Utilities for PCL printers */
 #include "gdevprn.h"
 #include "gdevpcl.h"
@@ -98,9 +95,10 @@ gdev_pcl_paper_size(gx_device * dev)
 
 /* Map an RGB color to a printer color. */
 gx_color_index
-gdev_pcl_3bit_map_rgb_color(gx_device * dev,
-		       gx_color_value r, gx_color_value g, gx_color_value b)
+gdev_pcl_3bit_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
+    gx_color_value r, g, b;
+    r = cv[0]; g = cv[1]; b = cv[2];
     return (((b >> cv_shift) << 2) + ((g >> cv_shift) << 1) + (r >> cv_shift)) ^ 7;
 }
 
