@@ -1,20 +1,17 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfb.c,v 1.28 2005/04/04 20:28:27 igor Exp $ */
+/* $Id: gdevpdfb.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Low-level bitmap image handling for PDF-writing driver */
 #include "string_.h"
 #include "gx.h"
@@ -23,6 +20,8 @@
 #include "gdevpdfg.h"
 #include "gdevpdfo.h"		/* for data stream */
 #include "gxcspace.h"
+#include "gxdcolor.h"
+#include "gxpcolor.h"
 #include "gxhldevc.h"
 
 /* We need this color space type for constructing temporary color spaces. */
@@ -31,7 +30,7 @@ extern const gs_color_space_type gs_color_space_type_Indexed;
 /* ---------------- Utilities ---------------- */
 
 /* Fill in the image parameters for a bitmap image. */
-private void
+static void
 pdf_make_bitmap_image(gs_image_t * pim, int x, int y, int w, int h)
 {
     pim->Width = w;
@@ -43,7 +42,7 @@ pdf_make_bitmap_image(gs_image_t * pim, int x, int y, int w, int h)
 
 /* Copy a mask bitmap.  for_pattern = -1 means put the image in-line, */
 /* 1 means put the image in a resource. */
-private int
+static int
 pdf_copy_mask_data(gx_device_pdf * pdev, const byte * base, int sourcex,
 		   int raster, gx_bitmap_id id, int x, int y, int w, int h,
 		   gs_image_t *pim, pdf_image_writer *piw,
@@ -90,10 +89,11 @@ pdf_copy_mask_data(gx_device_pdf * pdev, const byte * base, int sourcex,
     if (for_pattern < 0)
 	stream_puts(pdev->strm, "q ");
     pdf_image_writer_init(piw);
+    pdev->ParamCompatibilityLevel = pdev->CompatibilityLevel;
     if ((code = pdf_begin_write_image(pdev, piw, id, w, h, NULL, in_line)) < 0 ||
 	(code = psdf_setup_lossless_filters((gx_device_psdf *) pdev,
 					    &piw->binary[0],
-					    (gs_pixel_image_t *)pim)) < 0 ||
+					    (gs_pixel_image_t *)pim, in_line)) < 0 ||
 	(code = pdf_begin_image_data(pdev, piw, (const gs_pixel_image_t *)pim,
 				     NULL, 0)) < 0
 	)
@@ -103,18 +103,30 @@ pdf_copy_mask_data(gx_device_pdf * pdev, const byte * base, int sourcex,
     return pdf_end_write_image(pdev, piw);
 }
 
+static void 
+set_image_color(gx_device_pdf *pdev, gx_color_index c)
+{
+    pdf_set_pure_color(pdev, c, &pdev->saved_fill_color,
+			&pdev->fill_used_process_color,
+			&psdf_set_fill_color_commands);
+    if (!pdev->HaveStrokeColor)
+	pdf_set_pure_color(pdev, c, &pdev->saved_stroke_color,
+			    &pdev->stroke_used_process_color,
+			    &psdf_set_stroke_color_commands);
+}
+
 /* Copy a monochrome bitmap or mask. */
-private int
+static int
 pdf_copy_mono(gx_device_pdf *pdev,
 	      const byte *base, int sourcex, int raster, gx_bitmap_id id,
 	      int x, int y, int w, int h, gx_color_index zero,
 	      gx_color_index one, const gx_clip_path *pcpath)
 {
     int code;
-    gs_color_space cs;
+    gs_color_space *pcs = NULL;
     cos_value_t cs_value;
     cos_value_t *pcsvalue;
-    byte palette[sizeof(gx_color_index) * 2];
+    byte palette[arch_sizeof_color_index * 2];
     gs_image_t image;
     pdf_image_writer writer;
     pdf_stream_position_t ipos;
@@ -140,6 +152,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	    pres = pdf_find_resource_by_gs_id(pdev, resourceCharProc, id);
 	    if (pres == 0) {	/* Define the character in an embedded font. */
 		pdf_char_proc_t *pcp;
+		double x_offset;
 		int y_offset;
 
 		gs_image_t_init_mask(&image, false);
@@ -155,8 +168,11 @@ pdf_copy_mono(gx_device_pdf *pdev,
 		if (code < 0)
 		    return code;
 		y_offset = -y_offset;
-		pprintd3(pdev->strm, "0 0 0 %d %d %d d1\n", y_offset,
-			 w, h + y_offset);
+		x_offset = psdf_round(pdev->char_width.x, 100, 10); /* See 
+			pdf_write_Widths about rounding. We need to provide 
+			a compatible data for Tj. */
+		pprintg1(pdev->strm, "%g ", x_offset);
+		pprintd3(pdev->strm, "0 0 %d %d %d d1\n", y_offset, w, h + y_offset);
 		pprintd3(pdev->strm, "%d 0 0 %d 0 %d cm\n", w, h,
 			 y_offset);
 		pdf_image_writer_init(&writer);
@@ -168,28 +184,22 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	    } else if (pdev->pte) {
 		/* We're under pdf_text_process. It set a high level color. */
 	    } else
-		pdf_set_pure_color(pdev, one, &pdev->saved_fill_color,
-				   &pdev->fill_used_process_color,
-				   &psdf_set_fill_color_commands);
+		set_image_color(pdev, one);
 	    pdf_make_bitmap_matrix(&image.ImageMatrix, x, y, w, h, h);
 	    goto rx;
 	}
-	pdf_set_pure_color(pdev, one, &pdev->saved_fill_color,
-			   &pdev->fill_used_process_color,
-			   &psdf_set_fill_color_commands);
+	set_image_color(pdev, one);
 	gs_image_t_init_mask(&image, false);
 	invert = 0xff;
     } else if (one == gx_no_color_index) {
 	gs_image_t_init_mask(&image, false);
-	pdf_set_pure_color(pdev, zero, &pdev->saved_fill_color,
-			   &pdev->fill_used_process_color,
-			   &psdf_set_fill_color_commands);
+	set_image_color(pdev, zero);
     } else if (zero == pdev->black && one == pdev->white) {
-	gs_cspace_init_DeviceGray(pdev->memory, &cs);
-	gs_image_t_init(&image, &cs);
+	pcs = gs_cspace_new_DeviceGray(pdev->memory);
+	gs_image_t_init(&image, pcs);
     } else if (zero == pdev->white && one == pdev->black) {
-	gs_cspace_init_DeviceGray(pdev->memory, &cs);
-	gs_image_t_init(&image, &cs);
+	pcs = gs_cspace_new_DeviceGray(pdev->memory);
+	gs_image_t_init(&image, pcs);
 	invert = 0xff;
     } else {
 	/*
@@ -198,28 +208,33 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	 * internally, and high-level images don't go through this code.
 	 * However, we still want the code to work.
 	 */
-	gs_color_space cs_base;
+	gs_color_space *pcs_base;
 	gx_color_index c[2];
 	int i, j;
 	int ncomp = pdev->color_info.num_components;
 	byte *p;
 
-	code = pdf_cspace_init_Device(pdev->memory, &cs_base, ncomp);
+	code = pdf_cspace_init_Device(pdev->memory, &pcs_base, ncomp);
 	if (code < 0)
 	    return code;
 	c[0] = psdf_adjust_color_index((gx_device_vector *)pdev, zero);
 	c[1] = psdf_adjust_color_index((gx_device_vector *)pdev, one);
-	gs_cspace_init(&cs, &gs_color_space_type_Indexed, pdev->memory, false);
-	cs.params.indexed.base_space = *(gs_direct_color_space *)&cs_base;
-	cs.params.indexed.hival = 1;
+	pcs = gs_cspace_alloc(pdev->memory, &gs_color_space_type_Indexed);
+	if (pcs == NULL) {
+	    rc_decrement(pcs_base, "pdf_copy_mono");
+	    return_error(gs_error_VMerror);
+	}
+	pcs->base_space = pcs_base;
+	pcs->params.indexed.hival = 1;
+	pcs->params.indexed.n_comps = ncomp;
 	p = palette;
 	for (i = 0; i < 2; ++i)
 	    for (j = ncomp - 1; j >= 0; --j)
 		*p++ = (byte)(c[i] >> (j * 8));
-	cs.params.indexed.lookup.table.data = palette;
-	cs.params.indexed.lookup.table.size = p - palette;
-	cs.params.indexed.use_proc = false;
-	gs_image_t_init(&image, &cs);
+	pcs->params.indexed.lookup.table.data = palette;
+	pcs->params.indexed.lookup.table.size = p - palette;
+	pcs->params.indexed.use_proc = false;
+	gs_image_t_init(&image, pcs);
 	image.BitsPerComponent = 1;
     }
     pdf_make_bitmap_image(&image, x, y, w, h);
@@ -245,7 +260,7 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	 * We don't have to worry about color space scaling: the color
 	 * space is always a Device space.
 	 */
-	code = pdf_color_space(pdev, &cs_value, NULL, &cs,
+	code = pdf_color_space(pdev, &cs_value, NULL, pcs,
 			       &writer.pin->color_spaces, in_line);
 	if (code < 0)
 	    return code;
@@ -274,8 +289,9 @@ pdf_copy_mono(gx_device_pdf *pdev,
 	invert ^= 0xff;
     } else {
 	/* Use the Distiller compression parameters. */
+	pdev->ParamCompatibilityLevel = pdev->CompatibilityLevel;
 	psdf_setup_image_filters((gx_device_psdf *) pdev, &writer.binary[0],
-				 (gs_pixel_image_t *)&image, NULL, NULL, true);
+				 (gs_pixel_image_t *)&image, NULL, NULL, true, in_line);
     }
     pdf_begin_image_data(pdev, &writer, (const gs_pixel_image_t *)&image,
 			 pcsvalue, 0);
@@ -342,17 +358,17 @@ pdf_copy_color_data(gx_device_pdf * pdev, const byte * base, int sourcex,
 {
     int depth = pdev->color_info.depth;
     int bytes_per_pixel = depth >> 3;
-    gs_color_space cs;
+    gs_color_space *pcs;
     cos_value_t cs_value;
     ulong nbytes;
-    int code = pdf_cspace_init_Device(pdev->memory, &cs, bytes_per_pixel);
+    int code = pdf_cspace_init_Device(pdev->memory, &pcs, bytes_per_pixel);
     const byte *row_base;
     int row_step;
     bool in_line;
 
     if (code < 0)
 	return code;		/* can't happen */
-    gs_image_t_init(pim, &cs);
+    gs_image_t_init(pim, pcs);
     pdf_make_bitmap_image(pim, x, y, w, h);
     pim->BitsPerComponent = 8;
     nbytes = (ulong)w * bytes_per_pixel * h;
@@ -392,14 +408,15 @@ pdf_copy_color_data(gx_device_pdf * pdev, const byte * base, int sourcex,
      * space is always a Device space.
      */
     pdf_image_writer_init(piw);
+    pdev->ParamCompatibilityLevel = pdev->CompatibilityLevel;
     if ((code = pdf_begin_write_image(pdev, piw, id, w, h, NULL, in_line)) < 0 ||
-	(code = pdf_color_space(pdev, &cs_value, NULL, &cs,
+	(code = pdf_color_space(pdev, &cs_value, NULL, pcs,
 				&piw->pin->color_spaces, in_line)) < 0 ||
-	(for_pattern < 2 || (pdev->ForOPDFRead && nbytes < 512000) ?
+	(for_pattern < 2 || nbytes < 512000 ?
 	    (code = psdf_setup_lossless_filters((gx_device_psdf *) pdev,
-			&piw->binary[0], (gs_pixel_image_t *)pim)) :
+			&piw->binary[0], (gs_pixel_image_t *)pim, false)) :
 	    (code = psdf_setup_image_filters((gx_device_psdf *) pdev,
-			&piw->binary[0], (gs_pixel_image_t *)pim, NULL, NULL, false))
+			&piw->binary[0], (gs_pixel_image_t *)pim, NULL, NULL, false, false))
 	) < 0 ||
 	(code = pdf_begin_image_data(pdev, piw, (const gs_pixel_image_t *)pim,
 				     &cs_value, 0)) < 0
@@ -453,7 +470,7 @@ gdev_pdf_fill_mask(gx_device * dev,
 
     if (width <= 0 || height <= 0)
 	return 0;
-    if (depth > 1 || !gx_dc_is_pure(pdcolor) != 0)
+    if (depth > 1 || (!gx_dc_is_pure(pdcolor) != 0 && pdcolor->type != &gx_dc_pattern))
 	return gx_default_fill_mask(dev, data, data_x, raster, id,
 				    x, y, width, height, pdcolor, depth, lop,
 				    pcpath);
