@@ -1,20 +1,17 @@
-/* Copyright (C) 1999, 2000, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfg.c,v 1.61 2005/04/12 21:20:20 ray Exp $ */
+/* $Id: gdevpdfg.c 8550 2008-02-27 00:08:22Z leonardo $ */
 /* Graphics state management for pdfwrite driver */
 #include "math_.h"
 #include "string_.h"
@@ -61,6 +58,7 @@ pdf_save_viewer_state(gx_device_pdf *pdev, stream *s)
     pdev->vgstack[i].undercolor_removal_id = pdev->undercolor_removal_id;
     pdev->vgstack[i].overprint_mode = pdev->overprint_mode;
     pdev->vgstack[i].smoothness = pdev->state.smoothness;
+    pdev->vgstack[i].flatness = pdev->state.flatness;
     pdev->vgstack[i].text_knockout = pdev->state.text_knockout;
     pdev->vgstack[i].fill_overprint = pdev->fill_overprint;
     pdev->vgstack[i].stroke_overprint = pdev->stroke_overprint;
@@ -80,7 +78,7 @@ pdf_save_viewer_state(gx_device_pdf *pdev, stream *s)
 }
 
 /* Load the viewer's graphic state. */
-private void
+static void
 pdf_load_viewer_state(gx_device_pdf *pdev, pdf_viewer_state *s)
 {   
     pdev->transfer_ids[0] = s->transfer_ids[0];
@@ -96,6 +94,7 @@ pdf_load_viewer_state(gx_device_pdf *pdev, pdf_viewer_state *s)
     pdev->undercolor_removal_id = s->undercolor_removal_id;
     pdev->overprint_mode = s->overprint_mode;
     pdev->state.smoothness = s->smoothness;
+    pdev->state.flatness = s->flatness;
     pdev->state.text_knockout = s->text_knockout;
     pdev->fill_overprint = s->fill_overprint;
     pdev->stroke_overprint = s->stroke_overprint;
@@ -141,7 +140,7 @@ pdf_set_initial_color(gx_device_pdf * pdev, gx_hl_saved_color *saved_fill_color,
 }
 
 /* Prepare intitial values for viewer's graphics state parameters. */
-private void
+static void
 pdf_viewer_state_from_imager_state_aux(pdf_viewer_state *pvs, const gs_imager_state *pis)
 {
     pvs->transfer_not_identity = 
@@ -160,6 +159,7 @@ pdf_viewer_state_from_imager_state_aux(pdf_viewer_state *pvs, const gs_imager_st
     pvs->black_generation_id = (pis->black_generation != 0 ? pis->black_generation->id : 0);
     pvs->undercolor_removal_id = (pis->undercolor_removal != 0 ? pis->undercolor_removal->id : 0);
     pvs->overprint_mode = 0;
+    pvs->flatness = pis->flatness;
     pvs->smoothness = pis->smoothness;
     pvs->text_knockout = pis->text_knockout;
     pvs->fill_overprint = false;
@@ -215,7 +215,7 @@ pdf_prepare_initial_viewer_state(gx_device_pdf * pdev, const gs_imager_state *pi
 
 /* Reset the graphics state parameters to initial values. */
 /* Used if pdf_prepare_initial_viewer_state was not callad. */
-private void
+static void
 pdf_reset_graphics_old(gx_device_pdf * pdev)
 {
 
@@ -246,7 +246,7 @@ pdf_reset_graphics(gx_device_pdf * pdev)
 }
 
 /* Write client color. */
-private int
+static int
 pdf_write_ccolor(gx_device_pdf * pdev, const gs_imager_state * pis, 
 	        const gs_client_color *pcc)
 {   
@@ -259,17 +259,41 @@ pdf_write_ccolor(gx_device_pdf * pdev, const gs_imager_state * pis,
     return 0;
 }
 
+static inline bool
+is_cspace_allowed_in_strategy(gx_device_pdf * pdev, gs_color_space_index csi)
+{
+    if (pdev->params.ColorConversionStrategy == ccs_CMYK && 
+	    csi != gs_color_space_index_DeviceCMYK &&
+	    csi != gs_color_space_index_DeviceGray)
+	return false;
+    if (pdev->params.ColorConversionStrategy == ccs_sRGB && 
+	    csi != gs_color_space_index_DeviceRGB && 
+	    csi != gs_color_space_index_DeviceGray)
+	return false;
+    if (pdev->params.ColorConversionStrategy == ccs_Gray && 
+	    csi != gs_color_space_index_DeviceGray)
+	return false;
+    return true;
+}
+
+static inline bool
+is_pattern2_allowed_in_strategy(gx_device_pdf * pdev, const gx_drawing_color *pdc)
+{
+    const gs_color_space *pcs2 = gx_dc_pattern2_get_color_space(pdc);
+    gs_color_space_index csi = gs_color_space_get_index(pcs2);
+
+    return is_cspace_allowed_in_strategy(pdev, csi);
+}
 
 /* Set the fill or stroke color. */
-private int
+int
 pdf_reset_color(gx_device_pdf * pdev, const gs_imager_state * pis, 
 	        const gx_drawing_color *pdc, gx_hl_saved_color * psc,
 		bool *used_process_color,
 		const psdf_set_color_commands_t *ppscc)
 {
-    int code;
+    int code = 0;
     gx_hl_saved_color temp;
-    bool process_color;
     const gs_color_space *pcs, *pcs2;
     const gs_client_color *pcc; /* fixme: not needed due to gx_hld_get_color_component. */
     cos_value_t cs_value;
@@ -279,23 +303,14 @@ pdf_reset_color(gx_device_pdf * pdev, const gs_imager_state * pis,
 
     if (pdev->skip_colors)
 	return 0;
-    process_color = !gx_hld_save_color(pis, pdc, &temp);
+    gx_hld_save_color(pis, pdc, &temp);
     /* Since pdfwrite never applies halftones and patterns, but monitors
      * halftone/pattern IDs separately, we don't need to compare
      * halftone/pattern bodies here.
      */
     if (gx_hld_saved_color_equal(&temp, psc))
 	return 0;
-    /*
-     * In principle, we can set colors in either stream or text
-     * context.  However, since we currently enclose all text
-     * strings inside a gsave/grestore, this causes us to lose
-     * track of the color when we leave text context.  Therefore,
-     * we require stream context for setting colors.
-     */
-    code = pdf_open_page(pdev, PDF_IN_STREAM);
-    if (code < 0)
-	return code;
+
     switch (gx_hld_get_color_space_and_ccolor(pis, pdc, &pcs, &pcc)) {
 	case non_pattern_color_space:
 	    switch (gs_color_space_get_index(pcs)) {
@@ -303,17 +318,27 @@ pdf_reset_color(gx_device_pdf * pdev, const gs_imager_state * pis,
 		    command = ppscc->setgray; 
 		    break;
 		case gs_color_space_index_DeviceRGB:
+		    if (pdev->params.ColorConversionStrategy == ccs_CMYK)
+			goto write_process_color;
+		    if (pdev->params.ColorConversionStrategy == ccs_Gray)
+			goto write_process_color;
 		    command = ppscc->setrgbcolor; 
 		    break;
 		case gs_color_space_index_DeviceCMYK:
+		    if (pdev->params.ColorConversionStrategy == ccs_sRGB)
+			goto write_process_color;
+		    if (pdev->params.ColorConversionStrategy == ccs_Gray)
+			goto write_process_color;
 		    command = ppscc->setcmykcolor; 
 		    break;
 		case gs_color_space_index_Indexed:
 		    if (pdev->CompatibilityLevel <= 1.2) {
-			pcs2 = (const gs_color_space *)&pcs->params.indexed.base_space;
+			pcs2 = pcs->base_space;
 			csi = gs_color_space_get_index(pcs2);
+			if (!is_cspace_allowed_in_strategy(pdev, csi))
+			    goto write_process_color;
 			if (csi == gs_color_space_index_Separation) {
-			    pcs2 = (const gs_color_space *)&pcs2->params.separation.alt_space;
+			    pcs2 = pcs->base_space;
 			    goto check_pcs2;
 			}
 			goto check_pcs2;
@@ -321,9 +346,11 @@ pdf_reset_color(gx_device_pdf * pdev, const gs_imager_state * pis,
 		    goto scn;
 		case gs_color_space_index_Separation:
 		    if (pdev->CompatibilityLevel <= 1.2) {
-			pcs2 = (const gs_color_space *)&pcs->params.separation.alt_space;
+			pcs2 = pcs->base_space;
 			check_pcs2:
 			csi = gs_color_space_get_index(pcs2);
+			if (!is_cspace_allowed_in_strategy(pdev, csi))
+			    goto write_process_color;
 			switch(gs_color_space_get_index(pcs2)) {
 			    case gs_color_space_index_DevicePixel :
 			    case gs_color_space_index_DeviceN:
@@ -341,6 +368,12 @@ pdf_reset_color(gx_device_pdf * pdev, const gs_imager_state * pis,
 	    		goto write_process_color;
 		    goto scn;
 		default :
+		    if (pdev->params.ColorConversionStrategy == ccs_CMYK)
+			goto write_process_color;
+		    if (pdev->params.ColorConversionStrategy == ccs_sRGB)
+			goto write_process_color;
+		    if (pdev->params.ColorConversionStrategy == ccs_Gray)
+			goto write_process_color;
 	        scn:
 		    command = ppscc->setcolorn;
 		    if (!gx_hld_saved_color_same_cspace(&temp, psc)) {
@@ -383,6 +416,8 @@ pdf_reset_color(gx_device_pdf * pdev, const gs_imager_state * pis,
 		} else if (pdc->type == &gx_dc_pattern2) {
 		    if (pdev->CompatibilityLevel <= 1.2)
 	    		return_error(gs_error_rangecheck);
+		    if (!is_pattern2_allowed_in_strategy(pdev, pdc))
+	    		return_error(gs_error_rangecheck);
 		    code1 = pdf_put_pattern2(pdev, pdc, ppscc, &pres);
 		} else
 		    return_error(gs_error_rangecheck);
@@ -414,6 +449,34 @@ pdf_set_drawing_color(gx_device_pdf * pdev, const gs_imager_state * pis,
 		      bool *used_process_color,
 		      const psdf_set_color_commands_t *ppscc)
 {
+    gx_hl_saved_color temp;
+    int code;
+
+    /* This section of code was in pdf_reset_color above, but was moved into this 
+     * routine (and below) in order to isolate the switch to a stream context. This
+     * now allows us the opportunity to write colours in any context, in particular
+     * when in a text context, by using pdf_reset_color.
+     */
+    if (pdev->skip_colors)
+	return 0;
+    gx_hld_save_color(pis, pdc, &temp);
+    /* Since pdfwrite never applies halftones and patterns, but monitors
+     * halftone/pattern IDs separately, we don't need to compare
+     * halftone/pattern bodies here.
+     */
+    if (gx_hld_saved_color_equal(&temp, psc))
+	return 0;
+    /*
+     * In principle, we can set colors in either stream or text
+     * context.  However, since we currently enclose all text
+     * strings inside a gsave/grestore, this causes us to lose
+     * track of the color when we leave text context.  Therefore,
+     * we require stream context for setting colors.
+     */
+    code = pdf_open_page(pdev, PDF_IN_STREAM);
+    if (code < 0)
+	return code;
+
     return pdf_reset_color(pdev, pis, pdc, psc, used_process_color, ppscc);
 }
 int
@@ -423,8 +486,31 @@ pdf_set_pure_color(gx_device_pdf * pdev, gx_color_index color,
 		   const psdf_set_color_commands_t *ppscc)
 {
     gx_drawing_color dcolor;
+    gx_hl_saved_color temp;
+    int code;
 
     set_nonclient_dev_color(&dcolor, color);
+
+    if (pdev->skip_colors)
+	return 0;
+    gx_hld_save_color(NULL, &dcolor, &temp);
+    /* Since pdfwrite never applies halftones and patterns, but monitors
+     * halftone/pattern IDs separately, we don't need to compare
+     * halftone/pattern bodies here.
+     */
+    if (gx_hld_saved_color_equal(&temp, psc))
+	return 0;
+    /*
+     * In principle, we can set colors in either stream or text
+     * context.  However, since we currently enclose all text
+     * strings inside a gsave/grestore, this causes us to lose
+     * track of the color when we leave text context.  Therefore,
+     * we require stream context for setting colors.
+     */
+    code = pdf_open_page(pdev, PDF_IN_STREAM);
+    if (code < 0)
+	return code;
+
     return pdf_reset_color(pdev, NULL, &dcolor, psc, used_process_color, ppscc);
 }
 
@@ -460,8 +546,8 @@ pdf_string_to_cos_name(gx_device_pdf *pdev, const byte *str, uint len,
  * an identity map.  Return 1 if the map is the identity map, otherwise
  * return 0.
  */
-private data_source_proc_access(transfer_map_access); /* check prototype */
-private int
+static data_source_proc_access(transfer_map_access); /* check prototype */
+static int
 transfer_map_access(const gs_data_source_t *psrc, ulong start, uint length,
 		    byte *buf, const byte **ptr)
 {
@@ -474,7 +560,7 @@ transfer_map_access(const gs_data_source_t *psrc, ulong start, uint length,
 	buf[i] = frac2byte(map->values[(uint)start + i]);
     return 0;
 }
-private int
+static int
 transfer_map_access_signed(const gs_data_source_t *psrc,
 			   ulong start, uint length,
 			   byte *buf, const byte **ptr)
@@ -494,7 +580,7 @@ transfer_map_access_signed(const gs_data_source_t *psrc,
 	    ((frac2float(map->values[(uint)start + i]) + 1) * 127);
     return 0;
 }
-private int
+static int
 pdf_write_transfer_map(gx_device_pdf *pdev, const gx_transfer_map *map,
 		       int range0, bool check_identity,
 		       const char *key, char *ids)
@@ -569,7 +655,7 @@ pdf_write_transfer_map(gx_device_pdf *pdev, const gx_transfer_map *map,
     sprintf(ids, "%s%s%ld 0 R", key, (key[0] && key[0] != ' ' ? " " : ""), id);
     return 0;
 }
-private int
+static int
 pdf_write_transfer(gx_device_pdf *pdev, const gx_transfer_map *map,
 		   const char *key, char *ids)
 {
@@ -585,7 +671,7 @@ pdf_write_transfer(gx_device_pdf *pdev, const gx_transfer_map *map,
  * results.  Currently we only do this for a few of the functions.
  */
 #define HT_FUNC(name, expr)\
-  private floatp name(floatp xd, floatp yd) {\
+  static floatp name(floatp xd, floatp yd) {\
     float x = (float)xd, y = (float)yd;\
     return d2f(expr);\
   }
@@ -595,13 +681,13 @@ pdf_write_transfer(gx_device_pdf *pdev, const gx_transfer_map *map,
  * doesn't actually do the coercion.  Force this here.  Note that if we
  * use 'inline', it doesn't work.
  */
-private float
+static float
 d2f(floatp d)
 {
     float f = (float)d;
     return f;
 }
-private floatp
+static floatp
 ht_Round(floatp xf, floatp yf)
 {
     float x = (float)xf, y = (float)yf;
@@ -612,7 +698,7 @@ ht_Round(floatp xf, floatp yf)
     xabs -= 1, yabs -= 1;
     return d2f(d2f(d2f(xabs * xabs) + d2f(yabs * yabs)) - 1);
 }
-private floatp
+static floatp
 ht_Diamond(floatp xf, floatp yf)
 {
     float x = (float)xf, y = (float)yf;
@@ -625,7 +711,7 @@ ht_Diamond(floatp xf, floatp yf)
     xabs -= 1, yabs -= 1;
     return d2f(d2f(d2f(xabs * xabs) + d2f(yabs * yabs)) - 1);
 }
-private floatp
+static floatp
 ht_Ellipse(floatp xf, floatp yf)
 {
     float x = (float)xf, y = (float)yf;
@@ -651,11 +737,11 @@ ht_Ellipse(floatp xf, floatp yf)
  * Most of these are recognized properly even without d2f.  We've only
  * added d2f where it apparently makes a difference.
  */
-private float
+static float
 d2fsin_d(double x) {
     return d2f(gs_sin_degrees(d2f(x)));
 }
-private float
+static float
 d2fcos_d(double x) {
     return d2f(gs_cos_degrees(d2f(x)));
 }
@@ -681,7 +767,7 @@ typedef struct ht_function_s {
     const char *fname;
     floatp (*proc)(floatp, floatp);
 } ht_function_t;
-private const ht_function_t ht_functions[] = {
+static const ht_function_t ht_functions[] = {
     {"Round", ht_Round},
     {"Diamond", ht_Diamond},
     {"Ellipse", ht_Ellipse},
@@ -706,7 +792,7 @@ private const ht_function_t ht_functions[] = {
 };
 
 /* Write each kind of halftone. */
-private int
+static int
 pdf_write_spot_function(gx_device_pdf *pdev, const gx_ht_order *porder,
 			long *pid)
 {
@@ -775,7 +861,7 @@ pdf_write_spot_function(gx_device_pdf *pdev, const gx_ht_order *porder,
     gs_free_object(mem, values, "pdf_write_spot_function");
     return code;
 }
-private int
+static int
 pdf_write_spot_halftone(gx_device_pdf *pdev, const gs_spot_halftone *psht,
 			const gx_ht_order *porder, long *pid)
 {
@@ -844,7 +930,7 @@ pdf_write_spot_halftone(gx_device_pdf *pdev, const gs_spot_halftone *psht,
     stream_puts(s, ">>\n");
     return pdf_end_separate(pdev);
 }
-private int
+static int
 pdf_write_screen_halftone(gx_device_pdf *pdev, const gs_screen_halftone *psht,
 			  const gx_ht_order *porder, long *pid)
 {
@@ -856,7 +942,7 @@ pdf_write_screen_halftone(gx_device_pdf *pdev, const gs_screen_halftone *psht,
     spot.transfer_closure.proc = 0;
     return pdf_write_spot_halftone(pdev, &spot, porder, pid);
 }
-private int
+static int
 pdf_write_colorscreen_halftone(gx_device_pdf *pdev,
 			       const gs_colorscreen_halftone *pcsht,
 			       const gx_device_halftone *pdht, long *pid)
@@ -890,7 +976,7 @@ pdf_write_colorscreen_halftone(gx_device_pdf *pdev,
 #define CHECK(expr)\
   BEGIN if ((code = (expr)) < 0) return code; END
 
-private int
+static int
 pdf_write_threshold_halftone(gx_device_pdf *pdev,
 			     const gs_threshold_halftone *ptht,
 			     const gx_ht_order *porder, long *pid)
@@ -920,7 +1006,7 @@ pdf_write_threshold_halftone(gx_device_pdf *pdev,
     stream_write(writer.binary.strm, ptht->thresholds.data, ptht->thresholds.size);
     return pdf_end_data(&writer);
 }
-private int
+static int
 pdf_write_threshold2_halftone(gx_device_pdf *pdev,
 			      const gs_threshold2_halftone *ptht,
 			      const gx_ht_order *porder, long *pid)
@@ -969,7 +1055,7 @@ pdf_write_threshold2_halftone(gx_device_pdf *pdev,
     }
     return pdf_end_data(&writer);
 }
-private int 
+static int 
 pdf_get_halftone_component_index(const gs_multiple_halftone *pmht,
 				 const gx_device_halftone *pdht,
 				 int dht_index)
@@ -989,7 +1075,7 @@ pdf_get_halftone_component_index(const gs_multiple_halftone *pmht,
     }
     return j;
 }
-private int
+static int
 pdf_write_multiple_halftone(gx_device_pdf *pdev,
 			    const gs_multiple_halftone *pmht,
 			    const gx_device_halftone *pdht, long *pid)
@@ -1088,7 +1174,7 @@ pdf_write_multiple_halftone(gx_device_pdf *pdev,
  * Update the halftone.  This is a separate procedure only for
  * readability.
  */
-private int
+static int
 pdf_update_halftone(gx_device_pdf *pdev, const gs_imager_state *pis,
 		    char *hts)
 {
@@ -1135,14 +1221,14 @@ pdf_update_halftone(gx_device_pdf *pdev, const gs_imager_state *pis,
 
 /* ------ Graphics state updating ------ */
 
-private inline cos_dict_t *
+static inline cos_dict_t *
 resource_dict(pdf_resource_t *pres)
 {
     return (cos_dict_t *)pres->object;
 }
 
 /* Open an ExtGState. */
-private int
+static int
 pdf_open_gstate(gx_device_pdf *pdev, pdf_resource_t **ppres)
 {
     int code;
@@ -1168,35 +1254,15 @@ pdf_open_gstate(gx_device_pdf *pdev, pdf_resource_t **ppres)
     return 0;
 }
 
-private int 
-pdf_is_same_extgstate(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
-{
-    return true;
-}
-
 /* Finish writing an ExtGState. */
-private int
+int
 pdf_end_gstate(gx_device_pdf *pdev, pdf_resource_t *pres)
 {
     if (pres) {
-	pdf_resource_t *pres1 = pres;
-	int code;
-
-	code = pdf_find_same_resource(pdev, resourceExtGState, &pres, pdf_is_same_extgstate);
+	int code = pdf_substitute_resource(pdev, &pres, resourceExtGState, NULL, true);
+	
 	if (code < 0)
 	    return code;
-	if (code != 0) {
-	    code = pdf_cancel_resource(pdev, (pdf_resource_t *)pres1, resourceExtGState);
-	    if (code < 0)
-		return code;
-	    pdf_forget_resource(pdev, pres1, resourceExtGState);
-	} else {
-	    pdf_reserve_object_id(pdev, pres, gs_no_id);
-	    code = cos_write_object(pres->object, pdev);
-	    if (code < 0)
-		return code;
-	    pres->object->written = 1;
-	}
 	code = pdf_open_page(pdev, PDF_IN_STREAM);
 	if (code < 0)
 	    return code;
@@ -1213,7 +1279,7 @@ pdf_end_gstate(gx_device_pdf *pdev, pdf_resource_t *pres)
  * Update the transfer functions(s).  This is a separate procedure only
  * for readability.
  */
-private int
+static int
 pdf_update_transfer(gx_device_pdf *pdev, const gs_imager_state *pis,
 		    char *trs)
 {
@@ -1270,7 +1336,7 @@ pdf_update_transfer(gx_device_pdf *pdev, const gs_imager_state *pis,
  * stores separate opacity and shape alpha, a rangecheck will occur if
  * both are different from the current setting.
  */
-private int
+static int
 pdf_update_alpha(gx_device_pdf *pdev, const gs_imager_state *pis,
 		 pdf_resource_t **ppres)
 {
@@ -1278,6 +1344,22 @@ pdf_update_alpha(gx_device_pdf *pdev, const gs_imager_state *pis,
     floatp alpha;
     int code;
 
+    if (pdev->state.soft_mask_id != pis->soft_mask_id) {
+	char buf[20];
+
+	if (pis->soft_mask_id == 0)
+	    strcpy(buf, "/None");
+	else
+	    sprintf(buf, "%ld 0 R", pis->soft_mask_id);
+	code = pdf_open_gstate(pdev, ppres);
+	if (code < 0)
+	    return code;
+	code = cos_dict_put_c_key_string(resource_dict(*ppres), 
+		    "/SMask", (byte *)buf, strlen(buf));
+	if (code < 0)
+	    return code;
+	pdev->state.soft_mask_id = pis->soft_mask_id;
+    }
     if (pdev->state.opacity.alpha != pis->opacity.alpha) {
 	if (pdev->state.shape.alpha != pis->shape.alpha)
 	    return_error(gs_error_rangecheck);
@@ -1305,11 +1387,12 @@ pdf_update_alpha(gx_device_pdf *pdev, const gs_imager_state *pis,
 /*
  * Update the graphics subset common to all high-level drawing operations.
  */
-private int
+int
 pdf_prepare_drawing(gx_device_pdf *pdev, const gs_imager_state *pis,
 		    pdf_resource_t **ppres)
 {
     int code = 0;
+    int bottom;
 
     if (pdev->CompatibilityLevel >= 1.4) {
 	if (pdev->state.blend_mode != pis->blend_mode) {
@@ -1334,8 +1417,8 @@ pdf_prepare_drawing(gx_device_pdf *pdev, const gs_imager_state *pis,
 	 * If the graphics state calls for any transparency functions,
 	 * we can't represent them, so return a rangecheck.
 	 */
-	if (pis->opacity.alpha != 1 || pis->opacity.mask != 0 ||
-	    pis->shape.alpha != 1 || pis->shape.mask != 0 ||
+	if (pis->opacity.alpha != 1 || 
+	    pis->shape.alpha != 1 || 
 	    pis->transparency_stack != 0
 	    )
 	    return_error(gs_error_rangecheck);
@@ -1349,7 +1432,10 @@ pdf_prepare_drawing(gx_device_pdf *pdev, const gs_imager_state *pis,
      * removal, halftone phase, overprint mode, smoothness, blend mode, text
      * knockout.
      */
-    if (pdev->sbstack_depth == 0) {
+    bottom = (pdev->ResourcesBeforeUsage ? 1 : 0);
+    /* When ResourcesBeforeUsage != 0, one sbstack element 
+       appears from the page contents stream. */
+    if (pdev->sbstack_depth == bottom) {
 	gs_int_point phase, dev_phase;
 	char hts[5 + MAX_FN_CHARS + 1],
 	    trs[5 + MAX_FN_CHARS * 4 + 6 + 1],
@@ -1358,13 +1444,16 @@ pdf_prepare_drawing(gx_device_pdf *pdev, const gs_imager_state *pis,
 
 	hts[0] = trs[0] = bgs[0] = ucrs[0] = 0;
 	if (pdev->params.PreserveHalftoneInfo &&
-	    pdev->halftone_id != pis->dev_ht->id
+	    pdev->halftone_id != pis->dev_ht->id &&
+	    !pdev->PDFX
 	    ) {
 	    code = pdf_update_halftone(pdev, pis, hts);
 	    if (code < 0)
 		return code;
 	}
-	if (pdev->params.TransferFunctionInfo == tfi_Preserve) {
+	if (pdev->params.TransferFunctionInfo == tfi_Preserve &&
+	    !pdev->PDFX && !pdev->PDFA
+	    ) {
 	    code = pdf_update_transfer(pdev, pis, trs);
 	    if (code < 0)
 		return code;
@@ -1410,23 +1499,25 @@ pdf_prepare_drawing(gx_device_pdf *pdev, const gs_imager_state *pis,
 	    if (code < 0)
 		return code;
 	}
-	gs_currentscreenphase_pis(pis, &phase, 0);
-	gs_currentscreenphase_pis(&pdev->state, &dev_phase, 0);
-	if (dev_phase.x != phase.x || dev_phase.y != phase.y) {
-	    char buf[sizeof(int) * 3 + 5];
+	if (!pdev->PDFX) {
+	    gs_currentscreenphase_pis(pis, &phase, 0);
+	    gs_currentscreenphase_pis(&pdev->state, &dev_phase, 0);
+	    if (dev_phase.x != phase.x || dev_phase.y != phase.y) {
+		char buf[sizeof(int) * 3 + 5];
 
-	    code = pdf_open_gstate(pdev, ppres);
-	    if (code < 0)
-		return code;
-	    sprintf(buf, "[%d %d]", phase.x, phase.y);
-	    code = cos_dict_put_string_copy(resource_dict(*ppres), "/HTP", buf);
-	    if (code < 0)
-		return code;
-	    gx_imager_setscreenphase(&pdev->state, phase.x, phase.y,
-				     gs_color_select_all);
+		code = pdf_open_gstate(pdev, ppres);
+		if (code < 0)
+		    return code;
+		sprintf(buf, "[%d %d]", phase.x, phase.y);
+		code = cos_dict_put_string_copy(resource_dict(*ppres), "/HTP", buf);
+		if (code < 0)
+		    return code;
+		gx_imager_setscreenphase(&pdev->state, phase.x, phase.y,
+					 gs_color_select_all);
+	    }
 	}
     }
-    if (pdev->CompatibilityLevel >= 1.3 && pdev->sbstack_depth == 0) {
+    if (pdev->CompatibilityLevel >= 1.3 && pdev->sbstack_depth == bottom) {
 	if (pdev->overprint_mode != pdev->params.OPM) {
 	    code = pdf_open_gstate(pdev, ppres);
 	    if (code < 0)
@@ -1509,7 +1600,7 @@ pdf_prepare_fill(gx_device_pdf *pdev, const gs_imager_state *pis)
 }
 
 /* Update the graphics state for stroking. */
-private int
+static int
 pdf_try_prepare_stroke(gx_device_pdf *pdev, const gs_imager_state *pis)
 {
     pdf_resource_t *pres = 0;
