@@ -1,20 +1,17 @@
-/* Copyright (C) 1999, 2000, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfu.c,v 1.76 2005/04/05 15:44:44 igor Exp $ */
+/* $Id: gdevpdfu.c 8499 2008-01-23 16:49:51Z ken $ */
 /* Output utilities for PDF-writing driver */
 #include "memory_.h"
 #include "jpeglib_.h"		/* for sdct.h */
@@ -41,6 +38,12 @@
 #include "smd5.h"
 #include "sstring.h"
 #include "szlibx.h"
+#ifdef USE_LDF_JB2
+#include "sjbig2_luratech.h"
+#endif
+#ifdef USE_LWF_JP2
+#include "sjpx_luratech.h"
+#endif
 
 /* Define the size of internal stream buffers. */
 /* (This is not a limitation, it only affects performance.) */
@@ -75,18 +78,256 @@ private_st_pdf_pattern();
 
 /* ---------------- Utilities ---------------- */
 
+/*
+ * Strip whitespace and comments from a line of PostScript code as possible.
+ * Return a pointer to any string that remains, or NULL if none.
+ * Note that this may store into the string.
+ */
+/* This function copied from geninit.c . */
+static char *
+doit(char *line, bool intact)
+{
+    char *str = line;
+    char *from;
+    char *to;
+    int in_string = 0;
+
+    if (intact)
+	return str;
+    while (*str == ' ' || *str == '\t')		/* strip leading whitespace */
+	++str;
+    if (*str == 0)		/* all whitespace */
+	return NULL;
+    if (!strncmp(str, "%END", 4))	/* keep these for .skipeof */
+	return str;
+    if (str[0] == '%')    /* comment line */
+	return NULL;
+    /*
+     * Copy the string over itself removing:
+     *  - All comments not within string literals;
+     *  - Whitespace adjacent to '[' ']' '{' '}';
+     *  - Whitespace before '/' '(' '<';
+     *  - Whitespace after ')' '>'.
+     */
+    for (to = from = str; (*to = *from) != 0; ++from, ++to) {
+	switch (*from) {
+	    case '%':
+		if (!in_string)
+		    break;
+		continue;
+	    case ' ':
+	    case '\t':
+		if (to > str && !in_string && strchr(" \t>[]{})", to[-1]))
+		    --to;
+		continue;
+	    case '(':
+	    case '<':
+	    case '/':
+	    case '[':
+	    case ']':
+	    case '{':
+	    case '}':
+		if (to > str && !in_string && strchr(" \t", to[-1]))
+		    *--to = *from;
+                if (*from == '(')
+                    ++in_string;
+              	continue;
+	    case ')':
+		--in_string;
+		continue;
+	    case '\\':
+		if (from[1] == '\\' || from[1] == '(' || from[1] == ')')
+		    *++to = *++from;
+		continue;
+	    default:
+		continue;
+	}
+	break;
+    }
+    /* Strip trailing whitespace. */
+    while (to > str && (to[-1] == ' ' || to[-1] == '\t'))
+	--to;
+    *to = 0;
+    return str;
+}
+
+
+static int
+copy_ps_file_stripping(stream *s, const char *fname, bool HaveTrueTypes)
+{
+    FILE *f;
+    char buf[1024], *p, *q  = buf;
+    int n, l = 0, m = sizeof(buf) - 1, outl = 0;
+    bool skipping = false;
+
+    f = gp_fopen(fname, "rb");
+    if (f == NULL)
+	return_error(gs_error_undefinedfilename);
+    n = fread(buf, 1, m, f);
+    buf[n] = 0;
+    do {
+	if (*q == '\r' || *q == '\n') {
+	    q++;
+	    continue;
+	}
+	p = strchr(q, '\r');
+	if (p == NULL)
+	    p = strchr(q, '\n');
+	if (p == NULL) {
+	    if (n < m)
+		p = buf + n;
+	    else {
+		strcpy(buf, q);
+		l = strlen(buf);
+		m = sizeof(buf) - 1 - l;
+		if (!m) {
+		    eprintf1("The procset %s contains a too long line.", fname);
+		    return_error(gs_error_ioerror);
+		}
+		n = fread(buf + l, 1, m, f);
+		n += l;
+		m += l;
+		buf[n] = 0;
+		q = buf;
+		continue;
+	    }
+	}
+	*p = 0;
+	if (q[0] == '%')
+	    l = 0;
+	else {
+	    q = doit(q, false);
+	    if (q == NULL)
+		l = 0;
+	    else
+		l = strlen(q);
+	}
+	if (l) {
+	    if (!HaveTrueTypes && !strcmp("%%beg TrueType", q))
+		skipping = true;
+	    if (!skipping) {
+		outl += l + 1;
+		if (outl > 100) {
+		    q[l] = '\r';
+		    outl = 0;
+		} else
+		    q[l] = ' ';
+		stream_write(s, q, l + 1);
+	    }
+	    if (!HaveTrueTypes && !strcmp("%%end TrueType", q))
+		skipping = false;
+	}
+	q = p + 1;
+    } while (n == m || q < buf + n);
+    if (outl)
+	stream_write(s, "\r", 1);
+    fclose(f);
+    return 0;
+}
+
+static int
+copy_procsets(stream *s, const gs_param_string *path, bool HaveTrueTypes)
+{
+    char fname[gp_file_name_sizeof];
+    const byte *p = path->data, *e = path->data + path->size;
+    int l, i = 0, code;
+    const char *tt_encs[] = {"gs_agl.ps", "gs_mgl_e.ps"};
+
+    if (p != NULL) {
+	for (;; i++) {
+	    const byte *c = memchr(p, gp_file_name_list_separator, e - p);
+	    int k = 0; /* Initializing against a compiler warning only. */
+
+	    if (c == NULL)
+		c = e;
+	    l = c - p;
+	    if (l > 0) {
+		if (l > sizeof(fname) - 1)
+		    return_error(gs_error_limitcheck);
+		memcpy(fname, p, l);
+		fname[l] = 0;
+		if (!HaveTrueTypes) {
+		    for (k = count_of(tt_encs) - 1; k >= 0; k--) {
+			int L = strlen(tt_encs[k]);
+
+			if (!strcmp(fname + strlen(fname) - L, tt_encs[k]))
+			    break;
+		    }
+		}
+		if (HaveTrueTypes || k < 0) {
+		    code = copy_ps_file_stripping(s, fname, HaveTrueTypes);
+		    if (code < 0)
+			return code;
+		}
+	    }
+	    if (c == e)
+		break;
+	    p = c + 1;
+	}
+    }
+    if (!i)
+	return_error(gs_error_undefinedfilename);
+    return 0;
+}
+
+static int
+encode(stream **s, const stream_template *t, gs_memory_t *mem)
+{
+    stream_state *st = s_alloc_state(mem, t->stype, "pdf_open_document.encode");
+
+    if (st == 0)
+	return_error(gs_error_VMerror);
+    if (t->set_defaults)
+	t->set_defaults(st);
+    if (s_add_filter(s, t, st, mem) == 0) {
+	gs_free_object(mem, st, "pdf_open_document.encode");
+	return_error(gs_error_VMerror);
+    }
+    return 0;
+}
+
 /* ------ Document ------ */
 
 /* Open the document if necessary. */
-void
+int
 pdf_open_document(gx_device_pdf * pdev)
 {
     if (!is_in_page(pdev) && pdf_stell(pdev) == 0) {
 	stream *s = pdev->strm;
 	int level = (int)(pdev->CompatibilityLevel * 10 + 0.5);
 
-	pprintd2(s, "%%PDF-%d.%d\n", level / 10, level % 10);
 	pdev->binary_ok = !pdev->params.ASCII85EncodePages;
+	if (pdev->ForOPDFRead && pdev->OPDFReadProcsetPath.size) {
+	    int code, status;
+	    
+	    stream_write(s, (byte *)"%!PS-Adobe-2.0\r", 15);
+	    if (pdev->params.CompressPages || pdev->CompressEntireFile) {
+		/*  When CompressEntireFile is true and ASCII85EncodePages is false,
+		    the ASCII85Encode filter is applied, rather one may expect the opposite.
+		    Keeping it so due to no demand for this mode.
+		    A right implementation should compute the length of the compressed procset,
+		    write out an invocation of SubFileDecode filter, and write the length to
+		    there assuming the output file is positionable. */
+		stream_write(s, (byte *)"currentfile /ASCII85Decode filter /LZWDecode filter cvx exec\r", 61);
+		code = encode(&s, &s_A85E_template, pdev->pdf_memory);
+		if (code < 0)
+		    return code;
+		code = encode(&s, &s_LZWE_template, pdev->pdf_memory);
+		if (code < 0)
+		    return code;
+	    }
+	    code = copy_procsets(s, &pdev->OPDFReadProcsetPath, pdev->HaveTrueTypes);
+	    if (code < 0)
+		return code;
+	    if (!pdev->CompressEntireFile) {
+		status = s_close_filters(&s, pdev->strm);
+		if (status < 0)
+		    return_error(gs_error_ioerror);
+	    } else
+		pdev->strm = s;
+	    pdev->OPDFRead_procset_length = stell(s);
+	}
+	pprintd2(s, "%%PDF-%d.%d\n", level / 10, level % 10);
 	if (pdev->binary_ok)
 	    stream_puts(s, "%\307\354\217\242\n");
     }
@@ -103,12 +344,13 @@ pdf_open_document(gx_device_pdf * pdev)
 	pdev->compression = pdf_compress_none;
     else
 	pdev->compression = pdf_compress_Flate;
+    return 0;
 }
 
 /* ------ Objects ------ */
 
 /* Allocate an object ID. */
-private long
+static long
 pdf_next_id(gx_device_pdf * pdev)
 {
     return (pdev->next_id)++;
@@ -181,12 +423,12 @@ pdf_end_obj(gx_device_pdf * pdev)
 /* ------ Page contents ------ */
 
 /* Handle transitions between contexts. */
-private int
+static int
     none_to_stream(gx_device_pdf *), stream_to_text(gx_device_pdf *),
     string_to_text(gx_device_pdf *), text_to_stream(gx_device_pdf *),
     stream_to_none(gx_device_pdf *);
 typedef int (*context_proc) (gx_device_pdf *);
-private const context_proc context_procs[4][4] =
+static const context_proc context_procs[4][4] =
 {
     {0, none_to_stream, none_to_stream, none_to_stream},
     {stream_to_none, 0, stream_to_text, stream_to_text},
@@ -195,20 +437,20 @@ private const context_proc context_procs[4][4] =
 };
 
 /* Compute an object encryption key. */
-private int
+static int
 pdf_object_key(const gx_device_pdf * pdev, gs_id object_id, byte key[16])
 {
-    md5_state_t md5;
-    md5_byte_t zero[2] = {0, 0}, t;
+    gs_md5_state_t md5;
+    gs_md5_byte_t zero[2] = {0, 0}, t;
     int KeySize = pdev->KeyLength / 8;
 
-    md5_init(&md5);
-    md5_append(&md5, pdev->EncryptionKey, KeySize);
-    t = (byte)(object_id >>  0);  md5_append(&md5, &t, 1);
-    t = (byte)(object_id >>  8);  md5_append(&md5, &t, 1);
-    t = (byte)(object_id >> 16);  md5_append(&md5, &t, 1);
-    md5_append(&md5, zero, 2);
-    md5_finish(&md5, key);
+    gs_md5_init(&md5);
+    gs_md5_append(&md5, pdev->EncryptionKey, KeySize);
+    t = (byte)(object_id >>  0);  gs_md5_append(&md5, &t, 1);
+    t = (byte)(object_id >>  8);  gs_md5_append(&md5, &t, 1);
+    t = (byte)(object_id >> 16);  gs_md5_append(&md5, &t, 1);
+    gs_md5_append(&md5, zero, 2);
+    gs_md5_finish(&md5, key);
     return min(KeySize + 5, 16);
 }
 
@@ -228,7 +470,7 @@ pdf_begin_encrypt(gx_device_pdf * pdev, stream **s, gs_id object_id)
 {
     gs_memory_t *mem = pdev->v_memory;
     stream_arcfour_state *ss;
-    md5_byte_t key[16];
+    gs_md5_byte_t key[16];
     int code, keylength;
 
     if (!pdev->KeyLength)
@@ -269,7 +511,7 @@ pdf_end_encrypt(gx_device_pdf * pdev)
 }
 
 /* Enter stream context. */
-private int
+static int
 none_to_stream(gx_device_pdf * pdev)
 {
     stream *s;
@@ -294,8 +536,12 @@ none_to_stream(gx_device_pdf * pdev)
 	pdev->contents_length_id = pdf_obj_ref(pdev);
 	s = pdev->strm;
 	pprintld1(s, "<</Length %ld 0 R", pdev->contents_length_id);
-	if (pdev->compression == pdf_compress_Flate)
-	    pprints1(s, "/Filter /%s", compression_filter_name);
+	if (pdev->compression == pdf_compress_Flate) {
+	    if (pdev->binary_ok)
+		pprints1(s, "/Filter /%s", compression_filter_name);
+	    else 
+		pprints1(s, "/Filter [/ASCII85Decode /%s]", compression_filter_name);
+	}
 	stream_puts(s, ">>\nstream\n");
 	pdev->contents_pos = pdf_stell(pdev);
 	code = pdf_begin_encrypt(pdev, &s, pdev->contents_id);
@@ -303,14 +549,36 @@ none_to_stream(gx_device_pdf * pdev)
 	    return code;
 	pdev->strm = s;
 	if (pdev->compression == pdf_compress_Flate) {	/* Set up the Flate filter. */
-	    const stream_template *template = &compression_filter_template;
-	    stream *es = s_alloc(pdev->pdf_memory, "PDF compression stream");
-	    byte *buf = gs_alloc_bytes(pdev->pdf_memory, sbuf_size,
-				       "PDF compression buffer");
-	    compression_filter_state *st =
-		gs_alloc_struct(pdev->pdf_memory, compression_filter_state,
-				template->stype, "PDF compression state");
+	    const stream_template *template;
+	    stream *es;
+	    byte *buf;
+	    compression_filter_state *st;
 
+	    if (!pdev->binary_ok) {	/* Set up the A85 filter */
+		const stream_template *template = &s_A85E_template;
+		stream *as = s_alloc(pdev->pdf_memory, "PDF contents stream");
+		byte *buf = gs_alloc_bytes(pdev->pdf_memory, sbuf_size,
+					   "PDF contents buffer");
+		stream_A85E_state *ast = gs_alloc_struct(pdev->pdf_memory, stream_A85E_state,
+				template->stype, "PDF contents state");
+		if (as == 0 || ast == 0 || buf == 0)
+		    return_error(gs_error_VMerror);
+		s_std_init(as, buf, sbuf_size, &s_filter_write_procs,
+			   s_mode_write);
+		ast->memory = pdev->pdf_memory;
+		ast->template = template;
+		as->state = (stream_state *) ast;
+		as->procs.process = template->process;
+		as->strm = s;
+		(*template->init) ((stream_state *) ast);
+		pdev->strm = s = as;
+	    }
+	    template = &compression_filter_template;
+	    es = s_alloc(pdev->pdf_memory, "PDF compression stream");
+	    buf = gs_alloc_bytes(pdev->pdf_memory, sbuf_size,
+				       "PDF compression buffer");
+	    st = gs_alloc_struct(pdev->pdf_memory, compression_filter_state,
+				template->stype, "PDF compression state");
 	    if (es == 0 || st == 0 || buf == 0)
 		return_error(gs_error_VMerror);
 	    s_std_init(es, buf, sbuf_size, &s_filter_write_procs,
@@ -345,7 +613,7 @@ none_to_stream(gx_device_pdf * pdev)
     return PDF_IN_STREAM;
 }
 /* Enter text context from stream context. */
-private int
+static int
 stream_to_text(gx_device_pdf * pdev)
 {
     int code;
@@ -368,7 +636,7 @@ stream_to_text(gx_device_pdf * pdev)
     return (code < 0 ? code : PDF_IN_TEXT);
 }
 /* Exit string context to text context. */
-private int
+static int
 string_to_text(gx_device_pdf * pdev)
 {
     int code = pdf_from_string_to_text(pdev);
@@ -376,7 +644,7 @@ string_to_text(gx_device_pdf * pdev)
     return (code < 0 ? code : PDF_IN_TEXT);
 }
 /* Exit text context to stream context. */
-private int
+static int
 text_to_stream(gx_device_pdf * pdev)
 {
     int code;
@@ -389,7 +657,7 @@ text_to_stream(gx_device_pdf * pdev)
     return PDF_IN_STREAM;
 }
 /* Exit stream context. */
-private int
+static int
 stream_to_none(gx_device_pdf * pdev)
 {
     stream *s = pdev->strm;
@@ -403,10 +671,17 @@ stream_to_none(gx_device_pdf * pdev)
     } else {
 	if (pdev->vgstack_depth)
 	    pdf_restore_viewer_state(pdev, s);
-	if (pdev->compression_at_page_start == pdf_compress_Flate) {	/* Terminate the Flate filter. */
+	if (pdev->compression_at_page_start == pdf_compress_Flate) {	/* Terminate the filters. */
 	    stream *fs = s->strm;
 
-	    sclose(s);
+	    if (!pdev->binary_ok) {
+		sclose(s);	/* Terminate the ASCII85 filter. */
+		gs_free_object(pdev->pdf_memory, s->cbuf, "A85E contents buffer");
+		gs_free_object(pdev->pdf_memory, s, "A85E contents stream");
+		pdev->strm = s = fs;
+		fs = s->strm;
+	    }
+	    sclose(s);		/* Next terminate the compression filter */
 	    gs_free_object(pdev->pdf_memory, s->cbuf, "zlib buffer");
 	    gs_free_object(pdev->pdf_memory, s, "zlib stream");
 	    pdev->strm = s = fs;
@@ -512,6 +787,43 @@ pdf_forget_resource(gx_device_pdf * pdev, pdf_resource_t *pres1, pdf_resource_ty
     }
 }
 
+static int 
+nocheck(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
+{
+    return 1;
+}
+
+
+/* Substitute a resource with a same one. */
+int
+pdf_substitute_resource(gx_device_pdf *pdev, pdf_resource_t **ppres, 
+	pdf_resource_type_t rtype, 
+	int (*eq)(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1),
+	bool write)
+{
+    pdf_resource_t *pres1 = *ppres;
+    int code;
+
+    code = pdf_find_same_resource(pdev, rtype, ppres, (eq ? eq : nocheck));
+    if (code < 0)
+	return code;
+    if (code != 0) {
+	code = pdf_cancel_resource(pdev, (pdf_resource_t *)pres1, rtype);
+	if (code < 0)
+	    return code;
+	pdf_forget_resource(pdev, pres1, rtype);
+	return 0;
+    } else {
+	pdf_reserve_object_id(pdev, pres1, gs_no_id);
+	if (write) {
+	    code = cos_write_object(pres1->object, pdev);
+	    if (code < 0)
+		return code;
+	    pres1->object->written = 1;
+	}
+	return 1;
+    }
+}
 
 /* Find a resource of a given type by gs_id. */
 pdf_resource_t *
@@ -564,10 +876,13 @@ pdf_find_same_resource(gx_device_pdf * pdev, pdf_resource_type_t rtype, pdf_reso
     
     for (i = 0; i < NUM_RESOURCE_CHAINS; i++) {
 	for (pres = pchain[i]; pres != 0; pres = pres->next) {
-	    if (!pres->named && *ppres != pres) {
+	    if (*ppres != pres) {
+		int code;
 		cos_object_t *pco1 = pres->object;
-		int code = pco0->cos_procs->equal(pco0, pco1, pdev);
 
+		if (cos_type(pco0) != cos_type(pco1))
+		    continue;	    /* don't compare different types */
+		code = pco0->cos_procs->equal(pco0, pco1, pdev);
 		if (code < 0)
 		    return code;
 		if (code > 0) {
@@ -686,10 +1001,12 @@ pdf_alloc_aside(gx_device_pdf * pdev, pdf_resource_t ** plist,
     } else
 	pdf_reserve_object_id(pdev, pres, id);
     pres->next = *plist;
+    pres->rid = 0;
     *plist = pres;
     pres->prev = pdev->last_resource;
     pdev->last_resource = pres;
     pres->named = false;
+    pres->global = false;
     pres->where_used = pdev->used_mask;
     *ppres = pres;
     return 0;
@@ -783,16 +1100,18 @@ pdf_end_resource(gx_device_pdf * pdev)
 int
 pdf_write_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype)
 {
-    int j;
+    int j, code = 0;
 
-    for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
+    for (j = 0; j < NUM_RESOURCE_CHAINS && code >= 0; ++j) {
 	pdf_resource_t *pres = pdev->resources[rtype].chains[j];
 
 	for (; pres != 0; pres = pres->next)
-	    if (!pres->named && !pres->object->written)
-		cos_write_object(pres->object, pdev);
+	    if ((!pres->named || pdev->ForOPDFRead) 
+		&& !pres->object->written)
+		code = cos_write_object(pres->object, pdev);
+
     }
-    return 0;
+    return code;
 }
 
 /*
@@ -860,7 +1179,8 @@ pdf_write_and_free_all_resource_objects(gx_device_pdf *pdev)
 	if (code >= 0)
 	    code = code1;
     }
-    code1 = pdf_finish_font_descriptors(pdev, pdf_release_FontDescriptor_components);
+    code1 = pdf_finish_resources(pdev, resourceFontDescriptor,
+			pdf_release_FontDescriptor_components);
     if (code >= 0)
 	code = code1;
     for (i = 0; i < NUM_RESOURCE_TYPES; ++i) {
@@ -886,14 +1206,18 @@ pdf_store_page_resources(gx_device_pdf *pdev, pdf_page_t *page)
 	stream *s = 0;
 	int j;
 
+	if (i == resourceOther)
+	    continue;
 	page->resource_ids[i] = 0;
 	for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
 	    pdf_resource_t *pres = pdev->resources[i].chains[j];
 
 	    for (; pres != 0; pres = pres->next) {
 		if (pres->where_used & pdev->used_mask) {
-		    long id = pres->object->id;
+		    long id = pdf_resource_id(pres);
 
+		    if (id == -1L)
+			continue;
 		    if (s == 0) {
 			page->resource_ids[i] = pdf_begin_separate(pdev);
 			s = pdev->strm;
@@ -1023,9 +1347,13 @@ int
 pdf_open_page(gx_device_pdf * pdev, pdf_context_t context)
 {
     if (!is_in_page(pdev)) {
+	int code;
+
 	if (pdf_page_id(pdev, pdev->next_page + 1) == 0)
 	    return_error(gs_error_VMerror);
-	pdf_open_document(pdev);
+	code = pdf_open_document(pdev);
+	if (code < 0)
+	    return code;
     }
     /* Note that context may be PDF_IN_NONE here. */
     return pdf_open_contents(pdev, context);
@@ -1046,6 +1374,12 @@ pdf_unclip(gx_device_pdf * pdev)
 	if (code < 0)
 	    return code;
     }
+    if (pdev->context > PDF_IN_STREAM) {
+	int code = pdf_open_contents(pdev, PDF_IN_STREAM);
+
+	if (code < 0)
+	    return code;
+    }
     if (pdev->vgstack_depth > pdev->vgstack_bottom) {
 	int code = pdf_restore_viewer_state(pdev, pdev->strm);
 
@@ -1091,7 +1425,7 @@ pdf_put_matrix(gx_device_pdf * pdev, const char *before,
  * null <00>, and the machinery for selecting the put_name_chars procedure
  * depending on CompatibilityLevel is no longer needed.
  */
-private int
+static int
 pdf_put_name_chars_1_2(stream *s, const byte *nstr, uint size)
 {
     uint i;
@@ -1128,20 +1462,20 @@ pdf_put_name_chars_proc(const gx_device_pdf *pdev)
 {
     return &pdf_put_name_chars_1_2;
 }
-void
+int
 pdf_put_name_chars(const gx_device_pdf *pdev, const byte *nstr, uint size)
 {
-    DISCARD(pdf_put_name_chars_proc(pdev)(pdev->strm, nstr, size));
+    return pdf_put_name_chars_proc(pdev)(pdev->strm, nstr, size);
 }
-void
+int
 pdf_put_name(const gx_device_pdf *pdev, const byte *nstr, uint size)
 {
     stream_putc(pdev->strm, '/');
-    pdf_put_name_chars(pdev, nstr, size);
+    return pdf_put_name_chars(pdev, nstr, size);
 }
 
 /* Write an encoded string with encryption. */
-private int
+static int
 pdf_encrypt_encoded_string(const gx_device_pdf *pdev, const byte *str, uint size, gs_id object_id)
 {
     stream sinp, sstr, sout;
@@ -1149,13 +1483,16 @@ pdf_encrypt_encoded_string(const gx_device_pdf *pdev, const byte *str, uint size
     stream_state so;
     byte buf[100], bufo[100];
     stream_arcfour_state sarc4;
+
     if (pdf_encrypt_init(pdev, object_id, &sarc4) < 0) {
 	/* The interface can't pass an error. */
 	stream_write(pdev->strm, str, size);
 	return size;
     }
+    s_init(&sinp, NULL);
     sread_string(&sinp, str + 1, size);
     s_init(&sstr, NULL);
+    sstr.close_at_eod = false;
     s_init_state((stream_state *)&st, &s_PSSD_template, NULL);
     s_init_filter(&sstr, (stream_state *)&st, buf, sizeof(buf), &sinp);
     s_init(&sout, NULL);
@@ -1182,15 +1519,23 @@ pdf_encrypt_encoded_string(const gx_device_pdf *pdev, const byte *str, uint size
 }
 
 /* Write an encoded string with possible encryption. */
-private void
+static int
 pdf_put_encoded_string(const gx_device_pdf *pdev, const byte *str, uint size, gs_id object_id)
 {
-    if (!pdev->KeyLength || object_id == (gs_id)-1)
+    if (!pdev->KeyLength || object_id == (gs_id)-1) {
 	stream_write(pdev->strm, str, size);
-    else
-	DISCARD(pdf_encrypt_encoded_string(pdev, str, size, object_id));
+	return 0;
+    } else
+	return pdf_encrypt_encoded_string(pdev, str, size, object_id);
+}
+/* Write an encoded hexadecimal string with possible encryption. */
+static int
+pdf_put_encoded_hex_string(const gx_device_pdf *pdev, const byte *str, uint size, gs_id object_id)
+{
+    eprintf("Unimplemented function : pdf_put_encoded_hex_string\n");
+    stream_write(pdev->strm, str, size);
+    return_error(gs_error_unregistered);
 }
-
 /*  Scan an item in a serialized array or dictionary.
     This is a very simplified Postscript lexical scanner.
     It assumes the serialization with pdf===only defined in gs/lib/gs_pdfwr.ps .
@@ -1198,7 +1543,7 @@ pdf_put_encoded_string(const gx_device_pdf *pdev, const byte *str, uint size, gs
     Other items are passed identically.
     Note we don't reconstruct the nesting of arrays|dictionaries.
 */
-private int
+static int
 pdf_scan_item(const gx_device_pdf * pdev, const byte * p, uint l, gs_id object_id)
 {
     const byte *q = p;
@@ -1217,12 +1562,11 @@ pdf_scan_item(const gx_device_pdf * pdev, const byte * p, uint l, gs_id object_i
 }
 
 /* Write a serialized array or dictionary with possible encryption. */
-private void
+static int
 pdf_put_composite(const gx_device_pdf * pdev, const byte * vstr, uint size, gs_id object_id)
 {
     if (!pdev->KeyLength || object_id == (gs_id)-1) {
 	stream_write(pdev->strm, vstr, size);
-	return;
     } else {
 	const byte *p = vstr;
 	int l = size, n;
@@ -1238,6 +1582,7 @@ pdf_put_composite(const gx_device_pdf * pdev, const byte * vstr, uint size, gs_i
 	    p += n;
 	}
     }
+    return 0;
 }
 
 /*
@@ -1245,29 +1590,32 @@ pdf_put_composite(const gx_device_pdf * pdev, const byte * vstr, uint size, gs_i
  * this form is different depending on whether binary data are allowed.
  * We wish PDF supported ASCII85 strings ( <~ ~> ), but it doesn't.
  */
-void
+int
 pdf_put_string(const gx_device_pdf * pdev, const byte * str, uint size)
 {
     psdf_write_string(pdev->strm, str, size,
 		      (pdev->binary_ok ? PRINT_BINARY_OK : 0));
+    return 0;
 }
 
 /* Write a value, treating names specially. */
-void
+int
 pdf_write_value(const gx_device_pdf * pdev, const byte * vstr, uint size, gs_id object_id)
 {
     if (size > 0 && vstr[0] == '/')
-	pdf_put_name(pdev, vstr + 1, size - 1);
+	return pdf_put_name(pdev, vstr + 1, size - 1);
     else if (size > 3 && vstr[0] == 0 && vstr[1] == 0 && vstr[size - 1] == 0)
-	pdf_put_name(pdev, vstr + 3, size - 4);
-    else if (size > 1 && vstr[0] == '(')
-	pdf_put_encoded_string(pdev, vstr, size, object_id);
+	return pdf_put_name(pdev, vstr + 3, size - 4);
     else if (size > 1 && (vstr[0] == '[' || vstr[0] == '{'))
-	pdf_put_composite(pdev, vstr, size, object_id);
+	return pdf_put_composite(pdev, vstr, size, object_id);
     else if (size > 2 && vstr[0] == '<' && vstr[1] == '<')
-	pdf_put_composite(pdev, vstr, size, object_id);
-    else
-	stream_write(pdev->strm, vstr, size);
+	return pdf_put_composite(pdev, vstr, size, object_id);
+    else if (size > 1 && vstr[0] == '(')
+	return pdf_put_encoded_string(pdev, vstr, size, object_id);
+    else if (size > 1 && vstr[0] == '<')
+	return pdf_put_encoded_hex_string(pdev, vstr, size, object_id);
+    stream_write(pdev->strm, vstr, size);
+    return 0;
 }
 
 /* Store filters for a stream. */
@@ -1314,6 +1662,14 @@ pdf_put_filters(cos_dict_t *pcd, gx_device_pdf *pdev, stream *s,
 	    filter_name = pfn->FlateDecode;
 	else if (TEMPLATE_IS(s_LZWE_template))
 	    filter_name = pfn->LZWDecode;
+#ifdef USE_LDF_JB2
+	else if (TEMPLATE_IS(s_jbig2encode_template))
+	    filter_name = pfn->JBIG2Decode;
+#endif
+#ifdef USE_LWF_JP2
+	else if (TEMPLATE_IS(s_jpxe_template))
+	    filter_name = pfn->JPXDecode;
+#endif
 	else if (TEMPLATE_IS(s_PNGPE_template)) {
 	    /* This is a predictor for FlateDecode or LZWEncode. */
 	    const stream_PNGP_state *const ss =
@@ -1371,7 +1727,7 @@ pdf_put_filters(cos_dict_t *pcd, gx_device_pdf *pdev, stream *s,
 }
 
 /* Add a Flate compression filter to a binary writer. */
-private int
+static int
 pdf_flate_binary(gx_device_pdf *pdev, psdf_binary_writer *pbw)
 {
     const stream_template *template = (pdev->CompatibilityLevel < 1.3 ? 
@@ -1495,7 +1851,7 @@ pdf_end_data(pdf_data_writer_t *pdw)
 }
 
 /* Create a Function object. */
-private int pdf_function_array(gx_device_pdf *pdev, cos_array_t *pca,
+static int pdf_function_array(gx_device_pdf *pdev, cos_array_t *pca,
 			       const gs_function_info_t *pinfo);
 int
 pdf_function_scaled(gx_device_pdf *pdev, const gs_function_t *pfn,
@@ -1535,19 +1891,22 @@ pdf_function_scaled(gx_device_pdf *pdev, const gs_function_t *pfn,
 	return code;
     }
 }
-int
-pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn,
-	     cos_value_t *pvalue)
+static int
+pdf_function_aux(gx_device_pdf *pdev, const gs_function_t *pfn,
+	     pdf_resource_t **ppres)
 {
     gs_function_info_t info;
     cos_param_list_writer_t rlist;
     pdf_resource_t *pres;
     cos_object_t *pcfn;
     cos_dict_t *pcd;
-    int code = pdf_alloc_resource(pdev, resourceFunction, gs_no_id, &pres, 0L);
+    int code = pdf_alloc_resource(pdev, resourceFunction, gs_no_id, &pres, -1);
 
-    if (code < 0)
+    if (code < 0) {
+	*ppres = 0;
 	return code;
+    }
+    *ppres = pres;
     pcfn = pres->object;
     gs_function_get_info(pfn, &info);
     if (FunctionType(pfn) == function_type_ArrayedOutput) {
@@ -1560,11 +1919,7 @@ pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn,
 
 	cos_become(pcfn, cos_type_array);
 	pca = (cos_array_t *)pcfn;
-	code = pdf_function_array(pdev, pca, &info);
-	if (code < 0)
-	    return code;
-	COS_OBJECT_VALUE(pvalue, pca);
-	return 0;
+	return pdf_function_array(pdev, pca, &info);
     }
     if (info.DataSource != 0) {
 	psdf_binary_writer writer;
@@ -1630,13 +1985,28 @@ pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn,
     code = cos_param_list_writer_init(&rlist, pcd, PRINT_BINARY_OK);
     if (code < 0)
 	return code;
-    code = gs_function_get_params(pfn, (gs_param_list *)&rlist);
+    return gs_function_get_params(pfn, (gs_param_list *)&rlist);
+}
+static int 
+functions_equal(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
+{
+    return true;
+}
+int
+pdf_function(gx_device_pdf *pdev, const gs_function_t *pfn, cos_value_t *pvalue)
+{
+    pdf_resource_t *pres;
+    int code = pdf_function_aux(pdev, pfn, &pres);
+
     if (code < 0)
 	return code;
-    COS_OBJECT_VALUE(pvalue, pcd);
+    code = pdf_substitute_resource(pdev, &pres, resourceFunction, functions_equal, false);
+    if (code < 0)
+	return code;
+    COS_OBJECT_VALUE(pvalue, pres->object);
     return 0;
 }
-private int pdf_function_array(gx_device_pdf *pdev, cos_array_t *pca,
+static int pdf_function_array(gx_device_pdf *pdev, cos_array_t *pca,
 			       const gs_function_info_t *pinfo)
 {
     int i, code = 0;
@@ -1684,3 +2054,22 @@ pdf_write_font_bbox(gx_device_pdf *pdev, const gs_int_rect *pbox)
 	     pbox->p.x, pbox->p.y, x, y);
     return 0;
 }
+
+/* Write a FontBBox dictionary element using floats for the values. */
+int
+pdf_write_font_bbox_float(gx_device_pdf *pdev, const gs_rect *pbox)
+{
+    stream *s = pdev->strm;
+    /*
+     * AR 4 doesn't like fonts with empty FontBBox, which
+     * happens when the font contains only space characters.
+     * Small bbox causes AR 4 to display a hairline. So we use
+     * the full BBox.
+     */ 
+    float x = pbox->q.x + ((pbox->p.x == pbox->q.x) ? 1000 : 0);
+    float y = pbox->q.y + ((pbox->p.y == pbox->q.y) ? 1000 : 0);
+
+    pprintg4(s, "/FontBBox[%g %g %g %g]",
+	     pbox->p.x, pbox->p.y, x, y);
+    return 0;
+}
