@@ -1,20 +1,17 @@
-/* Copyright (C) 1996, 2000, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfx.h,v 1.115 2005/04/05 09:48:46 igor Exp $ */
+/* $Id: gdevpdfx.h 8360 2007-11-13 10:01:55Z ken $ */
 /* Internal definitions for PDF-writing driver. */
 
 #ifndef gdevpdfx_INCLUDED
@@ -114,6 +111,8 @@ typedef enum {
     resourceCIDFont,
     resourceCMap,
     resourceFontDescriptor,
+    resourceGroup,
+    resourceSoftMaskDict,
     resourceFunction,
     resourcePage,
     NUM_RESOURCE_TYPES
@@ -121,7 +120,7 @@ typedef enum {
 
 #define PDF_RESOURCE_TYPE_NAMES\
   "/ColorSpace", "/ExtGState", "/Pattern", "/Shading", "/XObject", 0, "/Font",\
-  0, "/Font", "/CMap", "/FontDescriptor", 0
+  0, "/Font", "/CMap", "/FontDescriptor", "/Group", "/Mask", 0, 0
 #define PDF_RESOURCE_TYPE_STRUCTS\
   &st_pdf_color_space,		/* gdevpdfg.h / gdevpdfc.c */\
   &st_pdf_resource,		/* see below */\
@@ -135,6 +134,8 @@ typedef enum {
   &st_pdf_resource,\
   &st_pdf_font_descriptor,	/* gdevpdff.h / gdevpdff.c */\
   &st_pdf_resource,\
+  &st_pdf_resource,\
+  &st_pdf_resource,\
   &st_pdf_resource
 
 /*
@@ -147,6 +148,7 @@ typedef enum {
     pdf_resource_t *prev;	/* previously allocated resource */\
     gs_id rid;			/* optional ID key */\
     bool named;\
+    bool global;                /* ps2write only */\
     char rname[1/*R*/ + (sizeof(long) * 8 / 3 + 1) + 1/*\0*/];\
     ulong where_used;		/* 1 bit per level of content stream */\
     cos_object_t *object
@@ -189,6 +191,7 @@ typedef enum {
 
 /* Define abstract types. */
 typedef struct pdf_char_proc_s pdf_char_proc_t;	/* gdevpdff.h */
+typedef struct pdf_char_proc_ownership_s pdf_char_proc_ownership_t;	/* gdevpdff.h */
 typedef struct pdf_font_s pdf_font_t;  /* gdevpdff.h */
 typedef struct pdf_text_data_s pdf_text_data_t;  /* gdevpdft.h */
 
@@ -279,9 +282,12 @@ typedef struct pdf_page_s {
     pdf_procset_t procsets;
     long contents_id;
     long resource_ids[resourceFont + 1]; /* resources thru Font, see above */
+    long group_id;
     cos_array_t *Annots;
     pdf_text_rotation_t text_rotation;
     pdf_page_dsc_info_t dsc_info;
+    bool NumCopies_set; /* ps2write only. */
+    int NumCopies;      /* ps2write only. */
 } pdf_page_t;
 #define private_st_pdf_page()	/* in gdevpdf.c */\
   gs_private_st_ptrs2(st_pdf_page, pdf_page_t, "pdf_page_t",\
@@ -341,6 +347,7 @@ typedef struct pdf_viewer_state_s {
     gs_id undercolor_removal_id;
     int overprint_mode;
     float smoothness; /* state.smoothness */
+    float flatness;
     bool text_knockout; /* state.text_knockout */
     bool fill_overprint;
     bool stroke_overprint;
@@ -351,6 +358,7 @@ typedef struct pdf_viewer_state_s {
     gx_hl_saved_color saved_stroke_color;
     gx_line_params line_params;
     float dash_pattern[max_dash];
+    gs_id soft_mask_id;
 } pdf_viewer_state;
 
 /*
@@ -370,13 +378,17 @@ typedef struct pdf_substream_save_s {
     pdf_resource_t      *font3;
     pdf_resource_t	*accumulating_substream_resource;
     bool		charproc_just_accumulated;
+    bool		accumulating_a_global_object;
+    pdf_resource_t      *pres_soft_mask_dict;
+    gs_const_string		objname;
+    int			last_charpath_op;
 } pdf_substream_save;
 
 #define private_st_pdf_substream_save()\
-    gs_private_st_ptrs6(st_pdf_substream_save, pdf_substream_save,\
+    gs_private_st_strings1_ptrs7(st_pdf_substream_save, pdf_substream_save,\
 	"pdf_substream_save", pdf_substream_save_enum,\
-	pdf_substream_save_reloc, text_state, clip_path, strm, \
-	substream_Resources, font3, accumulating_substream_resource)
+	pdf_substream_save_reloc, objname, text_state, clip_path, strm, \
+	substream_Resources, font3, accumulating_substream_resource, pres_soft_mask_dict)
 #define private_st_pdf_substream_save_element()\
   gs_private_st_element(st_pdf_substream_save_element, pdf_substream_save,\
     "pdf_substream_save[]", pdf_substream_save_elt_enum_ptrs,\
@@ -392,6 +404,7 @@ typedef enum {
 /* Define the device structure. */
 struct gx_device_pdf_s {
     gx_device_psdf_common;
+    bool is_ps2write;          /* ps2write (true) versus pdfwrite (false); never changed */
     /* PDF-specific distiller parameters */
     double CompatibilityLevel;
     int EndPage;
@@ -407,18 +420,30 @@ struct gx_device_pdf_s {
     bool UsePrologue;
     int OffOptimizations;
     /* End of distiller parameters */
+    /* PDF/X parameters */
+    gs_param_float_array PDFXTrimBoxToMediaBoxOffset;
+    gs_param_float_array PDFXBleedBoxToTrimBoxOffset;
+    bool PDFXSetBleedBoxToMediaBox;
     /* Other parameters */
     bool ReAssignCharacters;
     bool ReEncodeCharacters;
     long FirstObjectNumber;
     bool CompressFonts;
     bool PrintStatistics;
+    gs_param_string DocumentUUID;
+    gs_param_string InstanceUUID;
+    int DocumentTimeSeq;
     bool ForOPDFRead;          /* PS2WRITE only. */
+    gs_param_string OPDFReadProcsetPath; /* PS2WRITE only. */
+    bool CompressEntireFile;  /* PS2WRITE only. */
     bool ResourcesBeforeUsage; /* PS2WRITE only. */
     bool HavePDFWidths;        /* PS2WRITE only. */
     bool HaveStrokeColor;      /* PS2WRITE only. */
+    bool HaveTransparency;
     bool PatternImagemask; /* The target viewer|printer handles imagemask 
 			      with pattern color. */
+    bool PDFX;		   /* Generate PDF/X */
+    bool PDFA;		   /* Generate PDF/A */
     long MaxClipPathSize;  /* The maximal number of elements of a clipping path
 			      that the target viewer|printer can handle. */
     long MaxViewerMemorySize;
@@ -426,20 +451,22 @@ struct gx_device_pdf_s {
 			      a bitmap representation of a shading.
 			      (Bigger shadings to be downsampled). */
     long MaxInlineImageSize;
+    gs_param_int_array DSCEncodingToUnicode;
     /* Encryption parameters */
-    gs_const_string OwnerPassword;
-    gs_const_string UserPassword;
+    gs_param_string OwnerPassword;
+    gs_param_string UserPassword;
     uint KeyLength;
     uint Permissions;
     uint EncryptionR;
+    gs_param_string NoEncrypt;
+    bool EncryptMetadata;
     /* End of parameters */
+    bool ComputeDocumentDigest; /* Developer needs only; Always true in production. */
     /* Encryption data */
     byte EncryptionO[32];
     byte EncryptionU[32];
     byte EncryptionKey[16];
     uint EncryptionV;
-    bool EncryptMetadata;
-    gs_const_string NoEncrypt;
     /* Values derived from DSC comments */
     bool is_EPS;
     pdf_page_dsc_info_t doc_dsc_info; /* document default */
@@ -510,6 +537,8 @@ struct gx_device_pdf_s {
     pdf_resource_t *cs_Patterns[5];
     pdf_resource_t *Identity_ToUnicode_CMaps[2]; /* WMode = 0,1 */
     pdf_resource_t *last_resource;
+    pdf_resource_t *OneByteIdentityH;
+    gs_id IdentityCIDSystemInfo_id;
     pdf_outline_level_t outline_levels[MAX_OUTLINE_DEPTH];
     int outline_depth;
     int closed_outline_depth;
@@ -517,6 +546,8 @@ struct gx_device_pdf_s {
     pdf_article_t *articles;
     cos_dict_t *Dests;
     byte fileID[16];
+    /* Use a single time moment for all UUIDs to minimize an indeterminizm. */
+    long uuid_time[2];
     /*
      * global_named_objects holds named objects that are independent of
      * the current namespace: {Catalog}, {DocInfo}, {Page#}, {ThisPage},
@@ -593,6 +624,11 @@ struct gx_device_pdf_s {
     bool charproc_just_accumulated; /* A flag for controlling 
 			the glyph variation recognition. 
 			Used only with uncached charprocs. */
+    bool accumulating_a_global_object; /* ps2write only.
+			Accumulating a global object (such as a named Form,
+			so that resources used in it must also be global. 
+			Important for viewers with small memory,
+			which drops resources per page. */
     const pdf_char_glyph_pairs_t *cgp; /* A temporary pointer 
 			for pdf_is_same_charproc1.
 			Must be NULL when the garbager is invoked, 
@@ -604,8 +640,18 @@ struct gx_device_pdf_s {
        (which is being converted into a pattern)
        between 2 consecutive calls to pdf_image_end_image_data. */
     gs_id     image_mask_id;
+    bool      image_mask_is_SMask;
+    bool      image_mask_skip; /* A flag for pdf_begin_transparency_mask */
+    bool      image_with_SMask; /* A flag for pdf_begin_transparency_group. */
     gs_matrix converting_image_matrix;
     double    image_mask_scale;
+    /* Temporary data for soft mask form. */
+    pdf_resource_t *pres_soft_mask_dict;
+    /* Temporary data for pdfmark_BP. */
+    gs_const_string objname;
+    int OPDFRead_procset_length;      /* PS2WRITE only. */
+    void *find_resource_param; /* WARNING : not visible for garbager. */
+    int last_charpath_op; /* true or false state of last charpath */
 };
 
 #define is_in_page(pdev)\
@@ -628,13 +674,20 @@ struct gx_device_pdf_s {
  m(24,font_cache) m(25,clip_path)\
  m(26,PageLabels) m(27,PageLabels_current_label)\
  m(28,sbstack) m(29,substream_Resources) m(30,font3)\
- m(31,accumulating_substream_resource)
-#define gx_device_pdf_num_ptrs 32
-#define gx_device_pdf_do_strings(m) /* do nothing */
-#define gx_device_pdf_num_strings 0
+ m(31,accumulating_substream_resource) \
+ m(32,pres_soft_mask_dict) m(33,PDFXTrimBoxToMediaBoxOffset.data)\
+ m(34,PDFXBleedBoxToTrimBoxOffset.data) m(35, DSCEncodingToUnicode.data)
+#define gx_device_pdf_num_ptrs 36
+#define gx_device_pdf_do_param_strings(m)\
+    m(0, OPDFReadProcsetPath) m(1, OwnerPassword) m(2, UserPassword) m(3, NoEncrypt)\
+    m(4, DocumentUUID) m(5, InstanceUUID)
+#define gx_device_pdf_num_param_strings 6
+#define gx_device_pdf_do_const_strings(m)\
+    m(0, objname)
+#define gx_device_pdf_num_const_strings 1
 #define st_device_pdf_max_ptrs\
   (st_device_psdf_max_ptrs + gx_device_pdf_num_ptrs +\
-   gx_device_pdf_num_strings +\
+   gx_device_pdf_num_param_strings + gx_device_pdf_num_const_strings +\
    NUM_RESOURCE_TYPES * NUM_RESOURCE_CHAINS /* resources[].chains[] */ +\
    MAX_OUTLINE_DEPTH * 2 /* outline_levels[].{first,last}.action */
 
@@ -666,6 +719,13 @@ dev_proc_pattern_manage(gdev_pdf_pattern_manage);
 dev_proc_fill_rectangle_hl_color(gdev_pdf_fill_rectangle_hl_color);
     /* In gdevpdfv.c */
 dev_proc_include_color_space(gdev_pdf_include_color_space);
+    /* In gdevpdft.c */
+dev_proc_create_compositor(gdev_pdf_create_compositor);
+dev_proc_begin_transparency_group(gdev_pdf_begin_transparency_group);
+dev_proc_end_transparency_group(gdev_pdf_end_transparency_group);
+dev_proc_begin_transparency_mask(gdev_pdf_begin_transparency_mask);
+dev_proc_end_transparency_mask(gdev_pdf_end_transparency_mask);
+dev_proc_discard_transparency_layer(gdev_pdf_discard_transparency_layer);
 
 /* ================ Utility procedures ================ */
 
@@ -685,7 +745,7 @@ void pdf_reset_text(gx_device_pdf *pdev);
 /* ------ Document ------ */
 
 /* Open the document if necessary. */
-void pdf_open_document(gx_device_pdf * pdev);
+int pdf_open_document(gx_device_pdf * pdev);
 
 /* ------ Objects ------ */
 
@@ -782,6 +842,12 @@ int pdf_cancel_resource(gx_device_pdf * pdev, pdf_resource_t *pres,
 void pdf_forget_resource(gx_device_pdf * pdev, pdf_resource_t *pres1, 
 	pdf_resource_type_t rtype);
 
+/* Substitute a resource with a same one. */
+int pdf_substitute_resource(gx_device_pdf *pdev, pdf_resource_t **ppres, 
+	    pdf_resource_type_t rtype, 
+	    int (*eq)(gx_device_pdf *pdev, pdf_resource_t *pres0, pdf_resource_t *pres1),
+	    bool write);
+
 /* Get the object id of a resource. */
 long pdf_resource_id(const pdf_resource_t *pres);
 
@@ -867,16 +933,18 @@ int pdf_put_clip_path(gx_device_pdf * pdev, const gx_clip_path * pcpath);
 
 /* ------ Masked image convertion ------ */
 
-typedef struct {
+typedef struct pdf_lcvd_s {
     gx_device_memory mdev;
     gx_device_memory *mask;
     gx_device_pdf *pdev;
     dev_t_proc_fill_rectangle((*std_fill_rectangle), gx_device);
     dev_t_proc_close_device((*std_close_device), gx_device);
+    dev_t_proc_get_clipping_box((*std_get_clipping_box), gx_device);
     bool mask_is_empty;
     bool path_is_empty;
     bool mask_is_clean;
     bool write_matrix;
+    bool has_background;
     gs_matrix m;
     gs_point path_offset;
 } pdf_lcvd_t;
@@ -911,13 +979,15 @@ typedef struct pdf_filter_names_s {
     const char *FlateDecode;
     const char *LZWDecode;
     const char *RunLengthDecode;
+    const char *JBIG2Decode;
+    const char *JPXDecode;
 } pdf_filter_names_t;
 #define PDF_FILTER_NAMES\
   "/ASCII85Decode", "/ASCIIHexDecode", "/CCITTFaxDecode",\
   "/DCTDecode",  "/DecodeParms", "/Filter", "/FlateDecode",\
-  "/LZWDecode", "/RunLengthDecode"
+  "/LZWDecode", "/RunLengthDecode,", "/JBIG2Decode", "/JPXDecode"
 #define PDF_FILTER_NAMES_SHORT\
-  "/A85", "/AHx", "/CCF", "/DCT", "/DP", "/F", "/Fl", "/LZW", "/RL"
+  "/A85", "/AHx", "/CCF", "/DCT", "/DP", "/F", "/Fl", "/LZW", "/RL", "/???", "/???"
 
 /* Write matrix values. */
 void pdf_put_matrix(gx_device_pdf *pdev, const char *before,
@@ -927,15 +997,15 @@ void pdf_put_matrix(gx_device_pdf *pdev, const char *before,
 typedef int (*pdf_put_name_chars_proc_t)(stream *, const byte *, uint);
 pdf_put_name_chars_proc_t
     pdf_put_name_chars_proc(const gx_device_pdf *pdev);
-void pdf_put_name_chars(const gx_device_pdf *pdev, const byte *nstr,
+int pdf_put_name_chars(const gx_device_pdf *pdev, const byte *nstr,
 			uint size);
-void pdf_put_name(const gx_device_pdf *pdev, const byte *nstr, uint size);
+int pdf_put_name(const gx_device_pdf *pdev, const byte *nstr, uint size);
 
 /* Write a string in its shortest form ( () or <> ). */
-void pdf_put_string(const gx_device_pdf *pdev, const byte *str, uint size);
+int pdf_put_string(const gx_device_pdf *pdev, const byte *str, uint size);
 
 /* Write a value, treating names specially. */
-void pdf_write_value(const gx_device_pdf *pdev, const byte *vstr, uint size, gs_id object_id);
+int pdf_write_value(const gx_device_pdf *pdev, const byte *vstr, uint size, gs_id object_id);
 
 /* Store filters for a stream. */
 int pdf_put_filters(cos_dict_t *pcd, gx_device_pdf *pdev, stream *s,
@@ -999,6 +1069,7 @@ int pdf_write_function(gx_device_pdf *pdev, const gs_function_t *pfn,
 
 /* Write a FontBBox dictionary element. */
 int pdf_write_font_bbox(gx_device_pdf *pdev, const gs_int_rect *pbox);
+int pdf_write_font_bbox_float(gx_device_pdf *pdev, const gs_rect *pbox);
 
 /* ---------------- Exported by gdevpdfm.c ---------------- */
 
@@ -1131,6 +1202,7 @@ void pdf_text_state_copy(pdf_text_state_t *pts_to, pdf_text_state_t *pts_from);
 void pdf_reset_text_page(pdf_text_data_t *ptd);
 void pdf_reset_text_state(pdf_text_data_t *ptd);
 void pdf_close_text_page(gx_device_pdf *pdev);
+int  pdf_get_stoted_text_size(pdf_text_state_t *state);
 
 /* For gdevpdfb.c */
 
@@ -1152,11 +1224,10 @@ int pdf_do_char_image(gx_device_pdf * pdev, const pdf_char_proc_t * pcp,
 int pdf_start_charproc_accum(gx_device_pdf *pdev);
 /* Install charproc accumulator for a Type 3 font. */
 int pdf_set_charproc_attrs(gx_device_pdf *pdev, gs_font *font, const double *pw, int narg,
-		gs_text_cache_control_t control, gs_char ch, gs_const_string *gnstr);
+		gs_text_cache_control_t control, gs_char ch);
 /* Complete charproc accumulation for aType 3 font. */
-int pdf_end_charproc_accum(gx_device_pdf *pdev, gs_font *font, 
-		const pdf_char_glyph_pairs_t *cgp);
-
+int pdf_end_charproc_accum(gx_device_pdf *pdev, gs_font *font, const pdf_char_glyph_pairs_t *cgp, 
+		       gs_glyph glyph, gs_char output_char_code, const gs_const_string *gnstr);
 /* Open a stream object in the temporary file. */
 int pdf_open_aside(gx_device_pdf *pdev, pdf_resource_type_t rtype, 
 	gs_id id, pdf_resource_t **ppres, bool reserve_object_id, int options);
@@ -1173,7 +1244,7 @@ int pdf_exit_substream(gx_device_pdf *pdev);
 /* Add procsets to substream Resources. */
 int pdf_add_procsets(cos_dict_t *pcd, pdf_procset_t procsets);
 /* Add a resource to substream Resources. */
-int pdf_add_resource(gx_device_pdf *pdev, cos_dict_t *pcd, const char *key, const pdf_resource_t *pres);
+int pdf_add_resource(gx_device_pdf *pdev, cos_dict_t *pcd, const char *key, pdf_resource_t *pres);
 
 
 /* For gdevpdfu.c */
