@@ -1,33 +1,24 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gdevpsdu.c,v $ $Revision: 1.7.2.4 $ */
+/* $Id: gdevpsdu.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Common utilities for PostScript and PDF writers */
 #include "stdio_.h"		/* for FILE for jpeglib.h */
 #include "jpeglib_.h"		/* for sdct.h */
 #include "memory_.h"
-#include <stdlib.h>		/* for qsort */
 #include "gx.h"
 #include "gserrors.h"
 #include "gdevpsdf.h"
-#include "gxfont.h"
-#include "scanchar.h"
 #include "strimpl.h"
 #include "sa85x.h"
 #include "scfx.h"
@@ -35,6 +26,7 @@
 #include "sjpeg.h"
 #include "spprint.h"
 #include "sstring.h"
+#include "gsovrc.h"
 
 /* Structure descriptors */
 public_st_device_psdf();
@@ -111,18 +103,6 @@ psdf_setlogop(gx_device_vector * vdev, gs_logical_operation_t lop,
     return 0;
 }
 
-int
-psdf_setfillcolor(gx_device_vector * vdev, const gx_drawing_color * pdc)
-{
-    return psdf_set_color(vdev, pdc, &psdf_set_fill_color_commands);
-}
-
-int
-psdf_setstrokecolor(gx_device_vector * vdev, const gx_drawing_color * pdc)
-{
-    return psdf_set_color(vdev, pdc, &psdf_set_stroke_color_commands);
-}
-
 int
 psdf_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1, fixed y1,
 	    gx_path_type_t type)
@@ -164,7 +144,9 @@ psdf_curveto(gx_device_vector * vdev, floatp x0, floatp y0,
 	   floatp x1, floatp y1, floatp x2, floatp y2, floatp x3, floatp y3,
 	     gx_path_type_t type)
 {
-    if (x1 == x0 && y1 == y0)
+    if (x1 == x0 && y1 == y0 && x2 == x3 && y2 == y3)
+	pprintg2(gdev_vector_stream(vdev), "%g %g l\n", x3, y3);
+    else if (x1 == x0 && y1 == y0)
 	pprintg4(gdev_vector_stream(vdev), "%g %g %g %g v\n",
 		 x2, y2, x3, y3);
     else if (x3 == x2 && y3 == y2)
@@ -201,14 +183,30 @@ psdf_adjust_color_index(gx_device_vector *vdev, gx_color_index color)
     return (color == (gx_no_color_index ^ 1) ? gx_no_color_index : color);
 }
 
+/* Round a double value to a specified precision. */
+double 
+psdf_round(double v, int precision, int radix)
+{
+    double mul = 1;
+    double w = v;
+
+    if (w <= 0)
+	return w;
+    while (w < precision) {
+	w *= radix;
+	mul *= radix;
+    }
+    return (int)(w + 0.5) / mul;
+}
+
 /*
  * Since we only have 8 bits of color to start with, round the
  * values to 3 digits for more compact output.
  */
-private double
-round_byte_color(int cv)
+static inline double
+round_byte_color(gx_color_index cv)
 {
-    return (int)(cv * (1000.0 / 255.0) + 0.5) / 1000.0;
+    return (int)((uint)cv * (1000.0 / 255.0) + 0.5) / 1000.0;
 }
 int
 psdf_set_color(gx_device_vector * vdev, const gx_drawing_color * pdc,
@@ -232,7 +230,7 @@ psdf_set_color(gx_device_vector * vdev, const gx_drawing_color * pdc,
 	switch (vdev->color_info.num_components) {
 	case 4:
 	    /* if (v0 == 0 && v1 == 0 && v2 == 0 && ...) */
-	    if ((color & (0xffffff << 8)) == 0 && ppscc->setgray != 0) {
+	    if ((color & 0xffffff00) == 0 && ppscc->setgray != 0) {
 		v3 = 1.0 - v3;
 		goto g;
 	    }
@@ -273,6 +271,7 @@ psdf_begin_binary(gx_device_psdf * pdev, psdf_binary_writer * pbw)
 
     pbw->target = pdev->strm;
     pbw->dev = pdev;
+    pbw->strm = 0;		/* for GC in case of failure */
     /* If not binary, set up the encoding stream. */
     if (!pdev->binary_ok) {
 #define BUF_SIZE 100		/* arbitrary */
@@ -291,9 +290,8 @@ psdf_begin_binary(gx_device_psdf * pdev, psdf_binary_writer * pbw)
 	ss->template = &s_A85E_template;
 	s_init_filter(s, (stream_state *)ss, buf, BUF_SIZE, pdev->strm);
 #undef BUF_SIZE
-	pbw->strm = pbw->A85E = s;
+	pbw->strm = s;
     } else {
-	pbw->A85E = 0;		/* for GC */
 	pbw->strm = pdev->strm;
     }
     return 0;
@@ -347,14 +345,16 @@ psdf_DCT_filter(gs_param_list *plist /* may be NULL */,
 	/* Allocate space for IJG parameters. */
 	jcdp = gs_alloc_struct_immovable(mem, jpeg_compress_data,
            &st_jpeg_compress_data, "zDCTE");
-	if (jcdp == 0)
+        if (jcdp == 0)
 	    return_error(gs_error_VMerror);
 	ss->data.compress = jcdp;
 	jcdp->memory = ss->jpeg_memory = mem;	/* set now for allocation */
 	if ((code = gs_jpeg_create_compress(ss)) < 0)
 	    goto dcte_fail;	/* correct to do jpeg_destroy here */
 	/* Read parameters from dictionary */
-	s_DCTE_put_params((gs_param_list *)&rcc_list, ss); /* ignore errors */
+	code = s_DCTE_put_params((gs_param_list *)&rcc_list, ss);
+	if (code < 0)
+	    return code;
 	/* Create the filter. */
 	jcdp->template = s_DCTE_template;
 	/* Make sure we get at least a full scan line of input. */
@@ -409,9 +409,54 @@ psdf_CFE_binary(psdf_binary_writer * pbw, int w, int h, bool invert)
 int
 psdf_end_binary(psdf_binary_writer * pbw)
 {
-    int code = s_close_filters(&pbw->strm, pbw->target);
+    int status = s_close_filters(&pbw->strm, pbw->target);
 
-    /* s_close_filters freed the A85E stream, if any. */
-    pbw->A85E = 0;		/* for GC */
-    return code;
+    return (status >= 0 ? 0 : gs_note_error(gs_error_ioerror));
+}
+
+/* ---------------- Overprint, Get Bits ---------------- */
+
+/*
+ * High level devices cannot perform get_bits or get_bits_rectangle
+ * operations, for obvious reasons.
+ */
+int
+psdf_get_bits(gx_device * dev, int y, byte * data, byte ** actual_data)
+{
+    if (dev_proc(dev, get_alpha_bits)(dev, go_graphics) > 1)
+	eprintf1("Can't set GraphicsAlphaBits > 1 with a vector device %s.\n",
+	    dev->dname);
+    return_error(gs_error_unregistered);
+}
+
+int
+psdf_get_bits_rectangle(
+    gx_device *             dev,
+    const gs_int_rect *     prect,
+    gs_get_bits_params_t *  params,
+    gs_int_rect **          unread )
+{
+    return_error(gs_error_unregistered);
+}
+
+/*
+ * Create compositor procedure for PostScript/PDF writer. Since these
+ * devices directly support overprint (and have access to the imager
+ * state), no compositor is required for overprint support. Hence, this
+ * routine just recognizes and discards invocations of the overprint
+ * compositor.
+ */
+int
+psdf_create_compositor(
+    gx_device *             dev,
+    gx_device **            pcdev,
+    const gs_composite_t *  pct,
+    gs_imager_state * pis,
+    gs_memory_t *           mem )
+{
+    if (gs_is_overprint_compositor(pct)) {
+        *pcdev = dev;
+        return 0;
+    } else
+        return gx_default_create_compositor(dev, pcdev, pct, pis, mem);
 }
