@@ -1,20 +1,17 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpx.c,v 1.15 2004/12/07 23:25:04 stefan Exp $ */
+/* $Id: gdevpx.c 8551 2008-02-27 00:47:37Z marcos $ */
 /* H-P PCL XL driver */
 #include "math_.h"
 #include "memory_.h"
@@ -55,6 +52,13 @@ typedef struct gx_device_pclxl_s {
     gx_device_vector_common;
     /* Additional state information */
     pxeMediaSize_t media_size;
+    bool ManualFeed;            /* map ps setpage commands to pxl */
+    bool ManualFeed_set;         
+    int MediaPosition;		/* MediaPosition attribute */
+    int MediaPosition_set;
+    int page;			/* Page number starting at 0 */
+    bool Duplex;		/* Duplex attribute */
+    bool Tumble;		/* Tumble attribute */
     gx_path_type_t fill_rule;	/* ...winding_number or ...even_odd  */
     gx_path_type_t clip_rule;	/* ditto */
     pxeColorSpace_t color_space;
@@ -101,16 +105,19 @@ gs_public_st_suffix_add0_final(st_device_pclxl, gx_device_pclxl,
 			   (depth > 1 ? 256 : 2), (depth > 8 ? 256 : 1))
 
 /* Driver procedures */
-private dev_proc_open_device(pclxl_open_device);
-private dev_proc_output_page(pclxl_output_page);
-private dev_proc_close_device(pclxl_close_device);
-private dev_proc_copy_mono(pclxl_copy_mono);
-private dev_proc_copy_color(pclxl_copy_color);
-private dev_proc_fill_mask(pclxl_fill_mask);
+static dev_proc_open_device(pclxl_open_device);
+static dev_proc_output_page(pclxl_output_page);
+static dev_proc_close_device(pclxl_close_device);
+static dev_proc_copy_mono(pclxl_copy_mono);
+static dev_proc_copy_color(pclxl_copy_color);
+static dev_proc_fill_mask(pclxl_fill_mask);
 
-/*private dev_proc_draw_thin_line(pclxl_draw_thin_line); */
-private dev_proc_begin_image(pclxl_begin_image);
-private dev_proc_strip_copy_rop(pclxl_strip_copy_rop);
+static dev_proc_get_params(pclxl_get_params);
+static dev_proc_put_params(pclxl_put_params);
+
+/*static dev_proc_draw_thin_line(pclxl_draw_thin_line); */
+static dev_proc_begin_image(pclxl_begin_image);
+static dev_proc_strip_copy_rop(pclxl_strip_copy_rop);
 
 #define pclxl_device_procs(map_rgb_color, map_color_rgb)\
 {\
@@ -127,8 +134,8 @@ private dev_proc_strip_copy_rop(pclxl_strip_copy_rop);
 	pclxl_copy_color,\
 	NULL,			/* draw_line */\
 	NULL,			/* get_bits */\
-	gdev_vector_get_params,\
-	gdev_vector_put_params,\
+	pclxl_get_params,\
+	pclxl_put_params,\
 	NULL,			/* map_cmyk_color */\
 	NULL,			/* get_xfont_procs */\
 	NULL,			/* get_xfont_device */\
@@ -164,14 +171,14 @@ const gx_device_pclxl gs_pxlcolor_device = {
 
 /* ---------------- Other utilities ---------------- */
 
-inline private stream *
+static inline stream *
 pclxl_stream(gx_device_pclxl *xdev)
 {
     return gdev_vector_stream((gx_device_vector *)xdev);
 }
 
 /* Initialize for a page. */
-private void
+static void
 pclxl_page_init(gx_device_pclxl * xdev)
 {
     gdev_vector_init((gx_device_vector *)xdev);
@@ -187,7 +194,7 @@ pclxl_page_init(gx_device_pclxl * xdev)
 #define RGB_IS_GRAY(ci) ((ci) >> 8 == ((ci) & 0xffff))
 
 /* Set the color space and (optionally) palette. */
-private void
+static void
 pclxl_set_color_space(gx_device_pclxl * xdev, pxeColorSpace_t color_space)
 {
     if (xdev->color_space != color_space) {
@@ -198,7 +205,7 @@ pclxl_set_color_space(gx_device_pclxl * xdev, pxeColorSpace_t color_space)
 	xdev->color_space = color_space;
     }
 }
-private void
+static void
 pclxl_set_color_palette(gx_device_pclxl * xdev, pxeColorSpace_t color_space,
 			const byte * palette, uint palette_size)
 {
@@ -225,7 +232,7 @@ pclxl_set_color_palette(gx_device_pclxl * xdev, pxeColorSpace_t color_space,
 }
 
 /* Set a drawing RGB color. */
-private int
+static int
 pclxl_set_color(gx_device_pclxl * xdev, const gx_drawing_color * pdc,
 		px_attribute_t null_source, px_tag_t op)
 {
@@ -256,7 +263,7 @@ pclxl_set_color(gx_device_pclxl * xdev, const gx_drawing_color * pdc,
 
 /* Test whether we can handle a given color space in an image. */
 /* We cannot handle ICCBased color spaces. */
-private bool
+static bool
 pclxl_can_handle_color_space(const gs_color_space * pcs)
 {
     gs_color_space_index index = gs_color_space_get_index(pcs);
@@ -273,7 +280,7 @@ pclxl_can_handle_color_space(const gs_color_space * pcs)
 }
 
 /* Set brush, pen, and mode for painting a path. */
-private void
+static void
 pclxl_set_paints(gx_device_pclxl * xdev, gx_path_type_t type)
 {
     stream *s = pclxl_stream(xdev);
@@ -312,7 +319,7 @@ pclxl_set_paints(gx_device_pclxl * xdev, gx_path_type_t type)
 }
 
 /* Set the cursor. */
-private int
+static int
 pclxl_set_cursor(gx_device_pclxl * xdev, int x, int y)
 {
     stream *s = pclxl_stream(xdev);
@@ -325,13 +332,13 @@ pclxl_set_cursor(gx_device_pclxl * xdev, int x, int y)
 /* ------ Paths ------ */
 
 /* Flush any buffered path points. */
-private void
+static void
 px_put_np(stream * s, int count, pxeDataType_t dtype)
 {
     px_put_uba(s, (byte)count, pxaNumberOfPoints);
     px_put_uba(s, (byte)dtype, pxaPointType);
 }
-private int
+static int
 pclxl_flush_points(gx_device_pclxl * xdev)
 {
     int count = xdev->points.count;
@@ -443,14 +450,14 @@ pclxl_flush_points(gx_device_pclxl * xdev)
 
 /* ------ Images ------ */
 
-private image_enum_proc_plane_data(pclxl_image_plane_data);
-private image_enum_proc_end_image(pclxl_image_end_image);
-private const gx_image_enum_procs_t pclxl_image_enum_procs = {
+static image_enum_proc_plane_data(pclxl_image_plane_data);
+static image_enum_proc_end_image(pclxl_image_end_image);
+static const gx_image_enum_procs_t pclxl_image_enum_procs = {
     pclxl_image_plane_data, pclxl_image_end_image
 };
 
 /* Begin an image. */
-private void
+static void
 pclxl_write_begin_image(gx_device_pclxl * xdev, uint width, uint height,
 			uint dest_width, uint dest_height)
 {
@@ -464,7 +471,7 @@ pclxl_write_begin_image(gx_device_pclxl * xdev, uint width, uint height,
 
 /* Write rows of an image. */
 /****** IGNORES data_bit ******/
-private void
+static void
 pclxl_write_image_data(gx_device_pclxl * xdev, const byte * data, int data_bit,
 		       uint raster, uint width_bits, int y, int height)
 {
@@ -548,7 +555,7 @@ pclxl_write_image_data(gx_device_pclxl * xdev, const byte * data, int data_bit,
 }
 
 /* End an image. */
-private void
+static void
 pclxl_write_end_image(gx_device_pclxl * xdev)
 {
     spputc(xdev->strm, pxtEndImage);
@@ -557,7 +564,7 @@ pclxl_write_end_image(gx_device_pclxl * xdev)
 /* ------ Fonts ------ */
 
 /* Write a string (single- or double-byte). */
-private void
+static void
 px_put_string(stream * s, const byte * data, uint len, bool wide)
 {
     if (wide) {
@@ -572,7 +579,7 @@ px_put_string(stream * s, const byte * data, uint len, bool wide)
 }
 
 /* Write a 16-bit big-endian value. */
-private void
+static void
 px_put_us_be(stream * s, uint i)
 {
     spputc(s, (byte) (i >> 8));
@@ -581,7 +588,7 @@ px_put_us_be(stream * s, uint i)
 
 /* Define a bitmap font.  The client must call px_put_string */
 /* with the font name immediately before calling this procedure. */
-private void
+static void
 pclxl_define_bitmap_font(gx_device_pclxl * xdev)
 {
     stream *s = pclxl_stream(xdev);
@@ -608,7 +615,7 @@ pclxl_define_bitmap_font(gx_device_pclxl * xdev)
 
 /* Set the font.  The client must call px_put_string */
 /* with the font name immediately before calling this procedure. */
-private void
+static void
 pclxl_set_font(gx_device_pclxl * xdev)
 {
     stream *s = pclxl_stream(xdev);
@@ -622,7 +629,7 @@ pclxl_set_font(gx_device_pclxl * xdev)
 
 /* Define a character in a bitmap font.  The client must call px_put_string */
 /* with the font name immediately before calling this procedure. */
-private void
+static void
 pclxl_define_bitmap_char(gx_device_pclxl * xdev, uint ccode,
 	       const byte * data, uint raster, uint width_bits, uint height)
 {
@@ -650,7 +657,7 @@ pclxl_define_bitmap_char(gx_device_pclxl * xdev, uint ccode,
 }
 
 /* Write the name of the only font we define. */
-private void
+static void
 pclxl_write_font_name(gx_device_pclxl * xdev)
 {
     stream *s = pclxl_stream(xdev);
@@ -660,7 +667,7 @@ pclxl_write_font_name(gx_device_pclxl * xdev)
 
 /* Look up a bitmap id, return the index in the character table. */
 /* If the id is missing, return an index for inserting. */
-private int
+static int
 pclxl_char_index(gx_device_pclxl * xdev, gs_id id)
 {
     int i, i_empty = -1;
@@ -683,7 +690,7 @@ pclxl_char_index(gx_device_pclxl * xdev, gs_id id)
 }
 
 /* Remove the character table entry at a given index. */
-private void
+static void
 pclxl_remove_char(gx_device_pclxl * xdev, int index)
 {
     uint ccode = xdev->chars.table[index];
@@ -708,7 +715,7 @@ pclxl_remove_char(gx_device_pclxl * xdev, int index)
 /* Write a bitmap as a text character if possible. */
 /* The caller must set the color, cursor, and RasterOp. */
 /* We know id != gs_no_id. */
-private int
+static int
 pclxl_copy_text_char(gx_device_pclxl * xdev, const byte * data,
 		     int raster, gx_bitmap_id id, int w, int h)
 {
@@ -765,7 +772,7 @@ pclxl_copy_text_char(gx_device_pclxl * xdev, const byte * data,
 
 /* ---------------- Vector implementation procedures ---------------- */
 
-private int
+static int
 pclxl_beginpage(gx_device_vector * vdev)
 {
     gx_device_pclxl *const xdev = (gx_device_pclxl *)vdev;
@@ -774,14 +781,30 @@ pclxl_beginpage(gx_device_vector * vdev)
      * from there before in_page is set.
      */
     stream *s = vdev->strm;
+    byte media_source = eAutoSelect; /* default */
+
+    xdev->page ++;
+
+    errprintf("PAGE: %d %d\n", xdev->page, xdev->NumCopies);
+    errprintf("INFO: Printing page %d...\n", xdev->page);
+    errflush();
 
     px_write_page_header(s, (const gx_device *)vdev);
-    px_write_select_media(s, (const gx_device *)vdev, &xdev->media_size);
+
+    if (xdev->ManualFeed_set && xdev->ManualFeed) 
+	media_source = 2;
+    else if (xdev->MediaPosition_set && xdev->MediaPosition >= 0 )
+	media_source = xdev->MediaPosition;
+ 
+    px_write_select_media(s, (const gx_device *)vdev, &xdev->media_size,
+			  &media_source,
+                          xdev->page, xdev->Duplex, xdev->Tumble);
+
     spputc(s, pxtBeginPage);
     return 0;
 }
 
-private int
+static int
 pclxl_setlinewidth(gx_device_vector * vdev, floatp width)
 {
     stream *s = gdev_vector_stream(vdev);
@@ -791,7 +814,7 @@ pclxl_setlinewidth(gx_device_vector * vdev, floatp width)
     return 0;
 }
 
-private int
+static int
 pclxl_setlinecap(gx_device_vector * vdev, gs_line_cap cap)
 {
     stream *s = gdev_vector_stream(vdev);
@@ -802,7 +825,7 @@ pclxl_setlinecap(gx_device_vector * vdev, gs_line_cap cap)
     return 0;
 }
 
-private int
+static int
 pclxl_setlinejoin(gx_device_vector * vdev, gs_line_join join)
 {
     stream *s = gdev_vector_stream(vdev);
@@ -813,7 +836,7 @@ pclxl_setlinejoin(gx_device_vector * vdev, gs_line_join join)
     return 0;
 }
 
-private int
+static int
 pclxl_setmiterlimit(gx_device_vector * vdev, floatp limit)
 {
     stream *s = gdev_vector_stream(vdev);
@@ -828,7 +851,7 @@ pclxl_setmiterlimit(gx_device_vector * vdev, floatp limit)
     return 0;
 }
 
-private int
+static int
 pclxl_setdash(gx_device_vector * vdev, const float *pattern, uint count,
 	      floatp offset)
 {
@@ -861,7 +884,7 @@ pclxl_setdash(gx_device_vector * vdev, const float *pattern, uint count,
     return 0;
 }
 
-private int
+static int
 pclxl_setlogop(gx_device_vector * vdev, gs_logical_operation_t lop,
 	       gs_logical_operation_t diff)
 {
@@ -882,14 +905,14 @@ pclxl_setlogop(gx_device_vector * vdev, gs_logical_operation_t lop,
     return 0;
 }
 
-private int
+static int
 pclxl_can_handle_hl_color(gx_device_vector * vdev, const gs_imager_state * pis, 
                    const gx_drawing_color * pdc)
 {
     return false;
 }
 
-private int
+static int
 pclxl_setfillcolor(gx_device_vector * vdev, const gs_imager_state * pis, 
                    const gx_drawing_color * pdc)
 {
@@ -898,7 +921,7 @@ pclxl_setfillcolor(gx_device_vector * vdev, const gs_imager_state * pis,
     return pclxl_set_color(xdev, pdc, pxaNullBrush, pxtSetBrushSource);
 }
 
-private int
+static int
 pclxl_setstrokecolor(gx_device_vector * vdev, const gs_imager_state * pis, 
                      const gx_drawing_color * pdc)
 {
@@ -907,7 +930,7 @@ pclxl_setstrokecolor(gx_device_vector * vdev, const gs_imager_state * pis,
     return pclxl_set_color(xdev, pdc, pxaNullPen, pxtSetPenSource);
 }
 
-private int
+static int
 pclxl_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1,
 	     fixed y1, gx_path_type_t type)
 {
@@ -939,7 +962,7 @@ pclxl_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1,
     return 0;
 }
 
-private int
+static int
 pclxl_beginpath(gx_device_vector * vdev, gx_path_type_t type)
 {
     gx_device_pclxl *const xdev = (gx_device_pclxl *)vdev;
@@ -951,7 +974,7 @@ pclxl_beginpath(gx_device_vector * vdev, gx_path_type_t type)
     return 0;
 }
 
-private int
+static int
 pclxl_moveto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
 	     gx_path_type_t type)
 {
@@ -965,7 +988,7 @@ pclxl_moveto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
 			    xdev->points.current.y = (int)y);
 }
 
-private int
+static int
 pclxl_lineto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
 	     gx_path_type_t type)
 {
@@ -991,7 +1014,7 @@ pclxl_lineto(gx_device_vector * vdev, floatp x0, floatp y0, floatp x, floatp y,
     return 0;
 }
 
-private int
+static int
 pclxl_curveto(gx_device_vector * vdev, floatp x0, floatp y0,
 	   floatp x1, floatp y1, floatp x2, floatp y2, floatp x3, floatp y3,
 	      gx_path_type_t type)
@@ -1022,7 +1045,7 @@ pclxl_curveto(gx_device_vector * vdev, floatp x0, floatp y0,
     return 0;
 }
 
-private int
+static int
 pclxl_closepath(gx_device_vector * vdev, floatp x, floatp y,
 		floatp x_start, floatp y_start, gx_path_type_t type)
 {
@@ -1038,7 +1061,7 @@ pclxl_closepath(gx_device_vector * vdev, floatp x, floatp y,
     return 0;
 }
 
-private int
+static int
 pclxl_endpath(gx_device_vector * vdev, gx_path_type_t type)
 {
     gx_device_pclxl *const xdev = (gx_device_pclxl *)vdev;
@@ -1049,6 +1072,12 @@ pclxl_endpath(gx_device_vector * vdev, gx_path_type_t type)
     if (code < 0)
 	return code;
     if (type & (gx_path_type_fill | gx_path_type_stroke)) {
+	if (rule != xdev->fill_rule) {
+	    px_put_ub(s, (byte)(rule == gx_path_type_even_odd ? eEvenOdd :
+		       eNonZeroWinding));
+	    px_put_ac(s, pxaFillMode, pxtSetFillMode);
+	    xdev->fill_rule = rule;
+	}
 	pclxl_set_paints(xdev, type);
 	spputc(s, pxtPaintPath);
     }
@@ -1070,7 +1099,7 @@ pclxl_endpath(gx_device_vector * vdev, gx_path_type_t type)
 
 /* Vector implementation procedures */
 
-private const gx_device_vector_procs pclxl_vector_procs = {
+static const gx_device_vector_procs pclxl_vector_procs = {
 	/* Page management */
     pclxl_beginpage,
 	/* Imager state */
@@ -1101,7 +1130,7 @@ private const gx_device_vector_procs pclxl_vector_procs = {
 /* ------ Open/close/page ------ */
 
 /* Open the device. */
-private int
+static int
 pclxl_open_device(gx_device * dev)
 {
     gx_device_vector *const vdev = (gx_device_vector *)dev;
@@ -1114,6 +1143,12 @@ pclxl_open_device(gx_device * dev)
 					 VECTOR_OPEN_FILE_SEQUENTIAL);
     if (code < 0)
 	return code;
+
+    xdev->page = 0;
+    xdev->Duplex = false;
+    xdev->MediaPosition = 0;
+    xdev->Tumble = false;
+
     pclxl_page_init(xdev);
     px_write_file_header(vdev->strm, dev);
     xdev->media_size = pxeMediaSize_next;	/* no size selected */
@@ -1124,7 +1159,7 @@ pclxl_open_device(gx_device * dev)
 
 /* Wrap up ("output") a page. */
 /* We only support flush = true, and we don't support num_copies != 1. */
-private int
+static int
 pclxl_output_page(gx_device * dev, int num_copies, int flush)
 {
     gx_device_pclxl *const xdev = (gx_device_pclxl *)dev;
@@ -1145,7 +1180,7 @@ pclxl_output_page(gx_device * dev, int num_copies, int flush)
 /* Close the device. */
 /* Note that if this is being called as a result of finalization, */
 /* the stream may no longer exist. */
-private int
+static int
 pclxl_close_device(gx_device * dev)
 {
     gx_device_pclxl *const xdev = (gx_device_pclxl *)dev;
@@ -1159,12 +1194,12 @@ pclxl_close_device(gx_device * dev)
 
 /* ------ One-for-one images ------ */
 
-private const byte eBit_values[] = {
+static const byte eBit_values[] = {
     0, e1Bit, 0, 0, e4Bit, 0, 0, 0, e8Bit
 };
 
 /* Copy a monochrome bitmap. */
-private int
+static int
 pclxl_copy_mono(gx_device * dev, const byte * data, int data_x, int raster,
 		gx_bitmap_id id, int x, int y, int w, int h,
 		gx_color_index zero, gx_color_index one)
@@ -1246,7 +1281,7 @@ pclxl_copy_mono(gx_device * dev, const byte * data, int data_x, int raster,
 }
 
 /* Copy a color bitmap. */
-private int
+static int
 pclxl_copy_color(gx_device * dev,
 		 const byte * base, int sourcex, int raster, gx_bitmap_id id,
 		 int x, int y, int w, int h)
@@ -1286,7 +1321,7 @@ pclxl_copy_color(gx_device * dev,
 }
 
 /* Fill a mask. */
-private int
+static int
 pclxl_fill_mask(gx_device * dev,
 		const byte * data, int data_x, int raster, gx_bitmap_id id,
 		int x, int y, int w, int h,
@@ -1338,7 +1373,7 @@ pclxl_fill_mask(gx_device * dev,
 }
 
 /* Do a RasterOp. */
-private int
+static int
 pclxl_strip_copy_rop(gx_device * dev, const byte * sdata, int sourcex,
 		     uint sraster, gx_bitmap_id id,
 		     const gx_color_index * scolors,
@@ -1370,7 +1405,7 @@ gs_private_st_suffix_add1(st_pclxl_image_enum, pclxl_image_enum_t,
 			  rows.data);
 
 /* Start processing an image. */
-private int
+static int
 pclxl_begin_image(gx_device * dev,
 		  const gs_imager_state * pis, const gs_image_t * pim,
 		  gs_image_format_t format, const gs_int_rect * prect,
@@ -1505,24 +1540,28 @@ pclxl_begin_image(gx_device * dev,
     gs_free_object(mem, row_data, "pclxl_begin_image(rows)");
     gs_free_object(mem, pie, "pclxl_begin_image");
  use_default:
+    if (dev->color_info.num_components == 1)
+        pclxl_set_color_space(xdev, eGray);
+    else
+        pclxl_set_color_space(xdev, eRGB);
     return gx_default_begin_image(dev, pis, pim, format, prect,
 				  pdcolor, pcpath, mem, pinfo);
 }
 
 /* Write one strip of an image, from pie->rows.first_y to pie->y. */
-private int
+static int
 image_transform_x(const pclxl_image_enum_t *pie, int sx)
 {
     return (int)((pie->mat.tx + sx * pie->mat.xx + 0.5) /
 		 ((const gx_device_pclxl *)pie->dev)->scale.x);
 }
-private int
+static int
 image_transform_y(const pclxl_image_enum_t *pie, int sy)
 {
     return (int)((pie->mat.ty + sy * pie->mat.yy + 0.5) /
 		 ((const gx_device_pclxl *)pie->dev)->scale.y);
 }
-private int
+static int
 pclxl_image_write_rows(pclxl_image_enum_t *pie)
 {
     gx_device_pclxl *const xdev = (gx_device_pclxl *)pie->dev;
@@ -1551,7 +1590,7 @@ pclxl_image_write_rows(pclxl_image_enum_t *pie)
 }
 
 /* Process the next piece of an image. */
-private int
+static int
 pclxl_image_plane_data(gx_image_enum_common_t * info,
 		       const gx_image_plane_t * planes, int height,
 		       int *rows_used)
@@ -1584,7 +1623,7 @@ pclxl_image_plane_data(gx_image_enum_common_t * info,
 }
 
 /* Clean up by releasing the buffers. */
-private int
+static int
 pclxl_image_end_image(gx_image_enum_common_t * info, bool draw_last)
 {
     pclxl_image_enum_t *pie = (pclxl_image_enum_t *) info;
@@ -1597,3 +1636,101 @@ pclxl_image_end_image(gx_image_enum_common_t * info, bool draw_last)
     gs_free_object(pie->memory, pie, "pclxl_end_image");
     return code;
 }
+
+/*
+ * 'pclxl_get_params()' - Get pagedevice parameters.
+ */
+
+static int				/* O - Error status */
+pclxl_get_params(gx_device     *dev,	/* I - Device info */
+                 gs_param_list *plist)	/* I - Parameter list */
+{
+  gx_device_pclxl	*xdev;		/* PCL XL device */
+  int			code;		/* Return code */
+
+
+ /*
+  * First process the "standard" page device parameters...
+  */
+
+  if ((code = gdev_vector_get_params(dev, plist)) < 0)
+    return (code);
+
+ /*
+  * Then write the PCL-XL parameters...
+  */
+
+  xdev = (gx_device_pclxl *)dev;
+
+  if ((code = param_write_bool(plist, "Duplex", &(xdev->Duplex))) < 0)
+    return (code);
+
+  if ((code = param_write_int(plist, "MediaPosition",
+                              &(xdev->MediaPosition))) < 0)
+    return (code);
+
+  if ((code = param_write_bool(plist, "Tumble", &(xdev->Tumble))) < 0)
+    return (code);
+
+  return (0);
+}
+
+
+/*
+ * 'pclxl_put_params()' - Set pagedevice parameters.
+ */
+
+static int				/* O - Error status */
+pclxl_put_params(gx_device     *dev,	/* I - Device info */
+                 gs_param_list *plist)	/* I - Parameter list */
+{
+  gx_device_pclxl	*xdev;		/* PCL XL device */
+  int			code;		/* Error code */
+  int			intval;		/* Integer value */
+  bool			boolval;	/* Boolean value */
+
+
+ /*
+  * Process PCL-XL driver parameters...
+  */
+
+  xdev = (gx_device_pclxl *)dev;
+
+#define intoption(name, sname, type) \
+  if ((code = param_read_int(plist, sname, &intval)) < 0) \
+  { \
+    param_signal_error(plist, sname, code); \
+    return (code); \
+  } \
+  else if (code == 0) \
+  { \
+    xdev->name = (type)intval; \
+  }
+
+#define booloption(name, sname) \
+  if ((code = param_read_bool(plist, sname, &boolval)) < 0) \
+  { \
+    if ((code = param_read_null(plist, sname)) < 0) \
+    { \
+      param_signal_error(plist, sname, code); \
+      return (code); \
+    } \
+    if (code == 0) \
+      xdev->name = false; \
+  } \
+  else if (code == 0) \
+    xdev->name = (bool)boolval;
+
+  booloption(Duplex, "Duplex")
+  intoption(MediaPosition, "MediaPosition", int)
+  booloption(Tumble, "Tumble")
+
+ /*
+  * Then process standard page device parameters...
+  */
+
+  if ((code = gdev_vector_put_params(dev, plist)) < 0)
+    return (code);
+
+  return (0);
+}
