@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1992, 1994, 1996 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevsun.c,v 1.2.6.1 2002/02/22 19:45:56 ray Exp $*/
+/* $Id: gdevsun.c 8250 2007-09-25 13:31:24Z giles $*/
 /* SunView driver */
 #include "gx.h"			/* for gx_bitmap; includes std.h */
 
@@ -53,7 +50,7 @@ dev_proc_copy_color(sun_copy_color);
 dev_proc_draw_line(sun_draw_line);
 
 /* The device descriptor */
-private gx_device_procs sun_procs = {
+static gx_device_procs sun_procs = {
 	sun_open,
 	NULL,			/* get_initial_matrix */
 	sun_sync,
@@ -222,7 +219,7 @@ Display_does_not_support_this_many_colors
 #define TRUE_BLUE_COLS	(1 << TRUE_BLUE_BITS)
 
 /* Initialize the device. */
-private Notify_value destroy_func();
+static Notify_value destroy_func();
 int
 sun_open(register gx_device *dev)
 {
@@ -422,7 +419,7 @@ if ( gs_debug['X'] )
 	return 0;
 }
 /* Prevent the user from closing the window. */
-private Notify_value
+static Notify_value
 destroy_func(Frame frame, Destroy_status status)
 {	if ( status == DESTROY_CHECKING )
 	   {	notify_veto_destroy(frame);
