@@ -1,20 +1,17 @@
-/* Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gdevsunr.c,v 1.4.2.1 2002/02/22 19:45:56 ray Exp $ */
+/* $Id: gdevsunr.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Sun raster file driver */
 #include "gdevprn.h"
 
@@ -44,7 +41,7 @@ typedef struct sun_rasterfile_s {
 #  define Y_DPI 72
 #endif
 
-private dev_proc_print_page(sunhmono_print_page);
+static dev_proc_print_page(sunhmono_print_page);
 
 const gx_device_printer gs_sunhmono_device =
     prn_device(prn_std_procs, "sunhmono",
@@ -53,7 +50,7 @@ const gx_device_printer gs_sunhmono_device =
 	       0, 0, 0, 0,	/* margins */
 	       1, sunhmono_print_page);
 
-private int
+static int
 sunhmono_print_page(gx_device_printer * pdev, FILE * prn_stream)
 {
     int gsLineBytes = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
@@ -69,7 +66,7 @@ sunhmono_print_page(gx_device_printer * pdev, FILE * prn_stream)
       errprintf("pdev->width:%d (%d/%d) gsLineBytes:%d rasLineBytes:%d\n",
       pdev->width, pdev->width/8, pdev->width%8,gsLineBytes,rasLineBytes);
     */
-    lineStorage = gs_malloc(gsLineBytes, 1, "rasterfile_print_page(in)");
+    lineStorage = gs_malloc(pdev->memory, gsLineBytes, 1, "rasterfile_print_page(in)");
     if (lineStorage == 0) {
 	code = gs_note_error(gs_error_VMerror);
 	goto out;
@@ -96,6 +93,6 @@ sunhmono_print_page(gx_device_printer * pdev, FILE * prn_stream)
     fwrite("};\n", 1, 3, prn_stream);
 out:
     /* Clean up... */
-    gs_free(lineStorage, gsLineBytes, 1, "rasterfile_print_page(in)");
+    gs_free(pdev->memory, lineStorage, gsLineBytes, 1, "rasterfile_print_page(in)");
     return code;
 }
