@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevvec.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gdevvec.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Utilities for "vector" devices */
 #include "math_.h"
 #include "memory_.h"
@@ -42,7 +42,7 @@ gdev_vector_setflat(gx_device_vector * vdev, floatp flatness)
 }
 
 /* Put a path on the output file. */
-private bool
+static bool
 coord_between(fixed start, fixed mid, fixed end)
 {
     return (start <= end ? start <= mid && mid <= end :
@@ -224,7 +224,7 @@ gdev_vector_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1,
 /* ================ Utility procedures ================ */
 
 /* Recompute the cached color values. */
-private void
+static void
 gdev_vector_load_cache(gx_device_vector * vdev)
 {
     vdev->black = gx_device_black((gx_device *)vdev);
@@ -357,7 +357,7 @@ gdev_vector_update_log_op(gx_device_vector * vdev, gs_logical_operation_t lop)
 }
 
 /* Update color (fill or stroke). */
-private int
+static int
 gdev_vector_update_color(gx_device_vector * vdev,
 			      const gs_imager_state * pis,
 			      const gx_drawing_color * pdcolor,
@@ -392,7 +392,7 @@ gdev_vector_update_fill_color(gx_device_vector * vdev,
 }
 
 /* Update the state for filling a region. */
-private int
+static int
 update_fill(gx_device_vector * vdev, const gs_imager_state * pis, 
 	    const gx_drawing_color * pdcolor, gs_logical_operation_t lop)
 {
@@ -419,7 +419,7 @@ gdev_vector_prepare_fill(gx_device_vector * vdev, const gs_imager_state * pis,
 }
 
 /* Compare two dash patterns. */
-private bool
+static bool
 dash_pattern_eq(const float *stored, const gx_dash_params * set, floatp scale)
 {
     int i;
