@@ -1,35 +1,30 @@
-/* Copyright (C) 1994-2001, Ghostgum Software Pty Ltd.  All rights reserved.
-   Portions Copyright (C) 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
- */
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
 
 
-/* $Id: gsdll.h,v 1.3.2.2 2002/04/10 09:13:11 giles Exp $ */
+/* $Id: gsdll.h 8022 2007-06-05 22:23:38Z giles $ */
 
 /* This interface is deprecated and will be removed in future
  * ghostscript releases.  Use the interface described in
  * API.htm and iapi.h.
  */
 
-
 #ifndef gsdll_INCLUDED
 #  define gsdll_INCLUDED
 
 #include "iapi.h"
 
-#ifdef __MACINTOSH__
+#ifdef __MACOS__
 #define HWND char *
 #include <QDOffscreen.h>
 #pragma export on
@@ -96,7 +91,7 @@ typedef int (GSDLLAPIPTR PFN_gsdll_execute_end) (void);
 typedef int (GSDLLAPIPTR PFN_gsdll_exit) (void);
 typedef int (GSDLLAPIPTR PFN_gsdll_lock_device) (unsigned char *, int);
 
-#ifdef __MACINTOSH__
+#ifdef __MACOS__
 #pragma export off
 #endif
 
