@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,9 +11,10 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsimage.c 6673 2006-03-22 08:43:11Z leonardo $ */
+/* $Id: gsimage.c 8539 2008-02-22 20:18:08Z leonardo $ */
 /* Image setup procedures for Ghostscript library */
 #include "memory_.h"
+#include "math_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gsstruct.h"
@@ -24,8 +25,10 @@
 #include "gxdevice.h"
 #include "gxiparam.h"
 #include "gxpath.h"		/* for gx_effective_clip_path */
+#include "gximask.h"
 #include "gzstate.h"
-
+#include "gsutil.h"
+#include "vdtrace.h"
 
 /*
   The main internal invariant for the gs_image machinery is
@@ -117,7 +120,7 @@ gs_private_st_composite(st_gs_image_enum, gs_image_enum, "gs_image_enum",
 #define gs_image_enum_num_ptrs 2
 
 /* GC procedures */
-private 
+static 
 ENUM_PTRS_WITH(gs_image_enum_enum_ptrs, gs_image_enum *eptr)
 {
     /* Enumerate the data planes. */
@@ -132,7 +135,7 @@ ENUM_PTRS_WITH(gs_image_enum_enum_ptrs, gs_image_enum *eptr)
 ENUM_PTR(0, gs_image_enum, dev);
 ENUM_PTR(1, gs_image_enum, info);
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(gs_image_enum_reloc_ptrs, gs_image_enum *eptr)
+static RELOC_PTRS_WITH(gs_image_enum_reloc_ptrs, gs_image_enum *eptr)
 {
     int i;
 
@@ -145,6 +148,43 @@ private RELOC_PTRS_WITH(gs_image_enum_reloc_ptrs, gs_image_enum *eptr)
 }
 RELOC_PTRS_END
 
+static int
+is_image_visible(const gs_image_common_t * pic, gs_state * pgs, gx_clip_path *pcpath)
+{
+    /* HACK : We need the source image size here, 
+       but gs_image_common_t doesn't pass it.
+       We would like to move Width, Height to gs_image_common,
+       but gs_image2_t appears to have those fields of double type.
+     */
+    if (pic->type->begin_typed_image == gx_begin_image1) {
+	gs_image1_t *pim = (gs_image1_t *) pic;
+	gs_rect image_rect = {{0, 0}, {0, 0}};
+	gs_rect device_rect;
+	gs_int_rect device_int_rect;
+	gs_matrix mat;
+	int code;
+
+	image_rect.q.x = pim->Width;
+	image_rect.q.y = pim->Height;
+	code = gs_matrix_invert(&pic->ImageMatrix, &mat);
+	if (code < 0)
+	    return code;
+	code = gs_matrix_multiply(&mat, &ctm_only(pgs), &mat);
+	if (code < 0)
+	    return code;
+	code = gs_bbox_transform(&image_rect, &mat, &device_rect);
+	if (code < 0)
+	    return code;
+	device_int_rect.p.x = (int)floor(device_rect.p.x);
+	device_int_rect.p.y = (int)floor(device_rect.p.y);
+	device_int_rect.q.x = (int)ceil(device_rect.q.x);
+	device_int_rect.q.y = (int)ceil(device_rect.q.y);
+	if (!gx_cpath_rect_visible(pcpath, &device_int_rect))
+	    return 0;
+    }
+    return 1;
+}
+
 /* Create an image enumerator given image parameters and a graphics state. */
 int
 gs_image_begin_typed(const gs_image_common_t * pic, gs_state * pgs,
@@ -153,21 +193,47 @@ gs_image_begin_typed(const gs_image_common_t * pic, gs_state * pgs,
     gx_device *dev = gs_currentdevice(pgs);
     gx_clip_path *pcpath;
     int code = gx_effective_clip_path(pgs, &pcpath);
+    gx_device *dev2 = dev;
 
     if (code < 0)
 	return code;
+    /* Processing an image object operation */
+    gs_set_object_tag(pgs, GS_IMAGE_TAG);
+
     if (uses_color) {
 	gx_set_dev_color(pgs);
         code = gs_state_color_load(pgs);
         if (code < 0)
 	    return code;
     }
-    return gx_device_begin_typed_image(dev, (const gs_imager_state *)pgs,
+    /* Imagemask with shading color needs a special optimization
+       with converting the image into a clipping. 
+       Check for such case after gs_state_color_load is done,
+       because it can cause interpreter callout.
+     */
+    if (pic->type->begin_typed_image == &gx_begin_image1) {
+	gs_image_t *image = (gs_image_t *)pic;
+
+	if(image->ImageMask) {
+	    code = gx_image_fill_masked_start(dev, pgs->dev_color, pcpath, pgs->memory, &dev2);
+	    if (code < 0)
+		return code;
+	}
+    }
+    code = gx_device_begin_typed_image(dev2, (const gs_imager_state *)pgs,
 		NULL, pic, NULL, pgs->dev_color, pcpath, pgs->memory, ppie);
+    if (code < 0)
+	return code;
+    code = is_image_visible(pic, pgs, pcpath);
+    if (code < 0)
+	return code;
+    if (!code)	
+	(*ppie)->skipping = true;
+    return 0;
 }
 
 /* Allocate an image enumerator. */
-private void
+static void
 image_enum_init(gs_image_enum * penum)
 {
     /* Clean pointers for GC. */
@@ -207,17 +273,11 @@ gs_image_init(gs_image_enum * penum, const gs_image_t * pim, bool multi,
 	if (pgs->in_cachedevice)
 	    return_error(gs_error_undefined);
 	if (image.ColorSpace == NULL) {
-            /* parameterless color space - no re-entrancy problems */
-            static gs_color_space cs;
-
             /*
-             * Mutiple initialization of a DeviceGray color space is
-             * not harmful, as the space has no parameters. Use of a
-             * non-current color space is potentially incorrect, but
-             * it appears this case doesn't arise.
+             * Use of a non-current color space is potentially
+             * incorrect, but it appears this case doesn't arise.
              */
-            gs_cspace_init_DeviceGray(pgs->memory, &cs);
-	    image.ColorSpace = &cs;
+	    image.ColorSpace = gs_cspace_new_DeviceGray(pgs->memory);
         }
     }
     code = gs_image_begin_typed((const gs_image_common_t *)&image, pgs,
@@ -241,7 +301,7 @@ gs_image_bytes_per_plane_row(const gs_image_enum * penum, int plane)
 }
 
 /* Cache information when initializing, or after transferring plane data. */
-private void
+static void
 cache_planes(gs_image_enum *penum)
 {
     int i;
@@ -258,7 +318,7 @@ cache_planes(gs_image_enum *penum)
     }
 }
 /* Advance to the next wanted plane. */
-private void
+static void
 next_plane(gs_image_enum *penum)
 {
     int px = penum->plane_index;
@@ -273,7 +333,7 @@ next_plane(gs_image_enum *penum)
  * Initialize plane_index and (if appropriate) wanted and
  * wanted_varies at the beginning of a group of planes.
  */
-private void
+static void
 begin_planes(gs_image_enum *penum)
 {
     cache_planes(penum);
@@ -281,7 +341,7 @@ begin_planes(gs_image_enum *penum)
     next_plane(penum);
 }
 
-private int
+int
 gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
 	    const gs_data_image_t * pim, gx_device * dev)
 {
@@ -312,7 +372,8 @@ gs_image_common_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
     for (i = 0; i < pie->num_planes; ++i) {
 	penum->planes[i].pos = 0;
 	penum->planes[i].source.size = 0;	/* for gs_image_next_planes */
-	penum->planes[i].row.data = 0; /* for GC */
+	penum->planes[i].source.data = 0; /* for GC */
+        penum->planes[i].row.data = 0; /* for GC */
 	penum->planes[i].row.size = 0; /* ditto */
 	penum->image_planes[i].data_x = 0; /* just init once, never changes */
     }
@@ -359,14 +420,14 @@ gs_image_planes_wanted(gs_image_enum *penum)
  * Because some PostScript files use save/restore within an image data
  * reading procedure, this must be a stable allocator.
  */
-private gs_memory_t *
+static gs_memory_t *
 gs_image_row_memory(const gs_image_enum *penum)
 {
     return gs_memory_stable(penum->memory);
 }
 
 /* Free the row buffers when cleaning up. */
-private void
+static void
 free_row_buffers(gs_image_enum *penum, int num_planes, client_name_t cname)
 {
     int i;
@@ -417,6 +478,10 @@ gs_image_next_planes(gs_image_enum * penum,
     int code = 0;
 
 #ifdef DEBUG
+    vd_get_dc('i');
+    vd_set_shift(0, 0);
+    vd_set_scale(0.01);
+    vd_set_origin(0, 0);
     if (gs_debug_c('b')) {
 	int pi;
 
@@ -547,28 +612,43 @@ gs_image_next_planes(gs_image_enum * penum,
     /* Return the retained data pointers. */
     for (i = 0; i < num_planes; ++i)
 	plane_data[i] = penum->planes[i].source;
+    vd_release_dc;
     return code;
 }
 
 /* Clean up after processing an image. */
+/* Public for ghotpcl. */
 int
-gs_image_cleanup(gs_image_enum * penum)
+gs_image_cleanup(gs_image_enum * penum, gs_state *pgs)
 {
-    int code = 0;
+    int code = 0, code1;
 
     free_row_buffers(penum, penum->num_planes, "gs_image_cleanup(row)");
-    if (penum->info != 0)
-        code = gx_image_end(penum->info, !penum->error);
+    if (penum->info != 0) {
+	if (dev_proc(penum->info->dev, pattern_manage)(penum->info->dev, 
+		    gs_no_id, NULL, pattern_manage__is_cpath_accum)) {
+	    /* Performing a conversion of imagemask into a clipping path. */
+	    gx_device *cdev = penum->info->dev;
+
+	    code = gx_image_end(penum->info, !penum->error); /* Releases penum->info . */
+	    code1 = gx_image_fill_masked_end(cdev, penum->dev, pgs->dev_color);
+	    if (code == 0)
+		code = code1;
+	} else
+	    code = gx_image_end(penum->info, !penum->error);
+    }
     /* Don't free the local enumerator -- the client does that. */
+
     return code;
 }
 
 /* Clean up after processing an image and free the enumerator. */
 int
-gs_image_cleanup_and_free_enum(gs_image_enum * penum)
+gs_image_cleanup_and_free_enum(gs_image_enum * penum, gs_state *pgs)
 {
-    int code = gs_image_cleanup(penum);
+    int code = gs_image_cleanup(penum, pgs);
 
     gs_free_object(penum->memory, penum, "gs_image_cleanup_and_free_enum");
     return code;
 }
+
