@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsimage.h 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gsimage.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Generic image rendering interface */
 /* Requires gsstate.h */
 
@@ -131,6 +131,15 @@ gs_image_enum *gs_image_enum_alloc(gs_memory_t *, client_name_t);
 #  define gx_device_DEFINED
 typedef struct gx_device_s gx_device;
 #endif
+#ifndef gx_device_color_DEFINED
+#  define gx_device_color_DEFINED
+typedef struct gx_device_color_s gx_device_color;
+#endif
+
+/* Initialize the common part of the image class */
+int gs_image_common_init(gs_image_enum * penum,
+			 gx_image_enum_common_t * pie,
+			 const gs_data_image_t * pim, gx_device * dev);
 
 /* Initialize an enumerator for an ImageType 1 image. */
 int gs_image_init(gs_image_enum * penum, const gs_image_t * pim,
@@ -185,9 +194,9 @@ int gs_image_next(gs_image_enum * penum, const byte * dbytes,
 		  uint dsize, uint * pused);
 
 /* Clean up after processing an image. */
-int gs_image_cleanup(gs_image_enum * penum);
+int gs_image_cleanup(gs_image_enum * penum, gs_state *pgs);
 
 /* Clean up after processing an image and free the enumerator. */
-int gs_image_cleanup_and_free_enum(gs_image_enum * penum);
+int gs_image_cleanup_and_free_enum(gs_image_enum * penum, gs_state *pgs);
 
 #endif /* gsimage_INCLUDED */
