@@ -1,25 +1,18 @@
-/* Copyright (C) 1998, 2000, 2002 artofcode LLC. All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gsmalloc.c,v 1.3.2.2 2002/01/18 23:36:37 giles Exp $ */
+/* $Id: gsmalloc.c 8250 2007-09-25 13:31:24Z giles $ */
 /* C heap allocator */
-
 #include "malloc_.h"
 #include "gdebug.h"
 #include "gserror.h"
@@ -32,6 +25,7 @@
 #include "gsmemlok.h"		/* locking (multithreading) wrapper */
 #include "gsmemret.h"		/* retrying wrapper */
 
+
 /* ------ Heap allocator ------ */
 
 /*
@@ -40,26 +34,26 @@
  * blocks so we can free them at cleanup time.
  */
 /* Raw memory procedures */
-private gs_memory_proc_alloc_bytes(gs_heap_alloc_bytes);
-private gs_memory_proc_resize_object(gs_heap_resize_object);
-private gs_memory_proc_free_object(gs_heap_free_object);
-private gs_memory_proc_stable(gs_heap_stable);
-private gs_memory_proc_status(gs_heap_status);
-private gs_memory_proc_free_all(gs_heap_free_all);
+static gs_memory_proc_alloc_bytes(gs_heap_alloc_bytes);
+static gs_memory_proc_resize_object(gs_heap_resize_object);
+static gs_memory_proc_free_object(gs_heap_free_object);
+static gs_memory_proc_stable(gs_heap_stable);
+static gs_memory_proc_status(gs_heap_status);
+static gs_memory_proc_free_all(gs_heap_free_all);
 
 /* Object memory procedures */
-private gs_memory_proc_alloc_struct(gs_heap_alloc_struct);
-private gs_memory_proc_alloc_byte_array(gs_heap_alloc_byte_array);
-private gs_memory_proc_alloc_struct_array(gs_heap_alloc_struct_array);
-private gs_memory_proc_object_size(gs_heap_object_size);
-private gs_memory_proc_object_type(gs_heap_object_type);
-private gs_memory_proc_alloc_string(gs_heap_alloc_string);
-private gs_memory_proc_resize_string(gs_heap_resize_string);
-private gs_memory_proc_free_string(gs_heap_free_string);
-private gs_memory_proc_register_root(gs_heap_register_root);
-private gs_memory_proc_unregister_root(gs_heap_unregister_root);
-private gs_memory_proc_enable_free(gs_heap_enable_free);
-private const gs_memory_procs_t gs_malloc_memory_procs =
+static gs_memory_proc_alloc_struct(gs_heap_alloc_struct);
+static gs_memory_proc_alloc_byte_array(gs_heap_alloc_byte_array);
+static gs_memory_proc_alloc_struct_array(gs_heap_alloc_struct_array);
+static gs_memory_proc_object_size(gs_heap_object_size);
+static gs_memory_proc_object_type(gs_heap_object_type);
+static gs_memory_proc_alloc_string(gs_heap_alloc_string);
+static gs_memory_proc_resize_string(gs_heap_resize_string);
+static gs_memory_proc_free_string(gs_heap_free_string);
+static gs_memory_proc_register_root(gs_heap_register_root);
+static gs_memory_proc_unregister_root(gs_heap_unregister_root);
+static gs_memory_proc_enable_free(gs_heap_enable_free);
+static const gs_memory_procs_t gs_malloc_memory_procs =
 {
     /* Raw memory procedures */
     gs_heap_alloc_bytes,
@@ -103,13 +97,13 @@ struct gs_malloc_block_s {
     malloc_block_data;
 /* ANSI C does not allow zero-size arrays, so we need the following */
 /* unnecessary and wasteful workaround: */
-#define _npad (-size_of(struct malloc_block_data_s) & (arch_align_memory_mod - 1))
-    byte _pad[(_npad == 0 ? arch_align_memory_mod : _npad)];
+#define _npad (-size_of(struct malloc_block_data_s) & (ARCH_ALIGN_MEMORY_MOD - 1))
+    byte _pad[(_npad == 0 ? ARCH_ALIGN_MEMORY_MOD : _npad)];
 #undef _npad
 };
 
 /* Initialize a malloc allocator. */
-private long heap_available(P0());
+static long heap_available(void);
 gs_malloc_memory_t *
 gs_malloc_memory_init(void)
 {
@@ -122,6 +116,9 @@ gs_malloc_memory_init(void)
     mem->limit = max_long;
     mem->used = 0;
     mem->max_used = 0;
+    mem->gs_lib_ctx = 0;
+    mem->non_gc_memory = (gs_memory_t *)mem;
+
     return mem;
 }
 /*
@@ -131,8 +128,8 @@ gs_malloc_memory_init(void)
  */
 #define max_malloc_probes 20
 #define malloc_probe_size 64000
-private long
-heap_available(void)
+static long
+heap_available()
 {
     long avail = 0;
     void *probes[max_malloc_probes];
@@ -151,7 +148,7 @@ heap_available(void)
 }
 
 /* Allocate various kinds of blocks. */
-private byte *
+static byte *
 gs_heap_alloc_bytes(gs_memory_t * mem, uint size, client_name_t cname)
 {
     gs_malloc_memory_t *mmem = (gs_malloc_memory_t *) mem;
@@ -179,13 +176,13 @@ gs_heap_alloc_bytes(gs_memory_t * mem, uint size, client_name_t cname)
 	else {
 	    gs_malloc_block_t *bp = (gs_malloc_block_t *) ptr;
 
-#ifdef DEBUG
-	    /* malloc must align at least as strictly as the compiler! */
-	    if (ALIGNMENT_MOD(ptr, arch_align_memory_mod) != 0)
-		set_msg("malloc alignment failed!");
-	    else
-#endif
-		set_msg(ok_msg);
+	    /*
+	     * We would like to check that malloc aligns blocks at least as
+	     * strictly as the compiler (as defined by ARCH_ALIGN_MEMORY_MOD).
+	     * However, Microsoft VC 6 does not satisfy this requirement.
+	     * See gsmemory.h for more explanation.
+	     */
+	    set_msg(ok_msg);
 	    if (mmem->allocated)
 		mmem->allocated->prev = bp;
 	    bp->next = mmem->allocated;
@@ -209,7 +206,7 @@ gs_heap_alloc_bytes(gs_memory_t * mem, uint size, client_name_t cname)
     return ptr;
 #undef set_msg
 }
-private void *
+static void *
 gs_heap_alloc_struct(gs_memory_t * mem, gs_memory_type_ptr_t pstype,
 		     client_name_t cname)
 {
@@ -221,7 +218,7 @@ gs_heap_alloc_struct(gs_memory_t * mem, gs_memory_type_ptr_t pstype,
     ((gs_malloc_block_t *) ptr)[-1].type = pstype;
     return ptr;
 }
-private byte *
+static byte *
 gs_heap_alloc_byte_array(gs_memory_t * mem, uint num_elements, uint elt_size,
 			 client_name_t cname)
 {
@@ -231,7 +228,7 @@ gs_heap_alloc_byte_array(gs_memory_t * mem, uint num_elements, uint elt_size,
 	return 0;
     return gs_heap_alloc_bytes(mem, (uint) lsize, cname);
 }
-private void *
+static void *
 gs_heap_alloc_struct_array(gs_memory_t * mem, uint num_elements,
 			   gs_memory_type_ptr_t pstype, client_name_t cname)
 {
@@ -244,7 +241,7 @@ gs_heap_alloc_struct_array(gs_memory_t * mem, uint num_elements,
     ((gs_malloc_block_t *) ptr)[-1].type = pstype;
     return ptr;
 }
-private void *
+static void *
 gs_heap_resize_object(gs_memory_t * mem, void *obj, uint new_num_elements,
 		      client_name_t cname)
 {
@@ -276,21 +273,21 @@ gs_heap_resize_object(gs_memory_t * mem, void *obj, uint new_num_elements,
 		      gs_alloc_fill_alloc, new_size - old_size);
     return new_ptr + 1;
 }
-private uint
+static uint
 gs_heap_object_size(gs_memory_t * mem, const void *ptr)
 {
     return ((const gs_malloc_block_t *)ptr)[-1].size;
 }
-private gs_memory_type_ptr_t
-gs_heap_object_type(gs_memory_t * mem, const void *ptr)
+static gs_memory_type_ptr_t
+gs_heap_object_type(const gs_memory_t * mem, const void *ptr)
 {
     return ((const gs_malloc_block_t *)ptr)[-1].type;
 }
-private void
+static void
 gs_heap_free_object(gs_memory_t * mem, void *ptr, client_name_t cname)
 {
     gs_malloc_memory_t *mmem = (gs_malloc_memory_t *) mem;
-    gs_malloc_block_t *bp = mmem->allocated;
+    gs_malloc_block_t *bp;
     gs_memory_type_ptr_t pstype;
     struct_proc_finalize((*finalize));
 
@@ -307,6 +304,9 @@ gs_heap_free_object(gs_memory_t * mem, void *ptr, client_name_t cname)
 		  (ulong) ptr, client_name_string(cname));
 	(*finalize) (ptr);
     }
+    bp = mmem->allocated; /* If 'finalize' releases a memory,
+			     this function could be called recursively and
+			     change mmem->allocated. */
     if (ptr == bp + 1) {
 	mmem->allocated = bp->next;
 	mmem->used -= bp->size + sizeof(gs_malloc_block_t);
@@ -342,12 +342,12 @@ gs_heap_free_object(gs_memory_t * mem, void *ptr, client_name_t cname)
 	free((char *)((gs_malloc_block_t *) ptr - 1));
     }
 }
-private byte *
+static byte *
 gs_heap_alloc_string(gs_memory_t * mem, uint nbytes, client_name_t cname)
 {
     return gs_heap_alloc_bytes(mem, nbytes, cname);
 }
-private byte *
+static byte *
 gs_heap_resize_string(gs_memory_t * mem, byte * data, uint old_num, uint new_num,
 		      client_name_t cname)
 {
@@ -356,30 +356,30 @@ gs_heap_resize_string(gs_memory_t * mem, byte * data, uint old_num, uint new_num
 		 client_name_string(cname), (ulong) data);
     return gs_heap_resize_object(mem, data, new_num, cname);
 }
-private void
+static void
 gs_heap_free_string(gs_memory_t * mem, byte * data, uint nbytes,
 		    client_name_t cname)
 {
     /****** SHOULD CHECK SIZE IF DEBUGGING ******/
     gs_heap_free_object(mem, data, cname);
 }
-private int
+static int
 gs_heap_register_root(gs_memory_t * mem, gs_gc_root_t * rp,
 		      gs_ptr_type_t ptype, void **up, client_name_t cname)
 {
     return 0;
 }
-private void
+static void
 gs_heap_unregister_root(gs_memory_t * mem, gs_gc_root_t * rp,
 			client_name_t cname)
 {
 }
-private gs_memory_t *
+static gs_memory_t *
 gs_heap_stable(gs_memory_t *mem)
 {
     return mem;			/* heap memory is stable */
 }
-private void
+static void
 gs_heap_status(gs_memory_t * mem, gs_memory_status_t * pstat)
 {
     gs_malloc_memory_t *mmem = (gs_malloc_memory_t *) mem;
@@ -387,7 +387,7 @@ gs_heap_status(gs_memory_t * mem, gs_memory_status_t * pstat)
     pstat->allocated = mmem->used + heap_available();
     pstat->used = mmem->used;
 }
-private void
+static void
 gs_heap_enable_free(gs_memory_t * mem, bool enable)
 {
     if (enable)
@@ -399,7 +399,7 @@ gs_heap_enable_free(gs_memory_t * mem, bool enable)
 }
 
 /* Release all memory acquired by this allocator. */
-private void
+static void
 gs_heap_free_all(gs_memory_t * mem, uint free_mask, client_name_t cname)
 {
     gs_malloc_memory_t *const mmem = (gs_malloc_memory_t *) mem;
@@ -427,6 +427,7 @@ gs_heap_free_all(gs_memory_t * mem, uint free_mask, client_name_t cname)
 int
 gs_malloc_wrap(gs_memory_t **wrapped, gs_malloc_memory_t *contents)
 {
+#ifdef USE_RETRY_AND_LOCKING_MEMORY_WRAPPERS
     gs_memory_t *cmem = (gs_memory_t *)contents;
     gs_memory_locked_t *lmem = (gs_memory_locked_t *)
 	gs_alloc_bytes_immovable(cmem, sizeof(gs_memory_locked_t),
@@ -460,6 +461,7 @@ gs_malloc_wrap(gs_memory_t **wrapped, gs_malloc_memory_t *contents)
     }
 
     *wrapped = (gs_memory_t *)rmem;
+#endif 
     return 0;
 }
 
@@ -467,17 +469,23 @@ gs_malloc_wrap(gs_memory_t **wrapped, gs_malloc_memory_t *contents)
 gs_malloc_memory_t *
 gs_malloc_wrapped_contents(gs_memory_t *wrapped)
 {
+#ifdef USE_RETRY_AND_LOCKING_MEMORY_WRAPPERS
     gs_memory_retrying_t *rmem = (gs_memory_retrying_t *)wrapped;
     gs_memory_locked_t *lmem =
 	(gs_memory_locked_t *)gs_memory_retrying_target(rmem);
-
-    return (gs_malloc_memory_t *)gs_memory_locked_target(lmem);
+    if (lmem) 
+	return (gs_malloc_memory_t *)gs_memory_locked_target(lmem);
+    return (gs_malloc_memory_t *) wrapped;
+#else
+    return (gs_malloc_memory_t *)wrapped;
+#endif 
 }
 
 /* Free the wrapper, and return the wrapped contents. */
 gs_malloc_memory_t *
 gs_malloc_unwrap(gs_memory_t *wrapped)
 {
+#ifdef USE_RETRY_AND_LOCKING_MEMORY_WRAPPERS
     gs_memory_retrying_t *rmem = (gs_memory_retrying_t *)wrapped;
     gs_memory_locked_t *lmem =
 	(gs_memory_locked_t *)gs_memory_retrying_target(rmem);
@@ -487,29 +495,42 @@ gs_malloc_unwrap(gs_memory_t *wrapped)
     gs_memory_locked_release(lmem);
     gs_free_object(contents, lmem, "gs_malloc_unwrap(locked)");
     return (gs_malloc_memory_t *)contents;
+#else
+    return (gs_malloc_memory_t *)wrapped;
+#endif 
 }
 
-/* ------ Historical single-instance artifacts ------ */
 
-/* Define the default allocator. */
-gs_malloc_memory_t *gs_malloc_memory_default;
-gs_memory_t *gs_memory_t_default;
-
-/* Create the default allocator. */
+/* Create the default allocator, and return it. */
 gs_memory_t *
-gs_malloc_init(void)
+gs_malloc_init(const gs_memory_t *parent)
 {
-    gs_malloc_memory_default = gs_malloc_memory_init();
-    gs_malloc_wrap(&gs_memory_t_default, gs_malloc_memory_default);
-    return gs_memory_t_default;
+    gs_malloc_memory_t *malloc_memory_default = gs_malloc_memory_init();
+    gs_memory_t *memory_t_default;
+
+    if (parent)
+	malloc_memory_default->gs_lib_ctx = parent->gs_lib_ctx;
+    else 
+        gs_lib_ctx_init((gs_memory_t *)malloc_memory_default);
+
+#ifdef USE_RETRY_AND_LOCKING_MEMORY_WRAPPERS
+    gs_malloc_wrap(&memory_t_default, malloc_memory_default);
+#else
+    memory_t_default = (gs_memory_t *)malloc_memory_default;
+#endif
+    memory_t_default->stable_memory = memory_t_default;
+    return memory_t_default;
 }
 
 /* Release the default allocator. */
 void
-gs_malloc_release(void)
+gs_malloc_release(gs_memory_t *mem)
 {
-    gs_malloc_unwrap(gs_memory_t_default);
-    gs_memory_t_default = 0;
-    gs_malloc_memory_release(gs_malloc_memory_default);
-    gs_malloc_memory_default = 0;
+#ifdef USE_RETRY_AND_LOCKING_MEMORY_WRAPPERS
+    gs_malloc_memory_t * malloc_memory_default = gs_malloc_unwrap(mem);
+#else
+    gs_malloc_memory_t * malloc_memory_default = (gs_malloc_memory_t *)mem;
+#endif
+
+    gs_malloc_memory_release(malloc_memory_default);
 }
