@@ -1,20 +1,17 @@
-/* Copyright (C) 2002 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsovrc.c,v 1.9 2005/03/14 18:08:36 dan Exp $ */
+/* $Id: gsovrc.c 8488 2008-01-17 13:37:19Z leonardo $ */
 /* overprint/overprint mode compositor implementation */
 
 #include "memory_.h"
@@ -55,7 +52,7 @@ private_st_gs_overprint_t();
  * The read routine returns the number of bytes read on success, or < 0 in
  * the event of an error.
  */
-private int
+static int
 write_color_index(gx_color_index cindex, byte * data, uint * psize)
 {
     int             num_bytes = 0;
@@ -75,7 +72,7 @@ write_color_index(gx_color_index cindex, byte * data, uint * psize)
     return 0;
 }
 
-private int
+static int
 read_color_index(gx_color_index * pcindex, const byte * data, uint size)
 {
     gx_color_index  cindex = 0;
@@ -101,7 +98,7 @@ read_color_index(gx_color_index * pcindex, const byte * data, uint size)
  *
  * This is fairly simple.
  */
-private bool
+static bool
 c_overprint_equal(const gs_composite_t * pct0, const gs_composite_t * pct1)
 {
     if (pct0->type == pct1->type) {
@@ -131,7 +128,7 @@ c_overprint_equal(const gs_composite_t * pct0, const gs_composite_t * pct1)
  * Convert an overprint compositor to string form for use by the command
  * list device.
  */
-private int
+static int
 c_overprint_write(const gs_composite_t * pct, byte * data, uint * psize)
 {
     const gs_overprint_params_t *   pparams = &((const gs_overprint_t *)pct)->params;
@@ -162,6 +159,7 @@ c_overprint_write(const gs_composite_t * pct, byte * data, uint * psize)
     if (used > avail)
         return_error(gs_error_rangecheck);
     data[0] = flags;
+    if_debug1('v', "[v]c_overprint_write(%d)\n", flags);
     return 0;
 }
 
@@ -169,7 +167,7 @@ c_overprint_write(const gs_composite_t * pct, byte * data, uint * psize)
  * Convert the string representation of the overprint parameter into the
  * full compositor.
  */
-private int
+static int
 c_overprint_read(
     gs_composite_t **       ppct,
     const byte *            data,
@@ -183,8 +181,10 @@ c_overprint_read(
     if (size < 1)
         return_error(gs_error_rangecheck);
     flags = *data;
+    if_debug1('v', "[v]c_overprint_read(%d)\n", flags);
     params.retain_any_comps = (flags & OVERPRINT_ANY_COMPS) != 0;
     params.retain_spot_comps = (flags & OVERPRINT_SPOT_COMPS) != 0;
+    params.idle = 0;
 
     /* check if the drawn_comps array is present */
     if (params.retain_any_comps && !params.retain_spot_comps) {
@@ -199,8 +199,24 @@ c_overprint_read(
     return code < 0 ? code : nbytes;
 }
 
+/*
+ * Check for closing compositor. 
+ */
+static int
+c_overprint_is_closing(const gs_composite_t *this, gs_composite_t **ppcte, gx_device *dev)
+{
+    if (*ppcte != NULL && (*ppcte)->type->comp_id != GX_COMPOSITOR_OVERPRINT)
+	return 0;
+    return 3;
+}
 
-private composite_create_default_compositor_proc(c_overprint_create_default_compositor);
+static composite_create_default_compositor_proc(c_overprint_create_default_compositor);
+static composite_create_default_compositor_proc(c_overprint_create_default_compositor);
+static composite_equal_proc(c_overprint_equal);
+static composite_write_proc(c_overprint_write);
+static composite_is_closing_proc(c_overprint_is_closing);
+static composite_read_proc(c_overprint_read);
+static composite_adjust_ctm_proc(c_overprint_adjust_ctm);
 
 /* methods for the overprint compositor */
 const gs_composite_type_t   gs_composite_overprint_type = {
@@ -210,6 +226,9 @@ const gs_composite_type_t   gs_composite_overprint_type = {
         c_overprint_equal,                      /* procs.equal */
         c_overprint_write,                      /* procs.write */
         c_overprint_read,                       /* procs.read */
+	gx_default_composite_adjust_ctm,
+	c_overprint_is_closing,
+	gx_default_composite_is_friendly,
 	gx_default_composite_clist_write_update,/* procs.composite_clist_write_update */
 	gx_default_composite_clist_read_update	/* procs.composite_clist_reade_update */
     }                                           /* procs */
@@ -229,15 +248,14 @@ gs_create_overprint(
 {
     gs_overprint_t *                pct;
 
-    rc_alloc_struct_0( pct,
-                       gs_overprint_t,
-                       &st_overprint,
-                       mem,
-                       return_error(gs_error_VMerror),
-                       "gs_create_overprint" );
+    pct = gs_alloc_struct(mem, gs_overprint_t, &st_overprint,
+			     "gs_create_overprint");
+    if (pct == NULL)
+	return_error(gs_error_VMerror);
     pct->type = &gs_composite_overprint_type;
     pct->id = gs_next_ids(mem, 1);
     pct->params = *pparams;
+    pct->idle = false;
     *ppct = (gs_composite_t *)pct;
     return 0;
 }
@@ -327,12 +345,13 @@ gs_private_st_suffix_add0_final( st_overprint_device_t,
  * be initialized once via gx_device_forward_fill_in_procs. They are
  * constant once this initialization is complete.
  */
-private dev_proc_open_device(overprint_open_device);
-private dev_proc_put_params(overprint_put_params);
-private dev_proc_get_page_device(overprint_get_page_device);
-private dev_proc_create_compositor(overprint_create_compositor);
+static dev_proc_open_device(overprint_open_device);
+static dev_proc_put_params(overprint_put_params);
+static dev_proc_get_page_device(overprint_get_page_device);
+static dev_proc_create_compositor(overprint_create_compositor);
+static dev_proc_get_color_comp_index(overprint_get_color_comp_index);
 
-private gx_device_procs no_overprint_procs = {
+static gx_device_procs no_overprint_procs = {
     overprint_open_device,              /* open_device */
     0,                                  /* get_initial_matrix */
     0,                                  /* sync_output */
@@ -383,7 +402,7 @@ private gx_device_procs no_overprint_procs = {
     0,                                  /* end_transparency_mask */
     0,                                  /* discard_transparency_layer */
     0,                                  /* get_color_mapping_procs */
-    0,                                  /* get_color_comp_index */
+    overprint_get_color_comp_index,	/* get_color_comp_index */
     0,                                  /* encode_color */
     0                                   /* decode_color */
 };
@@ -414,11 +433,11 @@ private gx_device_procs no_overprint_procs = {
  *     of alpha-compositing and raster operations are not compatible in
  *     a strict sense.
  */
-private dev_proc_fill_rectangle(overprint_generic_fill_rectangle);
-private dev_proc_fill_rectangle(overprint_sep_fill_rectangle);
+static dev_proc_fill_rectangle(overprint_generic_fill_rectangle);
+static dev_proc_fill_rectangle(overprint_sep_fill_rectangle);
 /* other low-level overprint_sep_* rendering methods prototypes go here */
 
-private gx_device_procs generic_overprint_procs = {
+static gx_device_procs generic_overprint_procs = {
     overprint_open_device,              /* open_device */
     0,                                  /* get_initial_matrix */
     0,                                  /* sync_output */
@@ -469,12 +488,12 @@ private gx_device_procs generic_overprint_procs = {
     0,                                  /* end_transparency_mask */
     0,                                  /* discard_transparency_layer */
     0,                                  /* get_color_mapping_procs */
-    0,                                  /* get_color_comp_index */
+    overprint_get_color_comp_index,	/* get_color_comp_index */
     0,                                  /* encode_color */
     0                                   /* decode_color */
 };
 
-private gx_device_procs sep_overprint_procs = {
+static gx_device_procs sep_overprint_procs = {
     overprint_open_device,              /* open_device */
     0,                                  /* get_initial_matrix */
     0,                                  /* sync_output */
@@ -525,7 +544,7 @@ private gx_device_procs sep_overprint_procs = {
     0,                                  /* end_transparency_mask */
     0,                                  /* discard_transparency_layer */
     0,                                  /* get_color_mapping_procs */
-    0,                                  /* get_color_comp_index */
+    overprint_get_color_comp_index,	/* get_color_comp_index */
     0,                                  /* encode_color */
     0                                   /* decode_color */
 };
@@ -560,7 +579,7 @@ const overprint_device_t    gs_overprint_device = {
  */
 #if !arch_is_big_endian
 
-private gx_color_index
+static gx_color_index
 swap_color_index(int depth, gx_color_index color)
 {
     int             shift = depth - 8;
@@ -603,7 +622,7 @@ swap_color_index(int depth, gx_color_index color)
  * drawn_comps field. This is useful only if the device color model
  * is separable.
  */
-private void
+static void
 set_retain_mask(overprint_device_t * opdev)
 {
     int             i, ncomps = opdev->color_info.num_components;
@@ -624,7 +643,7 @@ set_retain_mask(overprint_device_t * opdev)
 }
 
 /* enlarge mask of non-zero components */
-private gx_color_index
+static gx_color_index
 check_drawn_comps(int ncomps, frac cvals[GX_DEVICE_COLOR_MAX_COMPONENTS])
 {
     int              i;
@@ -644,7 +663,7 @@ check_drawn_comps(int ncomps, frac cvals[GX_DEVICE_COLOR_MAX_COMPONENTS])
  * determined by mapping through the standard color spaces and check which
  * components assume non-zero values.
  */
-private int
+static int
 update_overprint_params(
     overprint_device_t *            opdev,
     const gs_overprint_params_t *   pparams )
@@ -652,7 +671,7 @@ update_overprint_params(
     int                             ncomps = opdev->color_info.num_components;
 
     /* check if overprint is to be turned off */
-    if (!pparams->retain_any_comps || opdev->color_info.num_components == 1) {
+    if (!pparams->retain_any_comps || pparams->idle) {
         /* if fill_rectangle forwards, overprint is already off */
         if (dev_proc(opdev, fill_rectangle) != gx_forward_fill_rectangle)
             memcpy( &opdev->procs,
@@ -734,7 +753,7 @@ update_overprint_params(
  * We assume this procedure is called only if the device is not already
  * open, and that gs_opendevice will take care of the is_open flag.
  */
-private int
+static int
 overprint_open_device(gx_device * dev)
 {
     overprint_device_t *    opdev = (overprint_device_t *)dev;
@@ -753,7 +772,7 @@ overprint_open_device(gx_device * dev)
  * The put_params method for the overprint device will check if the
  * target device has closed and, if so, close itself.
  */
-private int
+static int
 overprint_put_params(gx_device * dev, gs_param_list * plist)
 {
     overprint_device_t *    opdev = (overprint_device_t *)dev;
@@ -769,12 +788,35 @@ overprint_put_params(gx_device * dev, gs_param_list * plist)
     return code;
 }
 
+/*
+ * If the target device 'auto detects' new spot colors, then it will
+ * change its color_info data.  Make sure that we have a current copy.
+ */
+int
+overprint_get_color_comp_index(gx_device * dev, const char * pname,
+					int name_size, int component_type)
+{
+    overprint_device_t * opdev = (overprint_device_t *)dev;
+    gx_device * tdev = opdev->target;
+    int code;
+
+    if (tdev == 0)
+	code = gx_error_get_color_comp_index(dev, pname,
+				name_size, component_type);
+    else {
+	code = dev_proc(tdev, get_color_comp_index)(tdev, pname,
+				name_size, component_type);
+        opdev->color_info = tdev->color_info;
+    }
+    return code;
+}
+
 /*
  * The overprint device must never be confused with a page device.
  * Thus, we always forward the request for the page device to the
  * target, as should all forwarding devices.
  */
-private gx_device *
+static gx_device *
 overprint_get_page_device(gx_device * dev)
 {
     overprint_device_t *    opdev = (overprint_device_t *)dev;
@@ -787,7 +829,7 @@ overprint_get_page_device(gx_device * dev)
  * Calling create_compositor on the overprint device just updates the
  * overprint parameters; no new device is created.
  */
-private int
+static int
 overprint_create_compositor(
     gx_device *             dev,
     gx_device **            pcdev,
@@ -798,12 +840,14 @@ overprint_create_compositor(
     if (pct->type != &gs_composite_overprint_type)
         return gx_default_create_compositor(dev, pcdev, pct, pis, memory);
     else {
+	gs_overprint_params_t params = ((const gs_overprint_t *)pct)->params;
         int     code;
 
+	params.idle = pct->idle;
         /* device must already exist, so just update the parameters */
         code = update_overprint_params(
                        (overprint_device_t *)dev,
-                       &((const gs_overprint_t *)pct)->params );
+                       &params );
         if (code >= 0)
             *pcdev = dev;
         return code;
@@ -818,7 +862,7 @@ overprint_create_compositor(
  * the appropriate components, then invoke the copy_color procedure on the
  * target device.
  */
-private int
+static int
 overprint_generic_fill_rectangle(
     gx_device *     dev,
     int             x,
@@ -840,7 +884,7 @@ overprint_generic_fill_rectangle(
                                                     dev->memory );
 }
 
-private int
+static int
 overprint_sep_fill_rectangle(
     gx_device *     dev,
     int             x,
@@ -899,7 +943,7 @@ overprint_sep_fill_rectangle(
 
 
 /* complete a porcedure set */
-private void
+static void
 fill_in_procs(gx_device_procs * pprocs)
 {
     gx_device_forward   tmpdev;
@@ -939,7 +983,7 @@ fill_in_procs(gx_device_procs * pprocs)
  * elaborate special handling in the caching device create_compositor
  * procedure.
  */
-private int
+static int
 c_overprint_create_default_compositor(
     const gs_composite_t *  pct,
     gx_device **            popdev,
@@ -949,13 +993,17 @@ c_overprint_create_default_compositor(
 {
     const gs_overprint_t *  ovrpct = (const gs_overprint_t *)pct;
     overprint_device_t *    opdev = 0;
+    gs_overprint_params_t params;
 
     /* see if there is anything to do */
-    if ( !ovrpct->params.retain_any_comps    ||
-         tdev->color_info.num_components <= 1  ) {
+    if ( !ovrpct->params.retain_any_comps) {
         *popdev = tdev;
         return 0;
     }
+    if (pct->idle) {
+        *popdev = tdev;
+	return 0;
+    }
 
     /* check if the procedure arrays have been initialized */
     if (no_overprint_procs.get_xfont_procs == 0) {
@@ -978,7 +1026,9 @@ c_overprint_create_default_compositor(
     gx_device_copy_params((gx_device *)opdev, tdev);
     gx_device_set_target((gx_device_forward *)opdev, tdev);
 
+    params = ovrpct->params;
+    params.idle = ovrpct->idle;
+
     /* set up the overprint parameters */
-    return update_overprint_params( opdev,
-                                    &((const gs_overprint_t *)pct)->params );
+    return update_overprint_params( opdev, &params);
 }
