@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsptype1.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gsptype1.c 8500 2008-01-23 18:09:49Z leonardo $ */
 /* PatternType 1 pattern implementation */
 #include "math_.h"
 #include "gx.h"
@@ -48,7 +48,7 @@ private_st_pattern1_template();
 private_st_pattern1_instance();
 
 /* GC procedures */
-private ENUM_PTRS_BEGIN(pattern1_instance_enum_ptrs) {
+static ENUM_PTRS_BEGIN(pattern1_instance_enum_ptrs) {
     if (index < st_pattern1_template_max_ptrs) {
 	gs_ptr_type_t ptype =
 	    ENUM_SUPER_ELT(gs_pattern1_instance_t, st_pattern1_template,
@@ -60,17 +60,17 @@ private ENUM_PTRS_BEGIN(pattern1_instance_enum_ptrs) {
     }
     ENUM_PREFIX(st_pattern_instance, st_pattern1_template_max_ptrs);
 } ENUM_PTRS_END
-private RELOC_PTRS_BEGIN(pattern1_instance_reloc_ptrs) {
+static RELOC_PTRS_BEGIN(pattern1_instance_reloc_ptrs) {
     RELOC_PREFIX(st_pattern_instance);
     RELOC_SUPER(gs_pattern1_instance_t, st_pattern1_template, template);
 } RELOC_PTRS_END
 
 /* Define a PatternType 1 pattern. */
-private pattern_proc_uses_base_space(gs_pattern1_uses_base_space);
-private pattern_proc_make_pattern(gs_pattern1_make_pattern);
-private pattern_proc_get_pattern(gs_pattern1_get_pattern);
-private pattern_proc_set_color(gs_pattern1_set_color);
-private const gs_pattern_type_t gs_pattern1_type = {
+static pattern_proc_uses_base_space(gs_pattern1_uses_base_space);
+static pattern_proc_make_pattern(gs_pattern1_make_pattern);
+static pattern_proc_get_pattern(gs_pattern1_get_pattern);
+static pattern_proc_set_color(gs_pattern1_set_color);
+static const gs_pattern_type_t gs_pattern1_type = {
     1, {
 	gs_pattern1_uses_base_space, gs_pattern1_make_pattern,
 	gs_pattern1_get_pattern, gs_pattern1_remap_color,
@@ -83,23 +83,21 @@ private const gs_pattern_type_t gs_pattern1_type = {
  */
 int
 gs_cspace_build_Pattern1(gs_color_space ** ppcspace,
-		    const gs_color_space * pbase_cspace, gs_memory_t * pmem)
+		    gs_color_space * pbase_cspace, gs_memory_t * pmem)
 {
     gs_color_space *pcspace = 0;
-    int code;
 
     if (pbase_cspace != 0) {
 	if (gs_color_space_num_components(pcspace) < 0)		/* Pattern space */
 	    return_error(gs_error_rangecheck);
     }
-    code = gs_cspace_alloc(&pcspace, &gs_color_space_type_Pattern, pmem);
-    if (code < 0)
-	return code;
+    pcspace = gs_cspace_alloc(pmem, &gs_color_space_type_Pattern);
+    if (pcspace == NULL)
+	return_error(gs_error_VMerror);
     if (pbase_cspace != 0) {
 	pcspace->params.pattern.has_base_space = true;
-	gs_cspace_init_from((gs_color_space *) & (pcspace->params.pattern.base_space),
-			    pbase_cspace
-	    );
+	/* reference to base space shifts from pgs to pcs with no net change */
+	pcspace->base_space = pbase_cspace;
     } else
 	pcspace->params.pattern.has_base_space = false;
     *ppcspace = pcspace;
@@ -114,8 +112,8 @@ gs_pattern1_init(gs_pattern1_template_t * ppat)
 }
 
 /* Make an instance of a PatternType 1 pattern. */
-private int compute_inst_matrix(gs_pattern1_instance_t * pinst,
-	const gs_state * saved, gs_rect * pbbox, int width, int height);
+static int compute_inst_matrix(gs_pattern1_instance_t * pinst,
+	gs_state * saved, gs_rect * pbbox, int width, int height);
 int
 gs_makepattern(gs_client_color * pcc, const gs_pattern1_template_t * pcp,
 	       const gs_matrix * pmat, gs_state * pgs, gs_memory_t * mem)
@@ -123,7 +121,7 @@ gs_makepattern(gs_client_color * pcc, const gs_pattern1_template_t * pcp,
     return gs_pattern1_make_pattern(pcc, (const gs_pattern_template_t *)pcp,
 				    pmat, pgs, mem);
 }
-private int
+static int
 gs_pattern1_make_pattern(gs_client_color * pcc,
 			 const gs_pattern_template_t * ptemp,
 			 const gs_matrix * pmat, gs_state * pgs,
@@ -167,7 +165,8 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
 
 #define mat inst.step_matrix
     if_debug6('t', "[t]step_matrix=[%g %g %g %g %g %g]\n",
-	      mat.xx, mat.xy, mat.yx, mat.yy, mat.tx, mat.ty);
+	      inst.step_matrix.xx, inst.step_matrix.xy, inst.step_matrix.yx, 
+	      inst.step_matrix.yy, inst.step_matrix.tx, inst.step_matrix.ty);
     if_debug4('t', "[t]bbox=(%g,%g),(%g,%g)\n",
 	      bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y);
     {
@@ -188,21 +187,21 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
 	    /*
 	     * The pattern is empty: the stepping matrix doesn't matter.
 	     */
-	    gs_make_identity(&mat);
+	    gs_make_identity(&inst.step_matrix);
 	    bbox.p.x = bbox.p.y = bbox.q.x = bbox.q.y = 0;
 	} else {
 	    /* Check for singular stepping matrix. */
-	    if (fabs(mat.xx * mat.yy - mat.xy * mat.yx) < 1.0e-6) {
+	    if (fabs(inst.step_matrix.xx * inst.step_matrix.yy - inst.step_matrix.xy * inst.step_matrix.yx) < 1.0e-6) {
 		code = gs_note_error(gs_error_rangecheck);
 		goto fsaved;
 	    }
 	    if (ADJUST_SCALE_BY_GS_TRADITION &&
-	        mat.xy == 0 && mat.yx == 0 &&
-		fabs(fabs(mat.xx) - bbw) < 0.5 &&
-		fabs(fabs(mat.yy) - bbh) < 0.5
+	        inst.step_matrix.xy == 0 && inst.step_matrix.yx == 0 &&
+		fabs(fabs(inst.step_matrix.xx) - bbw) < 0.5 &&
+		fabs(fabs(inst.step_matrix.yy) - bbh) < 0.5
 		) {
-		gs_scale(saved, fabs(inst.size.x / mat.xx),
-			 fabs(inst.size.y / mat.yy));
+		gs_scale(saved, fabs(inst.size.x / inst.step_matrix.xx),
+			 fabs(inst.size.y / inst.step_matrix.yy));
 		code = compute_inst_matrix(&inst, saved, &bbox,
 						dev_width, dev_height);
 		if (code < 0)
@@ -221,13 +220,13 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
 		if_debug4('t', "[t]bbox=(%g,%g),(%g,%g)\n",
 			  bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y);
 	    } else if (ADJUST_AS_ADOBE) {
-		if (mat.xy == 0 && mat.yx == 0 &&
-		    fabs(fabs(mat.xx) - bbw) < 0.5 &&
-		    fabs(fabs(mat.yy) - bbh) < 0.5
+		if (inst.step_matrix.xy == 0 && inst.step_matrix.yx == 0 &&
+		    fabs(fabs(inst.step_matrix.xx) - bbw) < 0.5 &&
+		    fabs(fabs(inst.step_matrix.yy) - bbh) < 0.5
 		    ) {
 		    if (inst.step_matrix.xx <= 2) { 
 			/* Prevent a degradation - see -r72 mspro.pdf */
-			gs_scale(saved, fabs(inst.size.x / mat.xx), 1);
+			gs_scale(saved, fabs(inst.size.x / inst.step_matrix.xx), 1);
 			inst.step_matrix.xx = (float)inst.size.x;
 		    } else {
 			inst.step_matrix.xx = (float)floor(inst.step_matrix.xx + 0.5);
@@ -239,7 +238,7 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
 			    gs_scale(saved, (fabs(inst.size.x) - 1.0 / fixed_scale) / fabs(inst.size.x), 1);
 		    }
 		    if (inst.step_matrix.yy <= 2) {
-			gs_scale(saved, 1, fabs(inst.size.y / mat.yy));
+			gs_scale(saved, 1, fabs(inst.size.y / inst.step_matrix.yy));
 			inst.step_matrix.yy = (float)inst.size.y;
 		    } else {
 			inst.step_matrix.yy = (float)floor(inst.step_matrix.yy + 0.5);
@@ -253,12 +252,12 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
 	    }
 	}
     }
-    if ((code = gs_bbox_transform_inverse(&bbox, &mat, &inst.bbox)) < 0)
+    if ((code = gs_bbox_transform_inverse(&bbox, &inst.step_matrix, &inst.bbox)) < 0)
 	goto fsaved;
     if_debug4('t', "[t]ibbox=(%g,%g),(%g,%g)\n",
 	      inst.bbox.p.x, inst.bbox.p.y, inst.bbox.q.x, inst.bbox.q.y);
-    inst.is_simple = (fabs(mat.xx) == inst.size.x && mat.xy == 0 &&
-		      mat.yx == 0 && fabs(mat.yy) == inst.size.y);
+    inst.is_simple = (fabs(inst.step_matrix.xx) == inst.size.x && inst.step_matrix.xy == 0 &&
+		      inst.step_matrix.yx == 0 && fabs(inst.step_matrix.yy) == inst.size.y);
     if_debug6('t',
 	      "[t]is_simple? xstep=(%g,%g) ystep=(%g,%g) size=(%d,%d)\n",
 	      inst.step_matrix.xx, inst.step_matrix.xy,
@@ -266,22 +265,39 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
 	      inst.size.x, inst.size.y);
     /* Absent other information, instances always require a mask. */
     inst.uses_mask = true;
-    gx_translate_to_fixed(saved, float2fixed_rounded(mat.tx - bbox.p.x),
-			         float2fixed_rounded(mat.ty - bbox.p.y));
-    mat.tx = bbox.p.x;
-    mat.ty = bbox.p.y;
+    gx_translate_to_fixed(saved, float2fixed_rounded(inst.step_matrix.tx - bbox.p.x),
+			         float2fixed_rounded(inst.step_matrix.ty - bbox.p.y));
+    inst.step_matrix.tx = bbox.p.x;
+    inst.step_matrix.ty = bbox.p.y;
 #undef mat
     cbox.p.x = fixed_0;
     cbox.p.y = fixed_0;
     cbox.q.x = int2fixed(inst.size.x);
     cbox.q.y = int2fixed(inst.size.y);
     code = gx_clip_to_rectangle(saved, &cbox);
+    if (code < 0)
+	goto fsaved;
+    if (!inst.is_simple) {
+	code = gs_newpath(saved);
+	if (code >= 0)
+	    code = gs_moveto(saved, inst.template.BBox.p.x, inst.template.BBox.p.y);
+	if (code >= 0)
+	    code = gs_lineto(saved, inst.template.BBox.q.x, inst.template.BBox.p.y);
+	if (code >= 0)
+	    code = gs_lineto(saved, inst.template.BBox.q.x, inst.template.BBox.q.y);
+	if (code >= 0)
+	    code = gs_lineto(saved, inst.template.BBox.p.x, inst.template.BBox.q.y);
+	if (code >= 0)
+	    code = gs_clip(saved);
+	if (code < 0)
+	    goto fsaved;
+    }
+    code = gs_newpath(saved);
     if (code < 0)
 	goto fsaved;
     inst.id = gs_next_ids(mem, 1);
     *pinst = inst;
     return 0;
-#undef mat
   fsaved:gs_state_free(saved);
     gs_free_object(mem, pinst, "gs_makepattern");
     return code;
@@ -294,7 +310,7 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
  * the page.  We allocate a buffer for holding the pattern.  We need to
  * prevent this buffer from getting too large.
  */
-private int
+static int
 clamp_pattern_bbox(gs_pattern1_instance_t * pinst, gs_rect * pbbox,
 		    int width, int height, const gs_matrix * pmat)
 {
@@ -393,20 +409,38 @@ clamp_pattern_bbox(gs_pattern1_instance_t * pinst, gs_rect * pbbox,
 
 /* Compute the stepping matrix and device space instance bounding box */
 /* from the step values and the saved matrix. */
-private int
-compute_inst_matrix(gs_pattern1_instance_t * pinst, const gs_state * saved,
+static int
+compute_inst_matrix(gs_pattern1_instance_t * pinst, gs_state * saved,
 			    gs_rect * pbbox, int width, int height)
 {
-    double xx = pinst->template.XStep * saved->ctm.xx;
-    double xy = pinst->template.XStep * saved->ctm.xy;
-    double yx = pinst->template.YStep * saved->ctm.yx;
-    double yy = pinst->template.YStep * saved->ctm.yy;
+    float xx, xy, yx, yy, dx, dy, temp;
     int code;
 
-    /* Adjust the stepping matrix so all coefficients are >= 0. */
-    if (xx == 0 || yy == 0) {	/* We know that both xy and yx are non-zero. */
-	double temp;
+    code = gs_bbox_transform(&pinst->template.BBox, &ctm_only(saved), pbbox);
+    if (code < 0)
+	return code;
+    /*
+     * Adjust saved.ctm to map the bbox origin to pixels.
+     */
+    dx = pbbox->p.x - floor(pbbox->p.x + 0.5);
+    dy = pbbox->p.y - floor(pbbox->p.y + 0.5);
+    pbbox->p.x -= dx;
+    pbbox->p.y -= dy;
+    pbbox->q.x -= dx;
+    pbbox->q.y -= dy;
+    code = gx_translate_to_fixed(saved, float2fixed_rounded(saved->ctm.tx - dx), 
+					float2fixed_rounded(saved->ctm.ty - dy));
+    if (code < 0)
+	return code;
 
+    /* The stepping matrix : */
+    xx = pinst->template.XStep * saved->ctm.xx;
+    xy = pinst->template.XStep * saved->ctm.xy;
+    yx = pinst->template.YStep * saved->ctm.yx;
+    yy = pinst->template.YStep * saved->ctm.yy;
+
+    /* Adjust the stepping matrix so all coefficients are >= 0. */
+    if (xx == 0 || yy == 0) { /* We know that both xy and yx are non-zero. */
 	temp = xx, xx = yx, yx = temp;
 	temp = xy, xy = yy, yy = temp;
     }
@@ -421,14 +455,12 @@ compute_inst_matrix(gs_pattern1_instance_t * pinst, const gs_state * saved,
     pinst->step_matrix.yy = yy;
     pinst->step_matrix.tx = saved->ctm.tx;
     pinst->step_matrix.ty = saved->ctm.ty;
-    code = gs_bbox_transform(&pinst->template.BBox, &ctm_only(saved), pbbox);
     /*
      * Some applications produce patterns that are larger than the page.
      * If the bounding box for the pattern is larger than the page. clamp
      * the pattern to the page size.
      */
-    if (code >= 0 &&
-	(pbbox->q.x - pbbox->p.x > width || pbbox->q.y - pbbox->p.y > height))
+    if ((pbbox->q.x - pbbox->p.x > width || pbbox->q.y - pbbox->p.y > height))
 	code = clamp_pattern_bbox(pinst, pbbox, width,
 					height, &ctm_only(saved));
 
@@ -436,7 +468,7 @@ compute_inst_matrix(gs_pattern1_instance_t * pinst, const gs_state * saved,
 }
 
 /* Test whether a PatternType 1 pattern uses a base space. */
-private bool
+static bool
 gs_pattern1_uses_base_space(const gs_pattern_template_t *ptemp)
 {
     return ((const gs_pattern1_template_t *)ptemp)->PaintType == 2;
@@ -444,13 +476,20 @@ gs_pattern1_uses_base_space(const gs_pattern_template_t *ptemp)
 
 /* getpattern for PatternType 1 */
 /* This is only intended for the benefit of pattern PaintProcs. */
-private const gs_pattern_template_t *
+static const gs_pattern_template_t *
 gs_pattern1_get_pattern(const gs_pattern_instance_t *pinst)
 {
     return (const gs_pattern_template_t *)
 	&((const gs_pattern1_instance_t *)pinst)->template;
 }
 
+/* Check device color for Pattern Type 1. */
+bool
+gx_dc_is_pattern1_color(const gx_device_color *pdevc)
+{
+    return pdevc->type == &gx_dc_pattern;
+}
+
 /*
  * Perform actions required at setcolor time. This procedure resets the
  * overprint information (almost) as required by the pattern. The logic
@@ -490,7 +529,7 @@ gs_pattern1_get_pattern(const gs_pattern_instance_t *pinst)
  *     change in color space required to set a normal color will
  *     reset the overprint information as required.
  */
-private int
+static int
 gs_pattern1_set_color(const gs_client_color * pcc, gs_state * pgs)
 {
     gs_pattern1_instance_t * pinst = (gs_pattern1_instance_t *)pcc->pattern;
@@ -499,7 +538,7 @@ gs_pattern1_set_color(const gs_client_color * pcc, gs_state * pgs)
     if (ptmplt->PaintType == 2) {
         const gs_color_space *  pcs = pgs->color_space;
 
-        pcs = (const gs_color_space *)&(pcs->params.pattern.base_space);
+        pcs = pcs->base_space;
         return pcs->type->set_overprint(pcs, pgs);
     } else {
         gs_overprint_params_t   params;
@@ -546,7 +585,7 @@ public_st_gx_strip_bitmap();
  */
 typedef struct pixmap_info_s {
     gs_depth_bitmap bitmap;	/* must be first */
-    const gs_color_space *pcspace;
+    gs_color_space *pcspace;
     uint white_index;
     void (*free_proc)(gs_memory_t *, void *, client_name_t);
 } pixmap_info;
@@ -571,7 +610,7 @@ gs_private_st_suffix_add1(st_pixmap_info,
  *  Note that this routine does NOT release the data in the original pixmap;
  *  that remains the responsibility of the client.
  */
-private void
+static void
 free_pixmap_pattern(
     gs_memory_t *           pmem,
     void *                  pvpinst,
@@ -588,9 +627,9 @@ free_pixmap_pattern(
 /*
  *  PaintProcs for bitmap and pixmap patterns.
  */
-private int bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
+static int bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
 			 const gs_depth_bitmap * pbitmap, gs_state * pgs);
-private int
+static int
 mask_PaintProc(const gs_client_color * pcolor, gs_state * pgs)
 {
     const pixmap_info *ppmap = gs_getpattern(pcolor)->client_data;
@@ -607,15 +646,14 @@ mask_PaintProc(const gs_client_color * pcolor, gs_state * pgs)
     gs_image_init(pen, &mask, false, pgs);
     return bitmap_paint(pen, (gs_data_image_t *) & mask, pbitmap, pgs);
 }
-private int
+static int
 image_PaintProc(const gs_client_color * pcolor, gs_state * pgs)
 {
     const pixmap_info *ppmap = gs_getpattern(pcolor)->client_data;
     const gs_depth_bitmap *pbitmap = &(ppmap->bitmap);
     gs_image_enum *pen =
         gs_image_enum_alloc(gs_state_memory(pgs), "image_PaintProc");
-    gs_color_space cs;
-    const gs_color_space *pcspace;
+    gs_color_space *pcspace;
     gx_image_enum_common_t *pie;
     /*
      * If the image is transparent then we want to do image type4 processing.
@@ -640,8 +678,7 @@ image_PaintProc(const gs_client_color * pcolor, gs_state * pgs)
 	return_error(gs_error_VMerror);
 
     if (ppmap->pcspace == 0) {
-        gs_cspace_init_DeviceGray(pgs->memory, &cs);
-        pcspace = &cs;
+	pcspace = gs_cspace_new_DeviceGray(pgs->memory);
     } else
         pcspace = ppmap->pcspace;
     gs_gsave(pgs);
@@ -678,7 +715,7 @@ image_PaintProc(const gs_client_color * pcolor, gs_state * pgs)
     return code;
 }
 /* Finish painting any kind of bitmap pattern. */
-private int
+static int
 bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
 	     const gs_depth_bitmap * pbitmap, gs_state * pgs)
 {
@@ -694,7 +731,7 @@ bitmap_paint(gs_image_enum * pen, gs_data_image_t * pim,
     else
 	for (n = pim->Height; n > 0 && code >= 0; dp += raster, --n)
 	    code = gs_image_next(pen, dp, nbytes, &used);
-    code1 = gs_image_cleanup_and_free_enum(pen);
+    code1 = gs_image_cleanup_and_free_enum(pen, pgs);
     if (code >= 0 && code1 < 0)
 	code = code1;
     return code;
@@ -715,7 +752,7 @@ gs_makepixmappattern(
 			bool mask,
 			const gs_matrix * pmat,
 			long id,
-			const gs_color_space * pcspace,
+			gs_color_space * pcspace,
 			uint white_index,
 			gs_state * pgs,
 			gs_memory_t * mem
@@ -840,25 +877,25 @@ gs_makebitmappattern_xform(
  * 'masked_fill_rect' instead of 'masked_fill_rectangle' in order to limit
  * identifier lengths to 32 characters.
  */
-private dev_color_proc_get_dev_halftone(gx_dc_pattern_get_dev_halftone);
-private dev_color_proc_load(gx_dc_pattern_load);
+static dev_color_proc_get_dev_halftone(gx_dc_pattern_get_dev_halftone);
+static dev_color_proc_load(gx_dc_pattern_load);
 /*dev_color_proc_fill_rectangle(gx_dc_pattern_fill_rectangle); *//*gxp1fill.h */
-private dev_color_proc_equal(gx_dc_pattern_equal);
-private dev_color_proc_load(gx_dc_pure_masked_load);
+static dev_color_proc_equal(gx_dc_pattern_equal);
+static dev_color_proc_load(gx_dc_pure_masked_load);
 
-private dev_color_proc_get_dev_halftone(gx_dc_pure_masked_get_dev_halftone);
+static dev_color_proc_get_dev_halftone(gx_dc_pure_masked_get_dev_halftone);
 /*dev_color_proc_fill_rectangle(gx_dc_pure_masked_fill_rect); *//*gxp1fill.h */
-private dev_color_proc_equal(gx_dc_pure_masked_equal);
-private dev_color_proc_load(gx_dc_binary_masked_load);
+static dev_color_proc_equal(gx_dc_pure_masked_equal);
+static dev_color_proc_load(gx_dc_binary_masked_load);
 
-private dev_color_proc_get_dev_halftone(gx_dc_binary_masked_get_dev_halftone);
+static dev_color_proc_get_dev_halftone(gx_dc_binary_masked_get_dev_halftone);
 /*dev_color_proc_fill_rectangle(gx_dc_binary_masked_fill_rect); *//*gxp1fill.h */
-private dev_color_proc_equal(gx_dc_binary_masked_equal);
-private dev_color_proc_load(gx_dc_colored_masked_load);
+static dev_color_proc_equal(gx_dc_binary_masked_equal);
+static dev_color_proc_load(gx_dc_colored_masked_load);
 
-private dev_color_proc_get_dev_halftone(gx_dc_colored_masked_get_dev_halftone);
+static dev_color_proc_get_dev_halftone(gx_dc_colored_masked_get_dev_halftone);
 /*dev_color_proc_fill_rectangle(gx_dc_colored_masked_fill_rect); *//*gxp1fill.h */
-private dev_color_proc_equal(gx_dc_colored_masked_equal);
+static dev_color_proc_equal(gx_dc_colored_masked_equal);
 
 /* The device color types are exported for gxpcmap.c. */
 gs_private_st_composite(st_dc_pattern, gx_device_color, "dc_pattern",
@@ -917,7 +954,7 @@ const gx_device_color_type_t *const gx_dc_type_pattern = &gx_dc_pattern;
 #define gx_dc_type_pattern (&gx_dc_pattern)
 
 /* GC procedures */
-private 
+static 
 ENUM_PTRS_WITH(dc_pattern_enum_ptrs, gx_device_color *cptr)
 {
     return ENUM_USING(st_dc_pure_masked, vptr, size, index - 1);
@@ -929,7 +966,7 @@ case 0:
     ENUM_RETURN((tile == 0 ? tile : tile - tile->index));
 }
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(dc_pattern_reloc_ptrs, gx_device_color *cptr)
+static RELOC_PTRS_WITH(dc_pattern_reloc_ptrs, gx_device_color *cptr)
 {
     gx_color_tile *tile = cptr->colors.pattern.p_tile;
 
@@ -941,7 +978,7 @@ private RELOC_PTRS_WITH(dc_pattern_reloc_ptrs, gx_device_color *cptr)
     RELOC_USING(st_dc_pure_masked, vptr, size);
 }
 RELOC_PTRS_END
-private ENUM_PTRS_WITH(dc_masked_enum_ptrs, gx_device_color *cptr)
+static ENUM_PTRS_WITH(dc_masked_enum_ptrs, gx_device_color *cptr)
 ENUM_SUPER(gx_device_color, st_client_color, ccolor, 1);
 case 0:
 {
@@ -950,7 +987,7 @@ case 0:
     ENUM_RETURN((mask == 0 ? mask : mask - mask->index));
 }
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(dc_masked_reloc_ptrs, gx_device_color *cptr)
+static RELOC_PTRS_WITH(dc_masked_reloc_ptrs, gx_device_color *cptr)
 {
     gx_color_tile *mask = cptr->mask.m_tile;
 
@@ -962,7 +999,7 @@ private RELOC_PTRS_WITH(dc_masked_reloc_ptrs, gx_device_color *cptr)
     }
 }
 RELOC_PTRS_END
-private ENUM_PTRS_BEGIN(dc_binary_masked_enum_ptrs)
+static ENUM_PTRS_BEGIN(dc_binary_masked_enum_ptrs)
 {
     return ENUM_USING(st_dc_ht_binary, vptr, size, index - 2);
 }
@@ -970,7 +1007,7 @@ case 0:
 case 1:
 return ENUM_USING(st_dc_pure_masked, vptr, size, index);
 ENUM_PTRS_END
-private RELOC_PTRS_BEGIN(dc_binary_masked_reloc_ptrs)
+static RELOC_PTRS_BEGIN(dc_binary_masked_reloc_ptrs)
 {
     RELOC_USING(st_dc_pure_masked, vptr, size);
     RELOC_USING(st_dc_ht_binary, vptr, size);
@@ -1009,7 +1046,7 @@ gx_dc_pattern_save_dc(
  * halftones may be used by the PaintProc procedure. Hence, we can only
  * hope this is a contone device.
  */
-private const gx_device_halftone *
+static const gx_device_halftone *
 gx_dc_pattern_get_dev_halftone(const gx_device_color * pdevc)
 {
     return 0;
@@ -1020,19 +1057,19 @@ gx_dc_pattern_get_dev_halftone(const gx_device_color * pdevc)
  * base color. Ideally this would be returned via an inhereted method,
  * but the device color structure does not support such an arrangement.
  */
-private const gx_device_halftone *
+static const gx_device_halftone *
 gx_dc_pure_masked_get_dev_halftone(const gx_device_color * pdevc)
 {
     return 0;
 }
 
-private const gx_device_halftone *
+static const gx_device_halftone *
 gx_dc_binary_masked_get_dev_halftone(const gx_device_color * pdevc)
 {
     return pdevc->colors.binary.b_ht;
 }
 
-private const gx_device_halftone *
+static const gx_device_halftone *
 gx_dc_colored_masked_get_dev_halftone(const gx_device_color * pdevc)
 {
     return pdevc->colors.colored.c_ht;
@@ -1048,7 +1085,7 @@ gx_dc_colored_masked_get_dev_halftone(const gx_device_color * pdevc)
 	return code;
 
 /* Ensure that a colored Pattern is loaded in the cache. */
-private int
+static int
 gx_dc_pattern_load(gx_device_color * pdevc, const gs_imager_state * pis,
 		   gx_device * dev, gs_color_select_t select)
 {
@@ -1057,7 +1094,7 @@ gx_dc_pattern_load(gx_device_color * pdevc, const gs_imager_state * pis,
     FINISH_PATTERN_LOAD
 }
 /* Ensure that an uncolored Pattern is loaded in the cache. */
-private int
+static int
 gx_dc_pure_masked_load(gx_device_color * pdevc, const gs_imager_state * pis,
 		       gx_device * dev, gs_color_select_t select)
 {
@@ -1067,7 +1104,7 @@ gx_dc_pure_masked_load(gx_device_color * pdevc, const gs_imager_state * pis,
 	return code;
     FINISH_PATTERN_LOAD
 }
-private int
+static int
 gx_dc_binary_masked_load(gx_device_color * pdevc, const gs_imager_state * pis,
 			 gx_device * dev, gs_color_select_t select)
 {
@@ -1077,7 +1114,7 @@ gx_dc_binary_masked_load(gx_device_color * pdevc, const gs_imager_state * pis,
 	return code;
     FINISH_PATTERN_LOAD
 }
-private int
+static int
 gx_dc_colored_masked_load(gx_device_color * pdevc, const gs_imager_state * pis,
 			  gx_device * dev, gs_color_select_t select)
 {
@@ -1137,7 +1174,7 @@ gx_pattern_cache_lookup(gx_device_color * pdevc, const gs_imager_state * pis,
 #undef FINISH_PATTERN_LOAD
 
 /* Compare two Pattern colors for equality. */
-private bool
+static bool
 gx_dc_pattern_equal(const gx_device_color * pdevc1,
 		    const gx_device_color * pdevc2)
 {
@@ -1163,21 +1200,21 @@ gx_dc_pattern_get_nonzero_comps(
 }
 
 
-private bool
+static bool
 gx_dc_pure_masked_equal(const gx_device_color * pdevc1,
 			const gx_device_color * pdevc2)
 {
     return (*gx_dc_type_pure->equal) (pdevc1, pdevc2) &&
 	pdevc1->mask.id == pdevc2->mask.id;
 }
-private bool
+static bool
 gx_dc_binary_masked_equal(const gx_device_color * pdevc1,
 			  const gx_device_color * pdevc2)
 {
     return (*gx_dc_type_ht_binary->equal) (pdevc1, pdevc2) &&
 	pdevc1->mask.id == pdevc2->mask.id;
 }
-private bool
+static bool
 gx_dc_colored_masked_equal(const gx_device_color * pdevc1,
 			   const gx_device_color * pdevc2)
 {
