@@ -1,20 +1,17 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gstrans.c,v 1.19 2005/03/14 18:08:36 dan Exp $ */
+/* $Id: gstrans.c 8553 2008-02-27 19:22:57Z leonardo $ */
 /* Implementation of transparency, other than rendering */
 #include "math_.h"
 #include "memory_.h"
@@ -24,6 +21,8 @@
 #include "gsutil.h"
 #include "gzstate.h"
 #include "gxdevcli.h"
+#include "gdevdevn.h"
+#include "gxblend.h"
 #include "gdevp14.h"
 
 #define PUSH_TS 0
@@ -113,7 +112,7 @@ gs_private_st_ptrs1(st_transparency_state, gs_transparency_state_t,
 		    "gs_transparency_state_t", transparency_state_enum_ptrs,
 		    transparency_state_reloc_ptrs, saved);
 #if PUSH_TS
-private int
+static int
 push_transparency_stack(gs_state *pgs, gs_transparency_state_type_t type,
 			client_name_t cname)
 {
@@ -129,7 +128,7 @@ push_transparency_stack(gs_state *pgs, gs_transparency_state_type_t type,
     return 0;
 }
 #endif
-private void
+static void
 pop_transparency_stack(gs_state *pgs, client_name_t cname)
 {
     gs_transparency_state_t *pts = pgs->transparency_stack; /* known non-0 */
@@ -146,7 +145,7 @@ pop_transparency_stack(gs_state *pgs, client_name_t cname)
  * create_compositor will update its parameters but not create a new
  * compositor device.
  */
-private int
+static int
 gs_state_update_pdf14trans(gs_state * pgs, gs_pdf14trans_params_t * pparams)
 {
     gs_imager_state * pis = (gs_imager_state *)pgs;
@@ -174,6 +173,7 @@ gs_trans_group_params_init(gs_transparency_group_params_t *ptgp)
     ptgp->ColorSpace = 0;	/* bogus, but can't do better */
     ptgp->Isolated = false;
     ptgp->Knockout = false;
+    ptgp->image_with_SMask = false;
 }
 
 int
@@ -208,6 +208,7 @@ gs_begin_transparency_group(gs_state *pgs,
     params.pdf14_op = PDF14_BEGIN_TRANS_GROUP;
     params.Isolated = ptgp->Isolated;
     params.Knockout = ptgp->Knockout;
+    params.image_with_SMask = ptgp->image_with_SMask;
     params.opacity = pgs->opacity;
     params.shape = pgs->shape;
     params.blend_mode = pgs->blend_mode;
@@ -227,8 +228,12 @@ gx_begin_transparency_group(gs_imager_state * pis, gx_device * pdev,
     gs_transparency_group_params_t tgp = {0};
     gs_rect bbox;
 
+    if (pparams->Background_components != 0 && 
+	pparams->Background_components != pdev->color_info.num_components)
+	return_error(gs_error_rangecheck);
     tgp.Isolated = pparams->Isolated;
     tgp.Knockout = pparams->Knockout;
+    tgp.idle = pparams->idle;
     pis->opacity.alpha = pparams->opacity.alpha;
     pis->shape.alpha = pparams->shape.alpha;
     pis->blend_mode = pparams->blend_mode;
@@ -239,7 +244,7 @@ gx_begin_transparency_group(gs_imager_state * pis, gx_device * pdev,
 	    GS_COLOR_SPACE_TYPE_NAMES
 	};
 
-	dlprintf5("[v](0x%lx)begin_transparency_group [%g %g %g %g]\n",
+	dlprintf5("[v](0x%lx)gx_begin_transparency_group [%g %g %g %g]\n",
 		  (ulong)pis, bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y);
 	if (tgp.ColorSpace)
 	    dprintf1("     CS = %s",
@@ -262,6 +267,7 @@ gs_end_transparency_group(gs_state *pgs)
 {
     gs_pdf14trans_params_t params = { 0 };
 
+    if_debug0('v', "[v]gs_end_transparency_group\n");
     params.pdf14_op = PDF14_END_TRANS_GROUP;  /* Other parameters not used */
     return gs_state_update_pdf14trans(pgs, &params);
 }
@@ -269,6 +275,7 @@ gs_end_transparency_group(gs_state *pgs)
 int
 gx_end_transparency_group(gs_imager_state * pis, gx_device * pdev)
 {
+    if_debug0('v', "[v]gx_end_transparency_group\n");
     if (dev_proc(pdev, end_transparency_group) != 0)
 	return (*dev_proc(pdev, end_transparency_group)) (pdev, pis, NULL);
     else
@@ -278,7 +285,7 @@ gx_end_transparency_group(gs_imager_state * pis, gx_device * pdev)
 /*
  * Handler for identity mask transfer functions.
  */
-private int
+static int
 mask_transfer_identity(floatp in, float *out, void *proc_data)
 {
     *out = (float) in;
@@ -290,33 +297,38 @@ gs_trans_mask_params_init(gs_transparency_mask_params_t *ptmp,
 			  gs_transparency_mask_subtype_t subtype)
 {
     ptmp->subtype = subtype;
-    ptmp->has_Background = false;
+    ptmp->Background_components = 0;
     ptmp->TransferFunction = mask_transfer_identity;
     ptmp->TransferFunction_data = 0;
+    ptmp->replacing = false;
 }
 
 int
 gs_begin_transparency_mask(gs_state * pgs,
 			   const gs_transparency_mask_params_t * ptmp,
-			   const gs_rect * pbbox)
+			   const gs_rect * pbbox, bool mask_is_image)
 {
     gs_pdf14trans_params_t params = { 0 };
+    const int l = sizeof(params.Background[0]) * ptmp->Background_components;
     int i;
 
-    if_debug8('v', "[v](0x%lx)begin_transparency_mask [%g %g %g %g]\n\
-      subtype = %d  has_Background = %d  %s\n",
+    if_debug8('v', "[v](0x%lx)gs_begin_transparency_mask [%g %g %g %g]\n\
+      subtype = %d  Background_components = %d  %s\n",
 	      (ulong)pgs, pbbox->p.x, pbbox->p.y, pbbox->q.x, pbbox->q.y,
-	      (int)ptmp->subtype, ptmp->has_Background,
+	      (int)ptmp->subtype, ptmp->Background_components,
 	      (ptmp->TransferFunction == mask_transfer_identity ? "no TR" :
 	       "has TR"));
     params.pdf14_op = PDF14_BEGIN_TRANS_MASK;
     params.bbox = *pbbox;
     params.subtype = ptmp->subtype;
-    params.has_Background = ptmp->has_Background;
-    if (ptmp->has_Background)
-	memcpy(params.Background, ptmp->Background, size_of(ptmp->Background));
+    params.Background_components = ptmp->Background_components;
+    memcpy(params.Background, ptmp->Background, l);
+    params.GrayBackground = ptmp->GrayBackground;
+    params.transfer_function = ptmp->TransferFunction_data;
     params.function_is_identity =
 	    (ptmp->TransferFunction == mask_transfer_identity);
+    params.mask_is_image = mask_is_image;
+    params.replacing = ptmp->replacing;
     /* Sample the transfer function */
     for (i = 0; i < MASK_TRANSFER_FUNCTION_SIZE; i++) {
 	float in = (float)(i * (1.0 / (MASK_TRANSFER_FUNCTION_SIZE - 1)));
@@ -333,17 +345,21 @@ gx_begin_transparency_mask(gs_imager_state * pis, gx_device * pdev,
 				const gs_pdf14trans_params_t * pparams)
 {
     gx_transparency_mask_params_t tmp;
+    const int l = sizeof(pparams->Background[0]) * pparams->Background_components;
 
     tmp.subtype = pparams->subtype;
-    tmp.has_Background = pparams->has_Background;
-    memcpy(tmp.Background, pparams->Background, size_of(tmp.Background));
+    tmp.Background_components = pparams->Background_components;
+    memcpy(tmp.Background, pparams->Background, l);
+    tmp.GrayBackground = pparams->GrayBackground;
     tmp.function_is_identity = pparams->function_is_identity;
+    tmp.idle = pparams->idle;
+    tmp.replacing = pparams->replacing;
     memcpy(tmp.transfer_fn, pparams->transfer_fn, size_of(tmp.transfer_fn));
-    if_debug8('v', "[v](0x%lx)begin_transparency_mask [%g %g %g %g]\n\
-      subtype = %d  has_Background = %d  %s\n",
+    if_debug8('v', "[v](0x%lx)gx_begin_transparency_mask [%g %g %g %g]\n\
+      subtype = %d  Background_components = %d  %s\n",
 	      (ulong)pis, pparams->bbox.p.x, pparams->bbox.p.y,
 	      pparams->bbox.q.x, pparams->bbox.q.y,
-	      (int)tmp.subtype, tmp.has_Background,
+	      (int)tmp.subtype, tmp.Background_components,
 	      (tmp.function_is_identity ? "no TR" :
 	       "has TR"));
     if (dev_proc(pdev, begin_transparency_mask) != 0)
@@ -359,12 +375,11 @@ gs_end_transparency_mask(gs_state *pgs,
 {
     gs_pdf14trans_params_t params = { 0 };
 
-    if_debug2('v', "[v](0x%lx)end_transparency_mask(%d)\n", (ulong)pgs,
+    if_debug2('v', "[v](0x%lx)gs_end_transparency_mask(%d)\n", (ulong)pgs,
 	      (int)csel);
 
-    /* todo: route csel */
-
     params.pdf14_op = PDF14_END_TRANS_MASK;  /* Other parameters not used */
+    params.csel = csel;
     return gs_state_update_pdf14trans(pgs, &params);
 }
 
@@ -372,6 +387,8 @@ int
 gx_end_transparency_mask(gs_imager_state * pis, gx_device * pdev,
 				const gs_pdf14trans_params_t * pparams)
 {
+    if_debug2('v', "[v](0x%lx)gx_end_transparency_mask(%d)\n", (ulong)pis,
+	      (int)pparams->csel);
     if (dev_proc(pdev, end_transparency_mask) != 0)
 	return (*dev_proc(pdev, end_transparency_mask)) (pdev, NULL);
     else
@@ -384,42 +401,35 @@ gs_discard_transparency_layer(gs_state *pgs)
     /****** NYI, DUMMY ******/
     gs_transparency_state_t *pts = pgs->transparency_stack;
 
-    if_debug1('v', "[v](0x%lx)discard_transparency_layer\n", (ulong)pgs);
+    if_debug1('v', "[v](0x%lx)gs_discard_transparency_layer\n", (ulong)pgs);
     if (!pts)
 	return_error(gs_error_rangecheck);
     pop_transparency_stack(pgs, "gs_discard_transparency_layer");
     return 0;
 }
 
-int
-gs_init_transparency_mask(gs_state *pgs,
-			  gs_transparency_channel_selector_t csel)
+/*
+ * We really only care about the number of spot colors when we have
+ * a device which supports spot colors.  With the other devices we use
+ * the tint transform function for DeviceN and Separation color spaces
+ * and convert spot colors into process colors.
+ */
+static int
+get_num_pdf14_spot_colors(gs_state * pgs)
 {
-    gs_pdf14trans_params_t params = { 0 };
+    gx_device * dev = pgs->device;
+    gs_devn_params * pclist_devn_params = dev_proc(dev, ret_devn_params)(dev);
 
-    if_debug2('v', "[v](0x%lx)init_transparency_mask(%d)\n", (ulong)pgs,
-	      (int)csel);
-
-    params.pdf14_op = PDF14_INIT_TRANS_MASK;
-    params.csel = csel;
-    return gs_state_update_pdf14trans(pgs, &params);
-}
-
-int
-gx_init_transparency_mask(gs_imager_state * pis,
-				const gs_pdf14trans_params_t * pparams)
-{
-    gs_transparency_source_t *ptm;
-
-    if_debug2('v', "[v](0x%lx)init_transparency_mask(%d)\n", (ulong)pis,
-	      (int)pparams->csel);
-    switch (pparams->csel) {
-    case TRANSPARENCY_CHANNEL_Opacity: ptm = &pis->opacity; break;
-    case TRANSPARENCY_CHANNEL_Shape: ptm = &pis->shape; break;
-    default: return_error(gs_error_rangecheck);
+    /*
+     * Devices which support spot colors store the PageSpotColors device
+     * parameter inside their devn_params structure.  (This is done by the
+     * devn_put_params routine.)  The PageSpotColors device parameter is
+     * set by pdf_main whenever a PDF page is being processed.  See
+     * countspotcolors in lib/pdf_main.ps.
+     */
+    if (pclist_devn_params != NULL) {
+	return pclist_devn_params->page_spot_colors;
     }
-    rc_decrement_only(ptm->mask, "gs_init_transparency_mask");
-    ptm->mask = 0;
     return 0;
 }
 
@@ -428,7 +438,15 @@ gs_push_pdf14trans_device(gs_state * pgs)
 {
     gs_pdf14trans_params_t params = { 0 };
 
-    params.pdf14_op = PDF14_PUSH_DEVICE;  /* Other parameters not used */
+    params.pdf14_op = PDF14_PUSH_DEVICE;
+    /*
+     * We really only care about the number of spot colors when we have
+     * a device which supports spot colors.  With the other devices we use
+     * the tint transform function for DeviceN and Separation color spaces
+     * and convert spot colors into process colors.
+     */
+    params.num_spot_colors = get_num_pdf14_spot_colors(pgs);
+    /* Note: Other parameters not used */
     return gs_state_update_pdf14trans(pgs, &params);
 }
 
