@@ -1,20 +1,17 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxalloc.h,v 1.11 2004/08/04 19:36:12 stefan Exp $ */
+/* $Id: gxalloc.h 8496 2008-01-21 20:59:55Z leonardo $ */
 /* Structure definitions for standard allocator */
 /* Requires gsmemory.h, gsstruct.h */
 
@@ -338,6 +335,9 @@ struct gs_ref_memory_s {
 				/* its own chunk: must be */
 				/* 1 mod obj_align_mod */
     uint space;			/* a_local, a_global, a_system */
+#   if IGC_PTR_STABILITY_CHECK
+    unsigned space_id:3; /* r_space_bits + 1 bit for "instability". */
+#   endif
     /* Callers can change the following dynamically */
     /* (through a procedural interface). */
     gs_memory_gc_status_t gc_status;
@@ -353,11 +353,6 @@ struct gs_ref_memory_s {
     chunk_locator_t cfreed;	/* chunk where last object freed */
     ulong allocated;		/* total size of all chunks */
 				/* allocated at this save level */
-    long inherited;		/* chunks allocated at outer save */
-				/* levels that should be counted */
-				/* towards the GC threshold */
-				/* (may be negative, but allocated + */
-				/* inherited >= 0 always) */
     ulong gc_allocated;		/* value of (allocated + */
 				/* previous_status.allocated) after last GC */
     struct lost_ {		/* space freed and 'lost' (not put on a */
@@ -380,8 +375,10 @@ struct gs_ref_memory_s {
     /* Sharing / saved state information */
     int num_contexts;		/* # of contexts sharing this VM */
     struct alloc_change_s *changes;
+    struct alloc_change_s *scan_limit;
     struct alloc_save_s *saved;
     long total_scanned;
+    long total_scanned_after_compacting;
     struct alloc_save_s *reloc_saved;	/* for GC */
     gs_memory_status_t previous_status;		/* total allocated & used */
 				/* in outer save levels */
@@ -396,7 +393,7 @@ extern_st(st_ref_memory);
 #define public_st_ref_memory()	/* in gsalloc.c */\
   gs_public_st_composite(st_ref_memory, gs_ref_memory_t,\
     "gs_ref_memory", ref_memory_enum_ptrs, ref_memory_reloc_ptrs)
-#define st_ref_memory_max_ptrs 4  /* streams, names_array, changes, saved */
+#define st_ref_memory_max_ptrs 5  /* streams, names_array, changes, scan_limit, saved */
 
 /* Define the procedures for the standard allocator. */
 /* We export this for subclasses. */
