@@ -1,20 +1,17 @@
-/* Copyright (C) 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclbits.c,v 1.4.4.1 2002/02/22 19:45:58 ray Exp $ */
+/* $Id: gxclbits.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Halftone and bitmap writing for command lists */
 #include "memory_.h"
 #include "gx.h"
@@ -69,7 +66,7 @@ clist_bitmap_bytes(uint width_bits, uint height, int compression_mask,
  * Compress a bitmap, skipping extra padding bytes at the end of each row if
  * necessary.  We require height >= 1, raster >= bitmap_raster(width_bits).
  */
-private int
+static int
 cmd_compress_bitmap(stream_state * st, const byte * data, uint width_bits,
 		    uint raster, uint height, stream_cursor_write * pw)
 {
@@ -127,7 +124,7 @@ cmd_put_bits(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     clist_bitmap_bytes(width_bits, height, compression_mask,
 		       &uncompressed_raster, &full_raster);
     uint max_size = cbuf_size - op_size;
-    gs_memory_t *mem = (cldev->memory ? cldev->memory : &gs_memory_default);
+    gs_memory_t *mem = cldev->memory;
     byte *dp;
     int compress = 0;
 
@@ -239,7 +236,7 @@ out:
 }
 
 /* Add a command to set the tile size and depth. */
-private uint
+static uint
 cmd_size_tile_params(const gx_strip_bitmap * tile)
 {
     return 2 + cmd_size_w(tile->rep_width) + cmd_size_w(tile->rep_height) +
@@ -249,12 +246,12 @@ cmd_size_tile_params(const gx_strip_bitmap * tile)
 	 cmd_size_w(tile->size.y / tile->rep_height)) +
 	(tile->rep_shift == 0 ? 0 : cmd_size_w(tile->rep_shift));
 }
-private void
+static void
 cmd_store_tile_params(byte * dp, const gx_strip_bitmap * tile, int depth,
 		      uint csize)
 {
     byte *p = dp + 2;
-    byte bd = depth - 1;
+    byte bd = cmd_depth_to_code(depth);
 
     *dp = cmd_count_op(cmd_opv_set_tile_size, csize);
     p = cmd_put_w(tile->rep_width, p);
@@ -276,7 +273,7 @@ cmd_store_tile_params(byte * dp, const gx_strip_bitmap * tile, int depth,
 
 /* Add a command to set the tile index. */
 /* This is a relatively high-frequency operation, so we declare it `inline'. */
-inline private int
+static inline int
 cmd_put_tile_index(gx_device_clist_writer *cldev, gx_clist_state *pcls,
 		   uint indx)
 {
@@ -304,7 +301,7 @@ cmd_put_tile_index(gx_device_clist_writer *cldev, gx_clist_state *pcls,
 /* If necessary, write out data for a single color map. */
 int
 cmd_put_color_map(gx_device_clist_writer * cldev, cmd_map_index map_index,
-		  const gx_transfer_map * map, gs_id * pid)
+	int comp_num, const gx_transfer_map * map, gs_id * pid)
 {
     byte *dp;
     int code;
@@ -312,27 +309,30 @@ cmd_put_color_map(gx_device_clist_writer * cldev, cmd_map_index map_index,
     if (map == 0) {
 	if (pid && *pid == gs_no_id)
 	    return 0;	/* no need to write */
-	code = set_cmd_put_all_op(dp, cldev, cmd_opv_set_misc, 2);
+	code = set_cmd_put_all_op(dp, cldev, cmd_opv_set_misc, 3);
 	if (code < 0)
 	    return code;
 	dp[1] = cmd_set_misc_map + (cmd_map_none << 4) + map_index;
+	dp[2] = comp_num;
 	if (pid)
 	    *pid = gs_no_id;
     } else {
 	if (pid && map->id == *pid)
 	    return 0;	/* no need to write */
 	if (map->proc == gs_identity_transfer) {
-	    code = set_cmd_put_all_op(dp, cldev, cmd_opv_set_misc, 2);
+	    code = set_cmd_put_all_op(dp, cldev, cmd_opv_set_misc, 3);
 	    if (code < 0)
 		return code;
 	    dp[1] = cmd_set_misc_map + (cmd_map_identity << 4) + map_index;
+	    dp[2] = comp_num;
 	} else {
 	    code = set_cmd_put_all_op(dp, cldev, cmd_opv_set_misc,
-				      2 + sizeof(map->values));
+				      3 + sizeof(map->values));
 	    if (code < 0)
 		return code;
 	    dp[1] = cmd_set_misc_map + (cmd_map_other << 4) + map_index;
-	    memcpy(dp + 2, map->values, sizeof(map->values));
+	    dp[2] = comp_num;
+	    memcpy(dp + 3, map->values, sizeof(map->values));
 	}
 	if (pid)
 	    *pid = map->id;
@@ -356,7 +356,7 @@ typedef struct tile_loc_s {
 
 /* Look up a tile or character in the cache.  If found, set the index and */
 /* pointer; if not, set the index to the insertion point. */
-private bool
+static bool
 clist_find_bits(gx_device_clist_writer * cldev, gx_bitmap_id id, tile_loc * ploc)
 {
     uint index = tile_id_hash(id);
@@ -380,7 +380,7 @@ clist_find_bits(gx_device_clist_writer * cldev, gx_bitmap_id id, tile_loc * ploc
 }
 
 /* Delete a tile from the cache. */
-private void
+static void
 clist_delete_tile(gx_device_clist_writer * cldev, tile_slot * slot)
 {
     tile_hash *table = cldev->tile_table;
@@ -417,7 +417,7 @@ clist_delete_tile(gx_device_clist_writer * cldev, tile_slot * slot)
 /* Add a tile to the cache. */
 /* tile->raster holds the raster for the replicated tile; */
 /* we pass the raster of the actual data separately. */
-private int
+static int
 clist_add_tile(gx_device_clist_writer * cldev, const gx_strip_bitmap * tiles,
 	       uint sraster, int depth)
 {
@@ -497,7 +497,7 @@ clist_add_tile(gx_device_clist_writer * cldev, const gx_strip_bitmap * tiles,
 
 /* Change the tile parameters (size and depth). */
 /* Currently we do this for all bands at once. */
-private void
+static void
 clist_new_tile_params(gx_strip_bitmap * new_tile, const gx_strip_bitmap * tiles,
 		      int depth, const gx_device_clist_writer * cldev)
 {				/*
@@ -683,7 +683,7 @@ clist_change_bits(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     tile_loc loc;
     int code;
 
-  top:if (clist_find_bits(cldev, tiles->id, &loc)) {	/* The bitmap is in the cache.  Check whether this band */
+   top:if (clist_find_bits(cldev, tiles->id, &loc)) {	/* The bitmap is in the cache.  Check whether this band */
 	/* knows about it. */
 	uint band_index = pcls - cldev->states;
 	byte *bptr = ts_mask(loc.tile) + (band_index >> 3);
