@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclist.c 6651 2006-03-13 16:18:19Z stefan $ */
+/*$Id: gxclist.c 8528 2008-02-17 22:32:15Z leonardo $ */
 /* Command list document- and page-level code. */
 #include "memory_.h"
 #include "string_.h"
@@ -26,53 +26,77 @@
 #include "gsparams.h"
 #include "gxdcolor.h"
 
+extern dev_proc_open_device(pattern_clist_open_device);
+
 /* GC information */
-#define CLIST_IS_WRITER(cdev) ((cdev)->common.ymin < 0)
 extern_st(st_imager_state);
-private
+static
 ENUM_PTRS_WITH(device_clist_enum_ptrs, gx_device_clist *cdev)
     if (index < st_device_forward_max_ptrs) {
 	gs_ptr_type_t ret = ENUM_USING_PREFIX(st_device_forward, 0);
 
 	return (ret ? ret : ENUM_OBJ(0));
     }
-    if (!CLIST_IS_WRITER(cdev))
-	return 0;
     index -= st_device_forward_max_ptrs;
-    switch (index) {
-    case 0: return ENUM_OBJ((cdev->writer.image_enum_id != gs_no_id ?
-			     cdev->writer.clip_path : 0));
-    case 1: return ENUM_OBJ((cdev->writer.image_enum_id != gs_no_id ?
-			     cdev->writer.color_space.space : 0));
-    default:
-	return ENUM_USING(st_imager_state, &cdev->writer.imager_state,
-			  sizeof(gs_imager_state), index - 2);
+    if (CLIST_IS_WRITER(cdev)) {
+        switch (index) {
+        case 0: return ENUM_OBJ((cdev->writer.image_enum_id != gs_no_id ?
+                     cdev->writer.clip_path : 0));
+        case 1: return ENUM_OBJ((cdev->writer.image_enum_id != gs_no_id ?
+                     cdev->writer.color_space.space : 0));
+	case 2: return ENUM_OBJ(cdev->writer.pinst);
+        default:
+        return ENUM_USING(st_imager_state, &cdev->writer.imager_state,
+                  sizeof(gs_imager_state), index - 3);
+        }
+    }
+    else {
+        /* 041207
+         * clist is reader.
+         * We don't expect this code to be exercised at this time as the reader
+         * runs under gdev_prn_output_page which is an atomic function of the
+         * interpreter. We do this as this situation may change in the future.
+         */
+        if (index == 0)
+            return ENUM_OBJ(cdev->reader.band_complexity_array);
+        else if (index == 1)
+	    return ENUM_OBJ(cdev->reader.offset_map);
+	else
+            return 0;
     }
 ENUM_PTRS_END
-private
+static
 RELOC_PTRS_WITH(device_clist_reloc_ptrs, gx_device_clist *cdev)
 {
     RELOC_PREFIX(st_device_forward);
-    if (!CLIST_IS_WRITER(cdev))
-	return;
-    if (cdev->writer.image_enum_id != gs_no_id) {
-	RELOC_VAR(cdev->writer.clip_path);
-	RELOC_VAR(cdev->writer.color_space.space);
+    if (CLIST_IS_WRITER(cdev)) {
+        if (cdev->writer.image_enum_id != gs_no_id) {
+	    RELOC_VAR(cdev->writer.clip_path);
+	    RELOC_VAR(cdev->writer.color_space.space);
+        }
+	RELOC_VAR(cdev->writer.pinst);
+        RELOC_USING(st_imager_state, &cdev->writer.imager_state,
+            sizeof(gs_imager_state));
+    } else {
+        /* 041207
+         * clist is reader.
+         * See note above in ENUM_PTRS_WITH section.
+         */
+        RELOC_VAR(cdev->reader.band_complexity_array);
+        RELOC_VAR(cdev->reader.offset_map);
     }
-    RELOC_USING(st_imager_state, &cdev->writer.imager_state,
-		sizeof(gs_imager_state));
 } RELOC_PTRS_END
 public_st_device_clist();
 
 /* Forward declarations of driver procedures */
-private dev_proc_open_device(clist_open);
-private dev_proc_output_page(clist_output_page);
-private dev_proc_close_device(clist_close);
-private dev_proc_get_band(clist_get_band);
+static dev_proc_open_device(clist_open);
+static dev_proc_output_page(clist_output_page);
+static dev_proc_close_device(clist_close);
+static dev_proc_get_band(clist_get_band);
 /* Driver procedures defined in other files are declared in gxcldev.h. */
 
 /* Other forward declarations */
-private int clist_put_current_params(gx_device_clist_writer *cldev);
+static int clist_put_current_params(gx_device_clist_writer *cldev);
 
 /* The device procedures */
 const gx_device_procs gs_clist_device_procs = {
@@ -103,7 +127,7 @@ const gx_device_procs gs_clist_device_procs = {
     clist_fill_path,
     clist_stroke_path,
     clist_fill_mask,
-    gx_default_fill_trapezoid,
+    clist_fill_trapezoid,
     clist_fill_parallelogram,
     clist_fill_triangle,
     gx_default_draw_thin_line,
@@ -129,15 +153,34 @@ const gx_device_procs gs_clist_device_procs = {
     gx_forward_get_color_comp_index,
     gx_forward_encode_color,
     gx_forward_decode_color,
-    gx_default_pattern_manage,
+    clist_pattern_manage,
     gx_default_fill_rectangle_hl_color,
     gx_default_include_color_space,
     gx_default_fill_linear_color_scanline,
-    gx_default_fill_linear_color_trapezoid, /* fixme : write to clist. */
-    gx_default_fill_linear_color_triangle,
-    gx_forward_update_spot_equivalent_colors
+    clist_fill_linear_color_trapezoid,
+    clist_fill_linear_color_triangle,
+    gx_forward_update_spot_equivalent_colors,
+    gx_forward_ret_devn_params
 };
 
+/*------------------- Choose the implementation -----------------------
+
+   For chossing the clist i/o implementation by makefile options
+   we define global variables, which are initialized with
+   file/memory io procs when they are included into the build.
+ */
+const clist_io_procs_t *clist_io_procs_file_global = NULL;
+const clist_io_procs_t *clist_io_procs_memory_global = NULL;
+
+void 
+clist_init_io_procs(gx_device_clist *pclist_dev, bool in_memory)
+{   
+    if (in_memory || clist_io_procs_file_global == NULL)
+	pclist_dev->common.page_info.io_procs = clist_io_procs_memory_global;
+    else
+	pclist_dev->common.page_info.io_procs = clist_io_procs_file_global;
+}
+
 /* ------ Define the command set and syntax ------ */
 
 /* Initialization for imager state. */
@@ -160,7 +203,7 @@ const gs_imager_state clist_imager_state_initial =
 /*
  * Calculate the desired size for the tile cache.
  */
-private uint
+static uint
 clist_tile_cache_size(const gx_device * target, uint data_size)
 {
     uint bits_size =
@@ -180,7 +223,7 @@ clist_tile_cache_size(const gx_device * target, uint data_size)
  * Initialize the allocation for the tile cache.  Sets: tile_hash_mask,
  * tile_max_count, tile_table, chunk (structure), bits (structure).
  */
-private int
+static int
 clist_init_tile_cache(gx_device * dev, byte * init_data, ulong data_size)
 {
     gx_device_clist_writer * const cdev =
@@ -227,15 +270,23 @@ clist_init_tile_cache(gx_device * dev, byte * init_data, ulong data_size)
  * Initialize the allocation for the bands.  Requires: target.  Sets:
  * page_band_height (=page_info.band_params.BandHeight), nbands.
  */
-private int
+static int
 clist_init_bands(gx_device * dev, gx_device_memory *bdev, uint data_size,
 		 int band_width, int band_height)
 {
     gx_device_clist_writer * const cdev =
 	&((gx_device_clist *)dev)->writer;
     int nbands;
+    ulong space;
 
-    if (gdev_mem_data_size(bdev, band_width, band_height) > data_size)
+    if (dev->procs.open_device == pattern_clist_open_device) {
+	/* We don't need bands really. */
+	cdev->page_band_height = dev->height;
+	cdev->nbands = 1;
+	return 0;
+    }
+    if (gdev_mem_data_size(bdev, band_width, band_height, &space) < 0 ||
+	space > data_size)
 	return_error(gs_error_rangecheck);
     cdev->page_band_height = band_height;
     nbands = (cdev->target->height + band_height - 1) / band_height;
@@ -252,23 +303,27 @@ clist_init_bands(gx_device * dev, gx_device_memory *bdev, uint data_size,
  * Initialize the allocation for the band states, which are used only
  * when writing.  Requires: nbands.  Sets: states, cbuf, cend.
  */
-private int
+static int
 clist_init_states(gx_device * dev, byte * init_data, uint data_size)
 {
     gx_device_clist_writer * const cdev =
 	&((gx_device_clist *)dev)->writer;
     ulong state_size = cdev->nbands * (ulong) sizeof(gx_clist_state);
+    /* Align to the natural boundary for ARM processors, bug 689600 */
+    long alignment = (-(long)init_data) & (sizeof(init_data) - 1);
 
     /*
      * The +100 in the next line is bogus, but we don't know what the
      * real check should be. We're effectively assuring that at least 100
      * bytes will be available to buffer command operands.
      */
-    if (state_size + sizeof(cmd_prefix) + cmd_largest_size + 100 > data_size)
+    if (state_size + sizeof(cmd_prefix) + cmd_largest_size + 100 + alignment > data_size)
 	return_error(gs_error_rangecheck);
+    /* The end buffer position is not affected by alignment */
+    cdev->cend = init_data + data_size;
+    init_data +=  alignment;
     cdev->states = (gx_clist_state *) init_data;
     cdev->cbuf = init_data + state_size;
-    cdev->cend = init_data + data_size;
     return 0;
 }
 
@@ -277,16 +332,15 @@ clist_init_states(gx_device * dev, byte * init_data, uint data_size)
  * page_tile_cache_size, page_info.band_params.BandWidth,
  * page_info.band_params.BandBufferSpace, + see above.
  */
-private int
+static int
 clist_init_data(gx_device * dev, byte * init_data, uint data_size)
 {
     gx_device_clist_writer * const cdev =
 	&((gx_device_clist *)dev)->writer;
     gx_device *target = cdev->target;
+    /* BandWidth can't be smaller than target device width */
     const int band_width =
-	cdev->page_info.band_params.BandWidth =
-	(cdev->band_params.BandWidth ? cdev->band_params.BandWidth :
-	 target->width);
+	cdev->page_info.band_params.BandWidth = max(target->width, cdev->band_params.BandWidth);
     int band_height = cdev->band_params.BandHeight;
     bool page_uses_transparency = cdev->page_uses_transparency;
     const uint band_space =
@@ -301,20 +355,22 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     int code;
 
     /* Call create_buf_device to get the memory planarity set up. */
-    cdev->buf_procs.create_buf_device(&pbdev, target, NULL, NULL, true);
+    cdev->buf_procs.create_buf_device(&pbdev, target, 0, NULL, NULL, clist_get_band_complexity(0, 0));
     /* HACK - if the buffer device can't do copy_alpha, disallow */
     /* copy_alpha in the commmand list device as well. */
     if (dev_proc(pbdev, copy_alpha) == gx_no_copy_alpha)
 	cdev->disable_mask |= clist_disable_copy_alpha;
-    if (band_height) {
+    if (cdev->procs.open_device == pattern_clist_open_device) {
+	bits_size = data_size / 2;
+    } else if (band_height) {
 	/*
 	 * The band height is fixed, so the band buffer requirement
 	 * is completely determined.
 	 */
-	uint band_data_size =
-	    gdev_mem_data_size(&bdev, band_width, band_height);
+	ulong band_data_size;
 
-	if (band_data_size >= band_space)
+	if (gdev_mem_data_size(&bdev, band_width, band_height, &band_data_size) < 0 ||
+	    band_data_size >= band_space)
 	    return_error(gs_error_rangecheck);
 	bits_size = min(band_space - band_data_size, data_size >> 1);
     } else {
@@ -329,6 +385,7 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
 	if (band_height == 0)
 	    return_error(gs_error_rangecheck);
     }
+    cdev->ins_count = 0;
     code = clist_init_tile_cache(dev, data, bits_size);
     if (code < 0)
 	return code;
@@ -344,7 +401,7 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
  * Reset the device state (for writing).  This routine requires only
  * data, data_size, and target to be set, and is idempotent.
  */
-private int
+static int
 clist_reset(gx_device * dev)
 {
     gx_device_clist_writer * const cdev =
@@ -359,7 +416,7 @@ clist_reset(gx_device * dev)
     nbands = cdev->nbands;
     cdev->ymin = cdev->ymax = -1;	/* render_init not done yet */
     memset(cdev->tile_table, 0, (cdev->tile_hash_mask + 1) *
-	   sizeof(*cdev->tile_table));
+       sizeof(*cdev->tile_table));
     cdev->cnext = cdev->cbuf;
     cdev->ccl = 0;
     cdev->band_range_list.head = cdev->band_range_list.tail = 0;
@@ -407,13 +464,14 @@ clist_reset(gx_device * dev)
     cdev->undercolor_removal_id = gs_no_id;
     cdev->device_halftone_id = gs_no_id;
     cdev->image_enum_id = gs_no_id;
+    cdev->cropping_by_path = false;
     return 0;
 }
 /*
  * Initialize the device state (for writing).  This routine requires only
  * data, data_size, and target to be set, and is idempotent.
  */
-private int
+static int
 clist_init(gx_device * dev)
 {
     gx_device_clist_writer * const cdev =
@@ -430,7 +488,7 @@ clist_init(gx_device * dev)
 }
 
 /* (Re)init open band files for output (set block size, etc). */
-private int	/* ret 0 ok, -ve error code */
+static int	/* ret 0 ok, -ve error code */
 clist_reinit_output_file(gx_device *dev)
 {    gx_device_clist_writer * const cdev =
 	&((gx_device_clist *)dev)->writer;
@@ -451,16 +509,16 @@ clist_reinit_output_file(gx_device *dev)
     /* if partial page rendering is available */
     if ( clist_test_VMerror_recoverable(cdev) )
 	{ if (cdev->page_bfile != 0)
-	    code = clist_set_memory_warning(cdev->page_bfile, b_block);
+	    code = cdev->page_info.io_procs->set_memory_warning(cdev->page_bfile, b_block);
 	if (code >= 0 && cdev->page_cfile != 0)
-	    code = clist_set_memory_warning(cdev->page_cfile, c_block);
+	    code = cdev->page_info.io_procs->set_memory_warning(cdev->page_cfile, c_block);
 	}
     return code;
 }
 
 /* Write out the current parameters that must be at the head of each page */
 /* if async rendering is in effect */
-private int
+static int
 clist_emit_page_header(gx_device *dev)
 {
     gx_device_clist_writer * const cdev =
@@ -480,7 +538,7 @@ clist_emit_page_header(gx_device *dev)
 }
 
 /* Reset parameters for the beginning of a page. */
-private void
+static void
 clist_reset_page(gx_device_clist_writer *cwdev)
 {
     cwdev->page_bfile_end_pos = 0;
@@ -491,7 +549,7 @@ clist_reset_page(gx_device_clist_writer *cwdev)
 }
 
 /* Open the device's bandfiles */
-private int
+static int
 clist_open_output_file(gx_device *dev)
 {
     gx_device_clist_writer * const cdev =
@@ -511,12 +569,12 @@ clist_open_output_file(gx_device *dev)
     cdev->page_cfname[0] = 0;	/* create a new file */
     cdev->page_bfname[0] = 0;	/* ditto */
     clist_reset_page(cdev);
-    if ((code = clist_fopen(cdev->page_cfname, fmode, &cdev->page_cfile,
+    if ((code = cdev->page_info.io_procs->fopen(cdev->page_cfname, fmode, &cdev->page_cfile,
 			    cdev->bandlist_memory, cdev->bandlist_memory,
 			    true)) < 0 ||
-	(code = clist_fopen(cdev->page_bfname, fmode, &cdev->page_bfile,
+	(code = cdev->page_info.io_procs->fopen(cdev->page_bfname, fmode, &cdev->page_bfile,
 			    cdev->bandlist_memory, cdev->bandlist_memory,
-			    true)) < 0 ||
+			    false)) < 0 ||
 	(code = clist_reinit_output_file(dev)) < 0
 	) {
 	clist_close_output_file(dev);
@@ -532,11 +590,11 @@ int
 clist_close_page_info(gx_band_page_info_t *ppi)
 {
     if (ppi->cfile != NULL) {
-	clist_fclose(ppi->cfile, ppi->cfname, true);
+	ppi->io_procs->fclose(ppi->cfile, ppi->cfname, true);
 	ppi->cfile = NULL;
     }
     if (ppi->bfile != NULL) {
-	clist_fclose(ppi->bfile, ppi->bfname, true);
+	ppi->io_procs->fclose(ppi->bfile, ppi->bfname, true);
 	ppi->bfile = NULL;
     }
     return 0;
@@ -555,7 +613,7 @@ clist_close_output_file(gx_device *dev)
 
 /* Open the device by initializing the device state and opening the */
 /* scratch files. */
-private int
+static int
 clist_open(gx_device *dev)
 {
     gx_device_clist_writer * const cdev =
@@ -572,7 +630,7 @@ clist_open(gx_device *dev)
     return code;
 }
 
-private int
+static int
 clist_close(gx_device *dev)
 {
     gx_device_clist_writer * const cdev =
@@ -580,11 +638,15 @@ clist_close(gx_device *dev)
 
     if (cdev->do_not_open_or_close_bandfiles)
 	return 0;	
+    if (cdev->procs.open_device == pattern_clist_open_device) {
+	gs_free_object(cdev->bandlist_memory, cdev->data, "clist_close");
+	cdev->data = NULL;
+    }
     return clist_close_output_file(dev);
 }
 
 /* The output_page procedure should never be called! */
-private int
+static int
 clist_output_page(gx_device * dev, int num_copies, int flush)
 {
     return_error(gs_error_Fatal);
@@ -594,21 +656,26 @@ clist_output_page(gx_device * dev, int num_copies, int flush)
 int
 clist_finish_page(gx_device *dev, bool flush)
 {
-    gx_device_clist_writer * const cdev =
-	&((gx_device_clist *)dev)->writer;
+    gx_device_clist_writer * const cdev =	&((gx_device_clist *)dev)->writer;
     int code;
 
+    /* If this is a reader clist, which is about to be reset to a writer,
+     * free any band_complexity_array memory used by same.
+     */
+    if (!CLIST_IS_WRITER((gx_device_clist *)dev))
+       	gx_clist_reader_free_band_complexity_array( (gx_device_clist *)dev );
+
     if (flush) {
 	if (cdev->page_cfile != 0)
-	    clist_rewind(cdev->page_cfile, true, cdev->page_cfname);
+	    cdev->page_info.io_procs->rewind(cdev->page_cfile, true, cdev->page_cfname);
 	if (cdev->page_bfile != 0)
-	    clist_rewind(cdev->page_bfile, true, cdev->page_bfname);
+	    cdev->page_info.io_procs->rewind(cdev->page_bfile, true, cdev->page_bfname);
 	clist_reset_page(cdev);
     } else {
 	if (cdev->page_cfile != 0)
-	    clist_fseek(cdev->page_cfile, 0L, SEEK_END, cdev->page_cfname);
+	    cdev->page_info.io_procs->fseek(cdev->page_cfile, 0L, SEEK_END, cdev->page_cfname);
 	if (cdev->page_bfile != 0)
-	    clist_fseek(cdev->page_bfile, 0L, SEEK_END, cdev->page_bfname);
+	    cdev->page_info.io_procs->fseek(cdev->page_bfile, 0L, SEEK_END, cdev->page_bfname);
     }
     code = clist_init(dev);		/* reinitialize */
     if (code >= 0)
@@ -635,34 +702,40 @@ clist_end_page(gx_device_clist_writer * cldev)
 	 * Note that because of copypage, there may be many such entries.
 	 */
 	cb.band_min = cb.band_max = cmd_band_end;
-	cb.pos = (cldev->page_cfile == 0 ? 0 : clist_ftell(cldev->page_cfile));
-	code = clist_fwrite_chars(&cb, sizeof(cb), cldev->page_bfile);
+	cb.pos = (cldev->page_cfile == 0 ? 0 : cldev->page_info.io_procs->ftell(cldev->page_cfile));
+	code = cldev->page_info.io_procs->fwrite_chars(&cb, sizeof(cb), cldev->page_bfile);
 	if (code > 0)
 	    code = 0;
     }
     if (code >= 0) {
 	clist_compute_colors_used(cldev);
 	ecode |= code;
-	cldev->page_bfile_end_pos = clist_ftell(cldev->page_bfile);
+	cldev->page_bfile_end_pos = cldev->page_info.io_procs->ftell(cldev->page_bfile);
     }
     if (code < 0)
 	ecode = code;
 
     /* Reset warning margin to 0 to release reserve memory if mem files */
     if (cldev->page_bfile != 0)
-	clist_set_memory_warning(cldev->page_bfile, 0);
+	cldev->page_info.io_procs->set_memory_warning(cldev->page_bfile, 0);
     if (cldev->page_cfile != 0)
-	clist_set_memory_warning(cldev->page_cfile, 0);
+	cldev->page_info.io_procs->set_memory_warning(cldev->page_cfile, 0);
 
 #ifdef DEBUG
     if (gs_debug_c('l') | gs_debug_c(':'))
 	dlprintf2("[:]clist_end_page at cfile=%ld, bfile=%ld\n",
-		  cb.pos, cldev->page_bfile_end_pos);
+		  (long)cb.pos, (long)cldev->page_bfile_end_pos);
 #endif
     return 0;
 }
 
-/* Compute the set of used colors in the page_info structure. */
+/* Compute the set of used colors in the page_info structure. 
+ *
+ * NB: Area for improvement, move states[band] and page_info to clist
+ * rather than writer device, or remove completely as this is used by the old planar devices 
+ * to operate on a plane at a time.  
+ */
+
 void
 clist_compute_colors_used(gx_device_clist_writer *cldev)
 {
@@ -683,6 +756,7 @@ clist_compute_colors_used(gx_device_clist_writer *cldev)
 	    cldev->states[band].colors_used.or;
 	cldev->page_info.band_colors_used[entry].slow_rop |=
 	    cldev->states[band].colors_used.slow_rop;
+
     }
 }
 
@@ -759,7 +833,7 @@ clist_VMerror_recover_flush(gx_device_clist_writer *cldev,
 }
 
 /* Write the target device's current parameter list */
-private int	/* ret 0 all ok, -ve error */
+static int	/* ret 0 all ok, -ve error */
 clist_put_current_params(gx_device_clist_writer *cldev)
 {
     gx_device *target = cldev->target;
@@ -788,7 +862,7 @@ clist_put_current_params(gx_device_clist_writer *cldev)
 
 /* ---------------- Driver interface ---------------- */
 
-private int
+static int
 clist_get_band(gx_device * dev, int y, int *band_start)
 {
     gx_device_clist_writer * const cdev =
@@ -803,3 +877,27 @@ clist_get_band(gx_device * dev, int y, int *band_start)
     *band_start = start = y - y % band_height;
     return min(dev->height - start, band_height);
 }
+
+
+
+
+/* copy constructor if from != NULL
+ * default constructor if from == NULL
+ */
+void 
+clist_copy_band_complexity(gx_band_complexity_t *this, const gx_band_complexity_t *from)
+{
+    if (from) {
+	memcpy(this, from, sizeof(gx_band_complexity_t));
+    } else {
+	/* default */
+	this->uses_color = false;
+	this->nontrivial_rops = false;
+#if 0
+	/* todo: halftone phase */
+
+	this->x0 = 0;
+	this->y0 = 0;
+#endif
+    }
+}
