@@ -1,23 +1,17 @@
-/* Copyright (C) 1998 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gxclutil.c,v $ $Revision: 1.2.2.2 $ */
+/* $Id: gxclutil.c 8488 2008-01-17 13:37:19Z leonardo $ */
 /* Command list writing utilities. */
 
 #include "memory_.h"
@@ -37,13 +31,13 @@
 #ifdef DEBUG
 const char *const cmd_op_names[16] =
 {cmd_op_name_strings};
-private const char *const cmd_misc_op_names[16] =
+static const char *const cmd_misc_op_names[16] =
 {cmd_misc_op_name_strings};
-private const char *const cmd_misc2_op_names[16] =
+static const char *const cmd_misc2_op_names[16] =
 {cmd_misc2_op_name_strings};
-private const char *const cmd_segment_op_names[16] =
+static const char *const cmd_segment_op_names[16] =
 {cmd_segment_op_name_strings};
-private const char *const cmd_path_op_names[16] =
+static const char *const cmd_path_op_names[16] =
 {cmd_path_op_name_strings};
 const char *const *const cmd_sub_op_names[16] =
 {cmd_misc_op_names, 0, 0, 0, 0, 0, 0, 0,
@@ -70,7 +64,7 @@ cmd_count_op(int op, uint size)
 	else
 	    dlprintf3(", %s %d(%u)\n", cmd_op_names[op >> 4], op & 0xf,
 		      size);
-	fflush(dstderr);
+	dflush();
     }
     return op;
 }
@@ -131,9 +125,9 @@ cmd_print_stats(void)
 /* ---------------- Writing utilities ---------------- */
 
 /* Write the commands for one band or band range. */
-private int	/* ret 0 all ok, -ve error code, or +1 ok w/low-mem warning */
+static int	/* ret 0 all ok, -ve error code, or +1 ok w/low-mem warning */
 cmd_write_band(gx_device_clist_writer * cldev, int band_min, int band_max,
-	       cmd_list * pcl, byte cmd_end)
+	       cmd_list * pcl, gx_band_complexity_t *band_complexity, byte cmd_end)
 {
     const cmd_prefix *cp = pcl->head;
     int code_b = 0;
@@ -149,10 +143,11 @@ cmd_write_band(gx_device_clist_writer * cldev, int band_min, int band_max,
  	    return_error(gs_error_ioerror);
 	cb.band_min = band_min;
 	cb.band_max = band_max;
-	cb.pos = clist_ftell(cfile);
-	if_debug3('l', "[l]writing for bands (%d,%d) at %ld\n",
-		  band_min, band_max, cb.pos);
-	clist_fwrite_chars(&cb, sizeof(cb), bfile);
+	cb.pos = cldev->page_info.io_procs->ftell(cfile);
+	clist_copy_band_complexity(&cb.band_complexity, band_complexity);  
+	if_debug4('l', "[l]writing for bands (%d,%d) at %ld K %d \n",
+		  band_min, band_max, (long)cb.pos, cb.band_complexity.uses_color);
+	cldev->page_info.io_procs->fwrite_chars(&cb, sizeof(cb), bfile);
 	if (cp != 0) {
 	    pcl->tail->next = 0;	/* terminate the list */
 	    for (; cp != 0; cp = cp->next) {
@@ -165,14 +160,16 @@ cmd_write_band(gx_device_clist_writer * cldev, int band_min, int band_max,
 		    return_error(gs_error_Fatal);
 		}
 #endif
-		clist_fwrite_chars(cp + 1, cp->size, cfile);
+		if_debug2('L',"[L]Wrote cmd id=%ld at %ld\n", cp->id, 
+		    (long)cldev->page_info.io_procs->ftell(cfile));
+		cldev->page_info.io_procs->fwrite_chars(cp + 1, cp->size, cfile);
 	    }
 	    pcl->head = pcl->tail = 0;
 	}
-	clist_fwrite_chars(&end, 1, cfile);
-	process_interrupts();
-	code_b = clist_ferror_code(bfile);
-	code_c = clist_ferror_code(cfile);
+	cldev->page_info.io_procs->fwrite_chars(&end, 1, cfile);
+	process_interrupts(cldev->memory);
+	code_b = cldev->page_info.io_procs->ferror_code(bfile);
+	code_c = cldev->page_info.io_procs->ferror_code(cfile);
 	if (code_b < 0)
 	    return_error(code_b);
 	if (code_c < 0)
@@ -190,13 +187,16 @@ cmd_write_buffer(gx_device_clist_writer * cldev, byte cmd_end)
     int band;
     int code = cmd_write_band(cldev, cldev->band_range_min,
 			      cldev->band_range_max,
-			      &cldev->band_range_list, cmd_opv_end_run);
+			      &cldev->band_range_list, 
+			      NULL, 
+			      cmd_opv_end_run);
+
     int warning = code;
 
     for (band = 0, pcls = cldev->states;
 	 code >= 0 && band < nbands; band++, pcls++
 	 ) {
-	code = cmd_write_band(cldev, band, band, &pcls->list, cmd_end);
+	code = cmd_write_band(cldev, band, band, &pcls->list, &pcls->band_complexity, cmd_end);
 	warning |= code;
     }
     /* If an error occurred, finish cleaning up the pointers. */
@@ -208,7 +208,7 @@ cmd_write_buffer(gx_device_clist_writer * cldev, byte cmd_end)
     if (gs_debug_c('l'))
 	cmd_print_stats();
 #endif
-    return_check_interrupt(code != 0 ? code : warning);
+    return_check_interrupt(cldev->memory, code != 0 ? code : warning);
 }
 
 /*
@@ -216,7 +216,7 @@ cmd_write_buffer(gx_device_clist_writer * cldev, byte cmd_end)
  * data.  Return the pointer to the data area.  If an error or (low-memory
  * warning) occurs, set cldev->error_code and return 0.
  */
-#define cmd_headroom (sizeof(cmd_prefix) + arch_align_ptr_mod)
+#define cmd_headroom (sizeof(cmd_prefix) + ARCH_ALIGN_PTR_MOD)
 byte *
 cmd_put_list_op(gx_device_clist_writer * cldev, cmd_list * pcl, uint size)
 {
@@ -246,12 +246,13 @@ cmd_put_list_op(gx_device_clist_writer * cldev, cmd_list * pcl, uint size)
 	    lprintf1("cmd_put_list_op error at 0x%lx\n", (ulong) pcl->tail);
 	}
 #endif
+	if_debug2('L', ", to id=%ld , offset=%ld", pcl->tail->id, (long)pcl->tail->size);
 	pcl->tail->size += size;
     } else {
 	/* Skip to an appropriate alignment boundary. */
 	/* (We assume the command buffer itself is aligned.) */
 	cmd_prefix *cp = (cmd_prefix *)
-	    (dp + ((cldev->cbuf - dp) & (arch_align_ptr_mod - 1)));
+	    (dp + ((cldev->cbuf - dp) & (ARCH_ALIGN_PTR_MOD - 1)));
 
 	cmd_count_add1(stats_cmd.other_band);
 	dp = (byte *) (cp + 1);
@@ -270,6 +271,9 @@ cmd_put_list_op(gx_device_clist_writer * cldev, cmd_list * pcl, uint size)
 	pcl->tail = cp;
 	cldev->ccl = pcl;
 	cp->size = size;
+	cp->id = cldev->ins_count;
+	if_debug1('L', ", id=%ld ", cldev->ins_count);
+	cldev->ins_count++;
     }
     cldev->cnext = dp + size;
     return dp;
@@ -280,7 +284,7 @@ cmd_put_op(gx_device_clist_writer * cldev, gx_clist_state * pcls, uint size)
 {
     if_debug3('L', "[L]band %d: size=%u, left=%u",
 	      (int)(pcls - cldev->states),
-	      size, (uint) (cldev->cend - cldev->cnext));
+	      size, 0);
     return cmd_put_list_op(cldev, &pcls->list, size);
 }
 #endif
@@ -291,8 +295,7 @@ cmd_put_range_op(gx_device_clist_writer * cldev, int band_min, int band_max,
 		 uint size)
 {
     if_debug4('L', "[L]band range(%d,%d): size=%u, left=%u",
-	      band_min, band_max, size,
-	      (uint)(cldev->cend - cldev->cnext));
+	      band_min, band_max, size, 0);
     if (cldev->ccl != 0 && 
 	(cldev->ccl != &cldev->band_range_list ||
 	 band_min != cldev->band_range_min ||
@@ -332,128 +335,213 @@ cmd_put_w(register uint w, register byte * dp)
     *dp = w;
     return dp + 1;
 }
+/* Write a variable-size positive fractional. */
+int
+cmd_size_frac31(register frac31 w)
+{
+    register int size = 1;
+    register uint32_t v = w;
 
-/* Define the encodings of the different settable colors. */
+    while (v & 0x01FFFFFF)
+	v <<= 7, size++;
+    return size;
+}
+byte *
+cmd_put_frac31(register frac31 w, register byte * dp)
+{
+    register uint32_t v = w;
+
+    while (v & 0x01FFFFFF)
+	*dp++ = (v >> 24) | 1, v <<= 7;
+    *dp = (v >> 24);
+    return dp + 1;
+}
+
+
+
+/*
+ * This next two arrays are used for the 'delta' mode of placing a color
+ * in the clist.  These arrays are indexed by the number of bytes in the
+ * color value (the depth).
+ *
+ * Delta values are calculated by subtracting the old value for the color
+ * from the desired new value.  Then each byte of the differenece is
+ * examined.  For most bytes, if the difference fits into 4 bits (signed)
+ * then those bits are packed into the clist along with an opcode.  If
+ * the size of the color (the depth) is an odd number of bytes then instead
+ * of four bits per byte, extra bits are used for the upper three bytes
+ * of the color.  In this case, five bits are used for the first byte,
+ * six bits for the second byte, and five bits for third byte.  This
+ * maximizes the chance that the 'delta' mode can be used for placing
+ * colors in the clist.
+ */
+/*
+ * Depending upon the compiler and user choices, the size of a gx_color_index
+ * may be 4 to 8 bytes.  We will define table entries for up to 8 bytes.
+ * This macro is being used to prevent compiler warnings if gx_color_index is
+ * only 4 bytes.
+ */
+#define tab_entry(x) ((x) & (~((gx_color_index) 0)))
+
+const gx_color_index cmd_delta_offsets[] = {
+	tab_entry(0),
+	tab_entry(0),
+	tab_entry(0x0808),
+	tab_entry(0x102010),
+	tab_entry(0x08080808),
+	tab_entry(0x1020100808),
+	tab_entry(0x080808080808),
+	tab_entry(0x10201008080808),
+	tab_entry(0x0808080808080808),
+	};
+
+static const gx_color_index cmd_delta_masks[] = {
+	tab_entry(0),
+	tab_entry(0),
+	tab_entry(0x0f0f),
+	tab_entry(0x1f3f1f),
+	tab_entry(0x0f0f0f0f),
+	tab_entry(0x1f3f1f0f0f),
+	tab_entry(0x0f0f0f0f0f0f),
+	tab_entry(0x1f3f1f0f0f0f0f),
+	tab_entry(0x0f0f0f0f0f0f0f0f),
+	};
+
+#undef tab_entry
+
+/*
+ * There are currently only four different color "types" that can be placed
+ * into the clist.  These are called "color0", "color1", and "tile_color0",
+ * and "tile_color1".  There are separate command codes for color0 versus
+ * color1, both for the full value and delta commands - see cmd_put_color.
+ * Tile colors are preceded by a cmd_opv_set_tile_color command.
+ */
 const clist_select_color_t
-    clist_select_color0 = {cmd_op_set_color0, cmd_opv_delta2_color0, 0},
-    clist_select_color1 = {cmd_op_set_color1, cmd_opv_delta2_color1, 0},
-    clist_select_tile_color0 = {cmd_op_set_color0, cmd_opv_delta2_color0, 1},
-    clist_select_tile_color1 = {cmd_op_set_color1, cmd_opv_delta2_color1, 1};
+    clist_select_color0 = {cmd_op_set_color0, cmd_opv_delta_color0, 0},
+    clist_select_color1 = {cmd_op_set_color1, cmd_opv_delta_color1, 0},
+    clist_select_tile_color0 = {cmd_op_set_color0, cmd_opv_delta_color0, 1},
+    clist_select_tile_color1 = {cmd_op_set_color1, cmd_opv_delta_color1, 1};
+
+/*
+ * This routine is used to place a color into the clist.  Colors, in the
+ * clist, can be specified either as by a full value or by a "delta" value.
+ *
+ * See the comments before cmd_delta_offsets[] for a description of the
+ * 'delta' mode.  The delta mode may allow for a smaller command in the clist.
+ *
+ * For the full value mode, values are sent as a cmd code plus n bytes of
+ * data.  To minimize the number of bytes, a count is made of any low order
+ * bytes which are zero.  This count is packed into the low order 4 bits
+ * of the cmd code.  The data for these bytes are not sent.
+ *
+ * The gx_no_color_index value is treated as a special case.  This is done
+ * because it is both a commonly sent value and because it may require
+ * more bytes then the other color values.
+ *
+ * Parameters:
+ *   cldev - Pointer to clist device
+ *   pcls - Pointer to clist state
+ *   select - Descriptor record for type of color being sent.  See comments
+ *       by clist_select_color_t.
+ *   color - The new color value.
+ *   pcolor - Pointer to previous color value.  (If the color value is the
+ *       same as the previous value then nothing is placed into the clist.)
+ *
+ * Returns:
+ *   Error code
+ *   clist and pcls and cldev may be updated.
+ */
 int
 cmd_put_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	      const clist_select_color_t * select,
 	      gx_color_index color, gx_color_index * pcolor)
 {
-    byte *dp;
-    long diff = (long)color - (long)(*pcolor);
-    byte op, op_delta2;
+    byte * dp;		/* This is manipulated by the set_cmd_put_op macro */
+    gx_color_index diff = color - *pcolor;
+    byte op, op_delta;
     int code;
 
     if (diff == 0)
 	return 0;
+
+    /* If this is a tile color then send tile color type */
     if (select->tile_color) {
 	code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_set_tile_color, 1);
 	if (code < 0)
 	    return code;
     }
     op = select->set_op;
-    op_delta2 = select->delta2_op;
+    op_delta = select->delta_op;
     if (color == gx_no_color_index) {
 	/*
 	 * We must handle this specially, because it may take more
 	 * bytes than the color depth.
 	 */
-	code = set_cmd_put_op(dp, cldev, pcls, op + 15, 1);
+	code = set_cmd_put_op(dp, cldev, pcls, op + cmd_no_color_index, 1);
 	if (code < 0)
 	    return code;
     } else {
-	long delta;
-	byte operand;
-
-	switch ((cldev->color_info.depth + 15) >> 3) {
-	    case 5:
-		if (!((delta = diff + cmd_delta1_32_bias) &
-		      ~cmd_delta1_32_mask) &&
-		    (operand =
-		     (byte) ((delta >> 23) + ((delta >> 18) & 1))) != 0 &&
-		    operand != 15
-		    ) {
-		    code = set_cmd_put_op(dp, cldev, pcls,
-					  (byte) (op + operand), 2);
-		    if (code < 0)
-			return code;
-		    dp[1] = (byte) (((delta >> 10) & 0300) +
-				    (delta >> 5) + delta);
-		    break;
-		}
-		if (!((delta = diff + cmd_delta2_32_bias) &
-		      ~cmd_delta2_32_mask)
-		    ) {
-		    code = set_cmd_put_op(dp, cldev, pcls, op_delta2, 3);
-		    if (code < 0)
-			return code;
-		    dp[1] = (byte) ((delta >> 20) + (delta >> 16));
-		    dp[2] = (byte) ((delta >> 4) + delta);
-		    break;
-		}
-		code = set_cmd_put_op(dp, cldev, pcls, op, 5);
-		if (code < 0)
-		    return code;
-		*++dp = (byte) (color >> 24);
-		goto b3;
-	    case 4:
-		if (!((delta = diff + cmd_delta1_24_bias) &
-		      ~cmd_delta1_24_mask) &&
-		    (operand = (byte) (delta >> 16)) != 0 &&
-		    operand != 15
-		    ) {
-		    code = set_cmd_put_op(dp, cldev, pcls,
-					  (byte) (op + operand), 2);
-		    if (code < 0)
-			return code;
-		    dp[1] = (byte) ((delta >> 4) + delta);
-		    break;
-		}
-		if (!((delta = diff + cmd_delta2_24_bias) &
-		      ~cmd_delta2_24_mask)
-		    ) {
-		    code = set_cmd_put_op(dp, cldev, pcls, op_delta2, 3);
-		    if (code < 0)
-			return code;
-		    dp[1] = ((byte) (delta >> 13) & 0xf8) +
-			((byte) (delta >> 11) & 7);
-		    dp[2] = (byte) (((delta >> 3) & 0xe0) + delta);
-		    break;
-		}
-		code = set_cmd_put_op(dp, cldev, pcls, op, 4);
-		if (code < 0)
-		    return code;
-b3:		*++dp = (byte) (color >> 16);
-		goto b2;
-	    case 3:
-		code = set_cmd_put_op(dp, cldev, pcls, op, 3);
-		if (code < 0)
-		    return code;
-b2:		*++dp = (byte) (color >> 8);
-		goto b1;
-	    case 2:
-		if (diff >= -7 && diff < 7) {
-		    code = set_cmd_put_op(dp, cldev, pcls,
-					  op + (int)diff + 8, 1);
-		    if (code < 0)
-			return code;
-		    break;
-		}
-		code = set_cmd_put_op(dp, cldev, pcls, op, 2);
-		if (code < 0)
-		    return code;
-b1:		dp[1] = (byte) color;
+	/* Check if the "delta" mode command can be used. */
+	int num_bytes = (cldev->color_info.depth + 7) >> 3;
+	int delta_bytes = (num_bytes + 1) / 2;
+	gx_color_index delta_offset = cmd_delta_offsets[num_bytes];
+	gx_color_index delta_mask = cmd_delta_masks[num_bytes];
+	gx_color_index delta = (diff + delta_offset) & delta_mask;
+	bool use_delta = (color == (*pcolor + delta - delta_offset));
+	int bytes_dropped = 0;
+	gx_color_index data = color;
+	
+	/*
+	 * If we use the full value mode, we do not send low order bytes
+	 * which are zero. Determine how many low order bytes are zero.
+	 */
+	if (color == 0) {
+	    bytes_dropped = num_bytes;
+	}
+	else  {
+	    while ((data & 0xff) == 0) {
+	        bytes_dropped++;
+		data >>= 8; 
+	    }
+	}
+	
+	/* Now send one of the two command forms */
+	if (use_delta && delta_bytes < num_bytes - bytes_dropped) {
+	    code = set_cmd_put_op(dp, cldev, pcls,
+	    				op_delta, delta_bytes + 1);
+	    if (code < 0)
+	        return code;
+	    /*
+	     * If we have an odd number of bytes then use extra bits for
+	     * the high order three bytes of the color.
+	     */
+	    if ((num_bytes >= 3) && (num_bytes & 1)) {
+		data = delta >> ((num_bytes - 3) * 8);
+	        dp[delta_bytes--] = (byte)(((data >> 13) & 0xf8) + ((data >> 11) & 0x07));
+	        dp[delta_bytes--] = (byte)(((data >> 3) & 0xe0) + (data & 0x1f));
+	    }
+	    for(; delta_bytes>0; delta_bytes--) {
+	        dp[delta_bytes] = (byte)((delta >> 4) + delta);
+		delta >>= 16;
+	    }
+	}
+	else {
+	    num_bytes -= bytes_dropped;
+	    code = set_cmd_put_op(dp, cldev, pcls,
+	    			(byte)(op + bytes_dropped), num_bytes + 1);
+	    if (code < 0)
+	        return code;
+	    for(; num_bytes>0; num_bytes--) {
+	        dp[num_bytes] = (byte)data;
+		data >>= 8;
+	    }
 	}
     }
     *pcolor = color;
     return 0;
 }
 
+
 /* Put out a command to set the tile colors. */
 int
 cmd_set_tile_colors(gx_device_clist_writer * cldev, gx_clist_state * pcls,
@@ -580,14 +668,15 @@ cmd_put_params(gx_device_clist_writer *cldev,
 	gs_param_list_serialize(param_list, local_buf, sizeof(local_buf));
     if (param_length > 0) {
 	/* Get cmd buffer space for serialized */
-	code = set_cmd_put_all_op(dp, cldev, cmd_opv_put_params,
-				  1 + sizeof(unsigned) + param_length);
+	code = set_cmd_put_all_op(dp, cldev, cmd_opv_extend,
+				  2 + sizeof(unsigned) + param_length);
 	if (code < 0)
 	    return code;
 
 	/* write param list to cmd list: needs to all fit in cmd buffer */
 	if_debug1('l', "[l]put_params, length=%d\n", param_length);
-	++dp;
+	dp[1] = cmd_opv_ext_put_params;
+	dp += 2;
 	memcpy(dp, &param_length, sizeof(unsigned));
 	dp += sizeof(unsigned);
 	if (param_length > sizeof(local_buf)) {
@@ -611,12 +700,18 @@ cmd_put_params(gx_device_clist_writer *cldev,
 }
 
 /* Initialize CCITTFax filters. */
-private void
+static void
 clist_cf_init(stream_CF_state *ss, int width)
 {
     ss->K = -1;
     ss->Columns = width;
+#if 0 /* Disabled due to a crash with ppmraw -r216 c327.bin :
+	 the decoding filter overruns in 1 byte.         
+       */
     ss->EndOfBlock = false;
+#else
+    ss->EndOfBlock = true;
+#endif
     ss->BlackIs1 = true;
     ss->DecodedByteAlign = align_bitmap_mod;
 }
@@ -653,3 +748,16 @@ clist_rld_init(stream_RLD_state *ss)
     s_RLD_set_defaults_inline(ss);
     s_RLD_init_inline(ss);
 }
+
+/* Read a transformation matrix. */
+const byte *
+cmd_read_matrix(gs_matrix * pmat, const byte * cbp)
+{
+    stream s;
+
+    s_init(&s, NULL);
+    sread_string(&s, cbp, 1 + sizeof(*pmat));
+    sget_matrix(&s, pmat);
+    return cbp + stell(&s);
+}
+
