@@ -1,20 +1,17 @@
-/* Copyright (C) 1992, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxcmap.c,v 1.24 2005/03/14 18:08:36 dan Exp $ */
+/* $Id: gxcmap.c 8422 2007-12-03 21:31:16Z henrys $ */
 /* Color mapping for Ghostscript */
 #include "gx.h"
 #include "gserrors.h"
@@ -34,13 +31,13 @@
 
 /* Structure descriptor */
 public_st_device_color();
-private 
+static 
 ENUM_PTRS_WITH(device_color_enum_ptrs, gx_device_color *cptr)
 {
 	return ENUM_USING(*cptr->type->stype, vptr, size, index);
 }
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(device_color_reloc_ptrs, gx_device_color *cptr)
+static RELOC_PTRS_WITH(device_color_reloc_ptrs, gx_device_color *cptr)
 {
     RELOC_USING(*cptr->type->stype, vptr, size);
 }
@@ -298,7 +295,7 @@ rgb_cs_to_cmyk_cm(gx_device * dev, const gs_imager_state *pis,
         color_rgb_to_cmyk(r, g, b, pis, out);
     else {
         frac    c = frac_1 - r, m = frac_1 - g, y = frac_1 - b;
-        frac    k = min(c, min(m, g));
+        frac    k = min(c, min(m, y));
 
         out[0] = c - k;
         out[1] = m - k;
@@ -368,12 +365,21 @@ gx_error_get_color_mapping_procs(const gx_device * dev)
 {
     /*
      * We should never get here.  If we do then we do not have a "get_color_mapping_procs"
-     * routine for the device.
+     * routine for the device. This will be noisy, but better than returning NULL which
+     * would lead to SEGV (Segmentation Fault) errors when this is used.
      */
-#ifdef DEBUG
-    dprintf("No get_color_mapping_procs proc defined for device.\n");
-#endif
-    return NULL;
+    eprintf1("No get_color_mapping_procs proc defined for device '%s'\n", dev->dname);
+    switch (dev->color_info.num_components) {
+      case 1:     /* DeviceGray or DeviceInvertGray */
+	return gx_default_DevGray_get_color_mapping_procs(dev);
+
+      case 3:
+	return gx_default_DevRGB_get_color_mapping_procs(dev);
+
+      case 4:
+      default:		/* Unknown color model - punt with CMYK */
+        return gx_default_DevCMYK_get_color_mapping_procs(dev);
+    }
 }
     
 /* ----- Default color component name to colorant index conversion routines ------ */
@@ -461,32 +467,32 @@ gx_error_get_color_comp_index(gx_device * dev, const char * pname,
 
 /* ---------------- Device color rendering ---------------- */
 
-private cmap_proc_gray(cmap_gray_halftoned);
-private cmap_proc_gray(cmap_gray_direct);
+static cmap_proc_gray(cmap_gray_halftoned);
+static cmap_proc_gray(cmap_gray_direct);
 
-private cmap_proc_rgb(cmap_rgb_halftoned);
-private cmap_proc_rgb(cmap_rgb_direct);
+static cmap_proc_rgb(cmap_rgb_halftoned);
+static cmap_proc_rgb(cmap_rgb_direct);
 
 #define cmap_cmyk_halftoned cmap_cmyk_direct
-private cmap_proc_cmyk(cmap_cmyk_direct);
+static cmap_proc_cmyk(cmap_cmyk_direct);
 
-private cmap_proc_rgb_alpha(cmap_rgb_alpha_halftoned);
-private cmap_proc_rgb_alpha(cmap_rgb_alpha_direct);
+static cmap_proc_rgb_alpha(cmap_rgb_alpha_halftoned);
+static cmap_proc_rgb_alpha(cmap_rgb_alpha_direct);
 
 /* Procedure names are only guaranteed unique to 23 characters.... */
-private cmap_proc_rgb_alpha(cmap_rgb_alpha_halftoned);
-private cmap_proc_rgb_alpha(cmap_rgb_alpha_direct);
+static cmap_proc_rgb_alpha(cmap_rgb_alpha_halftoned);
+static cmap_proc_rgb_alpha(cmap_rgb_alpha_direct);
 
-private cmap_proc_separation(cmap_separation_halftoned);
-private cmap_proc_separation(cmap_separation_direct);
+static cmap_proc_separation(cmap_separation_halftoned);
+static cmap_proc_separation(cmap_separation_direct);
 
-private cmap_proc_devicen(cmap_devicen_halftoned);
-private cmap_proc_devicen(cmap_devicen_direct);
+static cmap_proc_devicen(cmap_devicen_halftoned);
+static cmap_proc_devicen(cmap_devicen_direct);
 
-private cmap_proc_is_halftoned(cmap_halftoned_is_halftoned);
-private cmap_proc_is_halftoned(cmap_direct_is_halftoned);
+static cmap_proc_is_halftoned(cmap_halftoned_is_halftoned);
+static cmap_proc_is_halftoned(cmap_direct_is_halftoned);
 
-private const gx_color_map_procs cmap_few = {
+static const gx_color_map_procs cmap_few = {
      cmap_gray_halftoned, 
      cmap_rgb_halftoned, 
      cmap_cmyk_halftoned,
@@ -495,7 +501,7 @@ private const gx_color_map_procs cmap_few = {
      cmap_devicen_halftoned,
      cmap_halftoned_is_halftoned
     };
-private const gx_color_map_procs cmap_many = {
+static const gx_color_map_procs cmap_many = {
      cmap_gray_direct,
      cmap_rgb_direct,
      cmap_cmyk_direct,
@@ -604,9 +610,7 @@ int
 gx_concretize_DeviceGray(const gs_client_color * pc, const gs_color_space * pcs,
 			 frac * pconc, const gs_imager_state * pis)
 {
-    float ftemp;
-
-    pconc[0] = unit_frac(pc->paint.values[0], ftemp);
+    pconc[0] = gx_unit_frac(pc->paint.values[0]);
     return 0;
 }
 int
@@ -614,6 +618,21 @@ gx_remap_concrete_DGray(const frac * pconc, const gs_color_space * pcs,
 	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
 			gs_color_select_t select)
 {
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    int code = pcb->client_procs->remap_DeviceGray(pcb, pconc,
+						pcs, pdc, pis, dev, select);
+
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
+
     if (pis->alpha == gx_max_color_value)
 	(*pis->cmap_procs->map_gray)
 	    (pconc[0], pdc, pis, dev, select);
@@ -628,13 +647,27 @@ gx_remap_DeviceGray(const gs_client_color * pc, const gs_color_space * pcs,
 	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
 		    gs_color_select_t select)
 {
-    float ftemp;
-    frac fgray = unit_frac(pc->paint.values[0], ftemp);
+    frac fgray = gx_unit_frac(pc->paint.values[0]);
 
     /* Save original color space and color info into dev color */
     pdc->ccolor.paint.values[0] = pc->paint.values[0];
     pdc->ccolor_valid = true;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    int code = pcb->client_procs->remap_DeviceGray(pcb, &fgray,
+						pcs, pdc, pis, dev, select);
+
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
+
     if (pis->alpha == gx_max_color_value)
 	(*pis->cmap_procs->map_gray)
 	    (fgray, pdc, pis, dev, select);
@@ -649,11 +682,9 @@ int
 gx_concretize_DeviceRGB(const gs_client_color * pc, const gs_color_space * pcs,
 			frac * pconc, const gs_imager_state * pis)
 {
-    float ftemp;
-
-    pconc[0] = unit_frac(pc->paint.values[0], ftemp);
-    pconc[1] = unit_frac(pc->paint.values[1], ftemp);
-    pconc[2] = unit_frac(pc->paint.values[2], ftemp);
+    pconc[0] = gx_unit_frac(pc->paint.values[0]);
+    pconc[1] = gx_unit_frac(pc->paint.values[1]);
+    pconc[2] = gx_unit_frac(pc->paint.values[2]);
     return 0;
 }
 int
@@ -661,6 +692,20 @@ gx_remap_concrete_DRGB(const frac * pconc, const gs_color_space * pcs,
 	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
 		       gs_color_select_t select)
 {
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    int code = pcb->client_procs->remap_DeviceRGB(pcb, pconc,
+						pcs, pdc, pis, dev, select);
+
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
     if (pis->alpha == gx_max_color_value)
 	gx_remap_concrete_rgb(pconc[0], pconc[1], pconc[2],
 			      pdc, pis, dev, select);
@@ -675,9 +720,8 @@ gx_remap_DeviceRGB(const gs_client_color * pc, const gs_color_space * pcs,
 	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
 		   gs_color_select_t select)
 {
-    float ftemp;
-    frac fred = unit_frac(pc->paint.values[0], ftemp), fgreen = unit_frac(pc->paint.values[1], ftemp),
-         fblue = unit_frac(pc->paint.values[2], ftemp);
+    frac fred = gx_unit_frac(pc->paint.values[0]), fgreen = gx_unit_frac(pc->paint.values[1]),
+         fblue = gx_unit_frac(pc->paint.values[2]);
 
     /* Save original color space and color info into dev color */
     pdc->ccolor.paint.values[0] = pc->paint.values[0];
@@ -685,6 +729,25 @@ gx_remap_DeviceRGB(const gs_client_color * pc, const gs_color_space * pcs,
     pdc->ccolor.paint.values[2] = pc->paint.values[2];
     pdc->ccolor_valid = true;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    frac conc[3];
+	    int code;
+
+	    conc[0] = fred;
+	    conc[1] = fgreen;
+	    conc[2] = fblue;
+	    code = pcb->client_procs->remap_DeviceRGB(pcb, conc,
+						pcs, pdc, pis, dev, select);
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
     if (pis->alpha == gx_max_color_value)
 	gx_remap_concrete_rgb(fred, fgreen, fblue,
 			      pdc, pis, dev, select);
@@ -699,12 +762,10 @@ int
 gx_concretize_DeviceCMYK(const gs_client_color * pc, const gs_color_space * pcs,
 			 frac * pconc, const gs_imager_state * pis)
 {
-    float ftemp;
-
-    pconc[0] = unit_frac(pc->paint.values[0], ftemp);
-    pconc[1] = unit_frac(pc->paint.values[1], ftemp);
-    pconc[2] = unit_frac(pc->paint.values[2], ftemp);
-    pconc[3] = unit_frac(pc->paint.values[3], ftemp);
+    pconc[0] = gx_unit_frac(pc->paint.values[0]);
+    pconc[1] = gx_unit_frac(pc->paint.values[1]);
+    pconc[2] = gx_unit_frac(pc->paint.values[2]);
+    pconc[3] = gx_unit_frac(pc->paint.values[3]);
     return 0;
 }
 int
@@ -713,6 +774,20 @@ gx_remap_concrete_DCMYK(const frac * pconc, const gs_color_space * pcs,
 			gs_color_select_t select)
 {
 /****** IGNORE alpha ******/
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    int code = pcb->client_procs->remap_DeviceCMYK(pcb, pconc,
+						pcs, pdc, pis, dev, select);
+
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
     gx_remap_concrete_cmyk(pconc[0], pconc[1], pconc[2], pconc[3], pdc,
 			   pis, dev, select);
     return 0;
@@ -723,8 +798,6 @@ gx_remap_DeviceCMYK(const gs_client_color * pc, const gs_color_space * pcs,
 		    gs_color_select_t select)
 {
 /****** IGNORE alpha ******/
-    float ft0, ft1, ft2, ft3;
-
     /* Save original color space and color info into dev color */
     pdc->ccolor.paint.values[0] = pc->paint.values[0];
     pdc->ccolor.paint.values[1] = pc->paint.values[1];
@@ -732,10 +805,30 @@ gx_remap_DeviceCMYK(const gs_client_color * pc, const gs_color_space * pcs,
     pdc->ccolor.paint.values[3] = pc->paint.values[3];
     pdc->ccolor_valid = true;
 
-    gx_remap_concrete_cmyk((frac)unit_frac(pc->paint.values[0], ft0),
-			   (frac)unit_frac(pc->paint.values[1], ft1),
-			   (frac)unit_frac(pc->paint.values[2], ft2),
-			   (frac)unit_frac(pc->paint.values[3], ft3),
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->custom_color_callback);
+
+        if (pcb != NULL) {
+	    frac conc[4];
+	    int code;
+
+	    conc[0] = gx_unit_frac(pc->paint.values[0]);
+	    conc[1] = gx_unit_frac(pc->paint.values[1]);
+	    conc[2] = gx_unit_frac(pc->paint.values[2]);
+	    conc[3] = gx_unit_frac(pc->paint.values[3]);
+	    code = pcb->client_procs->remap_DeviceCMYK(pcb, conc,
+						pcs, pdc, pis, dev, select);
+	    if (code == 0)
+		return 0;
+	}
+    }
+#endif
+    gx_remap_concrete_cmyk(gx_unit_frac(pc->paint.values[0]),
+			   gx_unit_frac(pc->paint.values[1]),
+			   gx_unit_frac(pc->paint.values[2]),
+			   gx_unit_frac(pc->paint.values[3]),
 			   pdc, pis, dev, select);
     return 0;
 }
@@ -743,7 +836,7 @@ gx_remap_DeviceCMYK(const gs_client_color * pc, const gs_color_space * pcs,
 
 /* ------ Render Gray color. ------ */
 
-private void
+static void
 cmap_gray_halftoned(frac gray, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
@@ -764,11 +857,11 @@ cmap_gray_halftoned(frac gray, gx_device_color * pdc,
 	    		(frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
 
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pis->dev_ht,
-    				&pis->screen_phase[select], true) == 1)
+	    				&pis->screen_phase[select]) == 1)
 	gx_color_load_select(pdc, pis, dev, select);
 }
 
-private void
+static void
 cmap_gray_direct(frac gray, gx_device_color * pdc, const gs_imager_state * pis,
 		 gx_device * dev, gs_color_select_t select)
 {
@@ -803,13 +896,12 @@ cmap_gray_direct(frac gray, gx_device_color * pdc, const gs_imager_state * pis,
 
 /* ------ Render RGB color. ------ */
 
-private void
+static void
 cmap_rgb_halftoned(frac r, frac g, frac b, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
     int i, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
-    bool gray_color = false;
 
     /* map to the color model */
     dev_proc(dev, get_color_mapping_procs)(dev)->map_rgb(dev, pis, r, g, b, cm_comps);
@@ -824,14 +916,12 @@ cmap_rgb_halftoned(frac r, frac g, frac b, gx_device_color * pdc,
             cm_comps[i] = frac_1 - gx_map_color_frac(pis,
 	    		(frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
 
-    if (ncomps == 3 && cm_comps[0] == cm_comps[1] && cm_comps[1] == cm_comps[2])
-	gray_color = true;
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pis->dev_ht,
-    				&pis->screen_phase[select], gray_color) == 1)
+	    				&pis->screen_phase[select]) == 1)
 	gx_color_load_select(pdc, pis, dev, select);
 }
 
-private void
+static void
 cmap_rgb_direct(frac r, frac g, frac b, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
@@ -866,7 +956,7 @@ cmap_rgb_direct(frac r, frac g, frac b, gx_device_color * pdc,
 
 /* ------ Render CMYK color. ------ */
 
-private void
+static void
 cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
@@ -892,7 +982,7 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
     /* duplicating most of the code of this procedure. */
     if (gx_device_must_halftone(dev)) {
 	if (gx_render_device_DeviceN(cm_comps, pdc, dev,
-		    pis->dev_ht, &pis->screen_phase[select], false) == 1)
+		    pis->dev_ht, &pis->screen_phase[select]) == 1)
 	    gx_color_load_select(pdc, pis, dev, select);
 	return;
     }
@@ -905,13 +995,13 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
 	color_set_pure(pdc, color);
     else {
 	if (gx_render_device_DeviceN(cm_comps, pdc, dev,
-		    pis->dev_ht, &pis->screen_phase[select], false) == 1)
+		    pis->dev_ht, &pis->screen_phase[select]) == 1)
 	    gx_color_load_select(pdc, pis, dev, select);
 	return;
     }
 }
 
-private void
+static void
 cmap_rgb_alpha_halftoned(frac r, frac g, frac b, frac alpha,
 	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
 			 gs_color_select_t select)
@@ -945,11 +1035,11 @@ cmap_rgb_alpha_halftoned(frac r, frac g, frac b, frac alpha,
 	    		(frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
 
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pis->dev_ht,
-    				&pis->screen_phase[select], false) == 1)
+	    				&pis->screen_phase[select]) == 1)
 	gx_color_load_select(pdc, pis, dev, select);
 }
 
-private void
+static void
 cmap_rgb_alpha_direct(frac r, frac g, frac b, frac alpha, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
@@ -1013,7 +1103,7 @@ cmap_rgb_alpha_direct(frac r, frac g, frac b, frac alpha, gx_device_color * pdc,
  * Returns:
  *    Mapped components in plist.
  */
-private inline void
+static inline void
 map_components_to_colorants(const frac * pcc,
 	const gs_devicen_color_map * pcolor_component_map, frac * plist)
 {
@@ -1033,7 +1123,7 @@ map_components_to_colorants(const frac * pcc,
     }
 }
 
-private void
+static void
 cmap_separation_halftoned(frac all, gx_device_color * pdc,
      const gs_imager_state * pis, gx_device * dev, gs_color_select_t select)
 {
@@ -1071,11 +1161,11 @@ cmap_separation_halftoned(frac all, gx_device_color * pdc,
 	    		(frac)(frac_1 - cm_comps[i]), effective_transfer[i]);
 
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pis->dev_ht,
-    				&pis->screen_phase[select], false) == 1)
+    					&pis->screen_phase[select]) == 1)
 	gx_color_load_select(pdc, pis, dev, select);
 }
 
-private void
+static void
 cmap_separation_direct(frac all, gx_device_color * pdc, const gs_imager_state * pis,
 		 gx_device * dev, gs_color_select_t select)
 {
@@ -1131,7 +1221,7 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_imager_state *
  * This routine is called to map a DeviceN colorspace to a DeviceN
  * output device which requires halftoning.  T
  */
-private void
+static void
 cmap_devicen_halftoned(const frac * pcc, 
     gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
     gs_color_select_t select)
@@ -1155,7 +1245,7 @@ cmap_devicen_halftoned(const frac * pcc,
     /* We need to finish halftoning */
 
     if (gx_render_device_DeviceN(cm_comps, pdc, dev, pis->dev_ht,
-    				&pis->screen_phase[select], false) == 1)
+    					&pis->screen_phase[select]) == 1)
 	gx_color_load_select(pdc, pis, dev, select);
 }
 
@@ -1163,7 +1253,7 @@ cmap_devicen_halftoned(const frac * pcc,
  * This routine is called to map a DeviceN colorspace to a DeviceN
  * output device which does not require halftoning.
  */
-private void
+static void
 cmap_devicen_direct(const frac * pcc, 
     gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
     gs_color_select_t select)
@@ -1198,13 +1288,13 @@ cmap_devicen_direct(const frac * pcc,
 
 /* ------ Halftoning check ----- */
 
-private bool
+static bool
 cmap_halftoned_is_halftoned(const gs_imager_state * pis, gx_device * dev)
 {
     return true;
 }
 
-private bool
+static bool
 cmap_direct_is_halftoned(const gs_imager_state * pis, gx_device * dev)
 {
     return false;
@@ -1256,7 +1346,7 @@ gx_color_frac_map(frac cv, const frac * values)
     if (rem == 0)
 	return mv;
     mdv = values[cmi + 1] - mv;
-#if arch_ints_are_short
+#if ARCH_INTS_ARE_SHORT
     /* Only use long multiplication if necessary. */
     if (mdv < -1 << (16 - cp_frac_bits) ||
 	mdv > 1 << (16 - cp_frac_bits)
@@ -1370,8 +1460,9 @@ gx_default_rgb_map_rgb_color(gx_device * dev, const gx_color_value cv[])
     else {
 	int bpc = dev->color_info.depth / 3;
 	int drop = sizeof(gx_color_value) * 8 - bpc;
-
-	return ((((cv[0] >> drop) << bpc) + (cv[1] >> drop)) << bpc) + (cv[2] >> drop);
+	return ( ( (((gx_color_index)cv[0] >> drop) << bpc) +
+		    ((gx_color_index)cv[1] >> drop)         ) << bpc) + 
+	       ((gx_color_index)cv[2] >> drop);
     }
 }
 
@@ -1508,3 +1599,16 @@ gx_default_map_color_rgb_alpha(gx_device * dev, gx_color_index color,
     prgba[3] = gx_max_color_value;	/* alpha = 1 */
     return (*dev_proc(dev, map_color_rgb)) (dev, color, prgba);
 }
+
+frac
+gx_unit_frac(float fvalue)
+{
+    frac f = frac_0;
+    if (is_fneg(fvalue))
+        f = frac_0;
+    else if (is_fge1(fvalue))
+        f = frac_1;
+    else
+        f = float2frac(fvalue);
+    return f;
+}
