@@ -1,20 +1,17 @@
-/* Copyright (C) 1992, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxdcconv.c,v 1.3.6.1 2002/02/22 19:45:58 ray Exp $ */
+/* $Id: gxdcconv.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Conversion between device color spaces for Ghostscript */
 #include "gx.h"
 #include "gsdcolor.h"		/* for gxcmap.h */
@@ -25,6 +22,8 @@
 #include "gxlum.h"
 #include "gxistate.h"
 
+extern bool CPSI_mode;		/* not worth polluting a header file */
+
 /*
  * The CMYK to RGB algorithms specified by Adobe are, e.g.,
  *      R = 1.0 - min(1.0, C + K)
@@ -32,9 +31,10 @@
  * We got better results on displays with
  *      R = (1.0 - C) * (1.0 - K)
  *      C = max(0.0, min(1.0, 1 - R / (1 - UCR)))
- * For utmost compatibility, we offer the Adobe algorithms as an option:
+ * For PLRM compatibility, we use the Adobe algorithms by default,
+ * but what Adobe says and what they do are two different things.
+ * Testing on CPSI shows that they use the 'better' algorithm.
  */
-#define USE_ADOBE_CMYK_RGB
 
 /* ------ Color space conversion ------ */
 
@@ -66,10 +66,10 @@ color_rgb_to_cmyk(frac r, frac g, frac b, const gs_imager_state * pis,
      * but they must agree with the ones in gs_init.ps.
      */
     frac bg =
-	(pis->black_generation == NULL ? frac_0 :
+	(pis == NULL ? k : pis->black_generation == NULL ? frac_0 :
 	 gx_map_color_frac(pis, k, black_generation));
     signed_frac ucr =
-	(pis->undercolor_removal == NULL ? frac_0 :
+	(pis == NULL ? k : pis->undercolor_removal == NULL ? frac_0 :
 	 gx_map_color_frac(pis, k, undercolor_removal));
 
     if (ucr == frac_1)
@@ -77,28 +77,29 @@ color_rgb_to_cmyk(frac r, frac g, frac b, const gs_imager_state * pis,
     else if (ucr == frac_0)
 	cmyk[0] = c, cmyk[1] = m, cmyk[2] = y;
     else {
-#ifdef USE_ADOBE_CMYK_RGB
-	/* C = max(0.0, min(1.0, 1 - R - UCR)), etc. */
-	signed_frac not_ucr = (ucr < 0 ? frac_1 + ucr : frac_1);
+	if (! CPSI_mode) {
+	    /* C = max(0.0, min(1.0, 1 - R - UCR)), etc. */
+	    signed_frac not_ucr = (ucr < 0 ? frac_1 + ucr : frac_1);
 
-	cmyk[0] = (c < ucr ? frac_0 : c > not_ucr ? frac_1 : c - ucr);
-	cmyk[1] = (m < ucr ? frac_0 : m > not_ucr ? frac_1 : m - ucr);
-	cmyk[2] = (y < ucr ? frac_0 : y > not_ucr ? frac_1 : y - ucr);
-#else
-	/* C = max(0.0, min(1.0, 1 - R / (1 - UCR))), etc. */
-	float denom = frac2float(frac_1 - ucr);		/* unscaled */
-	float v;
+	    cmyk[0] = (c < ucr ? frac_0 : c > not_ucr ? frac_1 : c - ucr);
+	    cmyk[1] = (m < ucr ? frac_0 : m > not_ucr ? frac_1 : m - ucr);
+	    cmyk[2] = (y < ucr ? frac_0 : y > not_ucr ? frac_1 : y - ucr);
+	} else {
+	    /* Adobe CPSI method */
+	    /* C = max(0.0, min(1.0, 1 - R / (1 - UCR))), etc. */
+	    float denom = frac2float(frac_1 - ucr);		/* unscaled */
+	    float v;
 
-	v = (float)frac_1 - r / denom;	/* unscaled */
-	cmyk[0] =
-	    (is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
-	v = (float)frac_1 - g / denom;	/* unscaled */
-	cmyk[1] =
-	    (is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
-	v = (float)frac_1 - b / denom;	/* unscaled */
-	cmyk[2] =
-	    (is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
-#endif
+	    v = (float)frac_1 - r / denom;	/* unscaled */
+	    cmyk[0] =
+		(is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
+	    v = (float)frac_1 - g / denom;	/* unscaled */
+	    cmyk[1] =
+		(is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
+	    v = (float)frac_1 - b / denom;	/* unscaled */
+	    cmyk[2] =
+		(is_fneg(v) ? frac_0 : v >= (float)frac_1 ? frac_1 : (frac) v);
+	}
     }
     cmyk[3] = bg;
     if_debug7('c', "[c]RGB 0x%x,0x%x,0x%x -> CMYK 0x%x,0x%x,0x%x,0x%x\n",
@@ -130,15 +131,14 @@ color_cmyk_to_rgb(frac c, frac m, frac y, frac k, const gs_imager_state * pis,
 	    rgb[0] = rgb[1] = rgb[2] = frac_0;
 	    break;
 	default:
-	    {
-#ifdef USE_ADOBE_CMYK_RGB
+	    if (! CPSI_mode) {
 		/* R = 1.0 - min(1.0, C + K), etc. */
 		frac not_k = frac_1 - k;
 
 		rgb[0] = (c > not_k ? frac_0 : not_k - c);
 		rgb[1] = (m > not_k ? frac_0 : not_k - m);
 		rgb[2] = (y > not_k ? frac_0 : not_k - y);
-#else
+	    } else {
 		/* R = (1.0 - C) * (1.0 - K), etc. */
 		ulong not_k = frac_1 - k;
 
@@ -151,7 +151,6 @@ color_cmyk_to_rgb(frac c, frac m, frac y, frac k, const gs_imager_state * pis,
 		rgb[1] = deduct_black(m);
 		rgb[2] = deduct_black(y);
 #undef deduct_black
-#endif
 	    }
     }
     if_debug7('c', "[c]CMYK 0x%x,0x%x,0x%x,0x%x -> RGB 0x%x,0x%x,0x%x\n",
