@@ -1,20 +1,17 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxdevcli.h,v 1.37 2005/04/04 20:28:27 igor Exp $ */
+/* $Id: gxdevcli.h 8250 2007-09-25 13:31:24Z giles $ */
 /* Definitions for device clients */
 
 #ifndef gxdevcli_INCLUDED
@@ -37,6 +34,7 @@
 #include "gxfixed.h"
 #include "gxtext.h"
 #include "gxcmap.h"
+#include "gsnamecl.h"
 
 /* See Drivers.htm for documentation of the driver interface. */
 
@@ -143,6 +141,10 @@ typedef struct gx_stroke_params_s gx_stroke_params;
 #  define gs_imager_state_DEFINED
 typedef struct gs_imager_state_s gs_imager_state;
 #endif
+#ifndef patch_fill_state_t_DEFINED
+#  define patch_fill_state_t_DEFINED
+typedef struct patch_fill_state_s  patch_fill_state_t;
+#endif
 
 /* We need an abstract type for the image enumeration state, */
 /* for begin[_typed]_image. */
@@ -320,7 +322,7 @@ typedef struct gx_device_color_info_s {
 
     /*
      * The number of bits of gx_color_index actually used. 
-     * This must be <= sizeof(gx_color_index), which is usually 64.
+     * This must be <= arch_sizeof_color_index, which is usually 64.
      */
     byte depth;
 
@@ -372,11 +374,11 @@ typedef struct gx_device_color_info_s {
      * with configurable color models which may be set for a single
      * non-'gray' color model.
      */
-    gx_color_value max_gray;	/* # of distinct color levels -1 */
-    gx_color_value max_color;
+    uint max_gray;		/* # of distinct color levels -1 */
+    uint max_color;
 
-    gx_color_value dither_grays;
-    gx_color_value dither_colors;
+    uint dither_grays;
+    uint dither_colors;
 
     /*
      * Information to control super-sampling of objects to support
@@ -496,7 +498,7 @@ typedef struct gx_device_color_info_s {
                              : "DeviceCMYK") )
 
 #define dci_std_polarity(nc)                    \
-    ( (nc) == 4 ? GX_CINFO_POLARITY_SUBTRACTIVE \
+    ( (nc) >= 4 ? GX_CINFO_POLARITY_SUBTRACTIVE \
                 : GX_CINFO_POLARITY_ADDITIVE )
 
       /*
@@ -569,16 +571,12 @@ typedef struct gx_device_color_info_s {
 #define dci_std_color_max_gray(nc, color_bits)            \
     ( (nc) == 3                                           \
         ? 0                                               \
-        : ( dci_std_gray_bits(nc, color_bits) >= 8        \
-            ? 255                                         \
-            : (1 << dci_std_gray_bits(nc, color_bits)) - 1 ) )
+        : (1 << dci_std_gray_bits(nc, color_bits)) - 1 ) 
 
 #define dci_std_color_max_color(nc, color_bits)               \
     ( (nc) == 1                                               \
         ? 0                                                   \
-        : ( dci_std_color_bits(nc, color_bits) >= 8           \
-            ? 255                                             \
-            : (1 << dci_std_color_bits(nc, color_bits)) - 1 ) )
+        : (1 << dci_std_color_bits(nc, color_bits)) - 1 )
 
 
 /*
@@ -692,7 +690,7 @@ typedef struct gx_device_cached_colors_s {
 	gx_device_cached_colors_t cached_colors;\
 	int width;			/* width in pixels */\
 	int height;			/* height in pixels */\
-        int TrayOrientation;            /* default 0 ( 90 180 270 ) if device supports */\
+        int LeadingEdge;                /* see below */\
 	float MediaSize[2];		/* media dimensions in points */\
 	float ImagingBBox[4];		/* imageable region in points */\
 	  bool ImagingBBox_set;\
@@ -710,9 +708,28 @@ typedef struct gx_device_cached_colors_s {
 	bool IgnoreNumCopies;		/* if true, force num_copies = 1 */\
 	bool UseCIEColor;		/* for PS LL3 */\
 	bool LockSafetyParams;		/* If true, prevent unsafe changes */\
+	long band_offset_x;		/* offsets of clist band base to (mem device) buffer */\
+	long band_offset_y;		/* for rendering that is phase sensitive (wtsimdi) */\
 	gx_page_device_procs page_procs;	/* must be last */\
 		/* end of std_device_body */\
 	gx_device_procs procs	/* object procedures */
+
+#define LEADINGEDGE_MASK 3
+#define LEADINGEDGE_SET_MASK (1 << 2)
+#define LEADINGEDGE_REQ_BIT (1 << 3)
+#define LEADINGEDGE_REQ_VAL_SHIFT 4
+#define LEADINGEDGE_REQ_VAL (LEADINGEDGE_MASK << LEADINGEDGE_REQ_VAL_SHIFT)
+/*
+ * The lower two bits of LeadingEdge correspond to the pagedevice
+ * parameter of the same name. The next bit (hex value 4) is set if
+ * the LeadingEdge was set explicitly by the user using setpagedevice.
+ * Otherwise, the value is the default as determined by the device (or
+ * zero if the device has no logic for setting LeadingEdge based on
+ * other parameters (such as %MediaSource). This field replaces the
+ * earlier TrayOrientation, which had a similar purpose but was not
+ * compatible with the PostScript spec.
+ */
+
 /*
  * Note: x/y_pixels_per_inch are here only for backward compatibility.
  * They should not be used in new code.
@@ -750,7 +767,6 @@ typedef struct gx_device_cached_colors_s {
 #define assign_dev_procs(todev, fromdev)\
   ((todev)->procs = (fromdev)->procs)
 
-
 /* ---------------- Device procedures ---------------- */
 
 /* Define an opaque type for parameter lists. */
@@ -1211,7 +1227,10 @@ typedef enum {
     pattern_manage__start_accum,
     pattern_manage__finish_accum,
     pattern_manage__load,
-    pattern_manage__shading_area
+    pattern_manage__shading_area,
+    pattern_manage__is_cpath_accum,
+    pattern_manage__shfill_doesnt_need_path,
+    pattern_manage__handles_clip_path
 } pattern_manage_t;
 
 #define dev_t_proc_pattern_manage(proc, dev_t)\
@@ -1260,6 +1279,7 @@ typedef struct gs_fill_attributes_s {
       const gx_device_halftone *ht; /* Reserved for possible use in future. */
       gs_logical_operation_t lop; /* Reserved for possible use in future. */
       fixed ystart, yend; /* Only for X-independent gradients. Base coordinates of the gradient. */
+      patch_fill_state_t *pfs; /* For gx_fill_triangle_small. Clients must not change. */
 } gs_fill_attributes;
 
 /* Fill a linear color scanline. */
@@ -1280,12 +1300,12 @@ typedef struct gs_fill_attributes_s {
 /* [p0 : p1] - left edge, from bottom to top.
    [p2 : p3] - right edge, from bottom to top.
    The filled area is within Y-spans of both edges. */
-/* If either (c0 and c1) or (c2 and c3) may be NULL.
+/* Either (c0 and c1) or (c2 and c3) may be NULL.
    In this case the color doesn't depend on X (on Y if fa->swap_axes).
    In this case the base coordinates for the color gradient
    may be unequal to p0, p1, p2, p3, and must be provided/taken
    in/from fa->ystart, fa->yend. 
-   The rerurn value 0 is not allowed in this case. */
+   The return value 0 is not allowed in this case. */
 /* Return values : 
   1 - success;
   0 - Too big. The area isn't filled. The client must decompose the area.
@@ -1327,6 +1347,20 @@ typedef struct gs_fill_attributes_s {
 #define dev_proc_update_spot_equivalent_colors(proc)\
   dev_t_proc_update_spot_equivalent_colors(proc, gx_device)
 
+/*
+ * return a pointer to the devn_params section of a device.  Return NULL
+ * if this field is not present within the device.
+ */
+#ifndef gs_devn_params_DEFINED
+#define gs_devn_params_DEFINED
+typedef struct gs_devn_params_s gs_devn_params;
+#endif
+
+#define dev_t_proc_ret_devn_params(proc, dev_t)\
+  gs_devn_params * proc(dev_t *dev)
+#define dev_proc_ret_devn_params(proc)\
+  dev_t_proc_ret_devn_params(proc, gx_device)
+
 /* Define the device procedure vector template proper. */
 
 #define gx_device_proc_struct(dev_t)\
@@ -1390,6 +1424,7 @@ typedef struct gs_fill_attributes_s {
 	dev_t_proc_fill_linear_color_trapezoid((*fill_linear_color_trapezoid), dev_t); \
 	dev_t_proc_fill_linear_color_triangle((*fill_linear_color_triangle), dev_t); \
 	dev_t_proc_update_spot_equivalent_colors((*update_spot_equivalent_colors), dev_t); \
+	dev_t_proc_ret_devn_params((*ret_devn_params), dev_t); \
 }
 
 
@@ -1548,6 +1583,8 @@ void gx_device_init(gx_device * dev, const gx_device * proto,
 /* or the allocator that was used to allocate it if it is a real object. */
 void gs_make_null_device(gx_device_null *dev_null, gx_device *target,
 			 gs_memory_t *mem);
+/* Is a null device ? */
+bool gs_is_null_device(gx_device *dev);
 
 /* Set the target of a (forwarding) device. */
 void gx_device_set_target(gx_device_forward *fdev, gx_device *target);
@@ -1606,7 +1643,7 @@ typedef struct gx_device_type_s {
 } gx_device_type;
 
 #define device_type(dtname, stype, initproc)\
-private dev_type_proc_initialize(initproc);\
+static dev_type_proc_initialize(initproc);\
 const gx_device_type dtname = { &stype, initproc }
 
 /*dev_type_proc_initialize(gdev_initialize); */
