@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxipixel.c,v 1.10 2004/03/13 22:31:19 ray Exp $ */
+/* $Id: gxipixel.c 8536 2008-02-20 21:02:48Z leonardo $ */
 /* Common code for ImageType 1 and 4 initialization */
 #include "gx.h"
 #include "math_.h"
@@ -46,12 +43,12 @@ private_st_gx_image_enum();
 extern_gx_image_class_table();
 
 /* Enumerator procedures */
-private const gx_image_enum_procs_t image1_enum_procs = {
+static const gx_image_enum_procs_t image1_enum_procs = {
     gx_image1_plane_data, gx_image1_end_image, gx_image1_flush
 };
 
 /* GC procedures */
-private 
+static 
 ENUM_PTRS_WITH(image_enum_enum_ptrs, gx_image_enum *eptr)
 {
     int bps;
@@ -79,7 +76,7 @@ ENUM_PTRS_WITH(image_enum_enum_ptrs, gx_image_enum *eptr)
 gx_image_enum_do_ptrs(e1)
 #undef e1
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(image_enum_reloc_ptrs, gx_image_enum *eptr)
+static RELOC_PTRS_WITH(image_enum_reloc_ptrs, gx_image_enum *eptr)
 {
     int i;
 
@@ -101,10 +98,10 @@ private RELOC_PTRS_WITH(image_enum_reloc_ptrs, gx_image_enum *eptr)
 RELOC_PTRS_END
 
 /* Forward declarations */
-private int color_draws_b_w(gx_device * dev,
+static int color_draws_b_w(gx_device * dev,
 			    const gx_drawing_color * pdcolor);
-private void image_init_map(byte * map, int map_size, const float *decode);
-private void image_init_colors(gx_image_enum * penum, int bps, int spp,
+static void image_init_map(byte * map, int map_size, const float *decode);
+static void image_init_colors(gx_image_enum * penum, int bps, int spp,
 			       gs_image_format_t format,
 			       const float *decode,
 			       const gs_imager_state * pis, gx_device * dev,
@@ -194,7 +191,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     const int bps = pim->BitsPerComponent;
     bool masked = penum->masked;
     const float *decode = pim->Decode;
-    gs_matrix mat;
+    gs_matrix_double mat;
     int index_bps;
     const gs_color_space *pcs = pim->ColorSpace;
     gs_logical_operation_t lop = (pis ? pis->log_op : lop_default);
@@ -205,36 +202,60 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     byte *buffer;
     fixed mtx, mty;
     gs_fixed_point row_extent, col_extent, x_extent, y_extent;
-    bool device_color;
+    bool device_color = true;
     gs_fixed_rect obox, cbox;
 
+    penum->Width = width;
+    penum->Height = height;
     if (pmat == 0)
 	pmat = &ctm_only(pis);
-    if ((code = gs_matrix_invert(&pim->ImageMatrix, &mat)) < 0 ||
-	(code = gs_matrix_multiply(&mat, pmat, &mat)) < 0
+    if ((code = gs_matrix_invert_to_double(&pim->ImageMatrix, &mat)) < 0 ||
+	(code = gs_matrix_multiply_double(&mat, pmat, &mat)) < 0
 	) {
 	gs_free_object(mem, penum, "gx_default_begin_image");
 	return code;
     }
-    penum->matrix = mat;
+    /*penum->matrix = mat;*/
+    penum->matrix.xx = mat.xx;
+    penum->matrix.xy = mat.xy;
+    penum->matrix.yx = mat.yx;
+    penum->matrix.yy = mat.yy;
+    penum->matrix.tx = mat.tx;
+    penum->matrix.ty = mat.ty;
     if_debug6('b', " [%g %g %g %g %g %g]\n",
 	      mat.xx, mat.xy, mat.yx, mat.yy, mat.tx, mat.ty);
     /* following works for 1, 2, 4, 8, 12, 16 */
     index_bps = (bps < 8 ? bps >> 1 : (bps >> 2) + 1);
-    mtx = float2fixed(mat.tx);
-    mty = float2fixed(mat.ty);
-    row_extent.x = float2fixed(width * mat.xx + mat.tx) - mtx;
+    /* 
+     * Compute extents with distance transformation.
+     */
+    if (mat.tx > 0)
+	mtx = float2fixed(mat.tx);
+    else { /* Use positive values to ensure round down. */
+	int f = (int)-mat.tx + 1;
+
+	mtx = float2fixed(mat.tx + f) - int2fixed(f);
+    }
+    if (mat.ty > 0)
+	mty = float2fixed(mat.ty);
+    else {  /* Use positive values to ensure round down. */
+	int f = (int)-mat.ty + 1;
+
+	mty = float2fixed(mat.ty + f) - int2fixed(f);
+    }
+
+    row_extent.x = float2fixed_rounded(width * mat.xx);
     row_extent.y =
 	(is_fzero(mat.xy) ? fixed_0 :
-	 float2fixed(width * mat.xy + mat.ty) - mty);
+	 float2fixed_rounded(width * mat.xy));
     col_extent.x =
 	(is_fzero(mat.yx) ? fixed_0 :
-	 float2fixed(height * mat.yx + mat.tx) - mtx);
-    col_extent.y = float2fixed(height * mat.yy + mat.ty) - mty;
+	 float2fixed_rounded(height * mat.yx));
+    col_extent.y = float2fixed_rounded(height * mat.yy);
     gx_image_enum_common_init((gx_image_enum_common_t *)penum,
 			      (const gs_data_image_t *)pim,
 			      &image1_enum_procs, dev,
-			      (masked ? 1 : cs_num_components(pcs)),
+			      (masked ? 1 : (penum->alpha ? cs_num_components(pcs)+1 : cs_num_components(pcs))),
 			      format);
     if (penum->rect.w == width && penum->rect.h == height) {
 	x_extent = row_extent;
@@ -242,14 +263,14 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     } else {
 	int rw = penum->rect.w, rh = penum->rect.h;
 
-	x_extent.x = float2fixed(rw * mat.xx + mat.tx) - mtx;
+	x_extent.x = float2fixed_rounded(rw * mat.xx);
 	x_extent.y =
 	    (is_fzero(mat.xy) ? fixed_0 :
-	     float2fixed(rw * mat.xy + mat.ty) - mty);
+	     float2fixed_rounded(rw * mat.xy));
 	y_extent.x =
 	    (is_fzero(mat.yx) ? fixed_0 :
-	     float2fixed(rh * mat.yx + mat.tx) - mtx);
-	y_extent.y = float2fixed(rh * mat.yy + mat.ty) - mty;
+	     float2fixed_rounded(rh * mat.yx));
+	y_extent.y = float2fixed_rounded(rh * mat.yy);
     }
     if (masked) {	/* This is imagemask. */
 	if (bps != 1 || pcs != NULL || penum->alpha || decode[0] == decode[1]) {
@@ -501,9 +522,16 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
 		  penum->clip_image);
 	dda_init(penum->dda.row.x, mtx, col_extent.x, height);
 	dda_init(penum->dda.row.y, mty, col_extent.y, height);
+	penum->dst_width = row_extent.x;
+	penum->dst_height = col_extent.y;
+	penum->yi0 = fixed2int_pixround_perfect(dda_current(penum->dda.row.y)); /* For gs_image_class_0_interpolate. */
 	if (penum->rect.y) {
-	    dda_advance(penum->dda.row.x, penum->rect.y);
-	    dda_advance(penum->dda.row.y, penum->rect.y);
+	    int y = penum->rect.y;
+
+	    while (y--) {
+		dda_next(penum->dda.row.x);
+		dda_next(penum->dda.row.y);
+	    }
 	}
 	penum->cur.x = penum->prev.x = dda_current(penum->dda.row.x);
 	penum->cur.y = penum->prev.y = dda_current(penum->dda.row.y);
@@ -532,24 +560,41 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
     penum->used.x = 0;
     penum->used.y = 0;
     {
-	static sample_unpack_proc_t procs[6] = {
-	    sample_unpack_1, sample_unpack_2,
+	static sample_unpack_proc_t procs[2][6] = {
+	{   sample_unpack_1, sample_unpack_2,
 	    sample_unpack_4, sample_unpack_8,
 	    0, 0
-	};
+	}, 
+	{   sample_unpack_1_interleaved, sample_unpack_2_interleaved,
+	    sample_unpack_4_interleaved, sample_unpack_8_interleaved,
+	    0, 0
+	}};
+	int num_planes = penum->num_planes;
+	bool interleaved = (num_planes == 1 && penum->plane_depths[0] != penum->bps);
 	int i;
 
-        procs[4] = sample_unpack_12_proc;
-        procs[5] = sample_unpack_16_proc;
+	procs[0][4] = procs[1][4] = sample_unpack_12_proc;
+	procs[0][5] = procs[1][5] = sample_unpack_16_proc;
+	if (interleaved) {
+	    int num_components = penum->plane_depths[0] / penum->bps;
+
+	    for (i = 1; i < num_components; i++) {
+		if (decode[0] != decode[i * 2 + 0] ||
+		    decode[1] != decode[i * 2 + 1])
+		    break;
+	    }
+	    if (i == num_components)
+		interleaved = false; /* Use single table. */
+	}
 	if (index_bps >= 4) {
-	    if ((penum->unpack = procs[index_bps]) == 0) {		/* bps case not supported. */
+	    if ((penum->unpack = procs[interleaved][index_bps]) == 0) {		/* bps case not supported. */
 		gx_default_end_image(dev,
 				     (gx_image_enum_common_t *) penum,
 				     false);
 		return_error(gs_error_rangecheck);
 	    }
 	} else {
-	    penum->unpack = procs[index_bps];
+	    penum->unpack = procs[interleaved][index_bps];
 	}
 	if_debug1('b', "[b]unpack=%d\n", bps);
 	/* Set up pixel0 for image class procedures. */
@@ -575,10 +620,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
 				 false);
 	    return_error(gs_error_VMerror);
 	}
-	gx_make_clip_translate_device(cdev, gx_cpath_list(pcpath), 0, 0, mem);
-	gx_device_retain((gx_device *)cdev, true); /* will free explicitly */
-	gx_device_set_target((gx_device_forward *)cdev, dev);
-	(*dev_proc(cdev, open_device)) ((gx_device *) cdev);
+	gx_make_clip_device_in_heap(cdev, pcpath, dev, mem);
 	penum->clip_dev = cdev;
     }
     if (penum->use_rop) {	/* Set up the RasterOp source device. */
@@ -602,7 +644,7 @@ gx_image_enum_begin(gx_device * dev, const gs_imager_state * pis,
 
 /* If a drawing color is black or white, return 0 or 1 respectively, */
 /* otherwise return -1. */
-private int
+static int
 color_draws_b_w(gx_device * dev, const gx_drawing_color * pdcolor)
 {
     if (color_is_pure(pdcolor)) {
@@ -660,7 +702,7 @@ image_init_clues(gx_image_enum * penum, int bps, int spp)
 }
 
 /* Initialize the color mapping tables for a non-mask image. */
-private void
+static void
 image_init_colors(gx_image_enum * penum, int bps, int spp,
 		  gs_image_format_t format, const float *decode /*[spp*2] */ ,
 		  const gs_imager_state * pis, gx_device * dev,
@@ -770,7 +812,7 @@ image_init_colors(gx_image_enum * penum, int bps, int spp,
 /* Construct a mapping table for sample values. */
 /* map_size is 2, 4, 16, or 256.  Note that 255 % (map_size - 1) == 0, */
 /* so the division 0xffffL / (map_size - 1) is always exact. */
-private void
+static void
 image_init_map(byte * map, int map_size, const float *decode)
 {
     float min_v = decode[0];
