@@ -1,20 +1,17 @@
-/* Copyright (C) 1995, 1996, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxline.h,v 1.2.6.1 2002/02/22 19:45:59 ray Exp $ */
+/* $Id: gxline.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Private line parameter definitions */
 
 #ifndef gxline_INCLUDED
@@ -59,20 +56,19 @@ typedef struct gx_line_params_s {
   ((plp)->half_width = (wid) / 2)
 #define gx_current_line_width(plp)\
   ((plp)->half_width * 2)
-int gx_set_miter_limit(P2(gx_line_params *, floatp));
+int gx_set_miter_limit(gx_line_params *, floatp);
 
 #define gx_current_miter_limit(plp)\
   ((plp)->miter_limit)
-int gx_set_dash(P5(gx_dash_params *, const float *, uint, floatp,
-		   gs_memory_t *));
+int gx_set_dash(gx_dash_params *, const float *, uint, floatp, gs_memory_t *);
 
 #define gx_set_dash_adapt(pdp, adpt) ((pdp)->adapt = (adpt))
-int gx_set_dot_length(P3(gx_line_params *, floatp, bool));
+int gx_set_dot_length(gx_line_params *, floatp, bool);
 
 /* See gsline.c for the computation of miter_check. */
 #define gx_line_params_initial\
- 0.0, gs_cap_butt, gs_join_miter, gs_join_bevel /* for Adobe compatibility */,\
- 10.0, 0.20305866, 0.0, 0/*false*/,\
+ 0.0, gs_cap_butt, gs_join_miter, -1,\
+ 10.0, (float)0.20305866, 0.0, 0/*false*/,\
   { identity_matrix_body }, { gx_dash_params_initial }
 
 #endif /* gxline_INCLUDED */
