@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxpaint.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gxpaint.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Graphics-state-aware fill and stroke procedures */
 #include "gx.h"
 #include "gzstate.h"
@@ -21,7 +21,7 @@
 #include "gxpath.h"
 #include "gxfont.h"
 
-private bool caching_an_outline_font(const gs_state * pgs)
+static bool caching_an_outline_font(const gs_state * pgs)
 {
     return pgs->in_cachedevice > 1 &&
 	    pgs->font != NULL &&
