@@ -1,29 +1,26 @@
-/* Copyright (C) 1997, 1998 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxpflat.c,v 1.44 2004/12/08 21:35:13 stefan Exp $ */
+/* $Id: gxpflat.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Path flattening algorithms */
 #include "string_.h"
 #include "gx.h"
+#include "gserrors.h"
 #include "gxarith.h"
 #include "gxfixed.h"
 #include "gzpath.h"
 #include "memory_.h"
 #include "vdtrace.h"
-#include <assert.h>
 
 /* ---------------- Curve flattening ---------------- */
 
@@ -103,7 +100,7 @@ gx_curve_log2_samples(fixed x0, fixed y0, const curve_segment * pc,
  * Curve and Surface Representation," B. A. Barsky and A. D. DeRose, IEEE,
  * 1985, courtesy of Crispin Goswell.
  */
-private void
+static void
 split_curve_midpoint(fixed x0, fixed y0, const curve_segment * pc,
 		     curve_segment * pc1, curve_segment * pc2)
 {				/*
@@ -136,7 +133,7 @@ split_curve_midpoint(fixed x0, fixed y0, const curve_segment * pc,
 #undef midpoint
 }
 
-private inline void
+static inline void
 print_points(const gs_fixed_point *points, int count)
 {
 #ifdef DEBUG    
@@ -145,7 +142,7 @@ print_points(const gs_fixed_point *points, int count)
     if (!gs_debug_c('3'))
 	return;
     for (i = 0; i < count; i++)
-	if_debug2('3', "[3]out x=%d y=%d\n", points[i].x, points[i].y);
+	if_debug2('3', "[3]out x=%ld y=%ld\n", points[i].x, points[i].y);
 #endif
 }
 
@@ -268,23 +265,77 @@ gx_flattened_iterator__init(gx_flattened_iterator *this,
     return true;
 }
 
+static inline bool 
+check_diff_overflow(fixed v0, fixed v1)
+{
+    if (v0 < v1) {
+	if (v1 - v0 < 0)
+	    return true;
+    } else {
+	if (v0 - v1 < 0)
+	    return true;
+    }
+    return false;
+}
+
+bool
+gx_check_fixed_diff_overflow(fixed v0, fixed v1)
+{
+    return check_diff_overflow(v0, v1);
+}
+bool
+gx_check_fixed_sum_overflow(fixed v0, fixed v1)
+{
+    /* We assume that clamp_point_aux have been applied to v1,
+       thus -v alweays exists.
+     */
+    return check_diff_overflow(v0, -v1);
+}
+
 /*  Initialize the iterator with a line. */
 bool
 gx_flattened_iterator__init_line(gx_flattened_iterator *this, 
 	    fixed x0, fixed y0, fixed x1, fixed y1)
 {
+    bool ox = check_diff_overflow(x0, x1);
+    bool oy = check_diff_overflow(y0, y1);
+
     this->x0 = this->lx0 = this->lx1 = x0;
     this->y0 = this->ly0 = this->ly1 = y0;
     this->x3 = x1;
     this->y3 = y1;
-    this->k = 0;
-    this->i = 1;
+    if (ox || oy) {
+	/* Subdivide a long line into 4 segments, because the filling algorithm 
+	   and the stroking algorithm need to compute differences 
+	   of coordinates of end points. 
+	   We can't use 2 segments, because gx_flattened_iterator__next
+	   implements a special code for that case, 
+	   which requires differences of coordinates as well.
+	 */
+	/* Note : the result of subdivision may be not strongly colinear. */
+	this->ax = this->bx = 0;
+	this->ay = this->by = 0;
+	this->cx = ((ox ? (x1 >> 1) - (x0 >> 1) : (x1 - x0) >> 1) + 1) >> 1;
+	this->cy = ((oy ? (y1 >> 1) - (y0 >> 1) : (y1 - y0) >> 1) + 1) >> 1;
+	this->rd3x = this->rd3y = this->id3x = this->id3y = 0;
+	this->rd2x = this->rd2y = this->id2x = this->id2y = 0;
+	this->idx = this->cx; 
+	this->idy = this->cy;
+	this->rdx = this->rdy = 0;
+	this->rx = this->ry = 0;
+	this->rmask = 0;
+	this->k = 2;
+	this->i = 4;
+    } else {
+	this->k = 0;
+	this->i = 1;
+    }
     this->curve = false;
     return true;
 }
 
 #ifdef DEBUG
-private inline void
+static inline void
 gx_flattened_iterator__print_state(gx_flattened_iterator *this)
 {
     if (!gs_debug_c('3'))
@@ -304,7 +355,7 @@ gx_flattened_iterator__print_state(gx_flattened_iterator *this)
 /* Move to the next segment and store it to this->lx0, this->ly0, this->lx1, this->ly1 .
  * Return true iff there exist more segments.
  */
-bool
+int
 gx_flattened_iterator__next(gx_flattened_iterator *this)
 {
     /*
@@ -343,6 +394,8 @@ gx_flattened_iterator__next(gx_flattened_iterator *this)
      * variables, we don't actually compute M, only M-1 (rmask).  */
     fixed x = this->lx1, y = this->ly1;
 
+    if (this->i <= 0)
+	return_error(gs_error_unregistered); /* Must not happen. */
     this->lx0 = this->lx1;
     this->ly0 = this->ly1;
     /* Fast check for N == 3, a common special case for small characters. */
@@ -356,7 +409,7 @@ gx_flattened_iterator__next(gx_flattened_iterator *this)
 #	undef poly2
 	if_debug2('3', "[3]dx=%f, dy=%f\n",
 		  fixed2float(x - this->x0), fixed2float(y - this->y0));
-	if_debug5('3', "[3]%s x=%g, y=%g x=%d y=%d\n",
+	if_debug5('3', "[3]%s x=%g, y=%g x=%ld y=%ld\n",
 		  (((x ^ this->x0) | (y ^ this->y0)) & float2fixed(-0.5) ?
 		   "add" : "skip"),
 		  fixed2float(x), fixed2float(y), x, y);
@@ -379,7 +432,7 @@ gx_flattened_iterator__next(gx_flattened_iterator *this)
 	accum(this->idy, this->rdy, this->id2y, this->rd2y, this->rmask);
 	accum(this->id2x, this->rd2x, this->id3x, this->rd3x, this->rmask);
 	accum(this->id2y, this->rd2y, this->id3y, this->rd3y, this->rmask);
-	if_debug5('3', "[3]%s x=%g, y=%g x=%d y=%d\n",
+	if_debug5('3', "[3]%s x=%g, y=%g x=%ld y=%ld\n",
 		  (((x ^ this->lx0) | (y ^ this->ly0)) & float2fixed(-0.5) ?
 		   "add" : "skip"),
 		  fixed2float(x), fixed2float(y), x, y);
@@ -392,13 +445,13 @@ gx_flattened_iterator__next(gx_flattened_iterator *this)
 last:
     this->lx1 = this->x3;
     this->ly1 = this->y3;
-    if_debug4('3', "[3]last x=%g, y=%g x=%d y=%d\n",
+    if_debug4('3', "[3]last x=%g, y=%g x=%ld y=%ld\n",
 	      fixed2float(this->lx1), fixed2float(this->ly1), this->lx1, this->ly1);
     vd_bar(this->lx0, this->ly0, this->lx1, this->ly1, 1, RGB(0, 255, 0));
     return false;
 }
 
-private inline void
+static inline void
 gx_flattened_iterator__unaccum(gx_flattened_iterator *this)
 {
 #   define unaccum(i, r, di, dr, rmask)\
@@ -417,12 +470,13 @@ gx_flattened_iterator__unaccum(gx_flattened_iterator *this)
  * This only works for states reached with gx_flattened_iterator__next.
  * Return true iff there exist more segments.
  */
-bool
+int
 gx_flattened_iterator__prev(gx_flattened_iterator *this)
 {
     bool last; /* i.e. the first one in the forth order. */
 
-    assert(this->i < 1 << this->k);
+    if (this->i >= 1 << this->k)
+	return_error(gs_error_unregistered); /* Must not happen. */
     this->lx1 = this->lx0;
     this->ly1 = this->ly0;
     if (this->k <= 1) {
@@ -441,7 +495,7 @@ gx_flattened_iterator__prev(gx_flattened_iterator *this)
     gx_flattened_iterator__unaccum(this);
     this->i++;
 #   ifdef DEBUG
-    if_debug5('3', "[3]%s x=%g, y=%g x=%d y=%d\n",
+    if_debug5('3', "[3]%s x=%g, y=%g x=%ld y=%ld\n",
 	      (((this->x ^ this->lx1) | (this->y ^ this->ly1)) & float2fixed(-0.5) ?
 	       "add" : "skip"),
 	      fixed2float(this->x), fixed2float(this->y), this->x, this->y);
@@ -452,7 +506,8 @@ gx_flattened_iterator__prev(gx_flattened_iterator *this)
     this->ly0 = this->y;
     vd_bar(this->lx0, this->ly0, this->lx1, this->ly1, 1, RGB(0, 0, 255));
     if (last)
-	assert(this->lx0 == this->x0 && this->ly0 == this->y0);
+	if (this->lx0 != this->x0 || this->ly0 != this->y0)
+	    return_error(gs_error_unregistered); /* Must not happen. */
     return !last;
 }
 
@@ -463,7 +518,7 @@ gx_flattened_iterator__switch_to_backscan(gx_flattened_iterator *this, bool not_
     /*	When scanning forth, the accumulator stands on the end of a segment,
 	except for the last segment.
 	When scanning back, the accumulator should stand on the beginning of a segment.
-	Asuuming that at least one forward step is done.
+	Assuming at least one forward step is done.
     */
     if (not_first)
 	if (this->i > 0 && this->k != 1 /* This case doesn't use the accumulator. */)
@@ -472,7 +527,7 @@ gx_flattened_iterator__switch_to_backscan(gx_flattened_iterator *this, bool not_
 
 #define max_points 50		/* arbitrary */
 
-private int
+static int
 generate_segments(gx_path * ppath, const gs_fixed_point *points, 
 		    int count, segment_notes notes)
 {
@@ -495,7 +550,7 @@ generate_segments(gx_path * ppath, const gs_fixed_point *points,
     }
 }
 
-private int
+static int
 gx_subdivide_curve_rec(gx_flattened_iterator *this, 
 		  gx_path * ppath, int k, curve_segment * pc,
 		  segment_notes notes, gs_fixed_point *points)
@@ -524,7 +579,11 @@ top :
 	bool more;
 
 	for(;;) {
-	    more = gx_flattened_iterator__next(this);
+	    code = gx_flattened_iterator__next(this);
+
+	    if (code < 0)
+		return code;
+	    more = code;
 	    ppt->x = this->lx1;
 	    ppt->y = this->ly1;
 	    ppt++;
