@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxstroke.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gxstroke.c 8286 2007-10-10 23:02:18Z leonardo $ */
 /* Path stroking procedures for Ghostscript library */
 #include "math_.h"
 #include "gx.h"
@@ -82,7 +82,7 @@
  * check and covers many common cases.  Clients that care always have the
  * option of using strokepath to get an exact result.
  */
-private float join_expansion_factor(const gs_imager_state *, gs_line_join);
+static float join_expansion_factor(const gs_imager_state *, gs_line_join);
 int
 gx_stroke_path_expansion(const gs_imager_state * pis, const gx_path * ppath,
 			 gs_fixed_point * ppt)
@@ -114,6 +114,7 @@ gx_stroke_path_expansion(const gs_imager_state * pis, const gx_path * ppath,
 		    goto not_exact;
 		break;
 	    case s_line:
+	    case s_dash:
 	    case s_line_close:
 		if (!(pseg->pt.x == prev.x || pseg->pt.y == prev.y))
 		    goto not_exact;
@@ -154,7 +155,7 @@ not_exact:
 
     return result;
 }
-private float
+static float
 join_expansion_factor(const gs_imager_state *pis, gs_line_join join)
 {
     switch (join) {
@@ -196,6 +197,7 @@ typedef struct partial_line_s {
     endpoint o;			/* starting coordinate */
     endpoint e;			/* ending coordinate */
     gs_fixed_point width;	/* one-half line width, see above */
+    gs_fixed_point vector;	/* The line segment direction */
     bool thin;			/* true if minimum-width line */
 } partial_line;
 typedef partial_line *pl_ptr;
@@ -206,18 +208,18 @@ typedef partial_line *pl_ptr;
   ((pp)->x = (p).x, (pp)->y = (p).y)
 
 /* Other forward declarations */
-private bool width_is_thin(pl_ptr);
-private void adjust_stroke(pl_ptr, const gs_imager_state *, bool);
-private int line_join_points(const gx_line_params * pgs_lp,
+static bool width_is_thin(pl_ptr);
+static void adjust_stroke(pl_ptr, const gs_imager_state *, bool, bool);
+static int line_join_points(const gx_line_params * pgs_lp,
 			     pl_ptr plp, pl_ptr nplp,
 			     gs_fixed_point * join_points,
 			     const gs_matrix * pmat, gs_line_join join,
 			     bool reflected);
-private void compute_caps(pl_ptr);
-private int add_points(gx_path *, const gs_fixed_point *,
+static void compute_caps(pl_ptr);
+static int add_points(gx_path *, const gs_fixed_point *,
 		       int, bool);
-private int add_round_cap(gx_path *, const_ep_ptr);
-private int cap_points(gs_line_cap, const_ep_ptr,
+static int add_round_cap(gx_path *, const_ep_ptr);
+static int cap_points(gs_line_cap, const_ep_ptr,
 		       gs_fixed_point * /*[3] */ );
 
 /* Define the default implementation of the device stroke_path procedure. */
@@ -234,7 +236,7 @@ gx_default_stroke_path(gx_device * dev, const gs_imager_state * pis,
 /* Fill a partial stroked path.  Free variables: */
 /* to_path, stroke_path_body, fill_params, always_thin, pis, dev, pdevc, */
 /* code, ppath, exit(label). */
-#define FILL_STROKE_PATH(thin)\
+#define FILL_STROKE_PATH(dev, thin, pcpath)\
   if(to_path==&stroke_path_body && !gx_path_is_void(&stroke_path_body)) {\
     fill_params.adjust.x = STROKE_ADJUSTMENT(thin, pis, x);\
     fill_params.adjust.y = STROKE_ADJUSTMENT(thin, pis, y);\
@@ -258,8 +260,12 @@ gx_default_stroke_path(gx_device * dev, const gs_imager_state * pis,
 	   gs_line_join, bool)
 typedef stroke_line_proc((*stroke_line_proc_t));
 
-private stroke_line_proc(stroke_add);
-private stroke_line_proc(stroke_fill);
+static stroke_line_proc(stroke_add);
+static stroke_line_proc(stroke_add_compat);
+static stroke_line_proc(stroke_fill);
+static int stroke_add_initial_cap_compat(gx_path * ppath, pl_ptr plp, bool adlust_longitude,
+	   const gx_device_color * pdevc, gx_device * dev,
+	   const gs_imager_state * pis);
 
 /* Define the orientations we handle specially. */
 typedef enum {
@@ -270,19 +276,20 @@ typedef enum {
 
 /*
  * Stroke a path.  If to_path != 0, append the stroke outline to it;
- * if to_path == 0, draw the strokes on dev.
+ * if to_path == 0, draw the strokes on pdev.
  *
  * Note that gx_stroke_path_only with to_path != NULL may clip the path to
  * the clipping path, as for to_path == NULL.  This is almost never
  * what is wanted.
  */
-private int
+static int
 gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 	       const gs_imager_state * pis, const gx_stroke_params * params,
 		 const gx_device_color * pdevc, const gx_clip_path * pcpath)
 {
+    extern bool CPSI_mode;
     stroke_line_proc_t line_proc =
-	(to_path == 0 ? stroke_fill : stroke_add);
+	(to_path == 0 ? stroke_fill : CPSI_mode ? stroke_add_compat : stroke_add);
     gs_fixed_rect ibox, cbox;
     gx_device_clip cdev;
     gx_device *dev = pdev;
@@ -398,7 +405,7 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
     if (pcpath)
 	gx_cpath_inner_box(pcpath, &cbox);
     else if (pdevc)
-	(*dev_proc(dev, get_clipping_box)) (dev, &cbox);
+	(*dev_proc(pdev, get_clipping_box)) (pdev, &cbox);
     else {
 	/* This is strokepath, not stroke.  Don't clip. */
 	cbox = ibox;
@@ -426,15 +433,23 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 	 * If we had to flatten the path, this is where we would
 	 * recompute its bbox and make the tests again,
 	 * but we don't bother right now.
-	 *
-	 * If there is a clipping path, set up a clipping device.
 	 */
-	if (pcpath) {
-	    gx_make_clip_device(&cdev, gx_cpath_list(pcpath));
-	    cdev.target = dev;
-	    cdev.max_fill_band = dev->max_fill_band;
-	    dev = (gx_device *) & cdev;
-	    (*dev_proc(dev, open_device)) (dev);
+	/*
+	 * If there is a clipping path, set up a clipping device.
+	 * for stroke_fill because, because the latter uses low level methods 
+	 * which don't accept a clipping path.
+	 * Note that in some cases stroke_fill appends the path to stroke_path_body
+	 * instead a real painting, and it is painted with FILL_STROKE_PATH.
+	 * 
+	 * Contrary to that, FILL_STROKE_PATH paints a path with 
+	 * the fill_path method, which handles a clipping path,
+	 * so we don't pass the clipper device to FILL_STROKE_PATH
+	 * to prevent an appearence of superposing clippers.
+	 */
+	if (pcpath && line_proc == stroke_fill) {
+	    gx_make_clip_device_on_stack(&cdev, pcpath, pdev);
+	    cdev.max_fill_band = pdev->max_fill_band;
+	    dev = (gx_device *)&cdev;
 	}
     }
     fill_params.rule = gx_rule_winding_number;
@@ -472,15 +487,23 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 	    default:
 		{
 		    /* The check is more complicated, but it's worth it. */
-		    double xsq = xx * xx + xy * xy;
-		    double ysq = yx * yx + yy * yy;
-		    double cross = xx * yx + xy * yy;
-
-		    if (cross < 0)
-			cross = 0;
-		    always_thin =
-			((max(xsq, ysq) + cross) * line_width * line_width
-			 < 0.25);
+		    /* Compute radii of the transformed round brush. */
+		    /* Let x = [a, sqrt(1-a^2)]' 
+		       radius^2 is an extremum of :
+		       rr(a)=(CTM*x)^2 = (a*xx + sqrt(1 - a^2)*xy)^2 + (a*yx + sqrt(1 - a^2)*yy)^2
+		       With solving D(rr(a),a)==0, got :
+		       max_rr = (xx^2 + xy^2 + yx^2 + yy^2 + sqrt(((xy + yx)^2 + (xx - yy)^2)*((xy - yx)^2 + (xx + yy)^2)))/2.
+		       r = sqrt(max_rr);
+		       Well we could use eigenvalues of the quadratic form,
+		       but it gives same result with a bigger calculus.
+		     */
+		    double max_rr = (xx*xx + xy*xy + yx*yx + yy*yy + 
+					sqrt( ((xy + yx)*(xy + yx) + (xx - yy)*(xx - yy)) * 
+					      ((xy - yx)*(xy - yx) + (xx + yy)*(xx + yy)) 
+					    )
+				     )/2;
+		    
+		    always_thin = max_rr * line_width * line_width < 0.25;
 		}
 	}
     }
@@ -502,7 +525,8 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 	device_dot_length *= fabs(pmat->xy) + fabs(pmat->yy);
     }
     /* Start by flattening the path.  We should do this on-the-fly.... */
-    if (!gx_path_has_curves(ppath)) {	/* don't need to flatten */
+    if (!gx_path_has_curves(ppath) && !gx_path_has_long_segments(ppath)) {	
+	/* don't need to flatten */
 	if (!ppath->first_subpath)
 	    return 0;
 	spath = ppath;
@@ -533,42 +557,57 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 	fixed y = pseg->pt.y;
 	bool is_closed = ((const subpath *)pseg)->is_closed;
 	partial_line pl, pl_prev, pl_first;
+	bool zero_length = true;
 
 	while ((pseg = pseg->next) != 0 &&
 	       pseg->type != s_start
 	    ) {
 	    /* Compute the width parameters in device space. */
 	    /* We work with unscaled values, for speed. */
-	    fixed sx = pseg->pt.x, udx = sx - x;
-	    fixed sy = pseg->pt.y, udy = sy - y;
+	    fixed sx, udx, sy, udy;
+	    bool is_dash_segment = false;
 
+         d1:if (pseg->type != s_dash) {
+		sx = pseg->pt.x;
+		sy = pseg->pt.y;
+		udx = sx - x;
+		udy = sy - y;
+	    } else {
+		dash_segment *pd = (dash_segment *)pseg;
+
+		sx = pd->pt.x;
+		sy = pd->pt.y;
+		udx = pd->tangent.x;
+		udy = pd->tangent.y;
+		is_dash_segment = true;
+	    }
+	    zero_length &= ((udx | udy) == 0);
 	    pl.o.p.x = x, pl.o.p.y = y;
 	  d:pl.e.p.x = sx, pl.e.p.y = sy;
-	    if (!(udx | udy)) {	/* degenerate */
+	    if (!(udx | udy) || pseg->type == s_dash) {	/* degenerate or short */
 		/*
 		 * If this is the first segment of the subpath,
 		 * check the entire subpath for degeneracy.
 		 * Otherwise, ignore the degenerate segment.
 		 */
-		if (index != 0)
+		if (index != 0 && pseg->type != s_dash)
 		    continue;
 		/* Check for a degenerate subpath. */
 		while ((pseg = pseg->next) != 0 &&
 		       pseg->type != s_start
 		    ) {
+		    if (is_dash_segment)
+			break;
+		    if (pseg->type == s_dash)
+			goto d1;
 		    sx = pseg->pt.x, udx = sx - x;
 		    sy = pseg->pt.y, udy = sy - y;
-		    if (udx | udy)
+		    if (udx | udy) {
+			zero_length = false;
 			goto d;
+		    }
 		}
-		/*
-		 * The entire subpath is degenerate, but it includes
-		 * more than one point.  If the dot length is non-zero,
-		 * draw the caps, otherwise do nothing.
-		 */
-		if (pgs_lp->dot_length != 0)
-		    break;
-		if (pgs_lp->cap != gs_cap_round) {
+		if (pgs_lp->dot_length == 0 && pgs_lp->cap != gs_cap_round && !is_dash_segment) {
 		    /* From PLRM, stroke operator :
 		       If a subpath is degenerate (consists of a single-point closed path 
 		       or of two or more points at the same coordinates), 
@@ -576,25 +615,27 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 		    break;
 		}
 		/*
-		 * Orient the dot according to the previous segment if
+		 * If the subpath is a dash, take the orientation from the dash segmnent.
+		 * Otherwise orient the dot according to the previous segment if
 		 * any, or else the next segment if any, or else
 		 * according to the specified dot orientation.
 		 */
 		{
-		    const segment *end = psub->prev;
+		    /* When passing here, either pseg == NULL or it points to the
+		       start of the next subpaph. So we can't use pseg 
+		       for determining the segment direction.
+		       In same time, psub->last may help, so use it. */
+		    const segment *end = psub->last;
 
-		    if (end != 0 && (end->pt.x != x || end->pt.y != y))
-			sx = end->pt.x, sy = end->pt.y;
-		    else if (pseg != 0 &&
-			     (pseg->pt.x != x || pseg->pt.y != y)
-			)
-			sx = pseg->pt.x, sy = pseg->pt.y;
+		    if (is_dash_segment) {
+			/* Nothing. */
+		    } else if (end != 0 && (end->pt.x != x || end->pt.y != y))
+			sx = end->pt.x, sy = end->pt.y,	udx = sx - x, udy = sy - y;
 		}
 		/*
 		 * Compute the properly oriented dot length, and then
 		 * draw the dot like a very short line.
 		 */
-		udx = sx - x, udy = sy - y;
 		if ((udx | udy) == 0) {
 		    if (is_fzero(pgs_lp->dot_orientation.xy)) {
 			/* Portrait orientation, dot length = X */
@@ -604,33 +645,38 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 			udy = fixed_1;
 		    }
 		}
-		{
+		if (sx == x && sy == y && (pseg == NULL || pseg->type == s_start)) {
 		    double scale = device_dot_length /
-		    hypot((double)udx, (double)udy);
-
+				hypot((double)udx, (double)udy);
+		    fixed udx1, udy1;
 		    /*
 		     * If we're using butt caps, make sure the "line" is
-		     * long enough to show up.
+    		     * long enough to show up.
+		     * Don't apply this with always_thin, becase
+		     * draw thin line always rounds the length up.
 		     */
-		    if (pgs_lp->cap == gs_cap_butt) {
+		    if (!always_thin && pgs_lp->cap == gs_cap_butt) {
 			fixed dmax = max(any_abs(udx), any_abs(udy));
 
 			if (dmax * scale < fixed_1)
 			    scale = (float)fixed_1 / dmax;
 		    }
-		    udx = (fixed) (udx * scale);
-		    udy = (fixed) (udy * scale);
-		    if ((udx | udy) == 0)
-			udy = fixed_epsilon;
-		    sx = x + udx;
-		    sy = y + udy;
+		    udx1 = (fixed) (udx * scale);
+		    udy1 = (fixed) (udy * scale);
+		    sx = x + udx1;
+		    sy = y + udy1;
 		}
 		/*
 		 * Back up 1 segment to keep the bookkeeping straight.
 		 */
 		pseg = (pseg != 0 ? pseg->prev : psub->last);
-		goto d;
+		if (!is_dash_segment)
+		    goto d;
+		pl.e.p.x = sx;
+		pl.e.p.y = sy;
 	    }
+	    pl.vector.x = udx;
+	    pl.vector.y = udy;
 	    if (always_thin) {
 		pl.e.cdelta.x = pl.e.cdelta.y = 0;
 		pl.width.x = pl.width.y = 0;
@@ -692,7 +738,10 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 		    pl.thin = width_is_thin(&pl);
 		}
 		if (!pl.thin) {
-		    adjust_stroke(&pl, pis, false);
+		    adjust_stroke(&pl, pis, false, 
+			    (pseg->prev == 0 || pseg->prev->type == s_start) && 
+			    (pseg->next == 0 || pseg->next->type == s_start) &&
+			    (zero_length || !is_closed));
 		    compute_caps(&pl);
 		}
 	    }
@@ -717,7 +766,7 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 				     uniform, join, initial_matrix_reflected);
 		if (code < 0)
 		    goto exit;
-		FILL_STROKE_PATH(always_thin);
+		FILL_STROKE_PATH(pdev, always_thin, pcpath);
 	    } else
 		pl_first = pl;
 	    pl_prev = pl;
@@ -731,7 +780,7 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 	    /* For some reason, the Borland compiler requires the cast */
 	    /* in the following statement. */
 	    pl_ptr lptr =
-		(!is_closed || join == gs_join_none ?
+		(!is_closed || join == gs_join_none || zero_length ?
 		 (pl_ptr) 0 : (pl_ptr) & pl_first);
 
 	    code = (*line_proc) (to_path, index - 1, &pl_prev, lptr, pdevc,
@@ -739,7 +788,14 @@ gx_stroke_path_only_aux(gx_path * ppath, gx_path * to_path, gx_device * pdev,
 				 initial_matrix_reflected);
 	    if (code < 0)
 		goto exit;
-	    FILL_STROKE_PATH(always_thin);
+	    FILL_STROKE_PATH(pdev, always_thin, pcpath);
+	    if (CPSI_mode && lptr == 0 && pgs_lp->cap != gs_cap_butt) {
+		/* Create the initial cap at last. */
+		code = stroke_add_initial_cap_compat(to_path, &pl_first, index == 1, pdevc, dev, pis);
+		if (code < 0)
+		    goto exit;
+		FILL_STROKE_PATH(pdev, always_thin, pcpath);
+	    }
 	}
 	psub = (const subpath *)pseg;
     }
@@ -784,7 +840,7 @@ gx_stroke_path_only(gx_path * ppath, gx_path * to_path, gx_device * pdev,
  * Unfortunately, the width values we computed are perpendicular to the
  * line in *user* space, so we may have to do some extra work.
  */
-private bool
+static bool
 width_is_thin(pl_ptr plp)
 {
     fixed dx, dy, wx = plp->width.x, wy = plp->width.y;
@@ -820,11 +876,9 @@ width_is_thin(pl_ptr plp)
     }
 }
 
-/* Adjust the endpoints and width of a stroke segment */
-/* to achieve more uniform rendering. */
-/* Only o.p, e.p, e.cdelta, and width have been set. */
-private void
-adjust_stroke(pl_ptr plp, const gs_imager_state * pis, bool thin)
+/* Adjust the endpoints and width of a stroke segment along a specified axis */
+static void
+adjust_stroke_transversal(pl_ptr plp, const gs_imager_state * pis, bool thin, bool horiz)
 {
     fixed *pw;
     fixed *pov;
@@ -832,9 +886,7 @@ adjust_stroke(pl_ptr plp, const gs_imager_state * pis, bool thin)
     fixed w, w2;
     fixed adj2;
 
-    if (!pis->stroke_adjust && plp->width.x != 0 && plp->width.y != 0)
-	return;			/* don't adjust */
-    if (any_abs(plp->width.x) < any_abs(plp->width.y)) {
+    if (horiz) {
 	/* More horizontal stroke */
 	pw = &plp->width.y, pov = &plp->o.p.y, pev = &plp->e.p.y;
 	adj2 = STROKE_ADJUSTMENT(thin, pis, y) << 1;
@@ -873,12 +925,80 @@ adjust_stroke(pl_ptr plp, const gs_imager_state * pis, bool thin)
     }
 }
 
+static void 
+adjust_stroke_longitude(pl_ptr plp, const gs_imager_state * pis, bool thin, bool horiz)
+{
+
+    fixed *pow = (horiz ? &plp->o.p.y : &plp->o.p.x);
+    fixed *pew = (horiz ? &plp->e.p.y : &plp->e.p.x);
+
+    /* Only adjust the endpoints if the line is horizontal or vertical. 
+       Debugged with pdfwrite->ppmraw 72dpi file2.pdf */
+    if (*pow == *pew) {
+	fixed *pov = (horiz ? &plp->o.p.x : &plp->o.p.y);
+	fixed *pev = (horiz ? &plp->e.p.x : &plp->e.p.y);
+	fixed length = any_abs(*pov - *pev);
+	fixed length_r, length_r_2;
+	fixed mv = (*pov + *pev) / 2, mv_r;
+	fixed adj2 = (horiz ? STROKE_ADJUSTMENT(thin, pis, x)
+			    : STROKE_ADJUSTMENT(thin, pis, y)) << 1;
+        
+	if (length > fixed_1) /* comparefiles/file2.pdf */
+	    return;
+	if (pis->line_params.cap == gs_cap_butt) {
+	    length_r = fixed_rounded(length);
+	    if (length_r < fixed_1)
+		length_r = fixed_1;
+	    length_r_2 = length_r / 2;
+	} else {
+	    /* Account width for proper placing cap centers. */
+	    fixed width = any_abs(horiz ? plp->width.y : plp->width.x);
+
+	    length_r = fixed_rounded(length + width * 2 + adj2);
+	    length_r_2 = fixed_rounded(length) / 2;
+	}
+	if (length_r & fixed_1)
+	    mv_r = fixed_floor(mv) + fixed_half;
+	else 
+	    mv_r = fixed_floor(mv);
+	if (*pov < *pev) {
+	    *pov = mv_r - length_r_2;
+	    *pev = mv_r + length_r_2;
+	} else {
+	    *pov = mv_r + length_r_2;
+	    *pev = mv_r - length_r_2;
+	}
+    }
+}
+
+/* Adjust the endpoints and width of a stroke segment */
+/* to achieve more uniform rendering. */
+/* Only o.p, e.p, e.cdelta, and width have been set. */
+static void
+adjust_stroke(pl_ptr plp, const gs_imager_state * pis, bool thin, bool adjust_longitude)
+{
+    bool horiz;
+
+    if (!pis->stroke_adjust && plp->width.x != 0 && plp->width.y != 0)
+	return;			/* don't adjust */
+    horiz = (any_abs(plp->width.x) <= any_abs(plp->width.y));
+    adjust_stroke_transversal(plp, pis, thin, horiz);
+    if (adjust_longitude)
+	adjust_stroke_longitude(plp, pis, thin, horiz);
+    /* fixme :
+       The best value for adjust_longitude is whether 
+       the dash is isolated and doesn't cover entire segment.
+       The current data structure can't pass this info.
+       Therefore we restrict adjust_stroke_longitude with 1 pixel length.
+    */
+}
+
 /* Compute the intersection of two lines.  This is a messy algorithm */
 /* that somehow ought to be useful in more places than just here.... */
 /* If the lines are (nearly) parallel, return -1 without setting *pi; */
 /* otherwise, return 0 if the intersection is beyond *pp1 and *pp2 in */
 /* the direction determined by *pd1 and *pd2, and 1 otherwise. */
-private int
+static int
 line_intersect(
 		  p_ptr pp1,	/* point on 1st line */
 		  p_ptr pd1,	/* slope of 1st line (dx,dy) */
@@ -924,7 +1044,7 @@ line_intersect(
 
 /* Set up the width and delta parameters for a thin line. */
 /* We only approximate the width and height. */
-private void
+static void
 set_thin_widths(register pl_ptr plp)
 {
     fixed dx = plp->e.p.x - plp->o.p.x, dy = plp->e.p.y - plp->o.p.y;
@@ -942,7 +1062,7 @@ set_thin_widths(register pl_ptr plp)
 
 /* Draw a line on the device. */
 /* Treat no join the same as a bevel join. */
-private int
+static int
 stroke_fill(gx_path * ppath, int first, register pl_ptr plp, pl_ptr nplp,
 	    const gx_device_color * pdevc, gx_device * dev,
 	    const gs_imager_state * pis, const gx_stroke_params * params,
@@ -1042,7 +1162,7 @@ stroke_fill(gx_path * ppath, int first, register pl_ptr plp, pl_ptr nplp,
 }
 
 /* Add a segment to the path.  This handles all the complex cases. */
-private int
+static int
 stroke_add(gx_path * ppath, int first, pl_ptr plp, pl_ptr nplp,
 	   const gx_device_color * pdevc, gx_device * dev,
 	   const gs_imager_state * pis, const gx_stroke_params * params,
@@ -1059,15 +1179,14 @@ stroke_add(gx_path * ppath, int first, pl_ptr plp, pl_ptr nplp,
 	/* We didn't set up the endpoint parameters before, */
 	/* because the line was thin.  Do it now. */
 	set_thin_widths(plp);
-	adjust_stroke(plp, pis, true);
+	adjust_stroke(plp, pis, true, first == 0 && nplp == 0);
 	compute_caps(plp);
     }
     /* Create an initial cap if desired. */
     if (first == 0 && pgs_lp->cap == gs_cap_round) {
 	vd_moveto(plp->o.co.x, plp->o.co.y);
 	if ((code = gx_path_add_point(ppath, plp->o.co.x, plp->o.co.y)) < 0 ||
-	    (code = add_round_cap(ppath, &plp->o)) < 0
-	    )
+	    (code = add_round_cap(ppath, &plp->o)) < 0)
 	    return code;
 	npoints = 0;
 	moveto_first = false;
@@ -1111,8 +1230,151 @@ stroke_add(gx_path * ppath, int first, pl_ptr plp, pl_ptr nplp,
     return gx_path_close_subpath(ppath);
 }
 
+/* Add a CPSI-compatible segment to the path.  This handles all the complex cases. */
+static int
+stroke_add_compat(gx_path * ppath, int first, pl_ptr plp, pl_ptr nplp,
+	   const gx_device_color * pdevc, gx_device * dev,
+	   const gs_imager_state * pis, const gx_stroke_params * params,
+	   const gs_fixed_rect * ignore_pbbox, int uniform, gs_line_join join,
+	   bool reflected)
+{
+    /* Actually it adds 2 contours : one for the segment itself,
+       and another one for line join or for the ending cap. 
+       Note CPSI creates negative contours. */
+    const gx_line_params *pgs_lp = gs_currentlineparams_inline(pis);
+    gs_fixed_point points[5];
+    int npoints;
+    bool const moveto_first = true; /* Keeping this code closer to "stroke_add". */
+    int code;
+
+    if (plp->thin) {
+	/* We didn't set up the endpoint parameters before, */
+	/* because the line was thin.  Do it now. */
+	set_thin_widths(plp);
+	adjust_stroke(plp, pis, true, first == 0 && nplp == 0);
+	compute_caps(plp);
+    }
+    /* The segment itself : */
+    ASSIGN_POINT(&points[0], plp->o.ce);
+    ASSIGN_POINT(&points[1], plp->e.co);
+    ASSIGN_POINT(&points[2], plp->e.ce);
+    ASSIGN_POINT(&points[3], plp->o.co);
+    code = add_points(ppath, points, 4, moveto_first);
+    if (code < 0)
+	return code;
+    code = gx_path_close_subpath(ppath);
+    if (code < 0)
+	return code;
+    vd_closepath;
+    npoints = 0;
+    if (nplp == 0) {
+	/* Add a final cap. */
+	if (pgs_lp->cap == gs_cap_butt)
+	    return 0;
+	if (pgs_lp->cap == gs_cap_round) {
+	    ASSIGN_POINT(&points[npoints], plp->e.co);
+	    vd_lineto(points[npoints].x, points[npoints].y);
+	    ++npoints;
+	    if ((code = add_points(ppath, points, npoints, moveto_first)) < 0)
+		return code;
+	    return add_round_cap(ppath, &plp->e);
+	}
+	ASSIGN_POINT(&points[0], plp->e.ce);
+	++npoints;
+	ASSIGN_POINT(&points[npoints], plp->e.co);
+	++npoints;
+	code = cap_points(pgs_lp->cap, &plp->e, points + npoints);
+	if (code < 0)
+	    return code;
+	npoints += code;
+    } else if (join == gs_join_round) {
+	ASSIGN_POINT(&points[npoints], plp->e.co);
+	vd_lineto(points[npoints].x, points[npoints].y);
+	++npoints;
+	if ((code = add_points(ppath, points, npoints, moveto_first)) < 0)
+	    return code;
+	return add_round_cap(ppath, &plp->e);
+    } else if (nplp->thin) {	/* no join */
+	npoints = 0;
+    } else {			/* non-round join */
+	bool ccw =
+	    (double)(plp->width.x) /* x1 */ * (nplp->width.y) /* y2 */ >
+	    (double)(nplp->width.x) /* x2 */ * (plp->width.y) /* y1 */;
+
+	if (ccw ^ reflected) {
+	    ASSIGN_POINT(&points[0], plp->e.co);
+	    vd_lineto(points[0].x, points[0].y);
+	    ++npoints;
+	    code = line_join_points(pgs_lp, plp, nplp, points + npoints,
+				    (uniform ? (gs_matrix *) 0 : &ctm_only(pis)),
+				    join, reflected);
+	    if (code < 0)
+		return code;
+	    code--; /* Drop the last point of the non-compatible mode. */
+	    npoints += code;
+	} else {
+	    code = line_join_points(pgs_lp, plp, nplp, points,
+				    (uniform ? (gs_matrix *) 0 : &ctm_only(pis)),
+				    join, reflected);
+	    if (code < 0)
+		return code;
+	    ASSIGN_POINT(&points[0], plp->e.ce); /* Replace the starting point of the non-compatible mode. */
+	    npoints = code;
+	}
+    }
+    code = add_points(ppath, points, npoints, moveto_first);
+    if (code < 0)
+	return code;
+    code = gx_path_close_subpath(ppath);
+    vd_closepath;
+    return code;
+}
+
+/* Add a CPSI-compatible segment to the path.  This handles all the complex cases. */
+static int
+stroke_add_initial_cap_compat(gx_path * ppath, pl_ptr plp, bool adlust_longitude,
+	   const gx_device_color * pdevc, gx_device * dev,
+	   const gs_imager_state * pis)
+{
+    const gx_line_params *pgs_lp = gs_currentlineparams_inline(pis);
+    gs_fixed_point points[4];
+    int npoints = 0;
+    int code;
+
+    if (pgs_lp->cap == gs_cap_butt)
+	return 0;
+    if (plp->thin) {
+	/* We didn't set up the endpoint parameters before, */
+	/* because the line was thin.  Do it now. */
+	set_thin_widths(plp);
+	adjust_stroke(plp, pis, true, adlust_longitude);
+	compute_caps(plp);
+    }
+    /* Create an initial cap if desired. */
+    if (pgs_lp->cap == gs_cap_round) {
+	vd_moveto(plp->o.co.x, plp->o.co.y);
+	if ((code = gx_path_add_point(ppath, plp->o.co.x, plp->o.co.y)) < 0 ||
+	    (code = add_round_cap(ppath, &plp->o)) < 0
+	    )
+	    return code;
+	return 0;
+    } else {
+	ASSIGN_POINT(&points[0], plp->o.co);
+	++npoints;
+	if ((code = cap_points(pgs_lp->cap, &plp->o, points + npoints)) < 0)
+	    return npoints;
+	npoints += code;
+	ASSIGN_POINT(&points[npoints], plp->o.ce);
+	++npoints;
+	code = add_points(ppath, points, npoints, true);
+	if (code < 0)
+	    return code;
+	return gx_path_close_subpath(ppath);
+    }
+}
+
 /* Add lines with a possible initial moveto. */
-private int
+static int
 add_points(gx_path * ppath, const gs_fixed_point * points, int npoints,
 	   bool moveto_first)
 {
@@ -1139,7 +1401,7 @@ add_points(gx_path * ppath, const gs_fixed_point * points, int npoints,
 /* Treat no join the same as a bevel join. */
 /* If pmat != 0, we must inverse-transform the distances for */
 /* the miter check. */
-private int
+static int
 line_join_points(const gx_line_params * pgs_lp, pl_ptr plp, pl_ptr nplp,
 		 gs_fixed_point * join_points, const gs_matrix * pmat,
 		 gs_line_join join, bool reflected)
@@ -1247,8 +1509,8 @@ line_join_points(const gx_line_params * pgs_lp, pl_ptr plp, pl_ptr nplp,
 	 * computations in user space.
 	 */
 	float check = pgs_lp->miter_check;
-	double u1 = plp->e.cdelta.y, v1 = plp->e.cdelta.x;
-	double u2 = nplp->o.cdelta.y, v2 = nplp->o.cdelta.x;
+	double u1 = plp->vector.y, v1 = plp->vector.x;
+	double u2 = -nplp->vector.y, v2 = -nplp->vector.x;
 	double num, denom;
 	int code;
 
@@ -1319,6 +1581,8 @@ line_join_points(const gx_line_params * pgs_lp, pl_ptr plp, pl_ptr nplp,
 	 *              +       -               false
 	 *              -       -               T >= check
 	 */
+	if (num == 0 && denom == 0)
+	    return_error(gs_error_unregistered); /* Must not happen. */
 	if (denom < 0)
 	    num = -num, denom = -denom;
 	/* Now denom >= 0, so sign(num) = sign(T). */
@@ -1344,7 +1608,7 @@ line_join_points(const gx_line_params * pgs_lp, pl_ptr plp, pl_ptr nplp,
 
 /* Compute the endpoints of the two caps of a segment. */
 /* Only o.p, e.p, width, and cdelta have been set. */
-private void
+static void
 compute_caps(pl_ptr plp)
 {
     fixed wx2 = plp->width.x;
@@ -1380,7 +1644,7 @@ compute_caps(pl_ptr plp)
 
 /* Add a round cap to a path. */
 /* Assume the current point is the cap origin (endp->co). */
-private int
+static int
 add_round_cap(gx_path * ppath, const_ep_ptr endp)
 {
     int code;
@@ -1409,7 +1673,7 @@ add_round_cap(gx_path * ppath, const_ep_ptr endp)
 
 /* Compute the points for a non-round cap. */
 /* Return the number of points. */
-private int
+static int
 cap_points(gs_line_cap type, const_ep_ptr endp, gs_fixed_point *pts /*[3]*/)
 {
 #define PUT_POINT(i, px, py)\
