@@ -1,20 +1,17 @@
-/* Copyright (C) 1992, 1995, 1997, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxttfb.c,v 1.36 2004/11/15 01:12:06 ray Exp $ */
+/* $Id: gxttfb.c 8250 2007-09-25 13:31:24Z giles $ */
 /* A bridge to True Type interpreter. */
 
 #include "gx.h"
@@ -42,7 +39,7 @@
 gs_public_st_composite(st_gx_ttfReader, gx_ttfReader,
     "gx_ttfReader", gx_ttfReader_enum_ptrs, gx_ttfReader_reloc_ptrs);
 
-private 
+static 
 ENUM_PTRS_WITH(gx_ttfReader_enum_ptrs, gx_ttfReader *mptr)
     {
 	/* The fields 'pfont' and 'glyph_data' may contain pointers from global 
@@ -56,12 +53,12 @@ ENUM_PTRS_WITH(gx_ttfReader_enum_ptrs, gx_ttfReader *mptr)
     ENUM_PTR(0, gx_ttfReader, memory);
 ENUM_PTRS_END
 
-private RELOC_PTRS_WITH(gx_ttfReader_reloc_ptrs, gx_ttfReader *mptr)
+static RELOC_PTRS_WITH(gx_ttfReader_reloc_ptrs, gx_ttfReader *mptr)
     DISCARD(mptr);
     RELOC_PTR(gx_ttfReader, memory);
 RELOC_PTRS_END
 
-private bool gx_ttfReader__Eof(ttfReader *this)
+static bool gx_ttfReader__Eof(ttfReader *this)
 {
     gx_ttfReader *r = (gx_ttfReader *)this;
 
@@ -72,7 +69,7 @@ private bool gx_ttfReader__Eof(ttfReader *this)
     return false;
 }
 
-private void gx_ttfReader__Read(ttfReader *this, void *p, int n)
+static void gx_ttfReader__Read(ttfReader *this, void *p, int n)
 {
     gx_ttfReader *r = (gx_ttfReader *)this;
     const byte *q;
@@ -98,28 +95,28 @@ private void gx_ttfReader__Read(ttfReader *this, void *p, int n)
     r->pos += n;
 }
 
-private void gx_ttfReader__Seek(ttfReader *this, int nPos)
+static void gx_ttfReader__Seek(ttfReader *this, int nPos)
 {
     gx_ttfReader *r = (gx_ttfReader *)this;
 
     r->pos = nPos;
 }
 
-private int gx_ttfReader__Tell(ttfReader *this)
+static int gx_ttfReader__Tell(ttfReader *this)
 {
     gx_ttfReader *r = (gx_ttfReader *)this;
 
     return r->pos;
 }
 
-private bool gx_ttfReader__Error(ttfReader *this)
+static bool gx_ttfReader__Error(ttfReader *this)
 {
     gx_ttfReader *r = (gx_ttfReader *)this;
 
     return r->error;
 }
 
-private int gx_ttfReader__LoadGlyph(ttfReader *this, int glyph_index, const byte **p, int *size)
+static int gx_ttfReader__LoadGlyph(ttfReader *this, int glyph_index, const byte **p, int *size)
 {
     gx_ttfReader *r = (gx_ttfReader *)this;
     gs_font_type42 *pfont = r->pfont;
@@ -146,7 +143,7 @@ private int gx_ttfReader__LoadGlyph(ttfReader *this, int glyph_index, const byte
     return 2; /* found */
 }
 
-private void gx_ttfReader__ReleaseGlyph(ttfReader *this, int glyph_index)
+static void gx_ttfReader__ReleaseGlyph(ttfReader *this, int glyph_index)
 {
     gx_ttfReader *r = (gx_ttfReader *)this;
 
@@ -156,7 +153,7 @@ private void gx_ttfReader__ReleaseGlyph(ttfReader *this, int glyph_index)
     gs_glyph_data_free(&r->glyph_data, "gx_ttfReader__ReleaseExtraGlyph");
 }
 
-private void gx_ttfReader__Reset(gx_ttfReader *this)
+static void gx_ttfReader__Reset(gx_ttfReader *this)
 {
     if (this->extra_glyph_index != -1) {
 	this->extra_glyph_index = -1;
@@ -194,19 +191,45 @@ void gx_ttfReader__destroy(gx_ttfReader *this)
     gs_free_object(this->memory, this, "gx_ttfReader__destroy");
 }
 
+static int 
+gx_ttfReader__default_get_metrics(const ttfReader *ttf, uint glyph_index, bool bVertical, 
+				  short *sideBearing, unsigned short *nAdvance)
+{
+    gx_ttfReader *this = (gx_ttfReader *)ttf;
+    float sbw[4];
+    int sbw_offset = bVertical;
+    int code;
+    int factor = this->pfont->data.unitsPerEm;
+
+    if (bVertical)
+	factor = factor; /* See simple_glyph_metrics */    
+    code = this->pfont->data.get_metrics(this->pfont, glyph_index, bVertical, sbw);
+    if (code < 0)
+	return code;
+    /* Due to an obsolete convention, simple_glyph_metrics scales
+       the metrics into 1x1 rectangle as Postscript like.
+       In same time, the True Type interpreter needs 
+       the original design units.
+       Undo the scaling here with accurate rounding. */
+    *sideBearing = (short)floor(sbw[0 + sbw_offset] * factor + 0.5);
+    *nAdvance = (short)floor(sbw[2 + sbw_offset] * factor + 0.5);
+    return 0;
+}
+
 void gx_ttfReader__set_font(gx_ttfReader *this, gs_font_type42 *pfont)
 {
     this->pfont = pfont;
+    this->super.get_metrics = gx_ttfReader__default_get_metrics;
 }
 
 
 /*----------------------------------------------*/
 
-private void DebugRepaint(ttfFont *ttf)
+static void DebugRepaint(ttfFont *ttf)
 {
 }
 
-private void DebugPrint(ttfFont *ttf, const char *fmt, ...)
+static int DebugPrint(ttfFont *ttf, const char *fmt, ...)
 {
     char buf[500];
     va_list args;
@@ -220,9 +243,10 @@ private void DebugPrint(ttfFont *ttf, const char *fmt, ...)
 	errwrite(buf, count);
 	va_end(args);
     }
+    return 0;
 }
 
-private void WarnBadInstruction(gs_font_type42 *pfont, int glyph_index)
+static void WarnBadInstruction(gs_font_type42 *pfont, int glyph_index)
 {
     char buf[gs_font_name_max + 1];
     int l;
@@ -235,18 +259,18 @@ private void WarnBadInstruction(gs_font_type42 *pfont, int glyph_index)
 	memcpy(buf, base_font->font_name.chars, l);
 	buf[l] = 0;
 	if (glyph_index >= 0)
-	    eprintf2("Failed to interpret TT instructions of fhe glyph index %d of the font %s. "
+	    eprintf2("Failed to interpret TT instructions for glyph index %d of font %s. "
 			"Continue ignoring instructions of the font.\n", 
 			glyph_index, buf);
 	else
-	    eprintf1("Failed to interpret TT instructions of the font %s. "
+	    eprintf1("Failed to interpret TT instructions in font %s. "
 			"Continue ignoring instructions of the font.\n", 	    
 			buf);
 	base_font->data.warning_bad_instruction = true;
     }
 }
 
-private void WarnPatented(gs_font_type42 *pfont, ttfFont *ttf, const char *txt)
+static void WarnPatented(gs_font_type42 *pfont, ttfFont *ttf, const char *txt)
 {
     if (!ttf->design_grid) {
 	char buf[gs_font_name_max + 1];
@@ -267,29 +291,29 @@ private void WarnPatented(gs_font_type42 *pfont, ttfFont *ttf, const char *txt)
 
 /*----------------------------------------------*/
 
-typedef struct gx_ttfMemory_s {
+struct gx_ttfMemory_s {
     ttfMemory super;
     gs_memory_t *memory;
-} gx_ttfMemory;
+};
 
 gs_private_st_simple(st_gx_ttfMemory, gx_ttfMemory, "gx_ttfMemory");
 /* st_gx_ttfMemory::memory points to a root. */
 
-private void *gx_ttfMemory__alloc_bytes(ttfMemory *this, int size,  const char *cname)
+static void *gx_ttfMemory__alloc_bytes(ttfMemory *this, int size,  const char *cname)
 {
     gs_memory_t *mem = ((gx_ttfMemory *)this)->memory;
 
     return gs_alloc_bytes(mem, size, cname);
 }
 
-private void *gx_ttfMemory__alloc_struct(ttfMemory *this, const ttfMemoryDescriptor *d,  const char *cname)
+static void *gx_ttfMemory__alloc_struct(ttfMemory *this, const ttfMemoryDescriptor *d,  const char *cname)
 {
     gs_memory_t *mem = ((gx_ttfMemory *)this)->memory;
 
     return mem->procs.alloc_struct(mem, (const gs_memory_struct_type_t *)d, cname);
 }
 
-private void gx_ttfMemory__free(ttfMemory *this, void *p,  const char *cname)
+static void gx_ttfMemory__free(ttfMemory *this, void *p,  const char *cname)
 {
     gs_memory_t *mem = ((gx_ttfMemory *)this)->memory;
 
@@ -303,7 +327,7 @@ static inline float reminder(float v, int x)
     return ((v / x) - floor(v / x)) * x;
 }
 
-private void decompose_matrix(const gs_font_type42 *pfont, const gs_matrix * char_tm, 
+static void decompose_matrix(const gs_font_type42 *pfont, const gs_matrix * char_tm, 
     const gs_log2_scale_point *log2_scale, bool design_grid,
     gs_point *char_size, gs_point *subpix_origin, gs_matrix *post_transform, bool *dg)
 {
@@ -344,35 +368,43 @@ private void decompose_matrix(const gs_font_type42 *pfont, const gs_matrix * cha
 
 ttfFont *ttfFont__create(gs_font_dir *dir)
 {
-    gs_memory_t *mem = dir->memory;
-    gx_ttfMemory *m = gs_alloc_struct(mem, gx_ttfMemory, &st_gx_ttfMemory, "ttfFont__create");
+    gs_memory_t *mem = dir->memory->stable_memory;
     ttfFont *ttf;
 
-    if (!m)
+    if (dir->ttm == NULL) {
+	gx_ttfMemory *m = gs_alloc_struct(mem, gx_ttfMemory, &st_gx_ttfMemory, "ttfFont__create(gx_ttfMemory)");
+
+	if (!m)
+	    return 0;
+	m->super.alloc_struct = gx_ttfMemory__alloc_struct;
+	m->super.alloc_bytes = gx_ttfMemory__alloc_bytes;
+	m->super.free = gx_ttfMemory__free;
+	m->memory = mem;
+	dir->ttm = m;
+    }
+    if(ttfInterpreter__obtain(&dir->ttm->super, &dir->tti))
 	return 0;
-    m->super.alloc_struct = gx_ttfMemory__alloc_struct;
-    m->super.alloc_bytes = gx_ttfMemory__alloc_bytes;
-    m->super.free = gx_ttfMemory__free;
-    m->memory = mem;
-    if(ttfInterpreter__obtain(&m->super, &dir->tti))
-	return 0;
-    if(gx_san__obtain(mem->stable_memory, &dir->san))
+    if(gx_san__obtain(mem, &dir->san))
 	return 0;
     ttf = gs_alloc_struct(mem, ttfFont, &st_ttfFont, "ttfFont__create");
     if (ttf == NULL)
 	return 0;
-    ttfFont__init(ttf, &m->super, DebugRepaint, DebugPrint);
+    ttfFont__init(ttf, &dir->ttm->super, DebugRepaint, (gs_debug_c('Y') ? DebugPrint : NULL));
     return ttf;
 }
 
 void ttfFont__destroy(ttfFont *this, gs_font_dir *dir)
 {   
-    ttfMemory *mem = this->tti->ttf_memory;
+    gs_memory_t *mem = dir->memory->stable_memory;
 
     ttfFont__finit(this);
-    mem->free(mem, this, "ttfFont__destroy");
+    gs_free_object(mem, this, "ttfFont__destroy");
     ttfInterpreter__release(&dir->tti);
     gx_san__release(&dir->san);
+    if (dir->tti == NULL && dir->ttm != NULL) {
+	gs_free_object(mem, dir->ttm, "ttfFont__destroy(gx_ttfMemory)");
+	dir->ttm = NULL;
+    }
 }
 
 int ttfFont__Open_aux(ttfFont *this, ttfInterpreter *tti, gx_ttfReader *r, gs_font_type42 *pfont,
@@ -425,7 +457,7 @@ typedef struct gx_ttfExport_s {
     bool monotonize;
 } gx_ttfExport;
 
-private void gx_ttfExport__MoveTo(ttfExport *this, FloatPoint *p)
+static void gx_ttfExport__MoveTo(ttfExport *this, FloatPoint *p)
 {
     gx_ttfExport *e = (gx_ttfExport *)this;
 
@@ -433,7 +465,7 @@ private void gx_ttfExport__MoveTo(ttfExport *this, FloatPoint *p)
 	e->error = gx_path_add_point(e->path, float2fixed(p->x), float2fixed(p->y));
 }
 
-private void gx_ttfExport__LineTo(ttfExport *this, FloatPoint *p)
+static void gx_ttfExport__LineTo(ttfExport *this, FloatPoint *p)
 {
     gx_ttfExport *e = (gx_ttfExport *)this;
 
@@ -441,7 +473,7 @@ private void gx_ttfExport__LineTo(ttfExport *this, FloatPoint *p)
 	e->error = gx_path_add_line_notes(e->path, float2fixed(p->x), float2fixed(p->y), sn_none);
 }
 
-private void gx_ttfExport__CurveTo(ttfExport *this, FloatPoint *p0, FloatPoint *p1, FloatPoint *p2)
+static void gx_ttfExport__CurveTo(ttfExport *this, FloatPoint *p0, FloatPoint *p1, FloatPoint *p2)
 {
     gx_ttfExport *e = (gx_ttfExport *)this;
 
@@ -461,7 +493,7 @@ private void gx_ttfExport__CurveTo(ttfExport *this, FloatPoint *p0, FloatPoint *
     }
 }
 
-private void gx_ttfExport__Close(ttfExport *this)
+static void gx_ttfExport__Close(ttfExport *this)
 {
     gx_ttfExport *e = (gx_ttfExport *)this;
 
@@ -469,12 +501,12 @@ private void gx_ttfExport__Close(ttfExport *this)
 	e->error = gx_path_close_subpath_notes(e->path, sn_none);
 }
 
-private void gx_ttfExport__Point(ttfExport *this, FloatPoint *p, bool bOnCurve, bool bNewPath)
+static void gx_ttfExport__Point(ttfExport *this, FloatPoint *p, bool bOnCurve, bool bNewPath)
 {
     /* Never called. */
 }
 
-private void gx_ttfExport__SetWidth(ttfExport *this, FloatPoint *p)
+static void gx_ttfExport__SetWidth(ttfExport *this, FloatPoint *p)
 {
     gx_ttfExport *e = (gx_ttfExport *)this;
 
@@ -482,17 +514,18 @@ private void gx_ttfExport__SetWidth(ttfExport *this, FloatPoint *p)
     e->w.y = float2fixed(p->y); 
 }
 
-private void gx_ttfExport__DebugPaint(ttfExport *this)
+static void gx_ttfExport__DebugPaint(ttfExport *this)
 {
 }
 
 /*----------------------------------------------*/
 
-private int
+static int
 path_to_hinter(t1_hinter *h, gx_path *path)
 {   int code;
     gs_path_enum penum;
-    gs_fixed_point pts[3], p;
+    gs_fixed_point pts[3];
+    gs_fixed_point p = {0, 0}; /* initialize to avoid a warning */
     bool first = true;
     int op;
 
@@ -533,7 +566,7 @@ path_to_hinter(t1_hinter *h, gx_path *path)
 
 #define exch(a,b) a^=b; b^=a; a^=b;
 
-private void
+static void
 transpose_path(gx_path *path)
 {   segment *s = (segment *)path->first_subpath;
 
@@ -556,7 +589,7 @@ typedef struct {
     fixed midx;
 } t1_hinter_aux;
 
-private int 
+static int 
 stem_hint_handler(void *client_data, gx_san_sect *ss)
 {
     t1_hinter_aux *h = (t1_hinter_aux *)client_data;
@@ -587,7 +620,7 @@ stem_hint_handler(void *client_data, gx_san_sect *ss)
 			  fixme : remove side_mask from gxhintn.c .
 			  */
 
-private int grid_fit(gx_device_spot_analyzer *padev, gx_path *path, 
+static int grid_fit(gx_device_spot_analyzer *padev, gx_path *path, 
 	gs_font_type42 *pfont, const gs_log2_scale_point *pscale, gx_ttfExport *e, ttfOutliner *o)
 {
     /* Not completed yet. */
