@@ -1,23 +1,17 @@
-/* Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: igcref.c,v $ $Revision: 1.2.2.1 $ */
+/* $Id: igcref.c 8418 2007-11-30 06:43:12Z lpd $ */
 /* ref garbage collector for Ghostscript */
 #include "memory_.h"
 #include "ghost.h"
@@ -39,19 +33,20 @@
 
 /* Forward references */
 ptr_proc_reloc(igc_reloc_ref_ptr, ref_packed);
+ptr_proc_reloc(igc_reloc_ref_ptr_nocheck, ref_packed);
 refs_proc_reloc(igc_reloc_refs);
 
 /*
  * Define the 'structure' type descriptor for refs.
  * This is special because it has different shared procs.
  */
-private gc_proc_clear_reloc(refs_clear_reloc);
-private gc_proc_set_reloc(refs_set_reloc);
-private gc_proc_compact(refs_compact);
-private const struct_shared_procs_t refs_shared_procs =
+static gc_proc_clear_reloc(refs_clear_reloc);
+static gc_proc_set_reloc(refs_set_reloc);
+static gc_proc_compact(refs_compact);
+static const struct_shared_procs_t refs_shared_procs =
 {refs_clear_reloc, refs_set_reloc, refs_compact};
-private struct_proc_clear_marks(refs_clear_marks);
-private struct_proc_reloc_ptrs(refs_do_reloc);
+static struct_proc_clear_marks(refs_clear_marks);
+static struct_proc_reloc_ptrs(refs_do_reloc);
 const gs_memory_struct_type_t st_refs =
 {sizeof(ref), "refs", &refs_shared_procs, refs_clear_marks, 0, refs_do_reloc};
 
@@ -82,11 +77,14 @@ ENUM_PTRS_BEGIN_PROC(ref_struct_enum_ptrs)
 }
 RELOC_PTRS_BEGIN(ref_struct_reloc_ptrs)
 {
+    vm_spaces spaces = gcst->spaces;
+    const gs_memory_t *cmem = space_system->stable_memory;
+
     ref *beg = vptr;
     ref *end = (ref *) ((char *)vptr + size);
 
     igc_reloc_refs((ref_packed *) beg, (ref_packed *) end, gcst);
-    ref_struct_clear_marks(vptr, size, pstype);
+    ref_struct_clear_marks(cmem, vptr, size, pstype);
 } RELOC_PTRS_END
 
 /* ------ Unmarking phase ------ */
@@ -104,8 +102,9 @@ ptr_ref_unmark(enum_ptr_t *pep, gc_state_t * ignored)
 }
 
 /* Unmarking routine for ref objects. */
-private void
-refs_clear_marks(void /*obj_header_t */ *vptr, uint size,
+static void
+refs_clear_marks(const gs_memory_t *cmem, 
+		 void /*obj_header_t */ *vptr, uint size,
 		 const gs_memory_struct_type_t * pstype)
 {
     ref_packed *rp = (ref_packed *) vptr;
@@ -118,7 +117,7 @@ refs_clear_marks(void /*obj_header_t */ *vptr, uint size,
 #ifdef DEBUG
 	    if (gs_debug_c('8')) {
 		dlprintf1("  [8]unmark packed 0x%lx ", (ulong) rp);
-		debug_print_ref((const ref *)rp);
+		debug_print_ref(cmem, (const ref *)rp);
 		dputs("\n");
 	    }
 #endif
@@ -130,7 +129,7 @@ refs_clear_marks(void /*obj_header_t */ *vptr, uint size,
 #ifdef DEBUG
 	    if (gs_debug_c('8')) {
 		dlprintf1("  [8]unmark ref 0x%lx ", (ulong) rp);
-		debug_print_ref(pref);
+		debug_print_ref(cmem, pref);
 		dputs("\n");
 	    }
 #endif
@@ -175,8 +174,8 @@ ptr_ref_mark(enum_ptr_t *pep, gc_state_t * ignored)
  */
 
 /* Clear the relocation for a ref object. */
-private void
-refs_clear_reloc(obj_header_t * hdr, uint size)
+static void
+refs_clear_reloc(obj_header_t *hdr, uint size)
 {
     ref_packed *rp = (ref_packed *) (hdr + 1);
     ref_packed *end = (ref_packed *) ((byte *) rp + size);
@@ -198,7 +197,7 @@ refs_clear_reloc(obj_header_t * hdr, uint size)
 }
 
 /* Set the relocation for a ref object. */
-private bool
+static bool
 refs_set_reloc(obj_header_t * hdr, uint reloc, uint size)
 {
     ref_packed *rp = (ref_packed *) (hdr + 1);
@@ -370,7 +369,7 @@ refs_set_reloc(obj_header_t * hdr, uint reloc, uint size)
 /* ------ Relocation phase ------ */
 
 /* Relocate all the pointers in a block of refs. */
-private void
+static void
 refs_do_reloc(void /*obj_header_t */ *vptr, uint size,
 	      const gs_memory_struct_type_t * pstype, gc_state_t * gcst)
 {
@@ -386,7 +385,10 @@ igc_reloc_refs(ref_packed * from, ref_packed * to, gc_state_t * gcst)
 {
     int min_trace = gcst->min_collect;
     ref_packed *rp = from;
-    bool do_all = gcst->relocating_untraced;
+    bool do_all = gcst->relocating_untraced; 
+
+    vm_spaces spaces = gcst->spaces;
+    const gs_memory_t *cmem = space_system->stable_memory;
 
     while (rp < to) {
 	ref *pref;
@@ -409,7 +411,7 @@ igc_reloc_refs(ref_packed * from, ref_packed * to, gc_state_t * gcst)
 	/* The following assignment is logically unnecessary; */
 	/* we do it only for convenience in debugging. */
 	pref = (ref *) rp;
-	if_debug3('8', "  [8]relocating %s %d ref at 0x%lx",
+	if_debug3('8', "  [8]relocating %s %d ref at 0x%lx\n",
 		  (r_has_attr(pref, l_mark) ? "marked" : "unmarked"),
 		  r_btype(pref), (ulong) pref);
 	if ((r_has_attr(pref, l_mark) || do_all) &&
@@ -510,7 +512,7 @@ igc_reloc_refs(ref_packed * from, ref_packed * to, gc_state_t * gcst)
 		    break;
 		case t_name:
 		    {
-			void *psub = name_ref_sub_table(pref);
+			void *psub = name_ref_sub_table(cmem, pref);
 			void *rsub = RELOC_OBJ(psub); /* gcst implicit */
 
 			SET_RELOC(pref->value.pname,
@@ -537,10 +539,10 @@ igc_reloc_refs(ref_packed * from, ref_packed * to, gc_state_t * gcst)
 		default:
 		    goto no_reloc; /* don't print trace message */
 	    }
-	    if_debug2('8', ", 0x%lx => 0x%lx", (ulong)before, (ulong)after);
+	    if_debug2('8', "  [8]relocated 0x%lx => 0x%lx\n",
+		      (ulong)before, (ulong)after);
 	}
 no_reloc:
-	if_debug0('8', "\n");
 	rp += packed_per_ref;
     }
 }
@@ -548,7 +550,7 @@ no_reloc:
 /* Relocate a pointer to a ref. */
 /* See gsmemory.h for why the argument is const and the result is not. */
 ref_packed *
-igc_reloc_ref_ptr(const ref_packed * prp, gc_state_t * ignored)
+igc_reloc_ref_ptr_nocheck(const ref_packed * prp, gc_state_t *gcst)
 {
     /*
      * Search forward for relocation.  This algorithm is intrinsically very
@@ -562,19 +564,6 @@ igc_reloc_ref_ptr(const ref_packed * prp, gc_state_t * ignored)
 #else
 # define RP_REF(rp) ((const ref *)rp)
 #endif
-
-    /*
-     * Iff this pointer points into a space that wasn't traced,
-     * the referent won't be marked.  In this case, we shouldn't
-     * do any relocation.  Check for this first.
-     */
-    if (r_is_packed(rp)) {
-	if (!r_has_pmark(rp))
-	    goto ret_rp;
-    } else {
-	if (!r_has_attr(RP_REF(rp), l_mark))
-	    goto ret_rp;
-    }
     for (;;) {
 
 	if (r_is_packed(rp)) {
@@ -621,6 +610,42 @@ igc_reloc_ref_ptr(const ref_packed * prp, gc_state_t * ignored)
 	rputc('u');
 	rp += packed_per_ref;
     }
+    /* Use a severely deprecated pun to remove the const property. */
+    {
+	union { const ref_packed *r; ref_packed *w; } u;
+
+	u.r = rp;
+	return u.w;
+    }
+#undef RP_REF
+}
+ref_packed *
+igc_reloc_ref_ptr(const ref_packed * prp, gc_state_t *gcst)
+{
+    /*
+     * Search forward for relocation.  This algorithm is intrinsically very
+     * inefficient; we hope eventually to replace it with a better one.
+     */
+    const ref_packed *rp = prp;
+#ifdef ALIGNMENT_ALIASING_BUG
+    const ref *rpref;
+# define RP_REF(rp) (rpref = (const ref *)rp, rpref)
+#else
+# define RP_REF(rp) ((const ref *)rp)
+#endif
+    /*
+     * Iff this pointer points into a space that wasn't traced,
+     * the referent won't be marked.  In this case, we shouldn't
+     * do any relocation.  Check for this first.
+     */
+    if (r_is_packed(rp)) {
+	if (!r_has_pmark(rp))
+	    goto ret_rp;
+    } else {
+	if (!r_has_attr(RP_REF(rp), l_mark))
+	    goto ret_rp;
+    }
+    return igc_reloc_ref_ptr_nocheck(prp, gcst);
 ret_rp:
     /* Use a severely deprecated pun to remove the const property. */
     {
@@ -635,18 +660,24 @@ ret_rp:
 
 /* Compact a ref object. */
 /* Remove the marks at the same time. */
-private void
-refs_compact(obj_header_t * pre, obj_header_t * dpre, uint size)
+static void
+refs_compact(const gs_memory_t *mem, obj_header_t * pre, obj_header_t * dpre, uint size)
 {
     ref_packed *dest;
     ref_packed *src;
     ref_packed *end;
     uint new_size;
 
+   /* The next switch controls an optimization 
+      for the loop termination condition.
+      It was useful during the development,
+      when some assumptions were temporary wrong.
+      We keep it for records. */
+
     src = (ref_packed *) (pre + 1);
     end = (ref_packed *) ((byte *) src + size);
     /*
-     * We know that a block of refs always ends with an unmarked
+     * We know that a block of refs always ends with a
      * full-size ref, so we only need to check for reaching the end
      * of the block when we see one of those.
      */
@@ -691,8 +722,8 @@ refs_compact(obj_header_t * pre, obj_header_t * dpre, uint size)
 		ref_assign_inline(&rtemp, (ref *) src);
 		r_clear_attrs(&rtemp, l_mark);
 		ref_assign_inline((ref *) dest, &rtemp);
-		dest += packed_per_ref;
 		src += packed_per_ref;
+		dest += packed_per_ref;
 	    } else {		/* check for end of block */
 		src += packed_per_ref;
 		if (src >= end)
@@ -708,7 +739,7 @@ refs_compact(obj_header_t * pre, obj_header_t * dpre, uint size)
 	lprintf3("Reloc error for refs 0x%lx: reloc = %lu, stored = %u\n",
 		 (ulong) dpre, (ulong) ((byte *) src - (byte *) dest),
 		 (uint) r_size((ref *) src - 1));
-	gs_exit(1);
+	gs_abort(mem);
     }
 #endif
     /* Pad to a multiple of sizeof(ref). */
