@@ -1,20 +1,17 @@
-/* Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: igcstr.c,v 1.7 2005/09/05 13:58:55 leonardo Exp $ */
+/* $Id: igcstr.c 8415 2007-11-29 01:42:16Z lpd $ */
 /* String GC routines for Ghostscript */
 #include "memory_.h"
 #include "ghost.h"
@@ -24,7 +21,7 @@
 #include "igcstr.h"
 
 /* Forward references */
-private bool gc_mark_string(const byte *, uint, bool, const chunk_t *);
+static bool gc_mark_string(const byte *, uint, bool, const chunk_t *);
 
 /* (Un)mark the strings in a chunk. */
 void
@@ -60,7 +57,7 @@ typedef string_mark_unit bword;
 #endif
 
 /* (Un)mark a string in a known chunk.  Return true iff any new marks. */
-private bool
+static bool
 gc_mark_string(const byte * ptr, uint size, bool set, const chunk_t * cp)
 {
     uint offset = ptr - cp->sbase;
@@ -113,6 +110,17 @@ gc_mark_string(const byte * ptr, uint size, bool set, const chunk_t * cp)
     return marks != 0;
 }
 
+/* Print a string for debugging.  We need this because there is no d---
+ * equivalent of fwrite.
+ */
+static void
+dfwrite(const byte *ptr, uint count)
+{
+    uint i;
+    for (i = 0; i < count; ++i)
+	dputc(ptr[i]);
+}
+
 /* Mark a string.  Return true if any new marks. */
 bool
 gc_string_mark(const byte * ptr, uint size, bool set, gc_state_t * gcst)
@@ -123,7 +131,7 @@ gc_string_mark(const byte * ptr, uint size, bool set, gc_state_t * gcst)
     if (size == 0)
 	return false;
 #define dprintstr()\
-  dputc('('); fwrite(ptr, 1, min(size, 20), dstderr);\
+  dputc('('); dfwrite(ptr, min(size, 20));\
   dputs((size <= 20 ? ")" : "...)"))
     if (!(cp = gc_locate(ptr, gcst))) {		/* not in a chunk */
 #ifdef DEBUG
@@ -192,7 +200,7 @@ gc_strings_clear_reloc(chunk_t * cp)
 }
 
 /* Count the 0-bits in a byte. */
-private const byte count_zero_bits_table[256] =
+static const byte count_zero_bits_table[256] =
 {
 #define o4(n) n,n-1,n-1,n-2
 #define o16(n) o4(n),o4(n-1),o4(n-1),o4(n-2)
