@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: iimage.h 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: iimage.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Image operator entry points */
 /* Requires gscspace.h, gxiparam.h */
 
@@ -38,7 +38,8 @@ int data_image_params(const gs_memory_t *mem,
                       bool has_alpha);
 int pixel_image_params(i_ctx_t *i_ctx_p, const ref *op,
                        gs_pixel_image_t *pim, image_params * pip,
-                       int max_bits_per_component, bool has_alpha);
+                       int max_bits_per_component, bool has_alpha,
+                       gs_color_space *csp);
 
 /* Exported for zimage3.c and ztrans.c */
 int zimage_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
