@@ -1,35 +1,31 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: interp.c,v $ $Revision: 1.6.2.1 $ */
+/* $Id: interp.c 8415 2007-11-29 01:42:16Z lpd $ */
 /* Ghostscript language interpreter */
 #include "memory_.h"
 #include "string_.h"
 #include "ghost.h"
 #include "gsstruct.h"		/* for iastruct.h */
+#include "gserrors.h"		/* for gpcheck.h */
 #include "stream.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "estack.h"
 #include "ialloc.h"
 #include "iastruct.h"
 #include "icontext.h"
 #include "icremap.h"
+#include "idebug.h"
 #include "igstate.h"		/* for handling e_RemapColor */
 #include "inamedef.h"
 #include "iname.h"		/* for the_name_table */
@@ -45,10 +41,12 @@
 #include "itoken.h"
 #include "iutil.h"		/* for array_get */
 #include "ivmspace.h"
+#include "iinit.h"
 #include "dstack.h"
 #include "files.h"		/* for file_check_read */
 #include "oper.h"
 #include "store.h"
+#include "gpcheck.h"
 
 /*
  * We may or may not optimize the handling of the special fast operators
@@ -77,18 +75,18 @@ public_st_op_stack();
  * The procedure to call if an operator requests rescheduling.
  * This causes an error unless the context machinery has been installed.
  */
-private int
+static int
 no_reschedule(i_ctx_t **pi_ctx_p)
 {
     return_error(e_invalidcontext);
 }
-int (*gs_interp_reschedule_proc)(P1(i_ctx_t **)) = no_reschedule;
+int (*gs_interp_reschedule_proc)(i_ctx_t **) = no_reschedule;
 
 /*
  * The procedure to call for time-slicing.
  * This is a no-op unless the context machinery has been installed.
  */
-int (*gs_interp_time_slice_proc)(P1(i_ctx_t **)) = 0;
+int (*gs_interp_time_slice_proc)(i_ctx_t **) = 0;
 
 /*
  * The number of interpreter "ticks" between calls on the time_slice_proc.
@@ -102,12 +100,16 @@ int gs_interp_time_slice_ticks = 0x7fff;
  * through a procedure.
  */
 #ifdef DEBUG
-private int
+static int
 call_operator(op_proc_t op_proc, i_ctx_t *i_ctx_p)
 {
-    int code = op_proc(i_ctx_p);
+    int code;
 
-    return code;
+#   ifdef DEBUG_TRACE_PS_OPERATORS
+    if_debug1('!', "[!]operator %s\n", op_get_name_string(op_proc));
+#   endif
+    code = op_proc(i_ctx_p);
+    return code; /* A good place for a conditional breakpoint. */
 }
 #else
 #  define call_operator(proc, p) ((*(proc))(p))
@@ -131,15 +133,20 @@ struct stats_interp_s {
 #endif
 
 /* Forward references */
-private int estack_underflow(P1(i_ctx_t *));
-private int interp(P3(i_ctx_t **, const ref *, ref *));
-private int interp_exit(P1(i_ctx_t *));
-private void set_gc_signal(P3(i_ctx_t *, int *, int));
-private int copy_stack(P3(i_ctx_t *, const ref_stack_t *, ref *));
-private int oparray_pop(P1(i_ctx_t *));
-private int oparray_cleanup(P1(i_ctx_t *));
-private int zsetstackprotect(P1(i_ctx_t *));
-private int zcurrentstackprotect(P1(i_ctx_t *));
+static int estack_underflow(i_ctx_t *);
+static int interp(i_ctx_t **, const ref *, ref *);
+static int interp_exit(i_ctx_t *);
+static void set_gc_signal(i_ctx_t *, int *, int);
+static int copy_stack(i_ctx_t *, const ref_stack_t *, int skip, ref *);
+static int oparray_pop(i_ctx_t *);
+static int oparray_cleanup(i_ctx_t *);
+static int zerrorexec(i_ctx_t *);
+static int zfinderrorobject(i_ctx_t *);
+static int errorexec_find(i_ctx_t *, ref *);
+static int errorexec_pop(i_ctx_t *);
+static int errorexec_cleanup(i_ctx_t *);
+static int zsetstackprotect(i_ctx_t *);
+static int zcurrentstackprotect(i_ctx_t *);
 
 /* Stack sizes */
 
@@ -168,7 +175,7 @@ const int gs_interp_max_op_num_args = MIN_BLOCK_OSTACK;		/* for iinit.c */
  * user parameter).
  */
 #ifndef MAX_ESTACK
-#  define MAX_ESTACK 250
+#  define MAX_ESTACK 5000
 #endif
 /*
  * The minimum block size for extending the execution stack is the largest
@@ -244,20 +251,16 @@ const int gs_interp_num_special_ops = num_special_ops;	/* for iinit.c */
 const int tx_next_index = tx_next_op;
 
 /*
- * Define the interpreter operators, which include the extended-type
- * operators defined in the list above.  NOTE: if the size of this table
- * ever exceeds 15 real entries, it will have to be split.
+ * NOTE: if the size of either table below ever exceeds 15 real entries, it
+ * will have to be split.
  */
-const op_def interp_op_defs[] = {
+/* Define the extended-type operators per the list above. */
+const op_def interp1_op_defs[] = {
     /*
      * The very first entry, which corresponds to operator index 0,
      * must not contain an actual operator.
      */
     op_def_begin_dict("systemdict"),
-    /*
-     * The next entries must be the extended-type operators, in the
-     * correct order.
-     */
     {"2add", zadd},
     {"2def", zdef},
     {"1dup", zdup},
@@ -268,13 +271,17 @@ const op_def interp_op_defs[] = {
     {"1pop", zpop},
     {"2roll", zroll},
     {"2sub", zsub},
-    /*
-     * The remaining entries are internal operators.
-     */
+    op_def_end(0)
+};
+/* Define the internal interpreter operators. */
+const op_def interp2_op_defs[] = {
     {"0.currentstackprotect", zcurrentstackprotect},
     {"1.setstackprotect", zsetstackprotect},
+    {"2.errorexec", zerrorexec},
+    {"0.finderrorobject", zfinderrorobject},
     {"0%interp_exit", interp_exit},
     {"0%oparray_pop", oparray_pop},
+    {"0%errorexec_pop", errorexec_pop},
     op_def_end(0)
 };
 
@@ -381,7 +388,7 @@ gs_interp_reset(i_ctx_t *i_ctx_p)
 }
 /* Report an e-stack block underflow.  The bottom guard slots of */
 /* e-stack blocks contain a pointer to this procedure. */
-private int
+static int
 estack_underflow(i_ctx_t *i_ctx_p)
 {
     return e_ExecStackUnderflow;
@@ -397,7 +404,7 @@ gs_interp_make_oper(ref * opref, op_proc_t proc, int idx)
 {
     int i;
 
-    for (i = num_special_ops; i > 0 && proc != interp_op_defs[i].proc; --i)
+    for (i = num_special_ops; i > 0 && proc != interp1_op_defs[i].proc; --i)
 	DO_NOTHING;
     if (i > 0)
 	make_tasv(opref, tx_op + (i - 1), a_executable, i, opproc, proc);
@@ -408,7 +415,7 @@ gs_interp_make_oper(ref * opref, op_proc_t proc, int idx)
 /*
  * Call the garbage collector, updating the context pointer properly.
  */
-private int
+int
 interp_reclaim(i_ctx_t **pi_ctx_p, int space)
 {
     i_ctx_t *i_ctx_p = *pi_ctx_p;
@@ -433,7 +440,7 @@ interp_reclaim(i_ctx_t **pi_ctx_p, int space)
  * In case of a quit or a fatal error, also store the exit code.
  * Set *perror_object to null or the error object.
  */
-private int gs_call_interp(P5(i_ctx_t **, ref *, int, int *, ref *));
+static int gs_call_interp(i_ctx_t **, ref *, int, int *, ref *);
 int
 gs_interpret(i_ctx_t **pi_ctx_p, ref * pref, int user_errors, int *pexit_code,
 	     ref * perror_object)
@@ -452,7 +459,7 @@ gs_interpret(i_ctx_t **pi_ctx_p, ref * pref, int user_errors, int *pexit_code,
     set_gc_signal(i_ctx_p, NULL, 0);
     return code;
 }
-private int
+static int
 gs_call_interp(i_ctx_t **pi_ctx_p, ref * pref, int user_errors,
 	       int *pexit_code, ref * perror_object)
 {
@@ -488,6 +495,10 @@ again:
     }
     code = interp(pi_ctx_p, epref, perror_object);
     i_ctx_p = *pi_ctx_p;
+    if (!r_has_type(&i_ctx_p->error_object, t__invalid)) {
+	*perror_object = i_ctx_p->error_object;
+	make_t(&i_ctx_p->error_object, t__invalid);
+    }
     /* Prevent a dangling reference to the GC signal in ticks_left */
     /* in the frame of interp, but be prepared to do a GC if */
     /* an allocation in this routine asks for it. */
@@ -524,6 +535,7 @@ again:
 	    epref = &doref;
 	    goto again;
 	case e_NeedInput:
+	case e_interrupt:
 	    return code;
     }
     /* Adjust osp in case of operand stack underflow */
@@ -533,17 +545,16 @@ again:
     /* we might be able to recover by adding or removing a block. */
     switch (code) {
 	case e_dictstackoverflow:
-	    if (ref_stack_extend(&d_stack, d_stack.requested) >= 0) {
-		dict_set_top();
-		doref = *perror_object;
-		epref = &doref;
-		goto again;
-	    }
+	    /* We don't have to handle this specially: */
+	    /* The only places that could generate it */
+	    /* use check_dstack, which does a ref_stack_extend, */
+	    /* so if` we get this error, it's a real one. */
 	    if (osp >= ostop) {
 		if ((ccode = ref_stack_extend(&o_stack, 1)) < 0)
 		    return ccode;
 	    }
-	    ccode = copy_stack(i_ctx_p, &d_stack, &saref);
+            /* Skip system dictionaries for CET 20-02-02 */
+	    ccode = copy_stack(i_ctx_p, &d_stack, min_dstack_size, &saref);
 	    if (ccode < 0)
 		return ccode;
 	    ref_stack_pop_to(&d_stack, min_dstack_size);
@@ -567,7 +578,7 @@ again:
 		if ((ccode = ref_stack_extend(&o_stack, 1)) < 0)
 		    return ccode;
 	    }
-	    ccode = copy_stack(i_ctx_p, &e_stack, &saref);
+	    ccode = copy_stack(i_ctx_p, &e_stack, 0, &saref);
 	    if (ccode < 0)
 		return ccode;
 	    {
@@ -607,7 +618,7 @@ again:
 		epref = &doref;
 		goto again;
 	    }
-	    ccode = copy_stack(i_ctx_p, &o_stack, &saref);
+	    ccode = copy_stack(i_ctx_p, &o_stack, 0, &saref);
 	    if (ccode < 0)
 		return ccode;
 	    ref_stack_clear(&o_stack);
@@ -625,25 +636,34 @@ again:
 	return code;
     if (gs_errorname(i_ctx_p, code, &error_name) < 0)
 	return code;		/* out-of-range error code! */
+    /*
+     * For greater Adobe compatibility, only the standard PostScript errors
+     * are defined in errordict; the rest are in gserrordict.
+     */
     if (dict_find_string(systemdict, "errordict", &perrordict) <= 0 ||
-	dict_find(perrordict, &error_name, &epref) <= 0
+	(dict_find(perrordict, &error_name, &epref) <= 0 &&
+	 (dict_find_string(systemdict, "gserrordict", &perrordict) <= 0 ||
+	  dict_find(perrordict, &error_name, &epref) <= 0))
 	)
 	return code;		/* error name not in errordict??? */
     doref = *epref;
     epref = &doref;
     /* Push the error object on the operand stack if appropriate. */
-    if (!ERROR_IS_INTERRUPT(code))
+    if (!ERROR_IS_INTERRUPT(code)) {
+	/* Replace the error object if within an oparray or .errorexec. */
 	*++osp = *perror_object;
+	errorexec_find(i_ctx_p, osp);
+    }
     goto again;
 }
-private int
+static int
 interp_exit(i_ctx_t *i_ctx_p)
 {
     return e_InterpreterExit;
 }
 
 /* Set the GC signal for all VMs. */
-private void
+static void
 set_gc_signal(i_ctx_t *i_ctx_p, int *psignal, int value)
 {
     gs_memory_gc_status_t stat;
@@ -669,10 +689,10 @@ set_gc_signal(i_ctx_t *i_ctx_p, int *psignal, int value)
 }
 
 /* Copy the contents of an overflowed stack into a (local) array. */
-private int
-copy_stack(i_ctx_t *i_ctx_p, const ref_stack_t * pstack, ref * arr)
+static int
+copy_stack(i_ctx_t *i_ctx_p, const ref_stack_t * pstack, int skip, ref * arr)
 {
-    uint size = ref_stack_count(pstack);
+    uint size = ref_stack_count(pstack) - skip;
     uint save_space = ialloc_space(idmemory);
     int code;
 
@@ -695,7 +715,7 @@ gs_errorname(i_ctx_t *i_ctx_p, int code, ref * perror_name)
 	dict_find_string(systemdict, "ErrorNames", &pErrorNames) <= 0
 	)
 	return_error(e_undefined);	/* errordict or ErrorNames not found?! */
-    return array_get(pErrorNames, (long)(-code - 1), perror_name);
+    return array_get(imemory, pErrorNames, (long)(-code - 1), perror_name);
 }
 
 /* Store an error string in $error.errorinfo. */
@@ -721,7 +741,7 @@ gs_errorinfo_put_string(i_ctx_t *i_ctx_p, const char *str)
 /* If execution terminates normally, return e_InterpreterExit. */
 /* If an error occurs, leave the current object in *perror_object */
 /* and return a (negative) error code. */
-private int
+static int
 interp(i_ctx_t **pi_ctx_p /* context for execution, updated if resched */,
        const ref * pref /* object to interpret */,
        ref * perror_object)
@@ -758,7 +778,8 @@ interp(i_ctx_t **pi_ctx_p /* context for execution, updated if resched */,
     int code;
     ref token;			/* token read from file or string, */
 				/* must be declared in this scope */
-    register const ref *pvalue;
+    register const ref *pvalue = 0;
+    uint opindex;		/* needed for oparrays */
     os_ptr whichp;
 
     /*
@@ -780,7 +801,7 @@ interp(i_ctx_t **pi_ctx_p /* context for execution, updated if resched */,
      * Get a pointer to the name table so that we can use the
      * inline version of name_index_ref.
      */
-    const name_table *const int_nt = the_name_table();
+    const name_table *const int_nt = imemory->gs_lib_ctx->gs_name_table;
 
 #define set_error(ecode)\
   { ierror.code = ecode; ierror.line = __LINE__; }
@@ -790,16 +811,29 @@ interp(i_ctx_t **pi_ctx_p /* context for execution, updated if resched */,
   { set_error(ecode); goto rwei; }
 #define return_with_code_iref()\
   { ierror.line = __LINE__; goto rweci; }
-#define return_with_error_code_op(nargs)\
-  return_with_code_iref()
 #define return_with_stackoverflow(objp)\
   { o_stack.requested = 1; return_with_error(e_stackoverflow, objp); }
 #define return_with_stackoverflow_iref()\
   { o_stack.requested = 1; return_with_error_iref(e_stackoverflow); }
+/*
+ * If control reaches the special operators (x_add, etc.) as a result of
+ * interpreting an executable name, iref points to the name, not the
+ * operator, so the name rather than the operator becomes the error object,
+ * which is wrong.  We detect and handle this case explicitly when an error
+ * occurs, so as not to slow down the non-error case.
+ */
+#define return_with_error_tx_op(err_code)\
+  { if (r_has_type(IREF, t_name)) {\
+        return_with_error(err_code, pvalue);\
+    } else {\
+        return_with_error_iref(err_code);\
+    }\
+  }
+
     int ticks_left = gs_interp_time_slice_ticks;
 
     /*
-     * If we exceed the VMThreshold, set ticks_left to -1
+     * If we exceed the VMThreshold, set ticks_left to -100
      * to alert the interpreter that we need to garbage collect.
      */
     set_gc_signal(i_ctx_p, &ticks_left, -100);
@@ -868,7 +902,6 @@ interp(i_ctx_t **pi_ctx_p /* context for execution, updated if resched */,
 	  r_packed_is_name(iref_packed) :
 	  r_has_type(IREF, t_name)))
 	) {
-	void debug_print_ref(P1(const ref *));
 	os_ptr save_osp = osp;	/* avoid side-effects */
 	es_ptr save_esp = esp;
 
@@ -877,15 +910,15 @@ interp(i_ctx_t **pi_ctx_p /* context for execution, updated if resched */,
 	dlprintf5("d%u,e%u<%u>0x%lx(%d): ",
 		  ref_stack_count(&d_stack), ref_stack_count(&e_stack),
 		  ref_stack_count(&o_stack), (ulong)IREF, icount);
-	debug_print_ref(IREF);
+	debug_print_ref(imemory, IREF);
 	if (iosp >= osbot) {
 	    dputs(" // ");
-	    debug_print_ref(iosp);
+	    debug_print_ref(imemory, iosp);
 	}
 	dputc('\n');
 	osp = save_osp;
 	esp = save_esp;
-	fflush(dstderr);
+	dflush();
     }
 #endif
 /* Objects that have attributes (arrays, dictionaries, files, and strings) */
@@ -960,61 +993,63 @@ interp(i_ctx_t **pi_ctx_p /* context for execution, updated if resched */,
 	case plain_exec(tx_op_add):
 x_add:	    INCR(x_add);
 	    if ((code = zop_add(iosp)) < 0)
-		return_with_error_code_op(2);
+		return_with_error_tx_op(code);
 	    iosp--;
 	    next_either();
 	case plain_exec(tx_op_def):
 x_def:	    INCR(x_def);
 	    osp = iosp;	/* sync o_stack */
 	    if ((code = zop_def(i_ctx_p)) < 0)
-		return_with_error_code_op(2);
+		return_with_error_tx_op(code);
 	    iosp -= 2;
 	    next_either();
 	case plain_exec(tx_op_dup):
 x_dup:	    INCR(x_dup);
 	    if (iosp < osbot)
-		return_with_error_iref(e_stackunderflow);
-	    if (iosp >= ostop)
-		return_with_stackoverflow_iref();
+		return_with_error_tx_op(e_stackunderflow);
+	    if (iosp >= ostop) {
+		o_stack.requested = 1;
+                return_with_error_tx_op(e_stackoverflow);
+            }
 	    iosp++;
 	    ref_assign_inline(iosp, iosp - 1);
 	    next_either();
 	case plain_exec(tx_op_exch):
 x_exch:	    INCR(x_exch);
 	    if (iosp <= osbot)
-		return_with_error_iref(e_stackunderflow);
+		return_with_error_tx_op(e_stackunderflow);
 	    ref_assign_inline(&token, iosp);
 	    ref_assign_inline(iosp, iosp - 1);
 	    ref_assign_inline(iosp - 1, &token);
 	    next_either();
 	case plain_exec(tx_op_if):
 x_if:	    INCR(x_if);
-	    if (!r_has_type(iosp - 1, t_boolean))
-		return_with_error_iref((iosp <= osbot ?
-					e_stackunderflow : e_typecheck));
 	    if (!r_is_proc(iosp))
-		return_with_error_iref(check_proc_failed(iosp));
+		return_with_error_tx_op(check_proc_failed(iosp));
+	    if (!r_has_type(iosp - 1, t_boolean))
+		return_with_error_tx_op((iosp <= osbot ?
+					e_stackunderflow : e_typecheck));
 	    if (!iosp[-1].value.boolval) {
 		iosp -= 2;
 		next_either();
 	    }
 	    if (iesp >= estop)
-		return_with_error_iref(e_execstackoverflow);
+		return_with_error_tx_op(e_execstackoverflow);
 	    store_state_either(iesp);
 	    whichp = iosp;
 	    iosp -= 2;
 	    goto ifup;
 	case plain_exec(tx_op_ifelse):
 x_ifelse:   INCR(x_ifelse);
-	    if (!r_has_type(iosp - 2, t_boolean))
-		return_with_error_iref((iosp < osbot + 2 ?
-					e_stackunderflow : e_typecheck));
-	    if (!r_is_proc(iosp - 1))
-		return_with_error_iref(check_proc_failed(iosp - 1));
 	    if (!r_is_proc(iosp))
-		return_with_error_iref(check_proc_failed(iosp));
+		return_with_error_tx_op(check_proc_failed(iosp));
+	    if (!r_is_proc(iosp - 1))
+		return_with_error_tx_op(check_proc_failed(iosp - 1));
+	    if (!r_has_type(iosp - 2, t_boolean))
+		return_with_error_tx_op((iosp < osbot + 2 ?
+					e_stackunderflow : e_typecheck));
 	    if (iesp >= estop)
-		return_with_error_iref(e_execstackoverflow);
+		return_with_error_tx_op(e_execstackoverflow);
 	    store_state_either(iesp);
 	    whichp = (iosp[-2].value.boolval ? iosp - 1 : iosp);
 	    iosp -= 3;
@@ -1037,25 +1072,25 @@ x_ifelse:   INCR(x_ifelse);
 x_index:    INCR(x_index);
 	    osp = iosp;	/* zindex references o_stack */
 	    if ((code = zindex(i_ctx_p)) < 0)
-		return_with_error_code_op(1);
+		return_with_error_tx_op(code);
 	    next_either();
 	case plain_exec(tx_op_pop):
 x_pop:	    INCR(x_pop);
 	    if (iosp < osbot)
-		return_with_error_iref(e_stackunderflow);
+		return_with_error_tx_op(e_stackunderflow);
 	    iosp--;
 	    next_either();
 	case plain_exec(tx_op_roll):
 x_roll:	    INCR(x_roll);
 	    osp = iosp;	/* zroll references o_stack */
 	    if ((code = zroll(i_ctx_p)) < 0)
-		return_with_error_code_op(2);
+		return_with_error_tx_op(code);
 	    iosp -= 2;
 	    next_either();
 	case plain_exec(tx_op_sub):
 x_sub:	    INCR(x_sub);
 	    if ((code = zop_sub(iosp)) < 0)
-		return_with_error_code_op(2);
+		return_with_error_tx_op(code);
 	    iosp--;
 	    next_either();
 	    /* Executable types. */
@@ -1064,15 +1099,17 @@ x_sub:	    INCR(x_sub);
 	case plain_exec(t_oparray):
 	    /* Replace with the definition and go again. */
 	    INCR(exec_array);
+	    opindex = op_index(IREF);
 	    pvalue = IREF->value.const_refs;
 	  opst:		/* Prepare to call a t_oparray procedure in *pvalue. */
 	    store_state(iesp);
 	  oppr:		/* Record the stack depths in case of failure. */
-	    if (iesp >= estop - 3)
+	    if (iesp >= estop - 4)
 		return_with_error_iref(e_execstackoverflow);
-	    iesp += 4;
+	    iesp += 5;
 	    osp = iosp;		/* ref_stack_count_inline needs this */
-	    make_mark_estack(iesp - 3, es_other, oparray_cleanup);
+	    make_mark_estack(iesp - 4, es_other, oparray_cleanup);
+	    make_int(iesp - 3, opindex); /* for .errorexec effect */
 	    make_int(iesp - 2, ref_stack_count_inline(&o_stack));
 	    make_int(iesp - 1, ref_stack_count_inline(&d_stack));
 	    make_op_estack(iesp, oparray_pop);
@@ -1148,7 +1185,7 @@ remap:		    if (iesp + 2 >= estop) {
 			    return_with_error_iref(code);
 			iesp = esp;
 		    }
-		    packed_get(iref_packed, iesp + 1);
+		    packed_get(imemory, iref_packed, iesp + 1);
 		    make_oper(iesp + 2, 0,
 			      r_ptr(&istate->remap_color_info,
 				    int_remap_color_info_t)->proc);
@@ -1218,6 +1255,7 @@ remap:		    if (iesp + 2 >= estop) {
 		    goto bot;
 		case plain_exec(t_oparray):
 		    INCR(name_oparray);
+		    opindex = op_index(pvalue);
 		    pvalue = (const ref *)pvalue->value.const_refs;
 		    goto opst;
 		case plain_exec(t_operator):
@@ -1263,7 +1301,7 @@ remap:		    if (iesp + 2 >= estop) {
 		    goto top;
 	    }
 	case exec(t_file):
-	    {			/* Executable file.  Read the next token and interpret it. */
+	    {	/* Executable file.  Read the next token and interpret it. */
 		stream *s;
 		scanner_state sstate;
 
@@ -1272,9 +1310,9 @@ remap:		    if (iesp + 2 >= estop) {
 		if (iosp >= ostop)	/* check early */
 		    return_with_stackoverflow_iref();
 		osp = iosp;	/* scan_token uses ostack */
-		scanner_state_init_options(&sstate, i_ctx_p->scanner_options);
+		scanner_init_options(&sstate, IREF, i_ctx_p->scanner_options);
 	    again:
-		code = scan_token(i_ctx_p, s, &token, &sstate);
+		code = scan_token(i_ctx_p, &token, &sstate);
 		iosp = osp;	/* ditto */
 		switch (code) {
 		    case 0:	/* read a token */
@@ -1302,6 +1340,9 @@ remap:		    if (iesp + 2 >= estop) {
 			SET_IREF(&token);
 			icount = 0;
 			goto top;
+		    case e_undefined:	/* //name undefined */
+			scanner_error_object(i_ctx_p, &sstate, &token);
+			return_with_error(code, &token);
 		    case scan_EOF:	/* end of file */
 			esfile_clear_cache();
 			goto bot;
@@ -1328,8 +1369,7 @@ remap:		    if (iesp + 2 >= estop) {
 			ref_assign_inline(iesp, &token);
 			esp = iesp;
 			osp = iosp;
-			code = scan_handle_refill(i_ctx_p, &token, &sstate,
-						  true, true,
+			code = scan_handle_refill(i_ctx_p, &sstate, true,
 						  ztokenexec_continue);
 		scan_cont:
 			iosp = osp;
@@ -1360,14 +1400,16 @@ remap:		    if (iesp + 2 >= estop) {
 			ref_assign_inline(iesp, &file_token);
 			esp = iesp;
 			osp = iosp;
-			code = ztoken_handle_comment(i_ctx_p, &file_token,
+			code = ztoken_handle_comment(i_ctx_p,
 						     &sstate, &token,
 						     code, true, true,
 						     ztokenexec_continue);
 		    }
 			goto scan_cont;
 		    default:	/* error */
-			return_with_code_iref();
+			ref_assign_inline(&token, IREF);
+			scanner_error_object(i_ctx_p, &sstate, &token);
+			return_with_error(code, &token);
 		}
 	    }
 	case exec(t_string):
@@ -1375,10 +1417,11 @@ remap:		    if (iesp + 2 >= estop) {
 		stream ss;
 		scanner_state sstate;
 
-		scanner_state_init_options(&sstate, SCAN_FROM_STRING);
+		s_init(&ss, NULL);
 		sread_string(&ss, IREF->value.bytes, r_size(IREF));
+		scanner_init_stream_options(&sstate, &ss, SCAN_FROM_STRING);
 		osp = iosp;	/* scan_token uses ostack */
-		code = scan_token(i_ctx_p, &ss, &token, &sstate);
+		code = scan_token(i_ctx_p, &token, &sstate);
 		iosp = osp;	/* ditto */
 		switch (code) {
 		    case 0:	/* read a token */
@@ -1411,7 +1454,9 @@ remap:		    if (iesp + 2 >= estop) {
 		    case scan_Refill:	/* error */
 			code = gs_note_error(e_syntaxerror);
 		    default:	/* error */
-			return_with_code_iref();
+			ref_assign_inline(&token, IREF);
+			scanner_error_object(i_ctx_p, &sstate, &token);
+			return_with_error(code, &token);
 		}
 	    }
 	    /* Handle packed arrays here by re-dispatching. */
@@ -1442,6 +1487,7 @@ remap:		    if (iesp + 2 >= estop) {
 			if (!op_index_is_operator(index)) {
 			    INCR(p_exec_oparray);
 			    store_state_short(iesp);
+			    opindex = index;
 			    /* Call the operator procedure. */
 			    index -= op_def_count;
 			    pvalue = (const ref *)
@@ -1646,6 +1692,7 @@ res:
     } else
 	code = 0;
     ticks_left = gs_interp_time_slice_ticks;
+    set_code_on_interrupt(imemory, &code);
     goto sched;
 
     /* Error exits. */
@@ -1662,7 +1709,7 @@ res:
 	 * We need a real object to return as the error object.
 	 * (It only has to last long enough to store in *perror_object.)
 	 */
-	packed_get((const ref_packed *)ierror.obj, &ierror.full);
+	packed_get(imemory, (const ref_packed *)ierror.obj, &ierror.full);
 	store_state_short(iesp);
 	if (IREF == ierror.obj)
 	    SET_IREF(&ierror.full);
@@ -1684,25 +1731,31 @@ res:
     esp = iesp;
     osp = iosp;
     ref_assign_inline(perror_object, ierror.obj);
+#ifdef DEBUG
+    if (ierror.code == e_InterpreterExit) {
+	/* Do not call gs_log_error to reduce the noise. */
+	return e_InterpreterExit;
+    }
+#endif
     return gs_log_error(ierror.code, __FILE__, ierror.line);
 }
 
 /* Pop the bookkeeping information for a normal exit from a t_oparray. */
-private int
+static int
 oparray_pop(i_ctx_t *i_ctx_p)
 {
-    esp -= 3;
+    esp -= 4;
     return o_pop_estack;
 }
 
 /* Restore the stack pointers after an error inside a t_oparray procedure. */
 /* This procedure is called only from pop_estack. */
-private int
+static int
 oparray_cleanup(i_ctx_t *i_ctx_p)
 {				/* esp points just below the cleanup procedure. */
     es_ptr ep = esp;
-    uint ocount_old = (uint) ep[2].value.intval;
-    uint dcount_old = (uint) ep[3].value.intval;
+    uint ocount_old = (uint) ep[3].value.intval;
+    uint dcount_old = (uint) ep[4].value.intval;
     uint ocount = ref_stack_count(&o_stack);
     uint dcount = ref_stack_count(&d_stack);
 
@@ -1716,14 +1769,14 @@ oparray_cleanup(i_ctx_t *i_ctx_p)
 }
 
 /* Don't restore the stack pointers. */
-private int
+static int
 oparray_no_cleanup(i_ctx_t *i_ctx_p)
 {
     return 0;
 }
 
 /* Find the innermost oparray. */
-private ref *
+static ref *
 oparray_find(i_ctx_t *i_ctx_p)
 {
     long i;
@@ -1739,9 +1792,101 @@ oparray_find(i_ctx_t *i_ctx_p)
     return 0;
 }
 
+/* <errorobj> <obj> .errorexec ... */
+/* Execute an object, substituting errorobj for the 'command' if an error */
+/* occurs during the execution.  Cf .execfile (in zfile.c). */
+static int
+zerrorexec(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+    int code;
+
+    check_op(2);
+    check_estack(4);		/* mark/cleanup, errobj, pop, obj */
+    push_mark_estack(es_other, errorexec_cleanup);
+    *++esp = op[-1];
+    push_op_estack(errorexec_pop);
+    code = zexec(i_ctx_p);
+    if (code >= 0)
+	pop(1);
+    else
+	esp -= 3;		/* undo our additions to estack */
+    return code;
+}
+
+/* - .finderrorobject <errorobj> true */
+/* - .finderrorobject false */
+/* If we are within an .errorexec or oparray, return the error object */
+/* and true, otherwise return false. */
+static int
+zfinderrorobject(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+    ref errobj;
+
+    if (errorexec_find(i_ctx_p, &errobj)) {
+	push(2);
+	op[-1] = errobj;
+	make_true(op);
+    } else {
+	push(1);
+	make_false(op);
+    }
+    return 0;
+}
+
+/*
+ * Find the innermost .errorexec or oparray.  If there is an oparray, or a
+ * .errorexec with errobj != null, store it in *perror_object and return 1,
+ * otherwise return 0;
+ */
+static int
+errorexec_find(i_ctx_t *i_ctx_p, ref *perror_object)
+{
+    long i;
+    const ref *ep;
+
+    for (i = 0; (ep = ref_stack_index(&e_stack, i)) != 0; ++i) {
+	if (r_is_estack_mark(ep)) {
+	    if (ep->value.opproc == oparray_cleanup) {
+		/* See oppr: above. */
+		uint opindex = (uint)ep[1].value.intval;
+		if (opindex == 0) /* internal operator, ignore */
+		    continue;
+		op_index_ref(opindex, perror_object);
+		return 1;
+	    }
+	    if (ep->value.opproc == oparray_no_cleanup)
+		return 0;	/* protection disabled */
+	    if (ep->value.opproc == errorexec_cleanup) {
+		if (r_has_type(ep + 1, t_null))
+		    return 0;
+		*perror_object = ep[1];	/* see .errorexec above */
+		return 1;
+	    }
+	}
+    }
+    return 0;
+}
+
+/* Pop the bookkeeping information on a normal exit from .errorexec. */
+static int
+errorexec_pop(i_ctx_t *i_ctx_p)
+{
+    esp -= 2;
+    return o_pop_estack;
+}
+
+/* Clean up when unwinding the stack on an error.  (No action needed.) */
+static int
+errorexec_cleanup(i_ctx_t *i_ctx_p)
+{
+    return 0;
+}
+
 /* <bool> .setstackprotect - */
 /* Set whether to protect the stack for the innermost oparray. */
-private int
+static int
 zsetstackprotect(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -1758,7 +1903,7 @@ zsetstackprotect(i_ctx_t *i_ctx_p)
 
 /* - .currentstackprotect <bool> */
 /* Return the stack protection status. */
-private int
+static int
 zcurrentstackprotect(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
