@@ -1,20 +1,17 @@
-/* Copyright (C) 1993, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: isave.c,v 1.11 2004/08/04 19:36:13 stefan Exp $ */
+/* $Id: isave.c 8496 2008-01-21 20:59:55Z leonardo $ */
 /* Save/restore manager for Ghostscript interpreter */
 #include "ghost.h"
 #include "memory_.h"
@@ -30,19 +27,19 @@
 #include "isstate.h"
 #include "store.h"		/* for ref_assign */
 #include "ivmspace.h"
+#include "igc.h"
 #include "gsutil.h"		/* gs_next_ids prototype */
 
-
 /* Structure descriptor */
 private_st_alloc_save();
 
 /* Define the maximum amount of data we are willing to scan repeatedly -- */
 /* see below for details. */
-private const long max_repeated_scan = 100000;
+static const long max_repeated_scan = 100000;
 
 /* Define the minimum space for creating an inner chunk. */
 /* Must be at least sizeof(chunk_head_t). */
-private const long min_inner_chunk_space = sizeof(chunk_head_t) + 500;
+static const long min_inner_chunk_space = sizeof(chunk_head_t) + 500;
 
 /*
  * The logic for saving and restoring the state is complex.
@@ -149,13 +146,16 @@ private const long min_inner_chunk_space = sizeof(chunk_head_t) + 500;
  */
 
 /* Tracing printout */
-private void
+static void
 print_save(const char *str, uint spacen, const alloc_save_t *sav)
 {
   if_debug5('u', "[u]%s space %u 0x%lx: cdata = 0x%lx, id = %lu\n",\
 	    str, spacen, (ulong)sav, (ulong)sav->client_data, (ulong)sav->id);
 }
 
+/* A link to igcref.c . */
+ptr_proc_reloc(igc_reloc_ref_ptr_nocheck, ref_packed);
+
 /*
  * Structure for saved change chain for save/restore.  Because of the
  * garbage collector, we need to distinguish the cases where the change
@@ -168,10 +168,11 @@ struct alloc_change_s {
     ref contents;
 #define AC_OFFSET_STATIC (-2)	/* static object */
 #define AC_OFFSET_REF (-1)	/* dynamic ref */
+#define AC_OFFSET_ALLOCATED (-3) /* a newly allocated ref array */
     short offset;		/* if >= 0, offset within struct */
 };
 
-private 
+static 
 CLEAR_MARKS_PROC(change_clear_marks)
 {
     alloc_change_t *const ptr = (alloc_change_t *)vptr;
@@ -181,18 +182,25 @@ CLEAR_MARKS_PROC(change_clear_marks)
     else
 	r_clear_attrs(&ptr->contents, l_mark);
 }
-private 
+static 
 ENUM_PTRS_WITH(change_enum_ptrs, alloc_change_t *ptr) return 0;
 ENUM_PTR(0, alloc_change_t, next);
 case 1:
     if (ptr->offset >= 0)
 	ENUM_RETURN((byte *) ptr->where - ptr->offset);
     else
-	ENUM_RETURN_REF(ptr->where);
+	if (ptr->offset != AC_OFFSET_ALLOCATED)
+	    ENUM_RETURN_REF(ptr->where);
+	else {
+	    /* Don't enumerate ptr->where, because it 
+	       needs a special processing with 
+	       alloc_save__filter_changes. */
+    	    ENUM_RETURN(0);
+	}
 case 2:
     ENUM_RETURN_REF(&ptr->contents);
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(change_reloc_ptrs, alloc_change_t *ptr)
+static RELOC_PTRS_WITH(change_reloc_ptrs, alloc_change_t *ptr)
 {
     RELOC_VAR(ptr->next);
     switch (ptr->offset) {
@@ -201,6 +209,24 @@ private RELOC_PTRS_WITH(change_reloc_ptrs, alloc_change_t *ptr)
 	case AC_OFFSET_REF:
 	    RELOC_REF_PTR_VAR(ptr->where);
 	    break;
+	case AC_OFFSET_ALLOCATED:
+	    /* We know that ptr->where may point to an unmarked object
+	       because change_enum_ptrs skipped it,
+	       and we know it always points to same space 
+	       because we took a special care when calling alloc_save_change_alloc.
+	       Therefore we must skip the check for the mark,
+	       which would happen if we call the regular relocation function
+	       igc_reloc_ref_ptr from RELOC_REF_PTR_VAR. 
+	       Calling igc_reloc_ref_ptr_nocheck instead. */
+	    {	/* A sanity check. */
+		obj_header_t *pre = (obj_header_t *)ptr->where - 1, *pre1 = 0;
+
+		if (pre->o_type != &st_refs)
+		    pre1->o_type = 0; /* issue a segfault. */
+	    }
+	    if (ptr->where != 0 && !gcst->relocating_untraced)
+		ptr->where = igc_reloc_ref_ptr_nocheck(ptr->where, gcst);
+	    break;
 	default:
 	    {
 		byte *obj = (byte *) ptr->where - ptr->offset;
@@ -223,7 +249,7 @@ gs_private_st_complex_only(st_alloc_change, alloc_change_t, "alloc_change",
 
 /* Debugging printout */
 #ifdef DEBUG
-private void
+static void
 alloc_save_print(alloc_change_t * cp, bool print_current)
 {
     dprintf2(" 0x%lx: 0x%lx: ", (ulong) cp, (ulong) cp->where);
@@ -250,10 +276,11 @@ alloc_save_print(alloc_change_t * cp, bool print_current)
 #endif
 
 /* Forward references */
-private void restore_resources(alloc_save_t *, gs_ref_memory_t *);
-private void restore_free(gs_ref_memory_t *);
-private long save_set_new(gs_ref_memory_t *, bool);
-private void save_set_new_changes(gs_ref_memory_t *, bool);
+static int  restore_resources(alloc_save_t *, gs_ref_memory_t *);
+static void restore_free(gs_ref_memory_t *);
+static int  save_set_new(gs_ref_memory_t * mem, bool to_new, bool set_limit, ulong *pscanned);
+static int  save_set_new_changes(gs_ref_memory_t *, bool, bool);
+static bool check_l_mark(void *obj);
 
 /* Initialize the save/restore machinery. */
 void
@@ -263,7 +290,7 @@ alloc_save_init(gs_dual_memory_t * dmem)
 }
 
 /* Record that we are in a save. */
-private void
+static void
 alloc_set_masks(gs_dual_memory_t *dmem, uint new_mask, uint test_mask)
 {
     int i;
@@ -294,18 +321,18 @@ alloc_set_not_in_save(gs_dual_memory_t *dmem)
 }
 
 /* Save the state. */
-private alloc_save_t *alloc_save_space(gs_ref_memory_t *mem,
+static alloc_save_t *alloc_save_space(gs_ref_memory_t *mem,
 				       gs_dual_memory_t *dmem,
 				       ulong sid);
-private void
+static void
 alloc_free_save(gs_ref_memory_t *mem, alloc_save_t *save, const char *scn)
 {
     gs_free_object((gs_memory_t *)mem, save, scn);
     /* Free any inner chunk structures.  This is the easiest way to do it. */
     restore_free(mem);
 }
-ulong
-alloc_save_state(gs_dual_memory_t * dmem, void *cdata)
+int
+alloc_save_state(gs_dual_memory_t * dmem, void *cdata, ulong *psid)
 {
     gs_ref_memory_t *lmem = dmem->space_local;
     gs_ref_memory_t *gmem = dmem->space_global;
@@ -338,8 +365,12 @@ alloc_save_state(gs_dual_memory_t * dmem, void *cdata)
     /* can have the attribute set are the ones on the changes chain, */
     /* and ones in objects allocated since the last save. */
     if (lmem->save_level > 1) {
-	long scanned = save_set_new(&lsave->state, false);
+	ulong scanned;
+	int code = save_set_new(&lsave->state, false, true, &scanned);
 
+	if (code < 0)
+	    return code;
+#if 0 /* Disable invisible save levels. */
 	if ((lsave->state.total_scanned += scanned) > max_repeated_scan) {
 	    /* Do a second, invisible save. */
 	    alloc_save_t *rsave;
@@ -347,25 +378,31 @@ alloc_save_state(gs_dual_memory_t * dmem, void *cdata)
 	    rsave = alloc_save_space(lmem, dmem, 0L);
 	    if (rsave != 0) {
 		rsave->client_data = cdata;
+#if 0 /* Bug 688153 */
 		rsave->id = lsave->id;
 		print_save("save", lmem->space, rsave);
 		lsave->id = 0;	/* mark as invisible */
 		rsave->state.save_level--; /* ditto */
 		lsave->client_data = 0;
+#else
+		rsave->id = 0;  /* mark as invisible */
+		print_save("save", lmem->space, rsave);
+		rsave->state.save_level--; /* ditto */
+		rsave->client_data = 0;
+#endif
 		/* Inherit the allocated space count -- */
 		/* we need this for triggering a GC. */
-		rsave->state.inherited =
-		    lsave->state.allocated + lsave->state.inherited;
-		lmem->inherited = rsave->state.inherited;
 		print_save("save", lmem->space, lsave);
 	    }
 	}
+#endif
     }
     alloc_set_in_save(dmem);
-    return sid;
+    *psid = sid;
+    return 0;
 }
 /* Save the state of one space (global or local). */
-private alloc_save_t *
+static alloc_save_t *
 alloc_save_space(gs_ref_memory_t * mem, gs_dual_memory_t * dmem, ulong sid)
 {
     gs_ref_memory_t save_mem;
@@ -421,6 +458,7 @@ alloc_save_space(gs_ref_memory_t * mem, gs_dual_memory_t * dmem, ulong sid)
 	      mem->space, (ulong) mem->streams);
     mem->streams = 0;
     mem->total_scanned = 0;
+    mem->total_scanned_after_compacting = 0;
     if (sid)
 	mem->save_level++;
     return save;
@@ -479,6 +517,82 @@ alloc_save_change(gs_dual_memory_t * dmem, const ref * pcont,
     return alloc_save_change_in(mem, pcont, where, cname);
 }
 
+/* Allocate a structure for recording an allocation event. */
+int
+alloc_save_change_alloc(gs_ref_memory_t *mem, client_name_t cname, ref_packed ***ppr)
+{
+    register alloc_change_t *cp;
+
+    if (mem->new_mask == 0)
+	return 0;		/* no saving */
+    cp = gs_alloc_struct((gs_memory_t *)mem, alloc_change_t,
+			 &st_alloc_change, "alloc_save_change");
+    if (cp == 0)
+	return_error(e_VMerror);
+    cp->next = mem->changes;
+    cp->where = 0;
+    cp->offset = AC_OFFSET_ALLOCATED;
+    make_null(&cp->contents);
+    mem->changes = cp;
+    *ppr = &cp->where;
+    return 1;
+}
+
+/* Remove an AC_OFFSET_ALLOCATED element. */
+void
+alloc_save_remove(gs_ref_memory_t *mem, ref_packed *obj, client_name_t cname)
+{
+    alloc_change_t **cpp = &mem->changes;
+    
+    for (; *cpp != NULL;) {
+	alloc_change_t *cp = *cpp;
+
+	if (cp->offset == AC_OFFSET_ALLOCATED && cp->where == obj) {
+	    if (mem->scan_limit == cp)
+		mem->scan_limit = cp->next;
+	    *cpp = cp->next;
+	    gs_free_object((gs_memory_t *)mem, cp, "alloc_save_remove");
+	} else
+	    cpp = &(*cpp)->next;
+    }
+}
+
+/* Filter save change lists. */
+static inline void
+alloc_save__filter_changes_in_space(gs_ref_memory_t *mem)
+{
+    /* This is a special function, which is called
+       from the garbager after setting marks and before collecting
+       unused space. Therefore it just resets marks for
+       elements being released instead releasing them really. */
+    alloc_change_t **cpp = &mem->changes;
+    
+    for (; *cpp != NULL; ) {
+	alloc_change_t *cp = *cpp;
+
+	if (cp->offset == AC_OFFSET_ALLOCATED && !check_l_mark(cp->where)) {
+	    obj_header_t *pre = (obj_header_t *)cp - 1;
+
+	    *cpp = cp->next;
+	    cp->where = 0;
+	    if (mem->scan_limit == cp)
+		mem->scan_limit = cp->next;
+	    o_set_unmarked(pre);
+	} else
+	    cpp = &(*cpp)->next;
+    }
+}
+
+/* Filter save change lists. */
+void
+alloc_save__filter_changes(gs_ref_memory_t *memory)
+{
+    gs_ref_memory_t *mem = memory;
+
+    for  (; mem; mem = &mem->saved->state)
+	alloc_save__filter_changes_in_space(mem);
+}
+
 /* Return (the id of) the innermost externally visible save object, */
 /* i.e., the innermost save with a non-zero ID. */
 ulong
@@ -534,13 +648,13 @@ alloc_is_since_save(const void *vptr, const alloc_save_t * save)
     }
 
     /*
-     * If we're about to do a global restore (save level = 1),
+     * If we're about to do a global restore (a restore to the level 0),
      * and there is only one context using this global VM
      * (the normal case, in which global VM is saved by the
      * outermost save), we also have to check the global save.
      * Global saves can't be nested, which makes things easy.
      */
-    if (mem->save_level == 1 &&
+    if (save->state.save_level == 0 /* Restoring to save level 0 - see bug 688157, 688161 */ &&
 	(mem = save->space_global) != save->space_local &&
 	save->space_global->num_contexts == 1
 	) {
@@ -628,10 +742,10 @@ alloc_save_client_data(const alloc_save_t * save)
  * if this is the outermost restore (which requires restoring both local
  * and global VM) or if we created extra save levels to reduce scanning.
  */
-private void restore_finalize(gs_ref_memory_t *);
-private void restore_space(gs_ref_memory_t *, gs_dual_memory_t *);
+static void restore_finalize(gs_ref_memory_t *);
+static void restore_space(gs_ref_memory_t *, gs_dual_memory_t *);
 
-bool
+int
 alloc_restore_step_in(gs_dual_memory_t *dmem, alloc_save_t * save)
 {
     /* Get save->space_* now, because the save object will be freed. */
@@ -639,6 +753,7 @@ alloc_restore_step_in(gs_dual_memory_t *dmem, alloc_save_t * save)
     gs_ref_memory_t *gmem = save->space_global;
     gs_ref_memory_t *mem = lmem;
     alloc_save_t *sprev;
+    int code;
 
     /* Finalize all objects before releasing resources or undoing changes. */
     do {
@@ -667,7 +782,9 @@ alloc_restore_step_in(gs_dual_memory_t *dmem, alloc_save_t * save)
 
 	sprev = mem->saved;
 	sid = sprev->id;
-	restore_resources(sprev, mem);	/* release other resources */
+	code = restore_resources(sprev, mem);	/* release other resources */
+	if (code < 0)
+	    return code;
 	restore_space(mem, dmem);	/* release memory */
 	if (sid != 0)
 	    break;
@@ -679,19 +796,25 @@ alloc_restore_step_in(gs_dual_memory_t *dmem, alloc_save_t * save)
 	/* need to restore global VM. */
 	mem = gmem;
 	if (mem != lmem && mem->saved != 0) {
-	    restore_resources(mem->saved, mem);
+	    code = restore_resources(mem->saved, mem);
+	    if (code < 0)
+		return code;
 	    restore_space(mem, dmem);
 	}
 	alloc_set_not_in_save(dmem);
     } else {			/* Set the l_new attribute in all slots that are now new. */
-	save_set_new(mem, true);
+	ulong scanned;
+
+	code = save_set_new(mem, true, false, &scanned);
+	if (code < 0)
+	    return code;
     }
 
     return sprev == save;
 }
 /* Restore the memory of one space, by undoing changes and freeing */
 /* memory allocated since the save. */
-private void
+static void
 restore_space(gs_ref_memory_t * mem, gs_dual_memory_t *dmem)
 {
     alloc_save_t *save = mem->saved;
@@ -710,6 +833,9 @@ restore_space(gs_ref_memory_t * mem, gs_dual_memory_t *dmem)
 		alloc_save_print(cp, true);
 	    }
 #endif
+	    if (cp->offset == AC_OFFSET_ALLOCATED)
+		DO_NOTHING;
+	    else
 	    if (r_is_packed(&cp->contents))
 		*cp->where = *(ref_packed *) & cp->contents;
 	    else
@@ -742,7 +868,7 @@ restore_space(gs_ref_memory_t * mem, gs_dual_memory_t *dmem)
 
 /* Restore to the initial state, releasing all resources. */
 /* The allocator is no longer usable after calling this routine! */
-void
+int
 alloc_restore_all(gs_dual_memory_t * dmem)
 {
     /*
@@ -753,10 +879,14 @@ alloc_restore_all(gs_dual_memory_t * dmem)
     gs_ref_memory_t *gmem = dmem->space_global;
     gs_ref_memory_t *smem = dmem->space_system;
     gs_ref_memory_t *mem;
+    int code;
 
     /* Restore to a state outside any saves. */
-    while (lmem->save_level != 0)
-	discard(alloc_restore_step_in(dmem, lmem->saved));
+    while (lmem->save_level != 0) {
+	code = alloc_restore_step_in(dmem, lmem->saved);
+	if (code < 0)
+	    return code;
+    }
 
     /* Finalize memory. */
     restore_finalize(lmem);
@@ -776,7 +906,9 @@ alloc_restore_all(gs_dual_memory_t * dmem)
 
 	empty_save.spaces = dmem->spaces;
 	empty_save.restore_names = false;	/* don't bother to release */
-	restore_resources(&empty_save, NULL);
+	code = restore_resources(&empty_save, NULL);
+	if (code < 0)
+	    return code;
     }
 
     /* Finally, release memory. */
@@ -791,7 +923,7 @@ alloc_restore_all(gs_dual_memory_t * dmem)
 	}
     }
     restore_free(smem);
-
+    return 0;
 }
 
 /*
@@ -799,7 +931,7 @@ alloc_restore_all(gs_dual_memory_t * dmem)
  * Note that we must temporarily disable the freeing operations
  * of the allocator while doing this.
  */
-private void
+static void
 restore_finalize(gs_ref_memory_t * mem)
 {
     chunk_t *cp;
@@ -823,9 +955,10 @@ restore_finalize(gs_ref_memory_t * mem)
 }
 
 /* Release resources for a restore */
-private void
+static int
 restore_resources(alloc_save_t * sprev, gs_ref_memory_t * mem)
 {
+    int code;
 #ifdef DEBUG
     if (mem) {
 	/* Note restoring of the file list. */
@@ -836,15 +969,18 @@ restore_resources(alloc_save_t * sprev, gs_ref_memory_t * mem)
 #endif
 
     /* Remove entries from font and character caches. */
-    font_restore(sprev);
+    code = font_restore(sprev);
+    if (code < 0)
+	return code;
 
     /* Adjust the name table. */
     if (sprev->restore_names)
 	names_restore(mem->gs_lib_ctx->gs_name_table, sprev);
+    return 0;
 }
 
 /* Release memory for a restore. */
-private void
+static void
 restore_free(gs_ref_memory_t * mem)
 {
     /* Free chunks allocated since the save. */
@@ -852,14 +988,16 @@ restore_free(gs_ref_memory_t * mem)
 }
 
 /* Forget a save, by merging this level with the next outer one. */
-private void file_forget_save(gs_ref_memory_t *);
-private void combine_space(gs_ref_memory_t *);
-private void forget_changes(gs_ref_memory_t *);
-void
+static void file_forget_save(gs_ref_memory_t *);
+static void combine_space(gs_ref_memory_t *);
+static void forget_changes(gs_ref_memory_t *);
+int
 alloc_forget_save_in(gs_dual_memory_t *dmem, alloc_save_t * save)
 {
     gs_ref_memory_t *mem = save->space_local;
     alloc_save_t *sprev;
+    ulong scanned;
+    int code;
 
     print_save("forget_save", mem->space, save);
 
@@ -871,7 +1009,9 @@ alloc_forget_save_in(gs_dual_memory_t *dmem, alloc_save_t * save)
 	if (mem->save_level != 0) {
 	    alloc_change_t *chp = mem->changes;
 
-	    save_set_new(&sprev->state, true);
+	    code = save_set_new(&sprev->state, true, false, &scanned);
+	    if (code < 0)
+		return code;
 	    /* Concatenate the changes chains. */
 	    if (chp == 0)
 		mem->changes = sprev->state.changes;
@@ -884,7 +1024,9 @@ alloc_forget_save_in(gs_dual_memory_t *dmem, alloc_save_t * save)
 	    combine_space(mem);	/* combine memory */
 	} else {
 	    forget_changes(mem);
-	    save_set_new(mem, false);
+	    code = save_set_new(mem, false, false, &scanned);
+	    if (code < 0)
+		return code;
 	    file_forget_save(mem);
 	    combine_space(mem);	/* combine memory */
 	    /* This is the outermost save, which might also */
@@ -892,7 +1034,9 @@ alloc_forget_save_in(gs_dual_memory_t *dmem, alloc_save_t * save)
 	    mem = save->space_global;
 	    if (mem != save->space_local && mem->saved != 0) {
 		forget_changes(mem);
-		save_set_new(mem, false);
+		code = save_set_new(mem, false, false, &scanned);
+		if (code < 0)
+		    return code;
 		file_forget_save(mem);
 		combine_space(mem);
 	    }
@@ -901,10 +1045,11 @@ alloc_forget_save_in(gs_dual_memory_t *dmem, alloc_save_t * save)
 	}
     }
     while (sprev != save);
+    return 0;
 }
 /* Combine the chunks of the next outer level with those of the current one, */
 /* and free the bookkeeping structures. */
-private void
+static void
 combine_space(gs_ref_memory_t * mem)
 {
     alloc_save_t *saved = mem->saved;
@@ -985,7 +1130,7 @@ combine_space(gs_ref_memory_t * mem)
 }
 /* Free the changes chain for a level 0 .forgetsave, */
 /* resetting the l_new flag in the changed refs. */
-private void
+static void
 forget_changes(gs_ref_memory_t * mem)
 {
     register alloc_change_t *chp = mem->changes;
@@ -995,6 +1140,9 @@ forget_changes(gs_ref_memory_t * mem)
 	ref_packed *prp = chp->where;
 
 	if_debug1('U', "[U]forgetting change 0x%lx\n", (ulong) chp);
+	if (chp->offset == AC_OFFSET_ALLOCATED)
+	    DO_NOTHING;
+	else
 	if (!r_is_packed(prp))
 	    r_clear_attrs((ref *) prp, l_new);
 	next = chp->next;
@@ -1003,7 +1151,7 @@ forget_changes(gs_ref_memory_t * mem)
     mem->changes = 0;
 }
 /* Update the streams list when forgetting a save. */
-private void
+static void
 file_forget_save(gs_ref_memory_t * mem)
 {
     const alloc_save_t *save = mem->saved;
@@ -1023,19 +1171,102 @@ file_forget_save(gs_ref_memory_t * mem)
     }
 }
 
-/* ------ Internal routines ------ */
+static inline int
+mark_allocated(void *obj, bool to_new, uint *psize)
+{   
+    obj_header_t *pre = (obj_header_t *)obj - 1;
+    uint size = pre_obj_contents_size(pre);
+    ref_packed *prp = (ref_packed *) (pre + 1);
+    ref_packed *next = (ref_packed *) ((char *)prp + size);
+#ifdef ALIGNMENT_ALIASING_BUG
+		ref *rpref;
+# define RP_REF(rp) (rpref = (ref *)rp, rpref)
+#else
+# define RP_REF(rp) ((ref *)rp)
+#endif
+
+    if (pre->o_type != &st_refs) {
+	/* Must not happen. */
+	if_debug0('u', "Wrong object type when expected a ref.\n");
+	return_error(e_Fatal);
+    }
+    /* We know that every block of refs ends with */
+    /* a full-size ref, so we only need the end check */
+    /* when we encounter one of those. */
+    if (to_new)
+	while (1) {
+	    if (r_is_packed(prp))
+		prp++;
+	    else {
+		RP_REF(prp)->tas.type_attrs |= l_new;
+		prp += packed_per_ref;
+		if (prp >= next)
+		    break;
+	    }
+    } else
+	while (1) {
+	    if (r_is_packed(prp))
+		prp++;
+	    else {
+		RP_REF(prp)->tas.type_attrs &= ~l_new;
+		prp += packed_per_ref;
+		if (prp >= next)
+		    break;
+	    }
+	}
+#undef RP_REF
+    *psize = size;
+    return 0;
+}
+
+/* Check if a block contains refs marked by garbager. */
+static bool
+check_l_mark(void *obj)
+{   
+    obj_header_t *pre = (obj_header_t *)obj - 1;
+    uint size = pre_obj_contents_size(pre);
+    ref_packed *prp = (ref_packed *) (pre + 1);
+    ref_packed *next = (ref_packed *) ((char *)prp + size);
+#ifdef ALIGNMENT_ALIASING_BUG
+		ref *rpref;
+# define RP_REF(rp) (rpref = (ref *)rp, rpref)
+#else
+# define RP_REF(rp) ((ref *)rp)
+#endif
+
+    /* We know that every block of refs ends with */
+    /* a full-size ref, so we only need the end check */
+    /* when we encounter one of those. */
+    while (1) {
+	if (r_is_packed(prp)) {
+	    if (r_has_pmark(prp))
+		return true;
+	    prp++;
+	} else {
+	    if (r_has_attr(RP_REF(prp), l_mark))
+		return true;
+	    prp += packed_per_ref;
+	    if (prp >= next)
+		return false;
+	}
+    }
+#undef RP_REF
+}
 
 /* Set or reset the l_new attribute in every relevant slot. */
 /* This includes every slot on the current change chain, */
 /* and every (ref) slot allocated at this save level. */
 /* Return the number of bytes of data scanned. */
-private long
-save_set_new(gs_ref_memory_t * mem, bool to_new)
+static int
+save_set_new(gs_ref_memory_t * mem, bool to_new, bool set_limit, ulong *pscanned)
 {
-    long scanned = 0;
+    ulong scanned = 0;
+    int code;
 
     /* Handle the change chain. */
-    save_set_new_changes(mem, to_new);
+    code = save_set_new_changes(mem, to_new, set_limit);
+    if (code < 0)
+	return code;
 
     /* Handle newly allocated ref objects. */
     SCAN_MEM_CHUNKS(mem, cp) {
@@ -1049,43 +1280,12 @@ save_set_new(gs_ref_memory_t * mem, bool to_new)
 	    if (pre->o_type == &st_refs) {
 		/* These are refs, scan them. */
 		ref_packed *prp = (ref_packed *) (pre + 1);
-		ref_packed *next = (ref_packed *) ((char *)prp + size);
-#ifdef ALIGNMENT_ALIASING_BUG
-		ref *rpref;
-# define RP_REF(rp) (rpref = (ref *)rp, rpref)
-#else
-# define RP_REF(rp) ((ref *)rp)
-#endif
-
-		if_debug2('U', "[U]refs 0x%lx to 0x%lx\n",
-			  (ulong) prp, (ulong) next);
-		has_refs = true;
+		uint size;
+		
+		code = mark_allocated(prp, to_new, &size);
+		if (code < 0)
+		    return code;
 		scanned += size;
-		/* We know that every block of refs ends with */
-		/* a full-size ref, so we only need the end check */
-		/* when we encounter one of those. */
-		if (to_new)
-		    while (1) {
-			if (r_is_packed(prp))
-			    prp++;
-			else {
-			    RP_REF(prp)->tas.type_attrs |= l_new;
-			    prp += packed_per_ref;
-			    if (prp >= next)
-				break;
-			}
-		} else
-		    while (1) {
-			if (r_is_packed(prp))
-			    prp++;
-			else {
-			    RP_REF(prp)->tas.type_attrs &= ~l_new;
-			    prp += packed_per_ref;
-			    if (prp >= next)
-				break;
-			}
-		    }
-#undef RP_REF
 	    } else
 		scanned += sizeof(obj_header_t);
 	    END_OBJECTS_SCAN
@@ -1095,26 +1295,104 @@ save_set_new(gs_ref_memory_t * mem, bool to_new)
     END_CHUNKS_SCAN
 	if_debug2('u', "[u]set_new (%s) scanned %ld\n",
 		  (to_new ? "restore" : "save"), scanned);
-    return scanned;
+    *pscanned = scanned;
+    return 0;
 }
 
+/* Drop redundant elements from the changes list and set l_new. */
+static void
+drop_redundant_changes(gs_ref_memory_t * mem)
+{
+    register alloc_change_t *chp = mem->changes, *chp_back = NULL, *chp_forth;
+
+    /* First reverse the list and set all. */
+    for (; chp; chp = chp_forth) {
+	chp_forth = chp->next;
+	if (chp->offset != AC_OFFSET_ALLOCATED) {
+	    ref_packed *prp = chp->where;
+
+	    if (!r_is_packed(prp)) {
+		ref *const rp = (ref *)prp;
+
+		rp->tas.type_attrs |= l_new;
+	    }
+	}
+	chp->next = chp_back;
+	chp_back = chp;
+    }
+    mem->changes = chp_back;
+    chp_back = NULL;
+    /* Then filter, reset and reverse again. */
+    for (chp = mem->changes; chp; chp = chp_forth) {
+	chp_forth = chp->next;
+	if (chp->offset != AC_OFFSET_ALLOCATED) {
+	    ref_packed *prp = chp->where;
+
+	    if (!r_is_packed(prp)) {
+		ref *const rp = (ref *)prp;
+
+		if ((rp->tas.type_attrs & l_new) == 0) {
+		    if (mem->scan_limit == chp)
+			mem->scan_limit = chp_back;
+		    if (mem->changes == chp)
+			mem->changes = chp_back;
+	    	    gs_free_object((gs_memory_t *)mem, chp, "alloc_save_remove");
+		    continue;
+		} else
+		    rp->tas.type_attrs &= ~l_new;
+	    }
+	}
+	chp->next = chp_back;
+	chp_back = chp;
+    }
+    mem->changes = chp_back;
+}
+
+
 /* Set or reset the l_new attribute on the changes chain. */
-private void
-save_set_new_changes(gs_ref_memory_t * mem, bool to_new)
+static int
+save_set_new_changes(gs_ref_memory_t * mem, bool to_new, bool set_limit)
 {
-    register alloc_change_t *chp = mem->changes;
+    register alloc_change_t *chp;
     register uint new = (to_new ? l_new : 0);
+    ulong scanned = 0;
 
-    for (; chp; chp = chp->next) {
-	ref_packed *prp = chp->where;
+    if (!to_new && mem->total_scanned_after_compacting > max_repeated_scan * 16) {
+	mem->total_scanned_after_compacting = 0;
+	drop_redundant_changes(mem);
+    } 
+    for (chp = mem->changes; chp; chp = chp->next) {
+	if (chp->offset == AC_OFFSET_ALLOCATED) {
+	    if (chp->where != 0) {
+		uint size;
+		int code = mark_allocated((void *)chp->where, to_new, &size);
 
-	if_debug3('U', "[U]set_new 0x%lx: (0x%lx, %d)\n",
-		  (ulong)chp, (ulong)prp, new);
-	if (!r_is_packed(prp)) {
-	    ref *const rp = (ref *) prp;
+		if (code < 0)
+		    return code;
+		scanned += size;
+	    }
+	} else {
+	    ref_packed *prp = chp->where;
 
-	    rp->tas.type_attrs =
-		(rp->tas.type_attrs & ~l_new) + new;
+	    if_debug3('U', "[U]set_new 0x%lx: (0x%lx, %d)\n",
+		    (ulong)chp, (ulong)prp, new);
+	    if (!r_is_packed(prp)) {
+		ref *const rp = (ref *) prp;
+
+		rp->tas.type_attrs =
+		    (rp->tas.type_attrs & ~l_new) + new;
+	    }
 	}
+	if (mem->scan_limit == chp)
+	    break;
     }
+    if (set_limit) {
+        mem->total_scanned_after_compacting += scanned;
+	if (scanned  + mem->total_scanned >= max_repeated_scan) {
+	    mem->scan_limit = mem->changes;
+	    mem->total_scanned = 0;
+	} else
+	    mem->total_scanned += scanned;
+    }
+    return 0;
 }
