@@ -1,23 +1,17 @@
-/* Copyright (C) 1992, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: scfd.c,v $ $Revision: 1.2.2.2 $ */
+/* $Id: scfd.c 8250 2007-09-25 13:31:24Z giles $ */
 /* CCITTFax decoding filter */
 #include "stdio_.h"		/* includes std.h */
 #include "memory_.h"
@@ -31,7 +25,7 @@
 private_st_CFD_state();
 
 /* Set default parameter values. */
-private void
+static void
 s_CFD_set_defaults(register stream_state * st)
 {
     stream_CFD_state *const ss = (stream_CFD_state *) st;
@@ -40,7 +34,7 @@ s_CFD_set_defaults(register stream_state * st)
 }
 
 /* Initialize CCITTFaxDecode filter */
-private int
+static int
 s_CFD_init(stream_state * st)
 {
     stream_CFD_state *const ss = (stream_CFD_state *) st;
@@ -52,19 +46,18 @@ s_CFD_init(stream_state * st)
     /* Because skip_white_pixels can look as many as 4 bytes ahead, */
     /* we need to allow 4 extra bytes at the end of the row buffers. */
     ss->lbuf = gs_alloc_bytes(st->memory, raster + 4, "CFD lbuf");
+    memset(ss->lbuf, white, raster + 4);  /* + 4 for Valgrind */
     ss->lprev = 0;
     if (ss->lbuf == 0)
-	return ERRC;
-/****** WRONG ******/
+	return ERRC;		/****** WRONG ******/
     if (ss->K != 0) {
 	ss->lprev = gs_alloc_bytes(st->memory, raster + 4, "CFD lprev");
 	if (ss->lprev == 0)
-	    return ERRC;
-/****** WRONG ******/
+	    return ERRC;	/****** WRONG ******/
 	/* Clear the initial reference line for 2-D encoding. */
-	memset(ss->lbuf, white, raster);
+	memset(ss->lprev, white, raster + 4); /* + 4 for Valgrind */
 	/* Ensure that the scan of the reference line will stop. */
-	ss->lbuf[raster] = 0xa0;
+	ss->lprev[raster] = 0xa0;
     }
     ss->k_left = min(ss->K, 0);
     ss->run_color = 0;
@@ -72,8 +65,9 @@ s_CFD_init(stream_state * st)
     ss->skipping_damage = false;
     ss->cbit = 0;
     ss->uncomp_run = 0;
-    ss->rows_left = (ss->Rows <= 0 || ss->EndOfBlock ? -1 : ss->Rows + 1);
-    ss->rpos = ss->wpos = raster - 1;
+    ss->rows_left = (ss->Rows <= 0 || ss->EndOfBlock ? -1 : ss->Rows);
+    ss->row = 0;
+    ss->rpos = ss->wpos = -1;
     ss->eol_count = 0;
     ss->invert = white;
     ss->min_left = 1;
@@ -81,7 +75,7 @@ s_CFD_init(stream_state * st)
 }
 
 /* Release the filter. */
-private void
+static void
 s_CFD_release(stream_state * st)
 {
     stream_CFD_state *const ss = (stream_CFD_state *) st;
@@ -201,11 +195,11 @@ d:			memset(q, black_byte, rlen >> 3);\
 		*q ^= ((1 << rlen) - 1) << qbit
 
 /* Buffer refill for CCITTFaxDecode filter */
-private int cf_decode_eol(P2(stream_CFD_state *, stream_cursor_read *));
-private int cf_decode_1d(P2(stream_CFD_state *, stream_cursor_read *));
-private int cf_decode_2d(P2(stream_CFD_state *, stream_cursor_read *));
-private int cf_decode_uncompressed(P2(stream_CFD_state *, stream_cursor_read *));
-private int
+static int cf_decode_eol(stream_CFD_state *, stream_cursor_read *);
+static int cf_decode_1d(stream_CFD_state *, stream_cursor_read *);
+static int cf_decode_2d(stream_CFD_state *, stream_cursor_read *);
+static int cf_decode_uncompressed(stream_CFD_state *, stream_cursor_read *);
+static int
 s_CFD_process(stream_state * st, stream_cursor_read * pr,
 	      stream_cursor_write * pw, bool last)
 {
@@ -227,9 +221,8 @@ s_CFD_process(stream_state * st, stream_cursor_read * pr,
     {
 	hcd_declare_state;
 	hcd_load_state();
-	if_debug8('w', "\
-[w]CFD_process top: eol_count=%d, k_left=%d, rows_left=%d\n\
-    bits=0x%lx, bits_left=%d, read %u, wrote %u%s\n",
+	if_debug8('w', "[w]CFD_process top: eol_count=%d, k_left=%d, rows_left=%d\n"
+    		"    bits=0x%lx, bits_left=%d, read %u, wrote %u%s\n",
 		  eol_count, k_left, rows_left,
 		  (ulong) bits, bits_left,
 		  (uint) (p - rstart), (uint) (pw->ptr - wstart),
@@ -285,10 +278,9 @@ s_CFD_process(stream_state * st, stream_cursor_read * pr,
 	    if (status)
 		goto out;
 	}
-	if (rows_left > 0 && --rows_left == 0) {
-	    status = EOFC;
-	    goto out;
-	}
+	ss->row++;
+	if (rows_left > 0 && --rows_left == 0) 
+	    goto ck_eol;	/* handle EOD if it is present */
 	if (ss->K != 0) {
 	    byte *prev_bits = ss->lprev;
 
@@ -308,11 +300,17 @@ s_CFD_process(stream_state * st, stream_cursor_read * pr,
 	 * in the string of initial zeros in the EOL.  If EndOfLine
 	 * is false, we aren't sure what we should do....
 	 */
-	if (ss->EncodedByteAlign & !ss->EndOfLine)
+	if (ss->EncodedByteAlign && !ss->EndOfLine)
 	    ss->bits_left &= ~7;
     }
     /* If we're between scan lines, scan for EOLs. */
     if (ss->wpos < 0) {
+	    /*
+	     * According to Adobe, the decoder should always check for
+	     * the EOD sequence, regardless of EndOfBlock: the Red Book's
+	     * documentation of EndOfBlock is wrong.
+	     */
+ck_eol:
 	while ((status = cf_decode_eol(ss, pr)) > 0) {
 	    if_debug0('w', "[w]EOL\n");
 	    /* If we are in a Group 3 mixed regime, */
@@ -326,16 +324,15 @@ s_CFD_process(stream_state * st, stream_cursor_read * pr,
 		hcd_store_state();
 	    }
 	    ++eol_count;
-	    /*
-	     * According to Adobe, the decoder should always check for
-	     * the EOD sequence, regardless of EndOfBlock: the Red Book's
-	     * documentation of EndOfBlock is wrong.
-	     */
 	    if (eol_count == (ss->K < 0 ? 2 : 6)) {
 		status = EOFC;
 		goto out;
 	    }
 	}
+	if (rows_left == 0) {
+	    status = EOFC;
+	    goto out;
+	}
 	if (status == 0)	/* input empty while scanning EOLs */
 	    goto out;
 	switch (eol_count) {
@@ -359,7 +356,7 @@ s_CFD_process(stream_state * st, stream_cursor_read * pr,
 	if_debug0('w', "[w1]new row\n");
 	status = cf_decode_1d(ss, pr);
     } else {
-	if_debug1('w', "[w1]new 2-D row, %d left\n", k_left);
+	if_debug1('w', "[w1]new 2-D row, k_left=%d\n", k_left);
 	status = cf_decode_2d(ss, pr);
     }
     if_debug3('w', "[w]CFD status = %d, wpos = %d, cbit = %d\n",
@@ -374,7 +371,7 @@ s_CFD_process(stream_state * st, stream_cursor_read * pr,
 		)
 		break;
 	    /* Substitute undamaged data if appropriate. */
-/****** NOT IMPLEMENTED YET ******/
+	    /****** NOT IMPLEMENTED YET ******/
 	    {
 		ss->wpos = wstop;
 		ss->cbit = -ss->Columns & 7;
@@ -401,7 +398,7 @@ s_CFD_process(stream_state * st, stream_cursor_read * pr,
  * if we detected an N-bit non-EOL, we know that at least N bits of data
  * are available in the buffer.
  */
-private int
+static int
 cf_decode_eol(stream_CFD_state * ss, stream_cursor_read * pr)
 {
     hcd_declare_state;
@@ -439,7 +436,7 @@ cf_decode_eol(stream_CFD_state * ss, stream_cursor_read * pr)
 }
 
 /* Decode a 1-D scan line. */
-private int
+static int
 cf_decode_1d(stream_CFD_state * ss, stream_cursor_read * pr)
 {
     cfd_declare_state;
@@ -492,7 +489,7 @@ cf_decode_1d(stream_CFD_state * ss, stream_cursor_read * pr)
     get_run(cf_black_decode, cfd_black_initial_bits, cfd_black_min_bits,
 	    bcnt, "[w1]black", dbl, out1);
     if (bcnt < 0) {		/* All exceptional codes are invalid here. */
-/****** WRONG, uncompressed IS ALLOWED ******/
+	/****** WRONG, uncompressed IS ALLOWED ******/
 	status = ERRC;
 	goto out;
     }
@@ -524,7 +521,7 @@ cf_decode_1d(stream_CFD_state * ss, stream_cursor_read * pr)
 }
 
 /* Decode a 2-D scan line. */
-private int
+static int
 cf_decode_2d(stream_CFD_state * ss, stream_cursor_read * pr)
 {
     cfd_declare_state;
@@ -710,6 +707,10 @@ v0:	    skip_bits(1);
     /* falls through */
   out:cfd_store_state();
     ss->invert = invert;
+    /* Ignore an error (missing EOFB/RTC when EndOfBlock == true) */
+    /* if we have finished all rows. */
+    if (status == ERRC && ss->Rows > 0 && ss->row > ss->Rows)
+	status = EOFC;
     return status;
     /*
      * We handle horizontal decoding here, so that we can
@@ -760,7 +761,7 @@ v0:	    skip_bits(1);
 }
 
 #if 1				/*************** */
-private int
+static int
 cf_decode_uncompressed(stream_CFD_state * ss, stream_cursor_read * pr)
 {
     return ERRC;
@@ -770,7 +771,7 @@ cf_decode_uncompressed(stream_CFD_state * ss, stream_cursor_read * pr)
 /* Decode uncompressed data. */
 /* (Not tested: no sample data available!) */
 /****** DOESN'T CHECK FOR OVERFLOWING SCAN LINE ******/
-private int
+static int
 cf_decode_uncompressed(stream * s)
 {
     cfd_declare_state;
@@ -810,8 +811,7 @@ cf_decode_uncompressed(stream * s)
 	    qbit += 8, q++;
 	rlen &= 1;
     }
-  out:
-/******* WRONG ******/
+  out:				/******* WRONG ******/
     cfd_store_state();
     return rlen;
 }
