@@ -1,23 +1,17 @@
-/* Copyright (C) 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: sdcparam.c,v $ $Revision: 1.2.2.1 $ */
+/* $Id: sdcparam.c 8250 2007-09-25 13:31:24Z giles $ */
 /* DCT filter parameter setting and reading */
 #include "memory_.h"
 #include "jpeglib_.h"
@@ -33,13 +27,13 @@
 
 /* Define the DCT parameters. */
 #define dctp(key, type, stype, memb) { key, type, offset_of(stype, memb) }
-private const gs_param_item_t s_DCT_param_items[] =
+static const gs_param_item_t s_DCT_param_items[] =
 {
 dctp("ColorTransform", gs_param_type_int, stream_DCT_state, ColorTransform),
     dctp("QFactor", gs_param_type_float, stream_DCT_state, QFactor),
     gs_param_item_end
 };
-private const gs_param_item_t jsd_param_items[] =
+static const gs_param_item_t jsd_param_items[] =
 {
     dctp("Picky", gs_param_type_int, jpeg_stream_data, Picky),
     dctp("Relax", gs_param_type_int, jpeg_stream_data, Relax),
@@ -89,7 +83,7 @@ static const byte inverse_natural_order[DCTSIZE2] =
 
 /* ================ Get parameters ================ */
 
-private int
+static int
 quant_param_string(gs_param_string * pstr, int count, const UINT16 * pvals,
 		   floatp QFactor, gs_memory_t * mem)
 {
@@ -112,7 +106,7 @@ quant_param_string(gs_param_string * pstr, int count, const UINT16 * pvals,
     return code & 1;
 }
 
-private int
+static int
 quant_param_array(gs_param_float_array * pfa, int count, const UINT16 * pvals,
 		  floatp QFactor, gs_memory_t * mem)
 {
@@ -158,7 +152,7 @@ s_DCT_get_quantization_tables(gs_param_list * plist,
 	    default_table_ptrs = defaults->data.compress->cinfo.quant_tbl_ptrs;
 	}
     } else {
-	/**************** quant_tables.size NOT INITIALIZED ****************/
+	quant_tables.size = count_of(d_comp_info);
 	num_in_tables = quant_tables.size;
 	for (i = 0; i < num_in_tables; ++i)
 	    d_comp_info[i].quant_tbl_no = i;
@@ -221,9 +215,8 @@ s_DCT_get_quantization_tables(gs_param_list * plist,
 		case 1:
 		    break;
 	    }
-	    /* break const to free the string */
-	    gs_free_string(mem, (byte *) str.data, str.size,
-			   "quant_param_string");
+	    gs_free_const_string(mem, str.data, str.size,
+				 "quant_param_string");
 	}
 	code = quant_param_array(&fa, DCTSIZE2,
 			    table_ptrs[comp_info[i].quant_tbl_no]->quantval,
@@ -237,7 +230,7 @@ s_DCT_get_quantization_tables(gs_param_list * plist,
     return param_end_write_dict(plist, "QuantTables", &quant_tables);
 }
 
-private int
+static int
 pack_huff_table(gs_param_string * pstr, const JHUFF_TBL * table,
 		gs_memory_t * mem)
 {
@@ -374,7 +367,7 @@ s_DCT_byte_params(gs_param_list * plist, gs_param_name key, int start,
 }
 
 /* Get N quantization values from an array or a string. */
-private int
+static int
 quant_params(gs_param_list * plist, gs_param_name key, int count,
 	     UINT16 * pvals, floatp QFactor)
 {
@@ -458,7 +451,7 @@ s_DCT_put_quantization_tables(gs_param_list * plist, stream_DCT_state * pdct,
 					  &quant_tables, true))
 	) {
 	case 1:
-	    return 0;
+	    return 1;
 	default:
 	    return param_signal_error(plist, "QuantTables", code);
 	case 0:
@@ -509,7 +502,7 @@ s_DCT_put_quantization_tables(gs_param_list * plist, stream_DCT_state * pdct,
 }
 
 /* Put Huffman tables. */
-private int
+static int
 find_huff_values(JHUFF_TBL ** table_ptrs, int num_tables,
 	       const UINT8 counts[16], const UINT8 * values, int codes_size)
 {
