@@ -1,26 +1,22 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: siscale.c,v 1.4.6.1 2002/02/22 19:46:00 ray Exp $ */
+/* $Id: siscale.c 8563 2008-02-29 20:49:04Z giles $ */
 /* Image scaling filters */
 #include "math_.h"
 #include "memory_.h"
 #include "stdio_.h"
-#include <assert.h>
-#include "gconfigv.h"
+#include "stdint_.h"
 #include "gdebug.h"
 #include "strimpl.h"
 #include "siscale.h"
@@ -32,49 +28,15 @@
 
 /* ---------------- ImageScaleEncode/Decode ---------------- */
 
-/* Define whether to accumulate pixels in fixed or floating point. */
-#if USE_FPU <= 0
-
-	/* Accumulate pixels in fixed point. */
-
-typedef int PixelWeight;
-
-#  if arch_ints_are_short
-typedef long AccumTmp;
-#  else
-typedef int AccumTmp;
-#  endif
-
-/*
- *    The optimal scaling for fixed point arithmetic is a function of the
- *    size of AccumTmp type, the size if the input pixel, the size of the
- *    intermediate pixel (PixelTmp) and the size of the output pixel.  This
- *    is set by these definitions and the fraction_bits variables in the
- *    functions.
- */
-#define num_weight_bits\
-  ((sizeof(AccumTmp) - maxSizeofPixel) * 8 - (LOG2_MAX_ISCALE_SUPPORT + 1))
-#define numScaleBits  ((maxSizeofPixel - sizeof(PixelTmp)) * 8 )
-#define fixedScaleFactor  ((int) (1 << numScaleBits))
-#define scale_PixelWeight(factor) ((int)((factor) * (1 << num_weight_bits)))
-#define unscale_AccumTmp(atemp, fraction_bits) arith_rshift(atemp, fraction_bits)
-#define NEED_FRACTION_BITS
-
-#else /* USE_FPU > 0 */
-
-	/* Accumulate pixels in floating point. */
-
 typedef float PixelWeight;
 typedef double AccumTmp;
 
 #define num_weight_bits 0		/* Not used for floating point */
 #define fixedScaleFactor 1		/* Straight scaling for floating point */
 #define scale_PixelWeight(factor) (factor)
-#define unscale_AccumTmp(atemp, fraction_bits) ((int)(atemp))
+#define unscale_AccumTmp(atemp, fraction_bits) ((int)(atemp + 0.5))
 /*#undef NEED_FRACTION_BITS*/
 
-#endif /* USE_FPU */
-
 /* Temporary intermediate values */
 typedef byte PixelTmp;
 typedef int PixelTmp2;		/* extra width for clamping sum */
@@ -107,7 +69,6 @@ typedef struct stream_IScale_state_s {
     /* The init procedure sets the following. */
     int sizeofPixelIn;		/* bytes per input value, 1 or 2 */
     int sizeofPixelOut;		/* bytes per output value, 1 or 2 */
-    double xscale, yscale;
     void /*PixelIn */ *src;
     void /*PixelOut */ *dst;
     PixelTmp *tmp;
@@ -117,6 +78,7 @@ typedef struct stream_IScale_state_s {
     int src_y;
     uint src_offset, src_size;
     int dst_y;
+    int src_y_offset;
     uint dst_offset, dst_size;
     CLIST dst_next_list;	/* for next output value */
     int dst_last_index;		/* highest index used in list */
@@ -134,7 +96,7 @@ gs_private_st_ptrs5(st_IScale_state, stream_IScale_state,
 #define Mitchell_support 2.0
 #define B (1.0 / 3.0)
 #define C (1.0 / 3.0)
-private double
+static double
 Mitchell_filter(double t)
 {
     double t2 = t * t;
@@ -179,16 +141,16 @@ Mitchell_filter(double t)
 
 /* Calculate the support for a given scale. */
 /* The value is always in the range 1 .. MAX_ISCALE_SUPPORT. */
-private int
+static int
 contrib_pixels(double scale)
 {
     return (int)(fWidthIn / (scale >= 1.0 ? 1.0 : max(scale, min_scale))
-		 * 2 + 1);
+		 * 2 + 1.5);
 }
 
 /* Pre-calculate filter contributions for a row or a column. */
 /* Return the highest input pixel index used. */
-private int
+static int
 calculate_contrib(
 	/* Return weight list parameters in contrib[0 .. size-1]. */
 		     CLIST * contrib,
@@ -197,8 +159,14 @@ calculate_contrib(
 		     CONTRIB * items,
 	/* The output image is scaled by 'scale' relative to the input. */
 		     double scale,
-	/* Start generating weights for input pixel 'input_index'. */
-		     int input_index,
+	/* Start generating weights for input pixel 'starting_output_index'. */
+		     int starting_output_index,
+	/* Offset of input subimage from the input image start. */
+		     int src_y_offset,
+	/* Entire output image size. */
+		     int dst_size,
+	/* Entire input image size. */
+		     int src_size,
 	/* Generate 'size' weight lists. */
 		     int size,
 	/* Limit pixel indices to 'limit', for clamping at the edges */
@@ -220,6 +188,7 @@ calculate_contrib(
     int i, j;
     int last_index = -1;
 
+    if_debug1('w', "[w]calculate_contrib scale=%lg\n", scale);
     if (scale < 1.0) {
 	double clamped_scale = max(scale, min_scale);
 
@@ -234,32 +203,34 @@ calculate_contrib(
     npixels = (int)(WidthIn * 2 + 1);
 
     for (i = 0; i < size; ++i) {
-	double center = (input_index + i) / scale;
-	int left = (int)ceil(center - WidthIn);
-	int right = (int)floor(center + WidthIn);
-
-	/*
-	 * In pathological cases, the limit may be much less
-	 * than the support.  We do need to deal with this.
+	/* Here we need :
+	   double scale = (double)dst_size / src_size;
+	   float dst_offset_fraction = floor(dst_offset) - dst_offset;
+	   double center = (starting_output_index  + i + dst_offset_fraction + 0.5) / scale - 0.5;
+	   int left = (int)ceil(center - WidthIn);
+	   int right = (int)floor(center + WidthIn);
+	   We can't compute 'right' in floats because float arithmetics is not associative.
+	   In older versions tt caused a 1 pixel bias of image bands due to 
+	   rounding direction appears to depend on src_y_offset. So compute in rationals.
+	   Since pixel center fall to half integers, we subtract 0.5 
+	   in the image space and add 0.5 in the device space.
 	 */
-#define clamp_pixel(j)\
-  (j < 0 ? (-j >= limit ? limit - 1 : -j) :\
-   j >= limit ? (j >> 1 >= limit ? 0 : (limit - j) + limit - 1) :\
-   j)
-	int lmin =
-	(left < 0 ? 0 : left);
-	int lmax =
-	(left < 0 ? (-left >= limit ? limit - 1 : -left) : left);
-	int rmin =
-	(right >= limit ?
-	 (right >> 1 >= limit ? 0 : (limit - right) + limit - 1) :
-	 right);
-	int rmax =
-	(right >= limit ? limit - 1 : right);
-	int first_pixel = min(lmin, rmin);
-	int last_pixel = max(lmax, rmax);
+	int dst_y_offset_fraction_num = (int)((int64_t)src_y_offset * dst_size % src_size) * 2 <= src_size
+			? -(int)((int64_t)src_y_offset * dst_size % src_size)
+			: src_size - (int)((int64_t)src_y_offset * dst_size % src_size);
+	int center_denom = dst_size * 2; 
+	int64_t center_num = /* center * center_denom * 2 = */ 
+	    (starting_output_index  + i) * src_size * 2 + src_size + dst_y_offset_fraction_num * 2 - dst_size;
+	int left = (int)ceil((center_num - WidthIn * center_denom) / center_denom);
+	int right = (int)floor((center_num + WidthIn * center_denom) / center_denom);
+	double center = (double)center_num / center_denom;
+#define clamp_pixel(j) (j < 0 ? 0 : j >= limit ? limit - 1 : j)
+	int first_pixel = clamp_pixel(left);
+	int last_pixel = clamp_pixel(right);
 	CONTRIB *p;
 
+	if_debug4('w', "[w]i=%d, i+offset=%lg scale=%lg center=%lg : ", starting_output_index + i, 
+		starting_output_index + i + (double)src_y_offset / src_size * dst_size, scale, center);
 	if (last_pixel > last_index)
 	    last_index = last_pixel;
 	contrib[i].first_pixel = (first_pixel % modulus) * stride;
@@ -269,32 +240,41 @@ calculate_contrib(
 	for (j = 0; j < npixels; ++j)
 	    p[j].weight = 0;
 	if (squeeze) {
+            double sum = 0;
+	    for (j = left; j <= right; ++j)
+		sum += fproc((center - j) / fscale) / fscale;
 	    for (j = left; j <= right; ++j) {
-		double weight =
-		fproc((center - j) / fscale) / fscale;
+		double weight = fproc((center - j) / fscale) / fscale / sum;
 		int n = clamp_pixel(j);
 		int k = n - first_pixel;
 
-		p[k].weight +=
+                p[k].weight +=
 		    (PixelWeight) (weight * scaled_factor);
+		if_debug2('w', " %d %f", k, (float)p[k].weight);
 	    }
+
 	} else {
+            double sum = 0;
+	    for (j = left; j <= right; ++j)
+		sum += fproc(center - j);
 	    for (j = left; j <= right; ++j) {
-		double weight = fproc(center - j);
+		double weight = fproc(center - j) / sum;
 		int n = clamp_pixel(j);
 		int k = n - first_pixel;
 
 		p[k].weight +=
 		    (PixelWeight) (weight * scaled_factor);
+		if_debug2('w', " %d %f", k, (float)p[k].weight);
 	    }
 	}
+	if_debug0('w', "\n");
     }
     return last_index;
 }
 
 
 /* Apply filter to zoom horizontally from src to tmp. */
-private void
+static void
 zoom_x(PixelTmp * tmp, const void /*PixelIn */ *src, int sizeofPixelIn,
        int tmp_width, int WidthIn, int Colors, const CLIST * contrib,
        const CONTRIB * items)
@@ -334,7 +314,7 @@ zoom_x(PixelTmp * tmp, const void /*PixelIn */ *src, int sizeofPixelIn,
 			  }\
 			}\
 			{ PixelIn2 pixel = unscale_AccumTmp(weight, fraction_bits);\
-			  if_debug1('W', " %ld", (long)pixel);\
+			  if_debug1('W', " %lx", (long)pixel);\
 			  *tp =\
 			    (PixelTmp)CLAMP(pixel, minPixelTmp, maxPixelTmp);\
 			}\
@@ -343,7 +323,7 @@ zoom_x(PixelTmp * tmp, const void /*PixelIn */ *src, int sizeofPixelIn,
 	if (sizeofPixelIn == 1) {
 	    zoom_x_loop(byte, int)
 	} else {		/* sizeofPixelIn == 2 */
-#if arch_ints_are_short
+#if ARCH_INTS_ARE_SHORT
 	    zoom_x_loop(bits16, long)
 #else
 	    zoom_x_loop(bits16, int)
@@ -359,7 +339,7 @@ zoom_x(PixelTmp * tmp, const void /*PixelIn */ *src, int sizeofPixelIn,
  * This is simpler because we can treat all columns identically
  * without regard to the number of samples per pixel.
  */
-private void
+static void
 zoom_y(void /*PixelOut */ *dst, int sizeofPixelOut, uint MaxValueOut,
        const PixelTmp * tmp, int WidthOut, int tmp_width,
        int Colors, const CLIST * contrib, const CONTRIB * items)
@@ -387,7 +367,7 @@ zoom_y(void /*PixelOut */ *dst, int sizeofPixelOut, uint MaxValueOut,
 		    weight += *pp * cp->weight;\
 		}\
 		{ PixelTmp2 pixel = unscale_AccumTmp(weight, fraction_bits);\
-		  if_debug1('W', " %d", pixel);\
+		  if_debug1('W', " %lx", (long)pixel);\
 		  ((PixelOut *)dst)[kc] =\
 		    (PixelOut)CLAMP(pixel, 0, max_weight);\
 		}\
@@ -396,7 +376,21 @@ zoom_y(void /*PixelOut */ *dst, int sizeofPixelOut, uint MaxValueOut,
     if (sizeofPixelOut == 1) {
 	zoom_y_loop(byte)
     } else {			/* sizeofPixelOut == 2 */
-	zoom_y_loop(bits16)
+	/* zoom_y_loop(bits16) */
+	for ( kc = 0; kc < kn; ++kc ) {
+		AccumTmp weight = 0;
+		{ const PixelTmp *pp = &tmp[kc + first_pixel];
+		  int j = cn;
+		  const CONTRIB *cp = cbp;
+		  for ( ; j > 0; pp += kn, ++cp, --j )
+		    weight += *pp * cp->weight;
+		}
+		{ PixelTmp2 pixel = unscale_AccumTmp(weight, fraction_bits);
+		  if_debug1('W', " %lx", (long)pixel);
+		  ((bits16 *)dst)[kc] =
+		    (bits16)CLAMP(pixel, 0, max_weight);
+		}
+	}
     }
     if_debug0('W', "\n");
 }
@@ -407,45 +401,46 @@ zoom_y(void /*PixelOut */ *dst, int sizeofPixelOut, uint MaxValueOut,
 #define tmp_height params.HeightIn
 
 /* Forward references */
-private void s_IScale_release(P1(stream_state * st));
+static void s_IScale_release(stream_state * st);
 
 /* Calculate the weights for an output row. */
-private void
+static void
 calculate_dst_contrib(stream_IScale_state * ss, int y)
 {
     uint row_size = ss->params.WidthOut * ss->params.Colors;
     int last_index =
-    calculate_contrib(&ss->dst_next_list, ss->dst_items, ss->yscale,
-		      y, 1, ss->params.HeightIn, MAX_ISCALE_SUPPORT, row_size,
+    calculate_contrib(&ss->dst_next_list, ss->dst_items, 
+		      (double)ss->params.EntireHeightOut / ss->params.EntireHeightIn,
+		      y, ss->src_y_offset, ss->params.EntireHeightOut, ss->params.EntireHeightIn, 
+		      1, ss->params.HeightIn, MAX_ISCALE_SUPPORT, row_size,
 		      (double)ss->params.MaxValueOut / (fixedScaleFactor * unitPixelTmp) );
     int first_index_mod = ss->dst_next_list.first_pixel / row_size;
 
+    if_debug2('w', "[W]calculate_dst_contrib for y = %d, y+offset=%d\n", y, y + ss->src_y_offset);
     ss->dst_last_index = last_index;
     last_index %= MAX_ISCALE_SUPPORT;
     if (last_index < first_index_mod) {		/* Shuffle the indices to account for wraparound. */
 	CONTRIB shuffle[MAX_ISCALE_SUPPORT];
 	int i;
 
-	for (i = 0; i < MAX_ISCALE_SUPPORT; ++i)
+	for (i = 0; i < MAX_ISCALE_SUPPORT; ++i) {
 	    shuffle[i].weight =
 		(i <= last_index ?
 		 ss->dst_items[i + MAX_ISCALE_SUPPORT - first_index_mod].weight :
 		 i >= first_index_mod ?
 		 ss->dst_items[i - first_index_mod].weight :
 		 0);
+	    if_debug1('W', " %f", shuffle[i].weight);
+	}
 	memcpy(ss->dst_items, shuffle, MAX_ISCALE_SUPPORT * sizeof(CONTRIB));
 	ss->dst_next_list.n = MAX_ISCALE_SUPPORT;
 	ss->dst_next_list.first_pixel = 0;
     }
-#ifdef DEBUG
-    if (gs_debug_c('w')) {
-	dprintf1("[w]calc dest contrib for y = %d\n", y);
-    }
-#endif
+    if_debug0('W', "\n");
 }
 
 /* Set default parameter values (actually, just clear pointers). */
-private void
+static void
 s_IScale_set_defaults(stream_state * st)
 {
     stream_IScale_state *const ss = (stream_IScale_state *) st;
@@ -458,7 +453,7 @@ s_IScale_set_defaults(stream_state * st)
 }
 
 /* Initialize the filter. */
-private int
+static int
 s_IScale_init(stream_state * st)
 {
     stream_IScale_state *const ss = (stream_IScale_state *) st;
@@ -466,13 +461,12 @@ s_IScale_init(stream_state * st)
 
     ss->sizeofPixelIn = ss->params.BitsPerComponentIn / 8;
     ss->sizeofPixelOut = ss->params.BitsPerComponentOut / 8;
-    ss->xscale = (double)ss->params.WidthOut / (double)ss->params.WidthIn;
-    ss->yscale = (double)ss->params.HeightOut / (double)ss->params.HeightIn;
 
     ss->src_y = 0;
     ss->src_size = ss->params.WidthIn * ss->sizeofPixelIn * ss->params.Colors;
     ss->src_offset = 0;
     ss->dst_y = 0;
+    ss->src_y_offset = ss->params.src_y_offset;
     ss->dst_size = ss->params.WidthOut * ss->sizeofPixelOut * ss->params.Colors;
     ss->dst_offset = 0;
 
@@ -485,7 +479,8 @@ s_IScale_init(stream_state * st)
 					   max(ss->params.WidthOut, ss->params.HeightOut),
 				      sizeof(CLIST), "image_scale contrib");
     ss->items = (CONTRIB *) gs_alloc_byte_array(mem,
-				  contrib_pixels(ss->xscale) * ss->params.WidthOut,
+				  contrib_pixels((double)ss->params.EntireWidthOut / 
+					ss->params.EntireWidthIn) * ss->params.WidthOut,
 				 sizeof(CONTRIB), "image_scale contrib[*]");
     /* Allocate buffers for 1 row of source and destination. */
     ss->dst = gs_alloc_byte_array(mem, ss->params.WidthOut * ss->params.Colors,
@@ -500,8 +495,10 @@ s_IScale_init(stream_state * st)
 /****** WRONG ******/
     }
     /* Pre-calculate filter contributions for a row. */
-    calculate_contrib(ss->contrib, ss->items, ss->xscale,
-		      0, ss->params.WidthOut, ss->params.WidthIn, ss->params.WidthIn,
+    calculate_contrib(ss->contrib, ss->items,
+		      (double)ss->params.EntireWidthOut / ss->params.EntireWidthIn,
+		      0, 0, ss->params.WidthOut, ss->params.WidthIn, 
+		      ss->params.WidthOut, ss->params.WidthIn, ss->params.WidthIn,
 		      ss->params.Colors, (double)unitPixelTmp * fixedScaleFactor / ss->params.MaxValueIn);
 
     /* Prepare the weights for the first output row. */
@@ -512,7 +509,7 @@ s_IScale_init(stream_state * st)
 }
 
 /* Process a buffer.  Note that this handles Encode and Decode identically. */
-private int
+static int
 s_IScale_process(stream_state * st, stream_cursor_read * pr,
 		 stream_cursor_write * pw, bool last)
 {
@@ -545,7 +542,8 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
 	    /* inter-assigned freely, but not compared! */
 	    if ((void *)row != ss->dst)		/* no buffering */
 		goto adv;
-	} {			/* We're delivering a buffered output row. */
+	} 
+	{			/* We're delivering a buffered output row. */
 	    uint wcount = ss->dst_size - ss->dst_offset;
 	    uint ncopy = min(wleft, wcount);
 
@@ -557,7 +555,7 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
 	    ss->dst_offset = 0;
 	}
 	/* Advance to the next output row. */
-      adv:++(ss->dst_y);
+      adv:++ss->dst_y;
 	if (ss->dst_y != ss->params.HeightOut)
 	    calculate_dst_contrib(ss, ss->dst_y);
     }
@@ -570,9 +568,8 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
 
 	if (rleft == 0)
 	    return 0;		/* need more input */
-#ifdef DEBUG
-	assert(ss->src_y < ss->params.HeightIn);
-#endif
+	if (ss->src_y >= ss->params.HeightIn)
+	    return ERRC;
 	if (rleft >= rcount) {	/* We're going to fill up a row. */
 	    const byte *row;
 
@@ -605,7 +602,7 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
 }
 
 /* Release the filter's storage. */
-private void
+static void
 s_IScale_release(stream_state * st)
 {
     stream_IScale_state *const ss = (stream_IScale_state *) st;
