@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: sjbig2_luratech.c 6745 2006-05-15 17:34:18Z giles $ */
+/* $Id: sjbig2_luratech.c 8456 2007-12-21 10:31:00Z ken $ */
 /* jbig2decode filter implementation -- hooks in luratech JBIG2 */
 
 #include "memory_.h"
@@ -59,7 +59,7 @@ typedef struct s_jbig2decode_global_data_s {
 } s_jbig2decode_global_data;
 
 /* create a global data struct and copy data into it */
-public int
+int
 s_jbig2decode_make_global_data(byte *data, uint size, void **result)
 {
     s_jbig2decode_global_data *global = NULL;
@@ -80,7 +80,7 @@ s_jbig2decode_make_global_data(byte *data, uint size, void **result)
 }
 
 /* free a global data struct and its data */
-public void
+void
 s_jbig2decode_free_global_data(void *data)
 {
     s_jbig2decode_global_data *global = (s_jbig2decode_global_data*)data;
@@ -93,31 +93,30 @@ s_jbig2decode_free_global_data(void *data)
 }
 
 /* store a global ctx pointer in our state structure */
-public int
-s_jbig2decode_set_global_data(stream_state *ss, void *data)
+int
+s_jbig2decode_set_global_data(stream_state *ss, s_jbig2_global_data_t *gs)
 {
     stream_jbig2decode_state *state = (stream_jbig2decode_state*)ss;
-    s_jbig2decode_global_data *global = (s_jbig2decode_global_data*)data;
+    if (state == NULL)
+        return gs_error_VMerror;
+    
+    state->global_struct = gs;
+    if (gs != NULL) {
+        s_jbig2decode_global_data *global = (s_jbig2decode_global_data*)(gs->data);
 
-    if (state != NULL) {
-      if (global != NULL) {
         state->global_data = global->data;
         state->global_size = global->size;
-        return 0;
-      } else {
-	state->global_data = NULL;
-	state->global_size = 0;
-	return 0;
-      }
+    } else {
+        state->global_data = NULL;
+        state->global_size = 0;
     }
-    
     return gs_error_VMerror;
 }
 
 /* invert the bits in a buffer */
 /* jbig2 and postscript have different senses of what pixel
    value is black, so we must invert the image */
-private void
+static void
 s_jbig2_invert_buffer(unsigned char *buf, int length)
 {
     int i;
@@ -129,7 +128,7 @@ s_jbig2_invert_buffer(unsigned char *buf, int length)
 /** callbacks passed to the luratech library */
 
 /* memory allocator */
-private void * JB2_Callback
+static void * JB2_Callback
 s_jbig2_alloc(unsigned long size, void *userdata)
 {
     void *result = malloc(size);
@@ -137,7 +136,7 @@ s_jbig2_alloc(unsigned long size, void *userdata)
 }
 
 /* memory release */
-private JB2_Error JB2_Callback
+static JB2_Error JB2_Callback
 s_jbig2_free(void *ptr, void *userdata)
 {
     free(ptr);
@@ -145,7 +144,7 @@ s_jbig2_free(void *ptr, void *userdata)
 }
 
 /* error callback for jbig2 codec */
-private void JB2_Callback
+static void JB2_Callback
 s_jbig2_message(const char *message, JB2_Message_Level level, void *userdata)
 {
     const char *type;
@@ -174,7 +173,7 @@ s_jbig2_message(const char *message, JB2_Message_Level level, void *userdata)
 }
 
 /* compressed read callback for jbig2 codec */
-private JB2_Size_T JB2_Callback
+static JB2_Size_T JB2_Callback
 s_jbig2_read(unsigned char *buffer, 
 	JB2_Size_T offset, JB2_Size_T size, void *userdata)
 {
@@ -200,7 +199,7 @@ s_jbig2_read(unsigned char *buffer,
 }
 
 /* uncompressed write callback for jbig2 codec */
-private JB2_Error JB2_Callback
+static JB2_Error JB2_Callback
 s_jbig2_write(unsigned char *buffer,
 		unsigned long row, unsigned long width,
 		unsigned long bbp, void *userdata)
@@ -221,7 +220,7 @@ s_jbig2_write(unsigned char *buffer,
 }
 
 
-private int
+static int
 s_jbig2decode_inbuf(stream_jbig2decode_state *state, stream_cursor_read * pr)
 {
     long in_size = pr->limit - pr->ptr;
@@ -262,7 +261,7 @@ s_jbig2decode_inbuf(stream_jbig2decode_state *state, stream_cursor_read * pr)
 }
 
 /* initialize the steam. */
-private int
+static int
 s_jbig2decode_init(stream_state * ss)
 {
     stream_jbig2decode_state *const state = (stream_jbig2decode_state *) ss;
@@ -285,7 +284,7 @@ s_jbig2decode_init(stream_state * ss)
 /* process a section of the input and return any decoded data.
    see strimpl.h for return codes.
  */
-private int
+static int
 s_jbig2decode_process(stream_state * ss, stream_cursor_read * pr,
 		  stream_cursor_write * pw, bool last)
 {
@@ -365,7 +364,7 @@ s_jbig2decode_process(stream_state * ss, stream_cursor_read * pr,
 }
 
 /* stream release. free all our decoder state. */
-private void
+static void
 s_jbig2decode_release(stream_state *ss)
 {
     stream_jbig2decode_state *const state = (stream_jbig2decode_state *) ss;
@@ -398,7 +397,7 @@ const stream_template s_jbig2decode_template = {
 private_st_jbig2encode_state();
 
 /* helper - start up the compression context */
-private int
+static int
 s_jbig2encode_start(stream_jbig2encode_state *state)
 {
     JB2_Error err;
@@ -431,7 +430,7 @@ s_jbig2encode_start(stream_jbig2encode_state *state)
 }
 
 /* callback for compressed data output */
-private JB2_Size_T JB2_Callback
+static JB2_Size_T JB2_Callback
 s_jbig2encode_write(const unsigned char *buffer,
 		JB2_Size_T pos, JB2_Size_T size, void *userdata)
 {
@@ -468,7 +467,7 @@ s_jbig2encode_write(const unsigned char *buffer,
 
 
 /* initialize the steam. */
-private int
+static int
 s_jbig2encode_init(stream_state * ss)
 {
     stream_jbig2encode_state *state = (stream_jbig2encode_state *)ss;
@@ -497,7 +496,7 @@ s_jbig2encode_init(stream_state * ss)
 /* process a section of the input and return any encoded data.
    see strimpl.h for return codes.
  */
-private int
+static int
 s_jbig2encode_process(stream_state * ss, stream_cursor_read * pr,
                   stream_cursor_write * pw, bool last)
 {
@@ -581,7 +580,7 @@ s_jbig2encode_process(stream_state * ss, stream_cursor_read * pr,
 
 /* stream release. free all our decoder state.
  */
-private void
+static void
 s_jbig2encode_release(stream_state *ss)
 {
     stream_jbig2encode_state *state = (stream_jbig2encode_state *)ss;
