@@ -1,30 +1,23 @@
-/* Copyright (C) 1995, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: szlibc.c,v $ $Revision: 1.2.2.2 $ */
+/* $Id: szlibc.c 8022 2007-06-05 22:23:38Z giles $ */
 /* Code common to zlib encoding and decoding streams */
 #include "std.h"
 #include "gserror.h"
 #include "gserrors.h"
 #include "gstypes.h"
 #include "gsmemory.h"
-#include "gsmalloc.h"
 #include "gsstruct.h"
 #include "strimpl.h"
 #include "szlibxx.h"
@@ -55,7 +48,7 @@ s_zlib_set_defaults(stream_state * st)
 int
 s_zlib_alloc_dynamic_state(stream_zlib_state *ss)
 {
-    gs_memory_t *mem = (ss->memory ? ss->memory : &gs_memory_default);
+    gs_memory_t *mem = ss->memory; 
     zlib_dynamic_state_t *zds =
 	gs_alloc_struct_immovable(mem, zlib_dynamic_state_t,
 				  &st_zlib_dynamic_state,
