@@ -1,20 +1,17 @@
-/* Copyright (C) 1992, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zcie.c,v 1.12 2004/08/04 19:36:13 stefan Exp $ */
+/* $Id: zcie.c 8250 2007-09-25 13:31:24Z giles $ */
 /* CIE color operators */
 #include "math_.h"
 #include "memory_.h"
@@ -84,10 +81,12 @@ dict_proc_array_param(const gs_memory_t *mem,
 	    check_proc_only(proc);
 	}
 	*pparray = *pvalue;
-    } else
+        return 0;
+    } else {
 	make_const_array(pparray, a_readonly | avm_foreign,
 			 count, &empty_procs[0]);
-    return 0;
+        return 1;
+    }
 }
 
 /* Get 3 ranges from a dictionary. */
@@ -110,13 +109,13 @@ dict_matrix3_param(const gs_memory_t *mem,
      * padding elements after each of the vectors.  However, we can be
      * confident that there is no padding within a single vector.
      */
-    float values[9];
+    float values[9], defaults[9];
     int code;
 
-    memcpy(&values[0], &Matrix3_default.cu, 3 * sizeof(float));
-    memcpy(&values[3], &Matrix3_default.cv, 3 * sizeof(float));
-    memcpy(&values[6], &Matrix3_default.cw, 3 * sizeof(float));
-    code = dict_floats_param(mem, pdref, kstr, 9, values, values);
+    memcpy(&defaults[0], &Matrix3_default.cu, 3 * sizeof(float));
+    memcpy(&defaults[3], &Matrix3_default.cv, 3 * sizeof(float));
+    memcpy(&defaults[6], &Matrix3_default.cw, 3 * sizeof(float));
+    code = dict_floats_param(mem, pdref, kstr, 9, values, defaults);
     if (code < 0)
 	return code;
     memcpy(&pmat3->cu, &values[0], 3 * sizeof(float));
@@ -157,7 +156,7 @@ cie_points_param(const gs_memory_t *mem,
 /* Process a 3- or 4-dimensional lookup table from a dictionary. */
 /* The caller has set pclt->n and pclt->m. */
 /* ptref is known to be a readable array of size at least n+1. */
-private int cie_3d_table_param(const ref * ptable, uint count, uint nbytes,
+static int cie_3d_table_param(const ref * ptable, uint count, uint nbytes,
 			       gs_const_string * strings);
 int
 cie_table_param(const ref * ptref, gx_color_lookup_table * pclt,
@@ -215,7 +214,7 @@ cie_table_param(const ref * ptref, gx_color_lookup_table * pclt,
     pclt->table = table;
     return 0;
 }
-private int
+static int
 cie_3d_table_param(const ref * ptable, uint count, uint nbytes,
 		   gs_const_string * strings)
 {
@@ -241,7 +240,7 @@ cie_3d_table_param(const ref * ptable, uint count, uint nbytes,
 /* ------ CIE setcolorspace ------ */
 
 /* Common code for the CIEBased* cases of setcolorspace. */
-private int
+static int
 cie_lmnp_param(const gs_memory_t *mem, const ref * pdref, gs_cie_common * pcie, ref_cie_procs * pcprocs)
 {
     int code;
@@ -257,7 +256,7 @@ cie_lmnp_param(const gs_memory_t *mem, const ref * pdref, gs_cie_common * pcie,
 }
 
 /* Common code for the CIEBasedABC/DEF[G] cases of setcolorspace. */
-private int
+static int
 cie_abc_param(const gs_memory_t *mem, const ref * pdref, gs_cie_abc * pcie, ref_cie_procs * pcprocs)
 {
     int code;
@@ -280,9 +279,7 @@ cie_set_finish(i_ctx_t *i_ctx_p, gs_color_space * pcs,
     if (code >= 0)
 	code = gs_setcolorspace(igs, pcs);
     /* Delete the extra reference to the parameter tables. */
-    gs_cspace_release(pcs);
-    /* Free the top-level object, which was copied by gs_setcolorspace. */
-    gs_free_object(gs_state_memory(igs), pcs, "cie_set_finish");
+    rc_decrement_only(pcs, "cie_set_finish");
     if (code < 0) {
 	ref_stack_pop_to(&e_stack, edepth);
 	return code;
@@ -293,14 +290,14 @@ cie_set_finish(i_ctx_t *i_ctx_p, gs_color_space * pcs,
 }
 
 /* Forward references */
-private int cache_common(i_ctx_t *, gs_cie_common *, const ref_cie_procs *,
+static int cache_common(i_ctx_t *, gs_cie_common *, const ref_cie_procs *,
 			 void *, gs_ref_memory_t *);
-private int cache_abc_common(i_ctx_t *, gs_cie_abc *, const ref_cie_procs *,
+static int cache_abc_common(i_ctx_t *, gs_cie_abc *, const ref_cie_procs *,
 			     void *, gs_ref_memory_t *);
 
 /* <dict> .setciedefgspace - */
-private int cie_defg_finish(i_ctx_t *);
-private int
+static int cie_defg_finish(i_ctx_t *);
+static int
 zsetciedefgspace(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -343,7 +340,7 @@ zsetciedefgspace(i_ctx_t *i_ctx_p)
 	DO_NOTHING;
     return cie_set_finish(i_ctx_p, pcs, &procs, edepth, code);
 }
-private int
+static int
 cie_defg_finish(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -358,8 +355,8 @@ cie_defg_finish(i_ctx_t *i_ctx_p)
 }
 
 /* <dict> .setciedefspace - */
-private int cie_def_finish(i_ctx_t *);
-private int
+static int cie_def_finish(i_ctx_t *);
+static int
 zsetciedefspace(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -402,7 +399,7 @@ zsetciedefspace(i_ctx_t *i_ctx_p)
 	DO_NOTHING;
     return cie_set_finish(i_ctx_p, pcs, &procs, edepth, code);
 }
-private int
+static int
 cie_def_finish(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -417,8 +414,8 @@ cie_def_finish(i_ctx_t *i_ctx_p)
 }
 
 /* <dict> .setcieabcspace - */
-private int cie_abc_finish(i_ctx_t *);
-private int
+static int cie_abc_finish(i_ctx_t *);
+static int
 zsetcieabcspace(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -446,7 +443,7 @@ zsetcieabcspace(i_ctx_t *i_ctx_p)
 	DO_NOTHING;
     return cie_set_finish(i_ctx_p, pcs, &procs, edepth, code);
 }
-private int
+static int
 cie_abc_finish(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -460,8 +457,8 @@ cie_abc_finish(i_ctx_t *i_ctx_p)
 }
 
 /* <dict> .setcieaspace - */
-private int cie_a_finish(i_ctx_t *);
-private int
+static int cie_a_finish(i_ctx_t *);
+static int
 zsetcieaspace(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -494,7 +491,7 @@ zsetcieaspace(i_ctx_t *i_ctx_p)
     pcie->DecodeA = DecodeA_default;
     return cie_set_finish(i_ctx_p, pcs, &procs, edepth, code);
 }
-private int
+static int
 cie_a_finish(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -509,7 +506,7 @@ cie_a_finish(i_ctx_t *i_ctx_p)
 
 /* Common cache code */
 
-private int
+static int
 cache_abc_common(i_ctx_t *i_ctx_p, gs_cie_abc * pcie,
 		 const ref_cie_procs * pcprocs,
 		 void *container, gs_ref_memory_t * imem)
@@ -524,7 +521,7 @@ cache_abc_common(i_ctx_t *i_ctx_p, gs_cie_abc * pcie,
 	    cache_common(i_ctx_p, &pcie->common, pcprocs, pcie, imem));
 }
 
-private int
+static int
 cache_common(i_ctx_t *i_ctx_p, gs_cie_common * pcie,
 	     const ref_cie_procs * pcprocs,
 	     void *container, gs_ref_memory_t * imem)
@@ -538,8 +535,8 @@ cache_common(i_ctx_t *i_ctx_p, gs_cie_common * pcie,
 /* ------ Internal routines ------ */
 
 /* Prepare to cache the values for one or more procedures. */
-private int cie_cache_finish1(i_ctx_t *);
-private int cie_cache_finish(i_ctx_t *);
+static int cie_cache_finish1(i_ctx_t *);
+static int cie_cache_finish(i_ctx_t *);
 int
 cie_prepare_cache(i_ctx_t *i_ctx_p, const gs_range * domain, const ref * proc,
 		  cie_cache_floats * pcache, void *container,
@@ -595,7 +592,7 @@ cie_prepare_caches_4(i_ctx_t *i_ctx_p, const gs_range * domains,
 }
 
 /* Store the result of caching one procedure. */
-private int
+static int
 cie_cache_finish_store(i_ctx_t *i_ctx_p, bool replicate)
 {
     os_ptr op = osp;
@@ -639,12 +636,12 @@ cie_cache_finish_store(i_ctx_t *i_ctx_p, bool replicate)
     esp -= 2;			/* pop pointer to cache */
     return o_pop_estack;
 }
-private int
+static int
 cie_cache_finish(i_ctx_t *i_ctx_p)
 {
     return cie_cache_finish_store(i_ctx_p, false);
 }
-private int
+static int
 cie_cache_finish1(i_ctx_t *i_ctx_p)
 {
     return cie_cache_finish_store(i_ctx_p, true);
