@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zcolor1.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: zcolor1.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Level 1 extended color operators */
 #include "ghost.h"
 #include "oper.h"
@@ -31,7 +31,7 @@
 #include "iimage.h"
 
 /* - currentblackgeneration <proc> */
-private int
+static int
 zcurrentblackgeneration(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -42,7 +42,7 @@ zcurrentblackgeneration(i_ctx_t *i_ctx_p)
 }
 
 /* - currentcolortransfer <redproc> <greenproc> <blueproc> <grayproc> */
-private int
+static int
 zcurrentcolortransfer(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -56,7 +56,7 @@ zcurrentcolortransfer(i_ctx_t *i_ctx_p)
 }
 
 /* - currentundercolorremoval <proc> */
-private int
+static int
 zcurrentundercolorremoval(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -67,7 +67,7 @@ zcurrentundercolorremoval(i_ctx_t *i_ctx_p)
 }
 
 /* <proc> setblackgeneration - */
-private int
+static int
 zsetblackgeneration(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -88,7 +88,7 @@ zsetblackgeneration(i_ctx_t *i_ctx_p)
 }
 
 /* <redproc> <greenproc> <blueproc> <grayproc> setcolortransfer - */
-private int
+static int
 zsetcolortransfer(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -134,7 +134,7 @@ zsetcolortransfer(i_ctx_t *i_ctx_p)
 }
 
 /* <proc> setundercolorremoval - */
-private int
+static int
 zsetundercolorremoval(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
