@@ -1,23 +1,17 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: zcontrol.c,v $ $Revision: 1.3.2.2 $ */
+/* $Id: zcontrol.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Control operators */
 #include "string_.h"
 #include "ghost.h"
@@ -30,17 +24,18 @@
 #include "store.h"
 
 /* Forward references */
-private int no_cleanup(P1(i_ctx_t *));
-private uint count_exec_stack(P2(i_ctx_t *, bool));
-private uint count_to_stopped(P2(i_ctx_t *, long));
-private int unmatched_exit(P2(os_ptr, op_proc_t));
+static int check_for_exec(const_os_ptr);
+static int no_cleanup(i_ctx_t *);
+static uint count_exec_stack(i_ctx_t *, bool);
+static uint count_to_stopped(i_ctx_t *, long);
+static int unmatched_exit(os_ptr, op_proc_t);
 
 /* See the comment in opdef.h for an invariant which allows */
 /* more efficient implementation of for, loop, and repeat. */
 
 /* <[test0 body0 ...]> .cond - */
-private int cond_continue(P1(i_ctx_t *));
-private int
+static int cond_continue(i_ctx_t *);
+static int
 zcond(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -58,12 +53,12 @@ zcond(i_ctx_t *i_ctx_p)
     esp = ep += 3;
     ref_assign(ep - 2, op);	/* the cond body */
     make_op_estack(ep - 1, cond_continue);
-    array_get(op, 0L, ep);
+    array_get(imemory, op, 0L, ep);
     esfile_check_cache();
     pop(1);
     return o_push_estack;
 }
-private int
+static int
 cond_continue(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -76,7 +71,7 @@ cond_continue(i_ctx_t *i_ctx_p)
     /* of the tail. */
     check_type(*op, t_boolean);
     if (op->value.boolval) {	/* true */
-	array_get(ep, 1L, ep);
+        array_get(imemory, ep, 1L, ep);
 	esfile_check_cache();
 	code = o_pop_estack;
     } else if (r_size(ep) > 2) {	/* false */
@@ -87,7 +82,7 @@ cond_continue(i_ctx_t *i_ctx_p)
 	elts = packed_next(elts);
 	elts = packed_next(elts);
 	ep->value.packed = elts;
-	array_get(ep, 0L, ep + 2);
+	array_get(imemory, ep, 0L, ep + 2);
 	make_op_estack(ep + 1, cond_continue);
 	esp = ep + 2;
 	esfile_check_cache();
@@ -105,10 +100,16 @@ int
 zexec(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
+    int code;
 
     check_op(1);
-    if (!r_has_attr(op, a_executable))
-	return 0;		/* literal object just gets pushed back */
+    code = check_for_exec(op);
+    if (code < 0) {
+	return code;
+    }
+    if (!r_has_attr(op, a_executable)) {
+	return 0;	/* shortcut, literal object just gets pushed back */
+    }
     check_estack(1);
     ++esp;
     ref_assign(esp, op);
@@ -118,7 +119,7 @@ zexec(i_ctx_t *i_ctx_p)
 }
 
 /* <obj1> ... <objn> <n> .execn - */
-private int
+static int
 zexecn(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -155,8 +156,8 @@ zexecn(i_ctx_t *i_ctx_p)
 }
 
 /* <obj> superexec - */
-private int end_superexec(P1(i_ctx_t *));
-private int
+static int end_superexec(i_ctx_t *);
+static int
 zsuperexec(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -175,7 +176,7 @@ zsuperexec(i_ctx_t *i_ctx_p)
     i_ctx_p->in_superexec++;
     return o_push_estack;
 }
-private int
+static int
 end_superexec(i_ctx_t *i_ctx_p)
 {
     i_ctx_p->in_superexec--;
@@ -189,15 +190,13 @@ end_superexec(i_ctx_t *i_ctx_p)
 /* 	After execution, the array will be placed on  the top of the	*/
 /*	operand stack (on top of any elemetns pushed by <executable>	*/
 /*	for both the normal case and for the error case.		*/
-private int end_runandhide(P1(i_ctx_t *));
-private int err_end_runandhide(P1(i_ctx_t *));
-private int
+static int end_runandhide(i_ctx_t *);
+static int err_end_runandhide(i_ctx_t *);
+static int
 zrunandhide(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     es_ptr ep;
-    uint size;
-    int code;
 
     check_op(2);
     if (!r_is_array(op - 1))
@@ -214,11 +213,12 @@ zrunandhide(i_ctx_t *i_ctx_p)
     make_int(ep - 3, (int)op[-1].tas.type_attrs);
     ref_assign(ep - 2, op - 1);
     r_clear_attrs(ep - 2, a_all);
+    /* replace the array with a special kind of mark that has a_read access */
     esfile_check_cache();
     pop(2);
     return o_push_estack;
 }
-private int
+static int
 runandhide_restore_hidden(i_ctx_t *i_ctx_p, ref *obj, ref *attrs)
 {
     os_ptr op = osp;
@@ -232,7 +232,7 @@ runandhide_restore_hidden(i_ctx_t *i_ctx_p, ref *obj, ref *attrs)
 }
 
 /* - %end_runandhide hiddenobject */
-private int
+static int
 end_runandhide(i_ctx_t *i_ctx_p)
 {
     int code;
@@ -244,7 +244,7 @@ end_runandhide(i_ctx_t *i_ctx_p)
 }
 
 /* restore hidden object for error returns */
-private int
+static int
 err_end_runandhide(i_ctx_t *i_ctx_p)
 {
     int code;
@@ -260,8 +260,8 @@ zif(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
-    check_type(op[-1], t_boolean);
     check_proc(*op);
+    check_type(op[-1], t_boolean);
     if (op[-1].value.boolval) {
 	check_estack(1);
 	++esp;
@@ -278,9 +278,9 @@ zifelse(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
-    check_type(op[-2], t_boolean);
-    check_proc(op[-1]);
     check_proc(*op);
+    check_proc(op[-1]);
+    check_type(op[-2], t_boolean);
     check_estack(1);
     ++esp;
     if (op[-2].value.boolval) {
@@ -294,20 +294,31 @@ zifelse(i_ctx_t *i_ctx_p)
 }
 
 /* <init> <step> <limit> <proc> for - */
-private int
-    for_pos_int_continue(P1(i_ctx_t *)),
-    for_neg_int_continue(P1(i_ctx_t *)),
-    for_real_continue(P1(i_ctx_t *));
+static int
+    for_pos_int_continue(i_ctx_t *),
+    for_neg_int_continue(i_ctx_t *),
+    for_real_continue(i_ctx_t *);
 int
 zfor(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     register es_ptr ep;
+    int code;
+    float params[3];
 
+ 	/* Mostly undocumented, and somewhat bizarre Adobe behavior discovered	*/
+	/* with the CET (28-05) and FTS (124-01) is that the proc is not run	*/
+	/* if BOTH the initial value and increment are zero.			*/
+    if ((code = float_params(op - 1, 3, params)) < 0)
+	return code;
+    if ( params[0] == 0.0 && params[1] == 0.0 ) {
+	pop(4);		/* don't run the proc */
+	return 0;
+    }
     check_estack(7);
     ep = esp + 6;
     check_proc(*op);
-    /* Push a mark, the control variable, the initial value, */
+    /* Push a mark, the control variable set to the initial value, */
     /* the increment, the limit, and the procedure, */
     /* and invoke the continuation operator. */
     if (r_has_type(op - 3, t_integer) &&
@@ -330,11 +341,6 @@ zfor(i_ctx_t *i_ctx_p)
 	else
 	    make_op_estack(ep, for_neg_int_continue);
     } else {
-	float params[3];
-	int code;
-
-	if ((code = float_params(op - 1, 3, params)) < 0)
-	    return code;
 	make_real(ep - 4, params[0]);
 	make_real(ep - 3, params[1]);
 	make_real(ep - 2, params[2]);
@@ -351,7 +357,7 @@ zfor(i_ctx_t *i_ctx_p)
 /* Execution stack contains mark, control variable, increment, */
 /* limit, and procedure (procedure is topmost.) */
 /* Continuation operator for positive integers. */
-private int
+static int
 for_pos_int_continue(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -370,7 +376,7 @@ for_pos_int_continue(i_ctx_t *i_ctx_p)
     return o_push_estack;
 }
 /* Continuation operator for negative integers. */
-private int
+static int
 for_neg_int_continue(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -389,7 +395,7 @@ for_neg_int_continue(i_ctx_t *i_ctx_p)
     return o_push_estack;
 }
 /* Continuation operator for reals. */
-private int
+static int
 for_real_continue(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -411,44 +417,68 @@ for_real_continue(i_ctx_t *i_ctx_p)
     return o_push_estack;
 }
 
-/* Here we provide an internal variant of 'for' that enumerates the */
-/* values 0, 1/N, 2/N, ..., 1 precisely.  The arguments must be */
-/* the integers 0, 1, and N.  We need this for */
-/* loading caches such as the transfer function cache. */
-private int for_fraction_continue(P1(i_ctx_t *));
+/*
+ * Here we provide an internal variant of 'for' that enumerates the values
+ * A, ((N-1)*A+1*B)/N, ((N-2)*A+2*B)/N, ..., B precisely.  The arguments are
+ * A (real), N (integer), and B (real).  We need this for loading caches such
+ * as the transfer function cache.
+ *
+ * NOTE: This computation must match the SAMPLE_LOOP_VALUE macro in gscie.h.
+ */
+static int for_samples_continue(i_ctx_t *);
+/* <first> <count> <last> <proc> %for_samples - */
 int
-zfor_fraction(i_ctx_t *i_ctx_p)
+zfor_samples(i_ctx_t *i_ctx_p)
 {
-    int code = zfor(i_ctx_p);
+    os_ptr op = osp;
+    es_ptr ep;
 
-    if (code < 0)
-	return code;		/* shouldn't ever happen! */
-    make_op_estack(esp, for_fraction_continue);
-    return code;
+    check_type(op[-3], t_real);
+    check_type(op[-2], t_integer);
+    check_type(op[-1], t_real);
+    check_proc(*op);
+    check_estack(8);
+    ep = esp + 7;
+    make_mark_estack(ep - 6, es_for, no_cleanup);
+    make_int(ep - 5, 0);
+    memcpy(ep - 4, op - 3, 3 * sizeof(ref));
+    ref_assign(ep - 1, op);
+    make_op_estack(ep, for_samples_continue);
+    esp = ep;
+    pop(4);
+    return o_push_estack;
 }
 /* Continuation procedure */
-private int
-for_fraction_continue(i_ctx_t *i_ctx_p)
+static int
+for_samples_continue(i_ctx_t *i_ctx_p)
 {
-    register es_ptr ep = esp;
-    int code = for_pos_int_continue(i_ctx_p);
+    os_ptr op = osp;
+    es_ptr ep = esp;
+    long var = ep[-4].value.intval;
+    float a = ep[-3].value.realval;
+    long n = ep[-2].value.intval;
+    float b = ep[-1].value.realval;
 
-    if (code != o_push_estack)
-	return code;
-    /* We must use osp instead of op here, because */
-    /* for_pos_int_continue pushes a value on the o-stack. */
-    make_real(osp, (float)osp->value.intval / ep[-1].value.intval);
-    return code;
+    if (var > n) {
+	esp -= 6;		/* pop everything */
+	return o_pop_estack;
+    }
+    push(1);
+    make_real(op, ((n - var) * a + var * b) / n);
+    ep[-4].value.intval = var + 1;
+    ref_assign_inline(ep + 2, ep);	/* saved proc */
+    esp = ep + 2;
+    return o_push_estack;
 }
 
 /* <int> <proc> repeat - */
-private int repeat_continue(P1(i_ctx_t *));
-private int
+static int repeat_continue(i_ctx_t *);
+int
 zrepeat(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
-    check_type(op[-1], t_integer);
     check_proc(*op);
+    check_type(op[-1], t_integer);
     if (op[-1].value.intval < 0)
 	return_error(e_rangecheck);
     check_estack(5);
@@ -462,7 +492,7 @@ zrepeat(i_ctx_t *i_ctx_p)
     return repeat_continue(i_ctx_p);
 }
 /* Continuation operator for repeat */
-private int
+static int
 repeat_continue(i_ctx_t *i_ctx_p)
 {
     es_ptr ep = esp;		/* saved proc */
@@ -478,8 +508,8 @@ repeat_continue(i_ctx_t *i_ctx_p)
 }
 
 /* <proc> loop */
-private int loop_continue(P1(i_ctx_t *));
-private int
+static int loop_continue(i_ctx_t *);
+static int
 zloop(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -495,7 +525,7 @@ zloop(i_ctx_t *i_ctx_p)
     return loop_continue(i_ctx_p);
 }
 /* Continuation operator for loop */
-private int
+static int
 loop_continue(i_ctx_t *i_ctx_p)
 {
     register es_ptr ep = esp;	/* saved proc */
@@ -506,7 +536,7 @@ loop_continue(i_ctx_t *i_ctx_p)
 }
 
 /* - exit - */
-private int
+static int
 zexit(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -545,7 +575,7 @@ zexit(i_ctx_t *i_ctx_p)
 
 /* In the normal (no-error) case, pop the mask from the e-stack, */
 /* and move the result to the o-stack. */
-private int
+static int
 stopped_push(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -560,7 +590,7 @@ stopped_push(i_ctx_t *i_ctx_p)
 /* Equivalent to true 1 .stop. */
 /* This is implemented in C because if were a pseudo-operator, */
 /* the stacks would get restored in case of an error. */
-private int
+static int
 zstop(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -585,7 +615,7 @@ zstop(i_ctx_t *i_ctx_p)
 }
 
 /* <result> <mask> .stop - */
-private int
+static int
 zzstop(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -618,10 +648,11 @@ zzstop(i_ctx_t *i_ctx_p)
 /* Equivalent to false 1 .stopped. */
 /* This is implemented in C because if were a pseudo-operator, */
 /* the stacks would get restored in case of an error. */
-private int
+static int
 zstopped(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
+
     check_op(1);
     /* Mark the execution stack, and push the default result */
     /* in case control returns normally. */
@@ -632,14 +663,12 @@ zstopped(i_ctx_t *i_ctx_p)
     ++esp;
     make_int(esp, 1);		/* save the signal mask */
     push_op_estack(stopped_push);
-    *++esp = *op;		/* execute the operand */
-    esfile_check_cache();
-    pop(1);
+    push_op_estack(zexec);	/* execute the operand */
     return o_push_estack;
 }
 
 /* <obj> <result> <mask> .stopped <result> */
-private int
+static int
 zzstopped(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -652,15 +681,14 @@ zzstopped(i_ctx_t *i_ctx_p)
     *++esp = op[-1];		/* save the result */
     *++esp = *op;		/* save the signal mask */
     push_op_estack(stopped_push);
-    *++esp = op[-2];		/* execute the operand */
-    esfile_check_cache();
-    pop(3);
+    push_op_estack(zexec);	/* execute the operand */
+    pop(2);
     return o_push_estack;
 }
 
 /* <mask> .instopped false */
 /* <mask> .instopped <result> true */
-private int
+static int
 zinstopped(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -680,7 +708,7 @@ zinstopped(i_ctx_t *i_ctx_p)
 /* <include_marks> .countexecstack <int> */
 /* - countexecstack <int> */
 /* countexecstack is an operator solely for the sake of the Genoa tests. */
-private int
+static int
 zcountexecstack(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -689,7 +717,7 @@ zcountexecstack(i_ctx_t *i_ctx_p)
     make_int(op, count_exec_stack(i_ctx_p, false));
     return 0;
 }
-private int
+static int
 zcountexecstack1(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -702,9 +730,9 @@ zcountexecstack1(i_ctx_t *i_ctx_p)
 /* <array> <include_marks> .execstack <subarray> */
 /* <array> execstack <subarray> */
 /* execstack is an operator solely for the sake of the Genoa tests. */
-private int execstack_continue(P1(i_ctx_t *));
-private int execstack2_continue(P1(i_ctx_t *));
-private int
+static int execstack_continue(i_ctx_t *);
+static int execstack2_continue(i_ctx_t *);
+static int
 push_execstack(i_ctx_t *i_ctx_p, os_ptr op1, bool include_marks,
 	       op_proc_t cont)
 {
@@ -718,11 +746,14 @@ push_execstack(i_ctx_t *i_ctx_p, os_ptr op1, bool include_marks,
      */
     uint depth;
 
-    check_write_type(*op1, t_array);
+    if (!r_is_array(op1))
+	return_op_typecheck(op1);
+    /* Check the length before the write access per CET 28-03 */
     size = r_size(op1);
     depth = count_exec_stack(i_ctx_p, include_marks);
     if (depth > size)
 	return_error(e_rangecheck);
+    check_write(*op1);
     {
 	int code = ref_stack_store_check(&e_stack, op1, size, 0);
 
@@ -734,14 +765,14 @@ push_execstack(i_ctx_t *i_ctx_p, os_ptr op1, bool include_marks,
     push_op_estack(cont);
     return o_push_estack;
 }
-private int
+static int
 zexecstack(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
     return push_execstack(i_ctx_p, op, false, execstack_continue);
 }
-private int
+static int
 zexecstack2(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -751,7 +782,7 @@ zexecstack2(i_ctx_t *i_ctx_p)
 }
 /* Continuation operator to do the actual transfer. */
 /* r_size(op1) was set just above. */
-private int
+static int
 do_execstack(i_ctx_t *i_ctx_p, bool include_marks, os_ptr op1)
 {
     os_ptr op = osp;
@@ -800,14 +831,14 @@ do_execstack(i_ctx_t *i_ctx_p, bool include_marks, os_ptr op1)
     pop(op - op1);
     return 0;
 }
-private int
+static int
 execstack_continue(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
 
     return do_execstack(i_ctx_p, false, op);
 }
-private int
+static int
 execstack2_continue(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -816,14 +847,14 @@ execstack2_continue(i_ctx_t *i_ctx_p)
 }
 
 /* - .needinput - */
-private int
+static int
 zneedinput(i_ctx_t *i_ctx_p)
 {
     return e_NeedInput;		/* interpreter will exit to caller */
 }
 
 /* <obj> <int> .quit - */
-private int
+static int
 zquit(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -834,8 +865,8 @@ zquit(i_ctx_t *i_ctx_p)
 }
 
 /* - currentfile <file> */
-private ref *zget_current_file(P1(i_ctx_t *));
-private int
+static ref *zget_current_file(i_ctx_t *);
+static int
 zcurrentfile(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -868,7 +899,7 @@ zcurrentfile(i_ctx_t *i_ctx_p)
     return 0;
 }
 /* Get the current file from which the interpreter is reading. */
-private ref *
+static ref *
 zget_current_file(i_ctx_t *i_ctx_p)
 {
     ref_stack_enum_t rsenum;
@@ -923,8 +954,8 @@ const op_def zcontrol3_op_defs[] = {
     {"0%for_pos_int_continue", for_pos_int_continue},
     {"0%for_neg_int_continue", for_neg_int_continue},
     {"0%for_real_continue", for_real_continue},
-    {"4%for_fraction", zfor_fraction},
-    {"0%for_fraction_continue", for_fraction_continue},
+    {"4%for_samples", zfor_samples},
+    {"0%for_samples_continue", for_samples_continue},
     {"0%loop_continue", loop_continue},
     {"0%repeat_continue", repeat_continue},
     {"0%stopped_push", stopped_push},
@@ -937,8 +968,29 @@ const op_def zcontrol3_op_defs[] = {
 
 /* ------ Internal routines ------ */
 
+/*
+ * Check the operand of exec or stopped.  Return 0 if OK to execute, or a
+ * negative error code.  We emulate an apparent bug in Adobe interpreters,
+ * which cause an invalidaccess error when 'exec'ing a noaccess literal
+ * (other than dictionaries).  We also match the Adobe interpreters in that
+ * we catch noaccess executable objects here, rather than waiting for the
+ * interpreter to catch them, so that we can signal the error with the
+ * object still on the operand stack.
+ */
+static bool
+check_for_exec(const_os_ptr op)
+{
+    if (!r_has_attr(op, a_execute) && /* only true if noaccess */
+	ref_type_uses_access(r_type(op)) &&
+	(r_has_attr(op, a_executable) || !r_has_type(op, t_dictionary))
+	) {
+	return_error(e_invalidaccess);
+    }
+    return 0;
+}
+
 /* Vacuous cleanup routine */
-private int
+static int
 no_cleanup(i_ctx_t *i_ctx_p)
 {
     return 0;
@@ -948,7 +1000,7 @@ no_cleanup(i_ctx_t *i_ctx_p)
  * Count the number of elements on the exec stack, with or without
  * the normally invisible elements (*op is a Boolean that indicates this).
  */
-private uint
+static uint
 count_exec_stack(i_ctx_t *i_ctx_p, bool include_marks)
 {
     uint count = ref_stack_count(&e_stack);
@@ -969,7 +1021,7 @@ count_exec_stack(i_ctx_t *i_ctx_p, bool include_marks)
  * mark on the e-stack with a given mask.  Return 0 if there is no 'stopped'
  * mark.
  */
-private uint
+static uint
 count_to_stopped(i_ctx_t *i_ctx_p, long mask)
 {
     ref_stack_enum_t rsenum;
@@ -981,12 +1033,13 @@ count_to_stopped(i_ctx_t *i_ctx_p, long mask)
 	es_ptr ep = rsenum.ptr + used - 1;
 	uint count = used;
 
-	for (; count; count--, ep--)
-	    if (r_is_estack_mark(ep) &&
-		estack_mark_index(ep) == es_stopped &&
-		(ep[2].value.intval & mask) != 0
-		)
-		return scanned + (used - count + 1);
+	for (; count; count--, ep--) {
+	    if (r_is_estack_mark(ep)) {
+		if (estack_mark_index(ep) == es_stopped &&
+		  (ep[2].value.intval & mask) != 0)
+		    return scanned + (used - count + 1);
+	    }
+	}	
 	scanned += used;
     } while (ref_stack_enum_next(&rsenum));
     return 0;
@@ -1021,7 +1074,7 @@ pop_estack(i_ctx_t *i_ctx_p, uint count)
  * enclosing control scope (loop or stopped).  The caller has already
  * ensured two free slots on the top of the o-stack.
  */
-private int
+static int
 unmatched_exit(os_ptr op, op_proc_t opproc)
 {
     make_oper(op - 1, 0, opproc);
