@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zcspixel.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: zcspixel.c 8250 2007-09-25 13:31:24Z giles $ */
 /* DevicePixel color space support */
 #include "ghost.h"
 #include "oper.h"
@@ -23,12 +23,12 @@
 #include "ialloc.h"
 
 /* <array> .setdevicepixelspace - */
-private int
+static int
 zsetdevicepixelspace(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     ref depth;
-    gs_color_space cs;
+    gs_color_space *pcs;
     int code;
 
     check_read_type(*op, t_array);
@@ -36,10 +36,12 @@ zsetdevicepixelspace(i_ctx_t *i_ctx_p)
 	return_error(e_rangecheck);
     array_get(imemory, op, 1L, &depth);
     check_type_only(depth, t_integer);
-    code = gs_cspace_init_DevicePixel(imemory, &cs, (int)depth.value.intval);
+    code = gs_cspace_new_DevicePixel(imemory, &pcs, (int)depth.value.intval);
     if (code < 0)
 	return code;
-    code = gs_setcolorspace(igs, &cs);
+    code = gs_setcolorspace(igs, pcs);
+    /* release reference from construction */
+    rc_decrement_only(pcs, "zsetseparationspace");
     if (code >= 0)
 	pop(1);
     return code;
