@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfarc4.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: zfarc4.c 8250 2007-09-25 13:31:24Z giles $ */
 
 /* this is the ps interpreter interface to the arcfour cipher filter
    used in PDF encryption. We provide both Decode and Encode filters;
@@ -31,7 +31,7 @@
 
 /* <source> <dict> arcfour/filter <file> */
 
-private int
+static int
 z_arcfour_d(i_ctx_t * i_ctx_p)
 {
     os_ptr op = osp;		/* i_ctx_p->op_stack.stack.p defined in osstack.h */
@@ -55,7 +55,7 @@ z_arcfour_d(i_ctx_t * i_ctx_p)
 }
 
 /* encode version of the filter */
-private int
+static int
 z_arcfour_e(i_ctx_t * i_ctx_p)
 {
     os_ptr op = osp;		/* i_ctx_p->op_stack.stack.p defined in osstack.h */
@@ -78,8 +78,8 @@ z_arcfour_e(i_ctx_t * i_ctx_p)
 			(stream_state *) & state, 0);
 }
 
-/* match the above routines to their postscript filter names
-   this is how our 'private' routines get called externally */
+/* Match the above routines to their postscript filter names.
+   This is how our static routines get called externally. */
 const op_def zfarc4_op_defs[] = {
     op_def_begin_filter(),
     {"2ArcfourDecode", z_arcfour_d},
