@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,11 +11,12 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfile.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: zfile.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Non-I/O file operators */
 #include "memory_.h"
 #include "string_.h"
 #include "unistd_.h"
+#include "stat_.h" /* get system header early to avoid name clash on Cygwin */
 #include "ghost.h"
 #include "gscdefs.h"		/* for gx_io_device_table */
 #include "gsutil.h"		/* for bytes_compare */
@@ -29,7 +30,6 @@
 #include "estack.h"		/* for filenameforall, .execfile */
 #include "ialloc.h"
 #include "ilevel.h"		/* %names only work in Level 2 */
-#include "interp.h"		/* gs_errorinfo_put_string prototype */
 #include "iname.h"
 #include "isave.h"		/* for restore */
 #include "idict.h"
@@ -50,21 +50,19 @@ extern_gx_io_device_table();
 extern const char iodev_dtype_stdio[];
 
 /* Forward references: file name parsing. */
-private int parse_file_name(const ref * op, gs_parsed_file_name_t * pfn, bool safemode);
-private int parse_real_file_name(const ref * op,
+static int parse_file_name(const ref * op, gs_parsed_file_name_t * pfn, bool safemode);
+static int parse_real_file_name(const ref * op,
 				 gs_parsed_file_name_t * pfn,
 				 gs_memory_t *mem, client_name_t cname);
-private int parse_file_access_string(const ref *op, char file_access[4]);
+static int parse_file_access_string(const ref *op, char file_access[4]);
 
 /* Forward references: other. */
-private int execfile_finish(i_ctx_t *);
-private int execfile_cleanup(i_ctx_t *);
-private int zopen_file(i_ctx_t *, const gs_parsed_file_name_t *pfn,
+static int execfile_finish(i_ctx_t *);
+static int execfile_cleanup(i_ctx_t *);
+static int zopen_file(i_ctx_t *, const gs_parsed_file_name_t *pfn,
 		       const char *file_access, stream **ps,
 		       gs_memory_t *mem);
-private iodev_proc_open_file(iodev_os_open_file);
-private void file_init_stream(stream *s, FILE *file, const char *fmode,
-			      byte *buffer, uint buffer_size);
+static iodev_proc_open_file(iodev_os_open_file);
 stream_proc_report_error(filter_report_error);
 
 /*
@@ -109,14 +107,14 @@ stream_proc_report_error(filter_report_error);
  * causing many aborted roundtrips through the JPEG filter code.
  */
 #define DEFAULT_BUFFER_SIZE 2048
-const uint file_default_buffer_size = DEFAULT_BUFFER_SIZE;
+extern const uint file_default_buffer_size;
 
 /* An invalid file object */
-private stream invalid_file_stream;
+static stream invalid_file_stream;
 stream *const invalid_file_entry = &invalid_file_stream;
 
 /* Initialize the file table */
-private int
+static int
 zfile_init(i_ctx_t *i_ctx_p)
 {
     /* Create and initialize an invalid (closed) stream. */
@@ -141,7 +139,7 @@ make_invalid_file(ref * fp)
 
 /* Check a file name for permission by stringmatch on one of the */
 /* strings of the permitgroup array. */
-private int
+static int
 check_file_permissions_reduced(i_ctx_t *i_ctx_p, const char *fname, int len,
 			const char *permitgroup)
 {
@@ -207,7 +205,7 @@ check_file_permissions_reduced(i_ctx_t *i_ctx_p, const char *fname, int len,
 
 /* Check a file name for permission by stringmatch on one of the */
 /* strings of the permitgroup array */
-private int
+static int
 check_file_permissions(i_ctx_t *i_ctx_p, const char *fname, int len,
 			const char *permitgroup)
 {
@@ -220,7 +218,7 @@ check_file_permissions(i_ctx_t *i_ctx_p, const char *fname, int len,
 }
 
 /* <name_string> <access_string> file <file> */
-private int
+int				/* exported for zsysvm.c */
 zfile(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -289,7 +287,7 @@ zfile(i_ctx_t *i_ctx_p)
  * temp directory is not explicitly named on the PermitFile... path 
  * The names 'SAFETY' and 'tempfiles' are defined by gs_init.ps
 */
-private bool
+static bool
 file_is_tempfile(i_ctx_t *i_ctx_p, const ref *op)
 {
     ref *SAFETY;
@@ -308,7 +306,7 @@ file_is_tempfile(i_ctx_t *i_ctx_p, const ref *op)
 /* ------ Level 2 extensions ------ */
 
 /* <string> deletefile - */
-private int
+static int
 zdeletefile(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -333,9 +331,9 @@ zdeletefile(i_ctx_t *i_ctx_p)
 }
 
 /* <template> <proc> <scratch> filenameforall - */
-private int file_continue(i_ctx_t *);
-private int file_cleanup(i_ctx_t *);
-private int
+static int file_continue(i_ctx_t *);
+static int file_cleanup(i_ctx_t *);
+static int
 zfilenameforall(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -379,7 +377,7 @@ zfilenameforall(i_ctx_t *i_ctx_p)
     return (code == o_pop_estack ? o_push_estack : code);
 }
 /* Continuation operator for enumerating files */
-private int
+static int
 file_continue(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -410,7 +408,7 @@ file_continue(i_ctx_t *i_ctx_p)
     }
 }
 /* Cleanup procedure for enumerating files */
-private int
+static int
 file_cleanup(i_ctx_t *i_ctx_p)
 {
     gx_io_device *iodev = r_ptr(esp + 2, gx_io_device);
@@ -420,18 +418,19 @@ file_cleanup(i_ctx_t *i_ctx_p)
 }
 
 /* <string1> <string2> renamefile - */
-private int
+static int
 zrenamefile(i_ctx_t *i_ctx_p)
 {
+    int code;
     os_ptr op = osp;
     gs_parsed_file_name_t pname1, pname2;
-    int code = parse_real_file_name(op - 1, &pname1, imemory,
-				    "renamefile(from)");
 
+    code = parse_real_file_name(op, &pname2, imemory, "renamefile(to)");
     if (code < 0)
 	return code;
-    pname2.fname = 0;
-    code = parse_real_file_name(op, &pname2, imemory, "renamefile(to)");
+
+    pname1.fname = 0;
+    code = parse_real_file_name(op - 1, &pname1, imemory, "renamefile(from)");
     if (code >= 0) {
 	if (pname1.iodev != pname2.iodev ) {
 	    if (pname1.iodev == iodev_default)
@@ -470,7 +469,7 @@ zrenamefile(i_ctx_t *i_ctx_p)
 /* <file> status <open_bool> */
 /* <string> status <pages> <bytes> <ref_time> <creation_time> true */
 /* <string> status false */
-private int
+static int
 zstatus(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -540,7 +539,7 @@ zstatus(i_ctx_t *i_ctx_p)
 /* ------ Non-standard extensions ------ */
 
 /* <executable_file> .execfile - */
-private int
+static int
 zexecfile(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -553,7 +552,7 @@ zexecfile(i_ctx_t *i_ctx_p)
     return zexec(i_ctx_p);
 }
 /* Finish normally. */
-private int
+static int
 execfile_finish(i_ctx_t *i_ctx_p)
 {
     check_ostack(1);
@@ -562,7 +561,7 @@ execfile_finish(i_ctx_t *i_ctx_p)
     return o_pop_estack;
 }
 /* Clean up by closing the file. */
-private int
+static int
 execfile_cleanup(i_ctx_t *i_ctx_p)
 {
     check_ostack(1);
@@ -571,7 +570,7 @@ execfile_cleanup(i_ctx_t *i_ctx_p)
 }
 
 /* - .filenamelistseparator <string> */
-private int
+static int
 zfilenamelistseparator(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -583,7 +582,7 @@ zfilenamelistseparator(i_ctx_t *i_ctx_p)
 }
 
 /* <name> .filenamesplit <dir> <base> <extension> */
-private int
+static int
 zfilenamesplit(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -595,7 +594,7 @@ zfilenamesplit(i_ctx_t *i_ctx_p)
 
 /* <string> .libfile <file> true */
 /* <string> .libfile <string> false */
-private int
+int				/* exported for zsysvm.c */
 zlibfile(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -655,7 +654,7 @@ zlibfile(i_ctx_t *i_ctx_p)
 
 /* A "simple" prefix is defined as a (possibly empty) string of
    alphanumeric, underscore, and hyphen characters. */
-private bool
+static bool
 prefix_is_simple(const char *pstr)
 {
     int i;
@@ -670,7 +669,7 @@ prefix_is_simple(const char *pstr)
 }
 
 /* <prefix|null> <access_string> .tempfile <name_string> <file> */
-private int
+static int
 ztempfile(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -760,7 +759,7 @@ const op_def zfile_op_defs[] =
 
 /* Parse a file name into device and individual name. */
 /* See gsfname.c for details. */
-private int
+static int
 parse_file_name(const ref * op, gs_parsed_file_name_t * pfn, bool safemode)
 {
     int code;
@@ -783,7 +782,7 @@ parse_file_name(const ref * op, gs_parsed_file_name_t * pfn, bool safemode)
 
 /* Parse a real (non-device) file name and convert to a C string. */
 /* See gsfname.c for details. */
-private int
+static int
 parse_real_file_name(const ref *op, gs_parsed_file_name_t *pfn,
 		     gs_memory_t *mem, client_name_t cname)
 {
@@ -794,7 +793,7 @@ parse_real_file_name(const ref *op, gs_parsed_file_name_t *pfn,
 
 /* Parse the access string for opening a file. */
 /* [4] is for r/w, +, b, \0. */
-private int
+static int
 parse_file_access_string(const ref *op, char file_access[4])
 {
     const byte *astr;
@@ -832,7 +831,7 @@ parse_file_access_string(const ref *op, char file_access[4])
  * Open a file specified by a parsed file name (which may be only a
  * device).
  */
-private int
+static int
 zopen_file(i_ctx_t *i_ctx_p, const gs_parsed_file_name_t *pfn,
 	   const char *file_access, stream **ps, gs_memory_t *mem)
 {
@@ -861,7 +860,7 @@ zopen_file(i_ctx_t *i_ctx_p, const gs_parsed_file_name_t *pfn,
  * Define the file_open procedure for the %os% IODevice (also used, as the
  * default, for %pipe% and possibly others).
  */
-private int
+static int
 iodev_os_open_file(gx_io_device * iodev, const char *fname, uint len,
 		   const char *file_access, stream ** ps, gs_memory_t * mem)
 {
@@ -887,53 +886,7 @@ make_stream_file(ref * pfile, stream * s, const char *access)
     }
 }
 
-private gp_file_name_combine_result 
-gp_file_name_combine_patch(const char *prefix, uint plen, const char *fname, uint flen, 
-			    bool no_sibling, char *buffer, uint *blen)
-{
-    return gp_file_name_combine(prefix, plen, fname, flen, no_sibling, buffer, blen);
-}
-
-/* Prepare a stream with a file name. */
-/* Return 0 if successful, error code if not. */
-/* On a successful return, the C file name is in the stream buffer. */
-/* If fname==0, set up stream, and buffer. */
-private int
-file_prepare_stream(const char *fname, uint len, const char *file_access, 
-		 uint buffer_size, stream ** ps, char fmode[4], 
-		 gx_io_device *iodev, gs_memory_t *mem)
-{
-    byte *buffer;
-    register stream *s;
-
-    /* Open the file, always in binary mode. */
-    strcpy(fmode, file_access);
-    strcat(fmode, gp_fmode_binary_suffix);
-    if (buffer_size == 0)
-	buffer_size = file_default_buffer_size;
-    if (len >= buffer_size)    /* we copy the file name into the buffer */
-	return_error(e_limitcheck);
-    /* Allocate the stream first, since it persists */
-    /* even after the file has been closed. */
-    s = file_alloc_stream(mem, "file_prepare_stream");
-    if (s == 0)
-	return_error(e_VMerror);
-    /* Allocate the buffer. */
-    buffer = gs_alloc_bytes(mem, buffer_size, "file_prepare_stream(buffer)");
-    if (buffer == 0)
-	return_error(e_VMerror);
-    if (fname != 0) {
-	memcpy(buffer, fname, len);
-	buffer[len] = 0;	/* terminate string */
-    } else
-	buffer[0] = 0;	/* safety */
-    s->cbuf = buffer;
-    s->bsize = s->cbsize = buffer_size;
-    *ps = s;
-    return 0;
-}
-
-private int
+static int
 check_file_permissions_aux(i_ctx_t *i_ctx_p, char *fname, uint flen)
 {   /* i_ctx_p is NULL running init files. */
     /* fname must be reduced. */
@@ -960,7 +913,12 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
     bool search_with_combine = false;
     char fmode[4] = { 'r', 0, 0, 0 };		/* room for binary suffix */
     stream *s;
+    gx_io_device *iodev = iodev_default;
 
+    /* when starting arg files (@ files) iodev_default is not yet set */
+    if (iodev == 0)
+        iodev = (gx_io_device *)gx_io_device_table[0];
+    
     strcat(fmode, gp_fmode_binary_suffix);
     if (gp_file_name_is_absolute(fname, flen)) {
        search_with_no_combine = true;
@@ -974,8 +932,7 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
 
 	if (gp_file_name_reduce(fname, flen, buffer, &blen1) != gp_combine_success)
 	    goto skip;
-	/* when starting arg files (@ files) iodev_default is not yet set */
-	if (iodev_os_open_file((gx_io_device *)gx_io_device_table[0], (const char *)buffer, blen1,
+	if (iodev_os_open_file(iodev, (const char *)buffer, blen1,
 				(const char *)fmode, &s, (gs_memory_t *)mem) == 0) {
 	    if (starting_arg_file ||
 			check_file_permissions_aux(i_ctx_p, buffer, blen1) >= 0) {
@@ -983,7 +940,7 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
 		make_stream_file(pfile, s, "r");
 		return 0;
 	    }
-	    iodev_os_fclose(iodev_default, s->file);
+	    iodev_os_fclose(iodev, s->file);
 	    return_error(e_invalidfileaccess);
 	}
 	skip:;
@@ -996,7 +953,6 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
 	    const ref *prdir = pfpath->list.value.refs + pi;
 	    const char *pstr = (const char *)prdir->value.const_bytes;
 	    uint plen = r_size(prdir), blen1 = blen;
-	    gx_io_device *iodev;
     	    gs_parsed_file_name_t pname;
 	    gp_file_name_combine_result r;
 
@@ -1010,10 +966,9 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
 		if (i_ctx_p == NULL)
 		    continue;		/* devices not yet initialized */
 		gs_parse_file_name(&pname, pstr, plen);
-		iodev = pname.iodev;
 		memcpy(buffer, pname.fname, pname.len);
 		memcpy(buffer+pname.len, fname, flen);
-		code = iodev->procs.open_file(iodev, buffer, pname.len + flen, fmode,
+		code = pname.iodev->procs.open_file(pname.iodev, buffer, pname.len + flen, fmode,
 					      &s, (gs_memory_t *)mem);
 		if (code < 0)
 		    continue;
@@ -1024,12 +979,11 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
 		*pclen = plen + flen;
 		return 0;
 	    } else {
-		iodev = iodev_default;
-		r = gp_file_name_combine_patch(pstr, plen, 
+		r = gp_file_name_combine(pstr, plen, 
 			fname, flen, false, buffer, &blen1);
 		if (r != gp_combine_success)
 		    continue;
-		if (iodev_os_open_file(iodev_default, (const char *)buffer, blen1, (const char *)fmode,
+		if (iodev_os_open_file(iodev, (const char *)buffer, blen1, (const char *)fmode,
 					&s, (gs_memory_t *)mem) == 0) {
 		    if (starting_arg_file ||
 			check_file_permissions_aux(i_ctx_p, buffer, blen1) >= 0) {
@@ -1037,7 +991,7 @@ lib_file_open(gs_file_path_ptr  lib_path, const gs_memory_t *mem, i_ctx_t *i_ctx
 			make_stream_file(pfile, s, "r");
 			return 0;
 		    }
-		    iodev_os_fclose(iodev_default, s->file);
+		    iodev_os_fclose(iodev, s->file);
 		    return_error(e_invalidfileaccess);
 		}
 	    }
@@ -1086,67 +1040,6 @@ file_read_string(const byte *str, uint len, ref *pfile, gs_ref_memory_t *imem)
     return 0;
 }
 
-/*
- * Set up a file stream on an OS file.  The caller has allocated the
- * stream and buffer.
- */
-private void
-file_init_stream(stream *s, FILE *file, const char *fmode, byte *buffer,
-		 uint buffer_size)
-{
-    switch (fmode[0]) {
-    case 'a':
-	sappend_file(s, file, buffer, buffer_size);
-	break;
-    case 'r':
-	/* Defeat buffering for terminals. */
-	{
-	    struct stat rstat;
-
-	    fstat(fileno(file), &rstat);
-	    sread_file(s, file, buffer,
-		       (S_ISCHR(rstat.st_mode) ? 1 : buffer_size));
-	}
-	break;
-    case 'w':
-	swrite_file(s, file, buffer, buffer_size);
-    }
-    if (fmode[1] == '+')
-	s->file_modes |= s_mode_read | s_mode_write;
-    s->save_close = s->procs.close;
-    s->procs.close = file_close_file;
-}
-
-/* Open a file stream, optionally on an OS file. */
-/* Return 0 if successful, error code if not. */
-/* On a successful return, the C file name is in the stream buffer. */
-/* If fname==0, set up the file entry, stream, and buffer, */
-/* but don't open an OS file or initialize the stream. */
-int
-file_open_stream(const char *fname, uint len, const char *file_access,
-		 uint buffer_size, stream ** ps, gx_io_device *iodev,
-		 iodev_proc_fopen_t fopen_proc, gs_memory_t *mem)
-{
-    int code;
-    FILE *file;
-    char fmode[4];  /* r/w/a, [+], [b], null */
-
-    if (!iodev)
-	iodev = iodev_default;
-    code = file_prepare_stream(fname, len, file_access, buffer_size, ps, fmode, 
-			    (!iodev ? iodev_default : iodev), mem);
-    if (code < 0)
-	return code;
-    if (fname == 0)
-	return 0;
-    code = (*fopen_proc)(iodev, (char *)(*ps)->cbuf, fmode, &file,
-			 (char *)(*ps)->cbuf, (*ps)->bsize);
-    if (code < 0)
-	return code;
-    file_init_stream(*ps, file, fmode, (*ps)->cbuf, (*ps)->bsize);
-    return 0;
-}
-
 /* Report an error by storing it in the stream's error_string. */
 int
 filter_report_error(stream_state * st, const char *str)
@@ -1207,130 +1100,6 @@ filter_open(const char *file_access, uint buffer_size, ref * pfile,
     return 0;
 }
 
-/* Allocate and return a file stream. */
-/* Return 0 if the allocation failed. */
-/* The stream is initialized to an invalid state, so the caller need not */
-/* worry about cleaning up if a later step in opening the stream fails. */
-stream *
-file_alloc_stream(gs_memory_t * mem, client_name_t cname)
-{
-    stream *s;
-    gs_ref_memory_t *imem = 0;
-
-    /*
-     * HACK: Figure out whether this is a gs_ref_memory_t.
-     * Avoiding this hack would require rippling a change
-     * from gs_memory_t to gs_ref_memory_t into the open_file and
-     * open_device procedures of gx_io_device, which in turn would
-     * impact other things we don't want to change.
-     */
-    if (mem->procs.free_object == gs_ref_memory_procs.free_object)
-	imem = (gs_ref_memory_t *) mem;
-
-    if (imem) {
-	/* Look first for a free stream allocated at this level. */
-	s = imem->streams;
-	while (s != 0) {
-	    if (!s_is_valid(s) && s->read_id != 0 /* i.e. !overflowed */ ) {
-		s->is_temp = 0;	/* not a temp stream */
-		return s;
-	    }
-	    s = s->next;
-	}
-    }
-    s = s_alloc(mem, cname);
-    if (s == 0)
-	return 0;
-    s_init_ids(s);
-    s->is_temp = 0;		/* not a temp stream */
-    /*
-     * Disable the stream now (in case we can't open the file,
-     * or a filter init procedure fails) so that `restore' won't
-     * crash when it tries to close open files.
-     */
-    s_disable(s);
-    if (imem) {
-	/* Add s to the list of files. */
-	if (imem->streams != 0)
-	    imem->streams->prev = s;
-	s->next = imem->streams;
-	imem->streams = s;
-    } else {
-	s->next = 0;
-    }
-    s->prev = 0;
-    return s;
-}
-
-/* ------ Stream closing ------ */
-
-/*
- * Finish closing a file stream.  This used to check whether it was
- * currentfile, but we don't have to do this any longer.  This replaces the
- * close procedure for the std* streams, which cannot actually be closed.
- *
- * This is exported for ziodev.c.  */
-int
-file_close_finish(stream * s)
-{
-    return 0;
-}
-
-/*
- * Close a file stream, but don't deallocate the buffer.  This replaces the
- * close procedure for %lineedit and %statementedit.  (This is WRONG: these
- * streams should allocate a new buffer each time they are opened, but that
- * would overstress the allocator right now.)  This is exported for ziodev.c.
- * This also replaces the close procedure for the string-reading stream
- * created for gs_run_string.
- */
-int
-file_close_disable(stream * s)
-{
-    int code = (*s->save_close)(s);
-
-    if (code)
-	return code;
-    /* Increment the IDs to prevent further access. */
-    s->read_id = s->write_id = (s->read_id | s->write_id) + 1;
-    return file_close_finish(s);
-}
-
-/* Close a file stream.  This replaces the close procedure in the stream */
-/* for normal (OS) files and for filters. */
-int
-file_close_file(stream * s)
-{
-    stream *stemp = s->strm;
-    gs_memory_t *mem;
-    int code = file_close_disable(s);
-
-    if (code)
-	return code;
-    /*
-     * Check for temporary streams created for filters.
-     * There may be more than one in the case of a procedure-based filter,
-     * or if we created an intermediate stream to ensure
-     * a large enough buffer.  Note that these streams may have been
-     * allocated by file_alloc_stream, so we mustn't free them.
-     */
-    while (stemp != 0 && stemp->is_temp != 0) {
-	stream *snext = stemp->strm;
-
-	mem = stemp->memory;
-	if (stemp->is_temp > 1)
-	    gs_free_object(mem, stemp->cbuf,
-			   "file_close(temp stream buffer)");
-	s_disable(stemp);
-	stemp = snext;
-    }
-    mem = s->memory;
-    gs_free_object(mem, s->cbuf, "file_close(buffer)");
-    if (s->close_strm && stemp != 0)
-	return sclose(stemp);
-    return 0;
-}
-
 /* Close a file object. */
 /* This is exported only for gsmain.c. */
 int
