@@ -1,20 +1,17 @@
-/* Copyright (C) 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfunc4.c,v 1.12 2004/08/04 19:36:13 stefan Exp $ */
+/* $Id: zfunc4.c 8250 2007-09-25 13:31:24Z giles $ */
 /* PostScript language support for FunctionType 4 (PS Calculator) Functions */
 #include "memory_.h"
 #include "ghost.h"
@@ -28,6 +25,9 @@
 #include "iname.h"
 #include "dstack.h"
 #include "ialloc.h"
+#include "gzstate.h"	    /* these are needed to check if device is pdfwrite */
+#include "gxdevcli.h"	    /* these are needed to check if device is pdfwrite */
+#include "string_.h"	    /* these are needed to check if device is pdfwrite */
 /*
  * FunctionType 4 functions are not defined in the PostScript language.  We
  * provide support for them because they are needed for PDF 1.3.  In
@@ -114,7 +114,7 @@ static const calc_op_t calc_ops[] = {
 };
 
 /* Fix up an if or ifelse forward reference. */
-private void
+static void
 psc_fixup(byte *p, byte *to)
 {
     int skip = to - (p + 3);
@@ -123,17 +123,38 @@ psc_fixup(byte *p, byte *to)
     p[2] = (byte)skip;
 }
 
+/* Check whether the ref is a given operator or resolves to it */
+static bool
+resolves_to_oper(i_ctx_t *i_ctx_p, const ref *pref, const op_proc_t proc)
+{
+    if (!r_has_attr(pref, a_executable))
+        return false;
+    if (r_btype(pref) == t_operator) {
+        return pref->value.opproc == proc;
+    } else if (r_btype(pref) == t_name) {
+        ref * val;
+        if (dict_find(systemdict, pref, &val) <= 0)
+	    return false;
+        if (r_btype(val) != t_operator)
+	    return false;
+        if (!r_has_attr(val, a_executable))
+	    return false;
+        return val->value.opproc == proc;
+    }
+   else
+      return false;
+}
+
 /*
  * Check a calculator function for validity, optionally storing its encoded
  * representation and add the size of the encoded representation to *psize.
  * Note that we arbitrarily limit the depth of procedure nesting.  pref is
  * known to be a procedure.
  */
-#define MAX_PSC_FUNCTION_NESTING 10
-private int
+static int
 check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int *psize)
 {
-    long i;
+    uint i;
     uint size = r_size(pref);
 
     for (i = 0; i < size; ++i) {
@@ -222,20 +243,28 @@ check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int
 	    code = check_psc_function(i_ctx_p, &elt, depth + 1, ops, psize);
 	    if (code < 0)
 		return code;
-	    /* Check for {proc} if | {proc1} {proc2} ifelse */
-#define R_IS_OPER(pref, proc)\
-  (r_btype(pref) == t_operator && r_has_attr(pref, a_executable) &&\
-   (pref)->value.opproc == proc)
-	    if (R_IS_OPER(&elt2, zif)) {
+	    /* Check for { proc } repeat | {proc} if | {proc1} {proc2} ifelse */
+	    if (resolves_to_oper(i_ctx_p, &elt2, zrepeat)) {
+		/* We can't handle 'repeat' with pdfwrite since it emits FunctionType 4 */
+		if (strcmp(i_ctx_p->pgs->device->dname, "pdfwrite") == 0)
+		    return_error(e_rangecheck);
+		if (ops) {
+		    *p = PtCr_repeat;
+		    psc_fixup(p, ops + *psize);
+		    p = ops + *psize;
+		    *p++ = PtCr_repeat_end;
+		}
+		*psize += 1;	/* extra room for repeat_end */
+	    } else if (resolves_to_oper(i_ctx_p, &elt2, zif)) {
 		if (ops) {
 		    *p = PtCr_if;
 		    psc_fixup(p, ops + *psize);
 		}
 	    } else if (!r_is_proc(&elt2))
 		return_error(e_rangecheck);
-	    else if ((code == array_get(imemory, pref, ++i, &elt3)) < 0)
+    	    else if ((code == array_get(imemory, pref, ++i, &elt3)) < 0)
 		return code;
-	    else if (R_IS_OPER(&elt3, zifelse)) {
+	    else if (resolves_to_oper(i_ctx_p, &elt3, zifelse)) {
 		if (ops) {
 		    *p = PtCr_if;
 		    psc_fixup(p, ops + *psize + 3);
@@ -248,10 +277,9 @@ check_psc_function(i_ctx_t *i_ctx_p, const ref *pref, int depth, byte *ops, int
 		    return code;
 		if (ops)
 		    psc_fixup(p, ops + *psize);
-	    } else
+	    } else 
 		return_error(e_rangecheck);
-#undef R_IS_OPER
-	}
+	    }	 /* end 'default' */
 	}
     next:
 	DO_NOTHING;
